/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

/**
 * This file contains all the internal leri header file.
 */

#ifndef AMO_H_
#define AMO_H_

/*----- Google -----*/
#include <gflags/gflags.h>
#include <glog/logging.h>
//#include <gtest/gtest.h>

/*----- Common -----*/
#include "leri/common.h"
#include "leri/util/timer.h"
#include "leri/util/util.h"
#include "leri/util/strutil.h"
#include "leri/util/fileutil.h"

// /*----- Lipid-----*/
// #include "leri/lipid/lipid.h"

// /*----- Omics -----*/
// #include "leri/atgc/atgc.h"
// #include "leri/atgc/dna.h"
// #include "leri/omics/de_brujin_graph.h"
// #include "leri/omics/omics.h"
// #include "leri/omics/genomics.h"
// #include "leri/omics/kmer.h"
// // #include "leri/fastx.h"
// #include "leri/nanopore/nanopore.h"

// /*----- Computational Biology Utils -----*/
// #include "leri/protein/cbutil.h"
// #include "leri/protein/rama.h"
// #include "leri/protein/grid3d.h"
// #include "leri/util/vtf.h"

// /*----- Protein -----*/
// #include "leri/protein/sequence.h"
// #include "leri/protein/protein.h"

// /*----- Protein Folding -----*/
// #include "leri/folding/driver.h"
// #include "leri/folding/modules.h"
// #include "leri/folding/simulation.h"
// /*----- Coarse-Grained Molecular Dynamics -----*/
// //#include "leri/cgmd/coarse_grained_md.h"
// #include "leri/folding/cgmd.h"

// /*----- Sampling -----*/
// #include "leri/sampling/nested_sampler.h"
// #include "leri/sampling/mcmc.h"

// /*----- Solvers -----*/
// #include "leri/bias/objfunc.h"
// #include "leri/solvers/solver.h"
// #include "leri/solvers/aco.h"
// #include "leri/solvers/chpso.h"
// #include "leri/solvers/jde.h"
// // #include "leri/solvers/lbfgs.h" 

// /*----- Deep Learning -----*/
#include "leri/img/image.h" 
// #include "leri/net/ninet.h" 
// #include "leri/net/phsior.h" 
#include "leri/net/imgor.h" 
#include "leri/net/senet.h" 
// #include "leri/net/optifel.h" 
// #include "leri/net/nirecor.h" 

#include "leri/clustering/hcluster.h" 

// /*----- Statistics -----*/
// #include "leri/stats/ica.h"
// // #include "leri/stats/tsne.h"
// #include "leri/stats/pearson.h"

// /*----- Report -----*/
// #include "leri/gio/gnuplot.h" 
#include "leri/gio/html.h" 
// #include "leri/gio/json.h" 

/*----- Sys Info. -----*/
#include "leri/util/sysinfo.h" 

/*----- Basic namespace -----*/
using leri::string;
// //using leri::Timer;
using leri::vector;
// using std::ifstream;
// using leri::ostringstream;

// /*----- Leri namespace -----*/
using leri::print_text;
using leri::print_info;
// // using leri::Leri;
// using leri::Lipid;
// using leri::Omics;
// using leri::Genomics;
// using leri::Nanopore;
// using leri::Protein;
// using leri::Simulation;
// using leri::LSequence;
// using leri::ATGC;
// using leri::DNA;
// // using leri::Fastx;
// using leri::Kmer;
// using leri::DeBrujinGraph;

// #ifdef USE_OPENCV
// using leri::Image;
// #endif // USE_OPENCV

// // using leri::Graph;
// // using leri::Stats_PCA;
// // using leri::Stats_ICA;
// using leri::ICA;
// using leri::TSNE;
// using leri::Pearson;

// using leri::Gnuplot;
using leri::HTML;


// A simple registry for leri commands.
typedef int (*BrewFunction)();
typedef std::map<leri::string, BrewFunction> BrewMap;
BrewMap g_brew_map;
#define RegisterBrewFunction(func) \
namespace { \
class __Registerer_##func { \
 public: /* NOLINT */ \
  __Registerer_##func() { \
    g_brew_map[#func] = &func; \
  } \
}; \
__Registerer_##func g_registerer_##func; \
}
static BrewFunction GetBrewFunction(const leri::string& name);

// void get_gpus(vector<int>* gpus); // Parse GPU ids or use all available devices
// /*--- Info. ---*/
// // Device Query: show diagnostic information for a GPU device.
// int device();
// RegisterBrewFunction(device);

// /*--- Lipid Data ---*/
// int lipid();
// RegisterBrewFunction(lipid);
// /*--- Sequencing Data ---*/
// int sff2fastq();
// RegisterBrewFunction(sff2fastq);
// int base_call();
// RegisterBrewFunction(base_call);
// int atgc_assembly();
// RegisterBrewFunction(atgc_assembly);
// int atgc_stats();
// RegisterBrewFunction(atgc_stats);
// int atgc_quality();
// RegisterBrewFunction(atgc_quality);
// int read_filter();
// RegisterBrewFunction(read_filter);
// int kmer_generator();
// RegisterBrewFunction(kmer_generator);
// int kmer_counter();
// RegisterBrewFunction(kmer_counter);
// int atgc_correct();
// RegisterBrewFunction(atgc_correct);
// int atgc_dna2aa();
// RegisterBrewFunction(atgc_dna2aa);
// // TODO: 
// int synteny();
// RegisterBrewFunction(synteny);
// int gene_exp();
// RegisterBrewFunction(gene_exp)
// int cnv();
// RegisterBrewFunction(cnv);
// // int omics_stats();
// // RegisterBrewFunction(omics_stats);
// // int metaomics();
// // RegisterBrewFunction(metaomics);
// // int fastq_check();
// // RegisterBrewFunction(fastq_check);
// // int seq_trim();
// // RegisterBrewFunction(seq_trim);

// /*--- Protein Sequence ---*/
// int sequence_convertor(); // Convertor: jackhmmer output to fasta
// RegisterBrewFunction(sequence_convertor);
// int sequence_mix();       // Mixture: spectial for G & GPCR
// RegisterBrewFunction(sequence_mix);
// int sequence_trim();
// RegisterBrewFunction(sequence_trim);
// int sequence_stats();     
// RegisterBrewFunction(sequence_stats);
// int sequence_eca();
// RegisterBrewFunction(sequence_eca);
// int sequence_energy();
// RegisterBrewFunction(sequence_energy);
// int mutant_energy();
// RegisterBrewFunction(mutant_energy);
// int point_mutation();
// RegisterBrewFunction(point_mutation);
// int sequence_design();
// RegisterBrewFunction(sequence_design);
// int residue_contact();
// RegisterBrewFunction(residue_contact);
// //The stability of a protein is generally represented by the change in the Gibbs free energy upon folding (DG), where an increasingly negative number represents greater stability. The substitution of a single amino acid in a protein sequence can result in a significant change in the protein’s stability (DDG), where a positive DDG represents a destabilizing mutation and a negative value represents a stabilizing mutation. 
// // ddG = dG_wt - dG_mutant
// int sequence_ddg();
// RegisterBrewFunction(sequence_ddg);

// int fasta2contact();
// RegisterBrewFunction(fasta2contact);
// /*--- Protein Structure ---*/
// int fasta2pdb();
// RegisterBrewFunction(fasta2pdb);
// int pdb_parser(); 
// RegisterBrewFunction(pdb_parser);
// int pdb_voxel();
// RegisterBrewFunction(pdb_voxel);
// int pdb_contact();
// RegisterBrewFunction(pdb_contact);
// int sidechain(); 
// RegisterBrewFunction(sidechain);
// int fold(); 
// RegisterBrewFunction(fold);
// int fold_energy(); 
// RegisterBrewFunction(fold_energy);
// int folding(); 
// RegisterBrewFunction(folding);
// int modyfing(); 
// RegisterBrewFunction(modyfing);
// int phasing(); 
// RegisterBrewFunction(phasing);
// int rama_map(); 
// RegisterBrewFunction(rama_map);
// int vtf2pdb(); 
// RegisterBrewFunction(vtf2pdb);


// /*--- GPCR-G proteins ---*/
// int selectivity_sector();
// RegisterBrewFunction(selectivity_sector);
// /*--- Optimizer ---*/
// int opt_chpso(); 
// RegisterBrewFunction(opt_chpso);
// int opt_jde(); 
// RegisterBrewFunction(opt_jde);
// // int opt_lbfgs(); 
// // RegisterBrewFunction(opt_lbfgs);
// // int opt_gd(); 
// // RegisterBrewFunction(opt_gd);

// /*--- Sampling ---*/
// int nested_sampler();
// RegisterBrewFunction(nested_sampler);
// int mcmc();
// RegisterBrewFunction(mcmc);

// // /*--- Statistical Analysis Tools ---*/
// int stats_pearson();
// RegisterBrewFunction(stats_pearson);
// int stats_ica();
// RegisterBrewFunction(stats_ica);
// int stats_tsne();
// RegisterBrewFunction(stats_tsne);

/*--- Deep learning ---*/
// int optifel();                 // CHPSO + fuzzy model
// RegisterBrewFunction(optifel);
// int nirecor();                 // CHPSO + FNN
// RegisterBrewFunction(nirecor);
// int nirecor_regression();                 // CHPSO + FNN:  regression
// RegisterBrewFunction(nirecor_regression);
int phsior();
RegisterBrewFunction(phsior);
int imgor();
RegisterBrewFunction(imgor);
int ninet();
RegisterBrewFunction(ninet);
int senet();
RegisterBrewFunction(senet);
int hcluster();
RegisterBrewFunction(hcluster);


// // int fmri_learning();
// // RegisterBrewFunction(fmri_learning);

// /*--- Clinical Research ---*/
// // int cancer_estimator();

// /*--- Image Tools ---*/
int img2ubyte();
RegisterBrewFunction(img2ubyte);

/*--- HTML report ---*/
int html();
RegisterBrewFunction(html);

/*--- Info. of Leri ---*/
int about();
RegisterBrewFunction(about);
int usage();
RegisterBrewFunction(usage);





// // int test();  // TODO(test): test
// // RegisterBrewFunction(test);
// // int test_gpu();
// // RegisterBrewFunction(test_gpu);



// // DEFINE_bool: boolean
// // DEFINE_int32: 32-bit integer
// // DEFINE_int64: 64-bit integer
// // DEFINE_uint64: unsigned 64-bit integer
// // DEFINE_double: double
// // DEFINE_string: C++ string
// // DEFINE_bool(info, false, 
// //     "show info of Leri or not");
// // DEFINE_bool(flag, false, 
// //     "flag");


DEFINE_bool(lhttp, false, 
    "'-lhttp 0', switch on/off leri web-server");
DEFINE_int32(hlog, 0, 
    "'-hlog 0', remove log file when it is completed");

// DEFINE_string(gpu, "0",
//     "optional | run in GPU mode on given device IDs separated by ',', "
//     "e.g. '-gpu 1,2'. '-gpu all' is to run it on all available GPUs");

// Parallel
DEFINE_int32(threads, 1,
    "number of threads");

// /*--- Directory ---*/
// DEFINE_string(cur_path, "",
//     "current directory where to launch Leri");
DEFINE_string(dirt, "",
    "directory where to load the data");
DEFINE_string(output, "",
    "optional | the path for the output of Leri");

// DEFINE_string(www, "",
//     "address of the HTML report");
DEFINE_string(jobname, "",
    "name of a job");

/*--- General Options ---*/
DEFINE_int32(type, -1,
    "type of compuation");
DEFINE_int32(phase, -1,
    "phase used in protein tasks"
    "0 for fasta, 1 for pdb, 2 for both of them, "
    "phase in learning tasks: 0 for training wile 1 for prediction");
// DEFINE_int32(kmer, 3,
//     "kmer");
// DEFINE_int32(n, -1,
//     "the value of n is different denpending on the function that is called");
DEFINE_int32(max_iter, 2000,
    "optional | the number of iterations to run");
// DEFINE_int32(intv, 1,
//     "optional | the interval for snapshot");
// DEFINE_int32(ntrial, 5,
//     "optional | the number of iterations to run");
// DEFINE_int32(npop, 16,
//     "optional | the number of population in some heuristic methods");
// DEFINE_int32(nvar, 30,
//     "optional | the number of population in some heuristic methods");
DEFINE_int32(nrule, 3,
    "optional | the number of fuzzy rules in OptiFel");
DEFINE_int32(nout, -1,
    "optional | the number of outputs in OptiFel and NiRecor");
// DEFINE_int32(offset, 0,
//     "optional | the offset of index");

// DEFINE_double(min, -1000,
//     "optional | lower boundary");
// DEFINE_double(max,  1000,
//     "optional | upper boundary");
// DEFINE_double(threshold, -1.0,
//     "threshold (optional), -1.0 means the value is not defined. "
//     "For protein sequence, the threshold is 0.8. "
//     "For Pearson's correlation, the threshold is 0.75. "
//     "For residue contact, the default value is 7.5");

DEFINE_bool(gz, false,
    "is gzip file or not");
// DEFINE_bool(re, false,
//     "re-do something");
// DEFINE_bool(backbone, false,
//     "only backbone");

// /*--- Sequence ---*/
// DEFINE_string(fastx, "",
//     "the FASTA or FASTQ file");
// DEFINE_string(sff, "",
//     "the SFF file");
// DEFINE_string(mat, "",
//     "the matrix text file");
// DEFINE_string(msa, "",
//     "the MSA text file");
// DEFINE_string(pdb, "",
//     "the PDB file");
// DEFINE_string(dseq, "",
//     "the FASTA file of the designed sequence");
// DEFINE_double(gap_frac, 0.20,
//     "optional | gap fraction in a sequence in (0, 1)");
// DEFINE_double(site_gap_frac, 1.0,
//     "optional | gap fraction in a sequence in (0, 1)");
// DEFINE_double(cnsv_frac, 0.50,
//     "optional | conservation fraction in a MSA in (0, 1)");
// DEFINE_double(temperature, 1.0,
//     "optional | temperature for simulation");

// /*--- Structure ---*/
DEFINE_string(index, "",
    "list of indices");
// DEFINE_string(chain, "",
//     "protein chain, e.g. A, or B, or C, ...");
// DEFINE_string(atom_type, "CA",
//     "residue contacts derived from atom CA or CB");
DEFINE_string(cfg, "",
    "the paramter settings text file");
// DEFINE_string(vtf, "",
//     "Valve texture format (VTF)");

// /*--- Control Parameters ---*/
// DEFINE_double(alpha, 0.0,
//     "parameter alpha");
// DEFINE_double(beta, 0.0,
//     "parameter beta");
// DEFINE_double(rho, 0.0,
//     "parameter rho");
// DEFINE_double(tau, 0.0,
//     "parameter tau");
// DEFINE_double(perplexity, 20.0,
//     "parameter used in t-SNE");

// /*--- Deep Learning ---*/
DEFINE_string(solver, "adam",
    "optional | the type of solver: 'sgd', 'rmsprop', 'adagrad', 'adam'");
DEFINE_int32(echo, 20,
    "optional | number of echoes");
DEFINE_int32(mbs, 32,
    "optional | size of the mini batch");
DEFINE_int32(win, 11,
    "optional | size of the sliding window, and it should be even number");
DEFINE_int32(w, 28,
    "optional | width of an image");
DEFINE_int32(h, 28,
    "optional | height of an image");
// DEFINE_int32(c, 1,
//     "optional | channel of an image");
DEFINE_double(lrate, 0.04f,
    "optional | learning rate");
DEFINE_string(param, "",
    "the paramter settings text file");
// DEFINE_string(net, "",
//     "configuration of network");

DEFINE_string(train_data, "",
    "training samples");
DEFINE_string(train_label, "",
    "training labels");
DEFINE_string(test_data, "",
    "test samples");
DEFINE_string(test_label, "",
    "test labels");

/*--- Job information ---*/
DEFINE_string(taskname, "Untitled",
    "task name");
DEFINE_string(username, "Anonymous",
    "user name");
DEFINE_string(jobid, "",
    "job id");
DEFINE_string(jobcpt, "0",
    "jobs completed");
DEFINE_string(jobwat, "0",
    "jobs in waiting");
DEFINE_string(jobrun, "0",
    "jobs in running");

#endif  // AMO_H_



















