/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

/**
 * This file contains algorithm of Burrows-Wheeler Transform and the Alignment.
 */

#ifndef LERI_BWTA_H_
#define LERI_BWTA_H_

#include "leri/common.h"

namespace leri {


//****************** BAM FLAG*********************************************
/*! @abstract the read is paired in sequencing, no matter whether it is mapped in a pair */
#define BAM_FPAIRED        1
/*! @abstract the read is mapped in a proper pair */
#define BAM_FPROPER_PAIR   2
/*! @abstract the read itself is unmapped; conflictive with BAM_FPROPER_PAIR */
#define BAM_FUNMAP         4
/*! @abstract the mate is unmapped */
#define BAM_FMUNMAP        8
/*! @abstract the read is mapped to the reverse strand */
#define BAM_FREVERSE      16
/*! @abstract the mate is mapped to the reverse strand */
#define BAM_FMREVERSE     32
/*! @abstract this is read1 */
#define BAM_FREAD1        64
/*! @abstract this is read2 */
#define BAM_FREAD2       128
/*! @abstract not primary alignment */
#define BAM_FSECONDARY   256


template <typename Dtype>
class BWTA {
public:
  BWTA();
  ~BWTA();

  void bwta_main();

  void alignment(string seq, string quality, vector<int>& result, int where_read_is_from);
  int * findline(char lastbase1, char base, int baseF, int baseL);
  int findone(char base, int baseF);
  void alignOne(string seq,vector<int>& result);
  void split(string target, vector<string>& strs);
  string read_reverse(string read);
  int flag(int read);

  int A1;
  int A2;
  int C1;
  int C2;
  int G1;
  int G2;
  int T1;
  int T2;
  int A, C, G, T, ref_length;

  int baseF_vector[20];
  int baseL_vector[20]; // ASCII of AGCT - 65 = 0，2，6，19

  vector<string> tally; 

  // FOR BAM FLAG
  bool paired = true;
  bool paired2 = true;
  bool proper_pair = true;
  bool proper_pair2 = true;
  bool unmap = true;
  bool unmap2 = true;
  bool mun_map = true;
  bool mun_map2 = true;
  bool reverse1 = false;
  bool reverse2 = false;
  bool mreverse = false;
  bool mreverse2 = false;

  // FOR SAM CIGAR
  bool exact_match = true;

protected:
  DISABLE_COPY_AND_ASSIGN(BWTA);
}; // class BWTA


} // namespace leri
#endif // LERI_BWTA_H_
