/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_LOSS_H_
#define LERI_LOSS_H_

#include "leri/common.h"
namespace leri {

// returns Energy (euclidian distance / 2) and max index
float match_labels(const float *out, const float *target, const int size, int *best_index = NULL);

namespace mse {
  inline float  cost(float out, float target) { return 0.5f*(out-target)*(out-target); };
  inline float  d_cost(float out, float target) { return (out-target); };
  const char name[] = "mse";
}
namespace cross_entropy {
  inline float  cost(float out, float target) {return (-target * std::log(out) - (1.f - target) * std::log(1.f - out));};
  inline float  d_cost(float out, float target) {return ((out - target) / (out*(1.f - out)));};
  const char name[] = "cross_entropy";
}

typedef struct {
public:
  float (*cost)(float, float);
  float (*d_cost)(float, float);
  const char *name;
} loss_func;

loss_func* new_loss_func(string loss);
loss_func* new_loss_func(const char *type);

} // namespace leri
#endif // LERI_LOSS_H_
