/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_OBJ_FUNC_H_
#define LERI_OBJ_FUNC_H_

#include "leri/common.h"
#include "leri/util/matrix.h"
#include "leri/solvers/chpso.h"
#include "leri/solvers/jde.h"

namespace leri {

template <typename Dtype>
class ObjFunc {
public:
  ObjFunc(string output_dirt = "", string job_name = "", int nvar = 20, int npop = 8, int max_iter = 2e3, int ntrial = 30)
    : output_dirt_(output_dirt),
      job_name_(job_name),
      max_iter_(max_iter), 
      nvar_(nvar),
      npop_(npop),
      ntrial_(ntrial) {
      base_name_ = output_dirt_ + job_name_;
    }
  ~ObjFunc(){}

  void init() {}
  void chpso();
  void jde();
  void finish();

private:
  static Dtype objective_func(const Dtype* xvar, const int n, void* instance, Dtype* gvar = NULL, Dtype** mout = NULL);

private:
  string output_dirt_;
  string job_name_; 
  string base_name_;
  int max_iter_;
  int nvar_;
  int npop_;
  int ntrial_;
  Dtype* xvar_;
protected:
  DISABLE_COPY_AND_ASSIGN(ObjFunc);
}; // class ObjFunc


}  // namespace leri

#endif  // LERI_OBJ_FUNC_H_
