/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_FORCE_H
#define LERI_FORCE_H

#include "leri/common.h"
#include "leri/cgmd/cgmdutil.h"

namespace leri {

template <typename Dtype>
class CGMDEngine {
public:
  CGMDEngine(int n_atom, int n_system)
  :potential(n_system) {
    nodes.emplace_back("pos", unique_ptr<DerivComputation>(new Pos(n_atom, n_system)));
    pos = dynamic_cast<Pos*>(nodes[0].computation.get());
  }
  CGMDEngine(int n_atom, int n_system, hid_t potential_group, bool quiet = false)
  :potential(n_system) {
    nodes.emplace_back("pos", unique_ptr<DerivComputation>(new Pos(n_atom, n_system)));
    pos = dynamic_cast<Pos*>(nodes[0].computation.get());
    // LOG(INFO)<<"xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx";
    initialize_engine_from_hdf5(n_atom, n_system, potential_group, quiet);
    // LOG(INFO)<<"xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx";
  }

  void initialize_engine_from_hdf5(int n_atom, int n_system, hid_t potential_group, bool quiet = false);
  float get_energy() { energy_ = 0; for (int i = 0; i < potential.size(); i++) { energy_ += potential[i]; } return energy_; }
  void add_node(const string& name, unique_ptr<DerivComputation>&& fcn, vector<string> argument_names);

  Node& get(const string& name);
  int get_idx(const string& name, bool must_exist = true);
  double get_n_hbond() {
    return get_idx("hbond_energy",false) == -1 ? -1.f : get_computation<HBondCounter>("hbond_energy").n_hbond;
  }

  template <typename T>
  T& get_computation(const string& name) {
    return dynamic_cast<T&>(*get(name).computation.get());
  }

  void compute(ComputeMode mode);
  enum IntegratorType {Verlet = 0, Predescu = 1};
  void integration_cycle(float* mom, float dt, float max_force, IntegratorType type = Verlet);
public:
  vector<Node> nodes;  // nodes[0] is the pos node
  Pos* pos;
  float energy_;
  vector<float> potential;
};


} // namespace leri
#endif // LERI_FORCE_H
