/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_MD_EXPORT_H
#define LERI_MD_EXPORT_H

#include "leri/cgmd/coord.h"

namespace leri {

// Yaan: from deriv_engine.h
struct DerivRecord {
  index_t atom;
  slot_t  loc, output_width;
  DerivRecord(index_t atom_, slot_t loc_, slot_t output_width_)
  :atom(atom_),
   loc(loc_),
   output_width(output_width_) {}
};

template <typename MutableCoordT, typename StaticCoordT>
void integration_stage_body(MutableCoordT &mom, MutableCoordT &pos, StaticCoordT  &deriv, float vel_factor, float pos_factor, float max_force) {
  // assumes unit mass for all particles
  float3 f = deriv.f3();
  if (max_force) {
    float f_mag = mag(f)+1e-6f;  // ensure no NaN when mag(f)==0.
    float scale_factor = atan(f_mag * ((0.5f*M_PI_F) / max_force)) * (max_force/f_mag * (2.f/M_PI_F));
    f *= scale_factor;
  }
  mom.set_value(mom.f3() - vel_factor*f);
  pos.set_value(pos.f3() + pos_factor*mom.f3());
}

void integration_stage(
        SysArray mom,
        SysArray pos,
        const SysArray deriv,
        float vel_factor,
        float pos_factor,
        float max_force,
        int n_atom, int n_system);

void deriv_accumulation(
        SysArray deriv, 
        const SysArray accum_buffer, 
        const DerivRecord* tape,
        int n_tape,
        int n_atom,
        int n_system);

void recenter(SysArray pos, bool xy_recenter_only, int n_atom, int n_system);

} // namespace leri
#endif // LERI_MD_EXPORT_H

