/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_THERMOSTAT_H_
#define LERI_THERMOSTAT_H_

#include "leri/common.h"
#include "leri/cgmd/coord.h"
#include "leri/cgmd/timing.h"
// #include "leri/util/random.h"
#include "leri/cgmd/random.h"

namespace leri {

// https://en.wikipedia.org/wiki/Ornstein%E2%80%93Uhlenbeck_process
template <typename Dtype>
class Thermostat {
public:
  Thermostat(uint32_t random_seed_, float timescale_, vector<float> temp_, float delta_t_)
  :n_invocations(0),
   n_system(temp_.size()),
   random_seed(random_seed_),
   timescale(timescale_),
   delta_t(delta_t_),
   temp(temp_),
   noise_scale(n_system) {
    update_parameters();
  }
  void set_timescale(float timescale_) {
    timescale = timescale_;
    update_parameters();
  }
  void set_temp (vector<float> temp_) {
    temp = temp_;
    update_parameters();
  }
  void set_delta_t  (float delta_t_) {
    delta_t = delta_t_;
    update_parameters();
  }
  void apply(SysArray mom, int n_atom) {
    TimerUpside timer(string("thermostat"));
    ornstein_uhlenbeck_thermostat(mom, mom_scale, noise_scale.data(), random_seed, n_atom, n_invocations++, n_system);
  }
private:
  void update_parameters();
  void ornstein_uhlenbeck_thermostat(SysArray mom, const float mom_scale,const float* noise_scale, uint32_t seed, int n_atoms, uint64_t n_round, int n_system);
public:
  const int n_system;
  uint32_t random_seed;
  float timescale;
  float delta_t;
  float mom_scale;
  vector<float> temp;
  vector<float> noise_scale;
private: 
  uint64_t n_invocations;

};

} // namespace leri

#endif
