/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright (C) 2015- All Rights Reserved - Yaan J. Jang.
 * Contact: yaan.jang 'AT' gmail.com
 *
 */

/**
 * This file contains of Coarse-Grained Molecular Dynamics
 */

#ifndef LERI_CGMD_H_
#define LERI_CGMD_H_

#include "leri/common.h"
#include "leri/util/random.h"
#include "leri/util/malloc.h"
#include "leri/util/strutil.h"
#include "leri/util/fileutil.h"
#include "leri/gio/figure.h"
#include "leri/protein/sequence.h"
#include "leri/protein/chain.h"
#include "leri/folding/energy.h"
#include "leri/folding/mdconfig.h"



#include "leri/cgmd/pivot_sampler.h"
#include "leri/cgmd/h5_support.h"
// #include "leri/cgmd/cgmdutil.h"
#include "leri/cgmd/deriv_engine.h"
// #include "leri/cgmd/timing.h"
#include "leri/cgmd/thermostat.h"
#include "leri/cgmd/md_export.h"
#include "leri/cgmd/random.h"
#include "leri/cgmd/state_logger.h"

#define BIG_PRIME 4294967291ul  // largest prime smaller than 2^32

namespace leri {

template <typename Dtype> 
class CGMD {
public:
  explicit CGMD(string fseq, string output_dirt = "", string job_name = "", int max_iter = 5e3, int intv = 1, string postfix = "_config.h5")
  :fseq_(fseq),
   output_dirt_(output_dirt),
   job_name_(job_name),
   postfix_(postfix),
   max_iter_(max_iter),
   intv_snapshot_(intv) {
    base_name_ = output_dirt_ + job_name_;
  }
  ~CGMD() {}

  void init(string cfg);
  void actuate();
  void finish();

  void set_temperature(Dtype temp) { temperature_ = temp; }
  void set_system(Chain<Dtype>* chain);

private:
  void init_system();
  // void build_init_struct();
  void parallel_tempering_step(
        uint32_t seed, uint64_t round,
        vector<int>& coord_indices,
        const vector<float>& temperatures, CGMDEngine<float>& engine);
  void deriv_matching(hid_t config, CGMDEngine<float>& engine, bool generate, double deriv_tol=1e-3);
  vector<float> potential_deriv_agreement(CGMDEngine<float>& engine);

private:
  string fseq_;
  string output_dirt_;    // output directory
  string job_name_;       // job name
  string base_name_;      // basic name
  string postfix_;

  string file_ndrd_;      // Neighbor-dependent Ramachandran Distributions
  string file_bdrl_;      // Backbone-dependent rotamer library
  string file_rmjp_;      // Radial MJ-1996 potential
  string file_phsi_;      // Torsion angles
  string file_dist_;      // Distance map
  string cfg_;

  int max_iter_;
  int intv_snapshot_;
  int seed_;
  int num_system_;
  int num_aa_;

  Dtype temperature_;     // TODO: different temperatures for multiple systems
  Dtype time_step_;

  LSequence<Dtype> sequence_;
  // Protein<Dtype>* protein_;
  Chain<Dtype>* chain_;
  int num_of_atoms_;

  // Dtype* coords_; // n_sys*n_N_CA_C*3
  Dtype* coords_cur_;
  Dtype* coords_opt_;
};

} // namespace leri
#endif // LERI_CGMD_H_
