/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright (C) 2015- All Rights Reserved - Yaan J. Jang.
 * Contact: yaan.jang 'AT' gmail.com
 *
 */

/**
 * This file contains the driver for protein folding simulation
 *
*/

#ifndef LERI_DRIVER_H_
#define LERI_DRIVER_H_

#include "leri/common.h"
#include "leri/util/random.h"
#include "leri/util/strutil.h"
#include "leri/sampling/dsa.h"

namespace leri {

template <typename Dtype>
class Driver {
public:
  Driver(string output_dirt = "", string job_name = "", int mod = 0)
  :output_dirt_(output_dirt),
   job_name_(job_name),
   sampling_mod_(mod) {
    base_name_ = output_dirt_ + job_name_;
  }
  ~Driver() {}

  void init(string cfg);
  void actuate();
  void finish(void);

  void set_system(Chain<Dtype>* chain) { chain_ = chain; }
private:
  void execute_init_func_sampling_module(int mod, const string cfgname, Chain<Dtype>* chain);
  void execute_prev_func_sampling_module(int mod);
  void execute_step_func_sampling_module(int mod);
  void execute_post_func_sampling_module(int mod);
  void execute_finish_func_sampling_module(int mod);

public:
  DSA<Dtype> dsa_;
  string sampling_name_;
  string sampling_cfg1_;
  string sampling_cfg2_;
  int num_steps_;
  bool prev_;
  bool post_;
private:
  string output_dirt_;    // output directory
  string job_name_;       // job name
  string base_name_;
  int sampling_mod_;

  Chain<Dtype>* chain_;
protected:
  DISABLE_COPY_AND_ASSIGN(Driver);
};

} // namespace leri
#endif // LERI_DRIVER_H_
