/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

/**
 * This file contains the implementation of calculating burial energy
 *
 */

#ifndef LERI_BURIAL_ENERGY_H_
#define LERI_BURIAL_ENERGY_H_
#include "leri/common.h" 
#include "leri/protein/chain.h"

namespace leri {

template <typename Dtype>
class BurialE {
public:
  BurialE() {}
  ~BurialE() {}

  void init(string cfg);
  void prev_calc_energy(Chain<Dtype>* chain);
  void calc_energy(Dtype* energy);
  void calc_gradient(Dtype* gradient);
  void post_calc_energy(void);
  void finish(void);

  
  void set_quiet(bool q = false) { quiet_ = q; }
  void set_system(Chain<Dtype>* chain);

private:
  void read_cfg(string cfg);
  void load_par_file(void);
  int is_helix(int idx_aa);
  int is_beta(int idx_aa);

  int is_NH_bonded(int idx_aa_1, int idx_aa_2);
  int is_CO_bonded(int idx_aa_1, int idx_aa_2);
  /* Calculate burial
   * NH = 1, CO = 2, and 0 is not to count the hbonds (i.e. on CB burial)
   */
  Dtype calc_burial(int idx_atom_1, int idx_atom_2, Dtype radius, int ignoreN, Dtype coneAngle, int idx_aa, int COorNH = 0, int *hbond = NULL);
  // Dtype calc_burial_cb(int idx_atom_1, int idx_atom_2, Dtype radius, int ignoreN, Dtype coneAngle, int idx_aa, int COorNH = 0, int *hbond = NULL);
private:
  bool quiet_;
  // // // Protein<Dtype>* protein_;
  Chain<Dtype>* chain_;
  int nchains_;
  int naas_;
  int natoms_;
  string CB_par_file_name_;
  string CO_par_file_name_;
  string NH_par_file_name_;
  vector<vector<Dtype> > co_par_;
  vector<vector<Dtype> > nh_par_;
  vector<vector<Dtype> > cb_par_;

  int no_helix_;
  Dtype beta_coeff_;
  struct P {
    Dtype CObinsize;
    Dtype NHbinsize;
    Dtype CBbinsize;
    Dtype CObins;
    Dtype NHbins;
    Dtype CBbins;
    Dtype COthreshold;
    Dtype NHthreshold;
    Dtype CBthreshold;
    int    COneighbors;
    int    NHneighbors;
    int    CBneighbors;
    Dtype COconeangle;
    Dtype NHconeangle;
    Dtype CBconeangle;
    Dtype COweight;
    Dtype NHweight;
    Dtype CBweight;
  } param_;

protected:
  DISABLE_COPY_AND_ASSIGN(BurialE);
};


} // namespace leri

#endif // __LERI_BURIAL_ENERGY_H_
