/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

/**
 * This file contains the implementation of calculating residue-contact bias energy
 *
 */

#ifndef LERI_CNCE_H_
#define LERI_CNCE_H_
#include "leri/common.h" 
#include "leri/protein/chain.h"

namespace leri {

template <typename Dtype>
class CNCE {
public:
  CNCE() {}
  ~CNCE() {}
  void init(string cfg);
  void prev_calc_energy(Chain<Dtype>* chain);
  void calc_energy(Dtype* energy);
  void calc_gradient(Dtype* gradient);
  void post_calc_energy(void);
  void finish(void);

  void set_quiet(bool q = false) { quiet_ = q; }
  void set_system(Chain<Dtype>* chain) {} // TODO: delete

private:
  void read_cfg(string cfg);
  // void load_par_file(void);
  int is_contact(int idx_aa_1, int idx_aa_2);

private:
  bool quiet_;
  Chain<Dtype>* chain_;
  string cm_file_;
  int num_aa_;
  // Dtype** res_contact_mat_;
  vector<vector<Dtype> > res_contact_mat_;
  Dtype weight_;
  Dtype cutoff_; // Default = 6.0 or 7.5
  // Protein<Dtype>* protein_;
protected:
  DISABLE_COPY_AND_ASSIGN(CNCE);
};

} // namespace leri
#endif // LERI_CNCE_H_
