/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

/**
 * This file contains the implementation of calculating DOPE-CB energy
 *
 */

#ifndef LERI_DOPE_CB_H_
#define LERI_DOPE_CB_H_
#include "leri/common.h" 
#include "leri/protein/chain.h"

namespace leri {

template <typename Dtype>
class DOPE_CB {
public:
  DOPE_CB() {}
  ~DOPE_CB() {}

  void init(string cfg);
  void prev_calc_energy(Chain<Dtype>* chain);
  void calc_energy(Dtype* energy);
  void calc_gradient(Dtype* gradient);
  void post_calc_energy(void);
  void finish(void);
  
  void set_quiet(bool q = false) { quiet_ = q; }
  void set_system(Chain<Dtype>* chain) {}

private:
  void read_cfg(string cfg);
  void load_par_file(void);

private:
  bool quiet_;
  // Protein<Dtype>* protein_;
  Chain<Dtype*> chain_;
  Dtype weight_;
  // vector<vector<Dtype> > res_contact_mat_;
protected:
  DISABLE_COPY_AND_ASSIGN(DOPE_CB);
};

} // namespace leri
#endif // LERI_DOPE_CB_H_
