/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

/**
 * This file contains the implementation of calculating DOPEPW energy
 *f
 */

#ifndef LERI_DOPE_PW_H_
#define LERI_DOPE_PW_H_
#include "leri/common.h"
#include "leri/gio/figure.h"
#include "leri/protein/atom.h"
#include "leri/protein/aminoacid.h"
#include "leri/protein/chain.h"
// #include "leri/protein/protein.h"

namespace leri {

// #define distance_index(atom0, atom1, natoms)  (atom0 * natoms + atom1)
#define convert_to_dopepw_row(term,aa0,at0,aa1,at1) (768000*term+aa0*38400+aa1*1920+at0*240+at1*30)
#define convert_to_dopepw_pos(term,aa0,at0,aa1,at1,bin) (768000*term+aa0*38400+aa1*1920+at0*240+at1*30+bin)

// IntValue convert_to_dope_row(TypeCode aa0,TypeCode at0,TypeCode aa1,TypeCode at1){return aa0*38400+aa1*1920+at0*240+at1*30;}
// IntValue convert_to_dope_pos(TypeCode aa0,TypeCode at0,TypeCode aa1,TypeCode at1,IntValue bin){return aa0*38400+aa1*1920+at0*240+at1*30+bin;}

#define MAXBINNUM 29
#define MAXATCODE 7
#define MAXAACODE 19
#define MINCD 2      // sets minimum chain distance to consider calculating
#define NUMTERMS  8

#define EECONT  0
#define HHCONT  1
#define CCCONT  2
#define EEap    3
#define EEp     4
#define PW0Ancont 5
#define PW1Ancont 6
#define PW2Ancont 7

#define HELIX 0
#define STRAND 1
#define COIL 2
#define PARALLEL 1
#define ANTIPARALLEL 2




template <typename Dtype>
class DOPE_PW {
public:
  // DOPE_PW():
  // num_of_bins_{MAXBINNUM+1, MAXBINNUM+1, MAXBINNUM+1, MAXBINNUM+1, MAXBINNUM+1, MAXBINNUM+1, MAXBINNUM+1, MAXBINNUM+1},
  // dist_cutoffs_{(Dtype)15.0, (Dtype)15.0, (Dtype)15.0, (Dtype)30.0, (Dtype)30.0, (Dtype)30.0, (Dtype)30.0, (Dtype)30.0},
  // coeffs_{(Dtype)1.0, (Dtype)1.0, (Dtype)1.0, (Dtype)1.0, (Dtype)1.0, (Dtype)1.0, (Dtype)1.0, (Dtype)1.0} {}
  DOPE_PW() {}
  ~DOPE_PW() {}

  void init(string cfg);
  void prev_calc_energy(Chain<Dtype>* chain);
  void calc_energy(Dtype* energy);
  void calc_gradient(Dtype* gradient);
  void post_calc_energy(void);
  void finish(void);

  
  void set_quiet(bool q = false) { quiet_ = q; }
  void set_system(Chain<Dtype>* chain);

private:
  void set_memory();
  void free_memory();
  void read_cfg(string cfg);
  void load_par_file(void);




  void set_active_pairs();
  void set_update_pairs();
  void is_helix();
  void is_beta();
  void set_continuity();
  void term_assign();


private:
  bool quiet_;
  string cfg_;
  // Protein<Dtype>* protein_;
  Chain<Dtype>* chain_;
  int nchains_;
  int naas_;
  int natoms_;
  // Dtype* dist_mtx_;

  Dtype* e_dope_; // dopeenergies











  // #define NUMTERMS  8
  static const int num_of_terms_ = 8;
  // // string par_file_name_;
  string par_file_names_[8];

  // int num_of_bins_[8];
  int* num_of_bins_;
  // Dtype dist_cutoffs_[8];// = {15.0,15.0,15.0,30.0,30.0,30.0,30.0,30.0};
  Dtype* dist_cutoffs_;// = {15.0,15.0,15.0,30.0,30.0,30.0,30.0,30.0};
  // Dtype coeffs_[8];// = {1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0};
  Dtype* coeffs_;// = {1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0};
  // int convtodope_[10];// = {-1, 0, 2, 6, 7, 1, 5, -1, 3, 4};
  map<string, int> conv2dope_;// = {-1, 0, 2, 6, 7, 1, 5, -1, 3, 4};
  Dtype bin_size_[8];
  // static const int idx_aa_min_ = 2;// sets minimum chain distance to consider calculating


  // vector<string> atom_names_;
  // vector<int> aa_idxs_;


  int n_update_pairs_;
  int n_active_pairs_;

  int * vec_pw_;
  int * vec_pap_;
  int * vec_ee_;
  int * parallel_align_;
  int * antiparallel_align_;
  int * vec_hoh_bond_;
  int * vec_ohh_bond_;
  int * coco_align_;
  int * nhnh_align_;
  int * nhco_align_;
  int * conh_align_;

  int * update_pairs_;
  int * active_pairs_;
  int * vec_sec_str_;  // array to hold secondary structure numbers. H<->0, E<->1, Other<->2
  int * vec_continuity_;

  int * pair_rows_;  // pairrows
  int * pair_terms_; // pairterms
  Dtype* pair_e_; // pairenergies
  Dtype* pair_coeffs_; // paircoeffs




  // // vector<vector<Dtype> > pairenergies;
  // Dtype** pairenergies;
  // Dtype** paircoeffs;
  // int** pairrows;
  // int** pairterms;

  // vector<int> updatepairs;
  // vector<int> active_pairs_;
  // vector<int> vec_sec_str_;  // array to hold secondary structure numbers. H<->0, E<->1, Other<->2
  // vector<int> vec_continuity_;
  // int** update_pairs_;
  // int* updatepairs;
  // int** active_pairs_;

  // int** papvec;
  // int** eevec;
  // int** parallel_align_;
  // int** antiparallel_align_;
  // int** vec_hoh_bond_;
  // int** vec_ohh_bond_;
  // int** coco_align_;
  // int** nhnh_align_;
  // int** nhco_align_;
  // int** coco_align_;

  // Dtype *distances;           // Array of atom pair-wise distances.
  // vector<vector<Dtype> > distances;
  // Dtype** distances;
  // bool *distmask;             // Array of distance masking values.  
  // vector<vector<bool> > distmask;
  // bool** distmask;

  
  // vector<Dtype> dopeenergies;










































public:
  // Module<Dtype> module;
   // Local copy of system variables.
  // struct AtomData *atoms;          // Array of atom data.
  // struct ResidueData *residues;    // Array of residue data.
  // struct ChainData *chains;        // Array of chain data.

  // Dtype *coords;              // Array of atom coordinates.
  // vector<vector<Dtype> > coords;

  // Dtype *invdistances;        // Array of atom pair-wise 1/distances.
  // bool *atommask;             // Array of atom masking values.








  int USE_FULL;
  // int n_active_pairs_;

  // int dopearraysize;

  // vector<int> updatepairs;
  // vector<int> active_pairs_;
  // // vector<vector<Dtype> > paircoeffs;
  // // vector<vector<Dtype> > pairrows;
  // // vector<vector<Dtype> > pairterms;


  // // emum {numbers = 8};

  // // Dtype* pairenergies;
  // //pair arrays
  // // vector<vector<int> > vec_pw_;
  // // vector<vector<int> > papvec;
  // // vector<vector<int> > eevec;
  // // vector<vector<int> > parallel_align_;
  // // vector<vector<int> > antiparallel_align_;
  // // vector<vector<int> > vec_hoh_bond_;
  // // vector<vector<int> > vec_ohh_bond_;
  // // vector<vector<int> > coco_align_;
  // // vector<vector<int> > nhnh_align_;
  // // vector<vector<int> > nhco_align_;
  // // vector<vector<int> > coco_align_;

  // vector<int> vec_sec_str_;  // array to hold secondary structure numbers. H<->0, E<->1, Other<->2
  // vector<int> vec_continuity_;
  // void four_atom_angle(Dtype *angle,int head1,int tail1,int head2,int tail2);
  // void print_pair_data_pw(int at1,int at2);
  // void setup_pw_pair_vecs(void);
  // void delete_pw_pair_vecs(void);
  void set_all_pair_vecs(int i, int j, int integer);
  // void set_system_for_dopepw(struct AtomData *_atoms, Dtype *_coords, 
  //                            Dtype *_distances, Dtype *_invdistances, 
  //                            bool *_atommask, bool *_distmask, int _natoms, 
  //                            struct ResidueData *_residues, int _nres, 
  //                            struct ChainData *_chains, int _nchains);
  // void load_dopepw_parfile(const char* parfilename);





  






private:
  // int natoms;               // Total number of atoms.
  // // int nres;                 // Total number of residues.
  // int nchains;              // Total number of chains.
  // vector<Atom<Dtype>> atoms_new;
  // vector<AminoAcid<Dtype>> residues_new;
  // Chain<Dtype> chains_new;
  // vector<vector<Dtype> > coords; // Array of atom coordinates.
  // vector<vector<Dtype> > coords_saved;
  // vector<vector<Dtype> > coords_opt;
protected:
  DISABLE_COPY_AND_ASSIGN(DOPE_PW);
};















} // namespace leri 

#endif
