/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

/**
 * This file contains the implementation of calculating hydrogen-bond energy
 *
 */

#ifndef LERI_HNCE_H_
#define LERI_HNCE_H_
#include "leri/common.h" 
#include "leri/protein/chain.h"

namespace leri {

template <typename Dtype>
class HNCE {
public:
  HNCE() {}
  ~HNCE() {}

  void init(string cfg);
  void prev_calc_energy(Chain<Dtype>* chain);
  void calc_energy(Dtype* energy);
  void post_calc_energy(void);
  void calc_gradient(Dtype* gradient);
  void finish(void);

  void set_quiet(bool q = false) { quiet_ = q; }
  void set_system(Chain<Dtype>* chain) {}
private:
  void read_cfg(string cfg);
  // void load_par_file(void);
  int is_NHbonded(int idx_aa_1, int idx_aa_2);

private:
  bool quiet_;
  Chain<Dtype>* chain_;
  string hbm_file_;
  int num_aa_;
  // Dtype** hydrogen_bond_mat_;
  vector<vector<Dtype> > hydrogen_bond_mat_;
  Dtype weight_;
  // Protein<Dtype>* protein_;

protected:
  DISABLE_COPY_AND_ASSIGN(HNCE);
};

} // namespace leri
#endif // LERI_HNCE_H_
