/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

/**
 * This file contains the implementation of calculating DOPE-CB energy
 *
 */

#ifndef LERI_TSP_H_
#define LERI_TSP_H_
#include "leri/common.h" 
#include "leri/protein/chain.h"

namespace leri {

template <typename Dtype>
class TSP {
public:
  TSP() {}
  ~TSP() {}

  void init(string cfg);
  void prev_calc_energy(Chain<Dtype>* chains);
  void calc_energy(Dtype* energy);
  void calc_gradient(Dtype* gradient);
  void post_calc_energy(void);
  void finish(void);

  void set_quiet(bool q = false) { quiet_ = q; }
  void set_system(Chain<Dtype>* chains) {}
private:
  void read_cfg(string cfg);
  void load_par_file(void);

private:
  bool quiet_;
  Chain<Dtype>* chain_;
  vector<vector<Dtype> > res_contact_mat_;
  // Protein<Dtype>* protein_;
  Dtype weight_;
protected:
  DISABLE_COPY_AND_ASSIGN(TSP);
};

} // namespace leri
#endif // LERI_TSP_H_
