/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

/**
 * This file contains the implementation of Protein Folding Simulation
 *
 */

#ifndef LERI_FOLDMALLOC_H_
#define LERI_FOLDMALLOC_H_
#include "leri/common.h"
#include "leri/protein/atom.h"
#include "leri/protein/bond.h"
#include "leri/protein/aminoacid.h"
#include "leri/protein/chain.h"
#include "leri/protein/protein.h"
// #include "leri/folding/driver.h"

namespace leri {

/**
 * Allocates memory
 */
void pl_allocate_memory(int natoms, int nres, int nchains);

template<typename Dtype>
void pl_copy_data(Protein<Dtype> protein) {
  int idx_chain = 0;
  // Chain<Dtype> chain = protein.chains_[idx_chain];
  int natoms = protein.chains_[idx_chain].get_num_of_atoms();
  int nres = protein.chains_[idx_chain].get_num_of_amino_acids();
  int nchains = protein.chains_.size(); 
  LOG(INFO)<<natoms;
  LOG(INFO)<<nres;

        // These lists store the first and last atom for each residue.
  // vector<int> firstatom;
  // vector<int> lastatom;

  // These lists store the first and last residue for each chain.
  vector<int> firstres;
  vector<int> lastres;
  // int nchains = 0;
  // nres = 0
  // natoms = 0
  int residx = 0;
  int atomidx = 0;
  int atomidx0 = 0;
  int atomidx1 = 0;
  // string aaname;
  // for (int i = 0; i < protein.chains_[idx_chain].amino_acids_.size(); i++) {
  //   for (int j = 0; j < protein.chains_[idx_chain].amino_acids_[i].atoms_.size(); j++) {
  //     LOG(INFO)<<protein.chains_[idx_chain].amino_acids_[i].atoms_[j].name_
  //              <<"\t"<<protein.chains_[idx_chain].amino_acids_[i].atoms_[j].coord_[0]
  //              <<"\t"<<protein.chains_[idx_chain].amino_acids_[i].atoms_[j].coord_[1]
  //              <<"\t"<<protein.chains_[idx_chain].amino_acids_[i].atoms_[j].coord_[2];
  //   }
  // }
  for (int i = 0; i < nchains; i++) {
    protein.chains_[idx_chain].firstres = residx;
    for (int j = 0; j < nres; j++) {
      atomidx0 = atomidx;
      int natm = protein.chains_[idx_chain].amino_acids_[j].atoms_.size();
      for (int k = 0; k < natm; k++) {
        protein.chains_[idx_chain].amino_acids_[j].atoms_[k].res = j;
        // protein.chains_[idx_chain].amino_acids_[j].atoms_[k].chain = i;
        protein.chains_[idx_chain].amino_acids_[j].atoms_[k].x = &(protein.chains_[idx_chain].amino_acids_[j].atoms_[k].coord_[0]);
        protein.chains_[idx_chain].amino_acids_[j].atoms_[k].y = &(protein.chains_[idx_chain].amino_acids_[j].atoms_[k].coord_[1]);
        protein.chains_[idx_chain].amino_acids_[j].atoms_[k].z = &(protein.chains_[idx_chain].amino_acids_[j].atoms_[k].coord_[2]);
        // LOG(INFO)<<*(protein.chains_[idx_chain].amino_acids_[j].atoms_[k].x)<<":"
        //          <<*(protein.chains_[idx_chain].amino_acids_[j].atoms_[k].y)<<":"
        //          <<*(protein.chains_[idx_chain].amino_acids_[j].atoms_[k].z);
        atomidx++;
        // LOG(INFO)<<atomidx;
      }
      atomidx1 = atomidx-1;
      // firstatom.push_back(atomidx0);
      // lastatom.push_back(atomidx1);
      protein.chains_[idx_chain].amino_acids_[j].firstatom = atomidx0;
      protein.chains_[idx_chain].amino_acids_[j].lastatom  = atomidx1;
      residx++;
    }
    protein.chains_[idx_chain].lastres = residx-1;
  }

}

template<typename Dtype>
void pl_build_connectivity(Protein<Dtype>* protein, int idx_chain, bool use_sc) {
  idx_chain = 0;
  // Chain<Dtype> chain = protein->chains_[idx_chain];
  // vector<Atom<Dtype> > atoms; // TODO: remove
  // int natoms = protein->chains_[idx_chain].get_num_of_atoms();
  // int nres = protein->chains_[idx_chain].get_num_of_atoms();
  // int nchains = protein->chains_.size(); 

  // for (int i = 0; i < protein.chains_[idx_chain].amino_acids_.size(); i++) {
  //   for (int j = 0; j < protein.chains_[idx_chain].amino_acids_[i].atoms_.size(); j++) {
  //     LOG(INFO)<<protein.chains_[idx_chain].amino_acids_[i].name_
  //              <<"\t"<<protein.chains_[idx_chain].amino_acids_[i].atoms_[j].name_
  //              <<"\t"<<protein.chains_[idx_chain].amino_acids_[i].atoms_[j].coord_[0]
  //              <<"\t"<<protein.chains_[idx_chain].amino_acids_[i].atoms_[j].coord_[1]
  //              <<"\t"<<protein.chains_[idx_chain].amino_acids_[i].atoms_[j].coord_[2];
  //   }
  // }
  // LOG(INFO)<<protein.chains_[idx_chain].aa_last_;
  protein->leri_create_protein_bonds(protein->chains_, idx_chain, use_sc);
}

/**
 * Allocates memory for an atoms array with natoms elements.
 * @param atoms Atom<double>**
 * @param natoms int
 * @return void
 */
void create_atoms_array(Atom<double> **atoms, int natoms);
/**
 * Frees the memory allocated for the atoms array, The natoms argument is set to zero.
 * @param atoms Atom<double>**
 * @param natoms int*
 * @return void
 */
void delete_atoms_array(Atom<double> **atoms, int *natoms);

/**
 * Allocates memory for a bonds array with nbonds elements.
 * @param bonds Bond<double>**
 * @param nbonds int
 * @return void
 */
void create_bonds_array(Bond<double> **bonds, int nbonds);
/**
 * Frees the memory allocated for the bonds array, The nbonds argument is set to zero.
 * @param bonds Bond<double>**
 * @param nbonds int*
 * @return void
 */
void delete_bonds_array(Bond<double> **bonds, int *nbonds);

/**
 * Allocates memory for a residues array with nres elements.
 * @param residues AminoAcid<double>**
 * @param nres int
 * @return void
 */
void create_residues_array(AminoAcid<double> **residues, int nres);
/**
 * Frees the memory allocated for the residues array, The nres argument is set to zero.
 * @param residues AminoAcid<double>**
 * @param nres int*
 * @return void
 */
void delete_residues_array(AminoAcid<double> **residues, int *nres);

/**
 * Allocates memory for a chains array with nchains elements.
 * @param chains Chain<double>**
 * @param nchains int
 * @return void
 */
void create_chains_array(Chain<double> **chains, int nchains);
/**
 * Frees the memory allocated for the chains array, The nchains argument is set to zero.
 * @param chains Chain<double>**
 * @param nchains int*
 * @return void
 */
void delete_chains_array(Chain<double> **chains, int *nchains);





/**
 * Allocates memory for a boolean array barray with nelements.
 * @param barray bool**
 * @param nelements int
 * @return void
 */
void create_bool_array(bool **barray, int nelements);
/**
 * Frees the memory allocated for the boolean array barray,
 * @param barray bool**
 * @param nelements int
 * @return void
 */
void delete_bool_array(bool **barray, int nelements);

/**
 * Allocates memory for a xywz coordinates array for natoms. This just means creating a floating-point
 * array with 4 * natoms.
 * @param coords double**
 * @param natoms int
 * @return void
 */
void create_coords_array(double **coords, int natoms);
/**
 * Frees the memory allocated for the coordinates array coords,
 * @param coords double**
 * @param natoms int
 * @return void
 */
void delete_coords_array(double **coords, int natoms);
/**
 * Allocates memory for a distance array for natoms. This just means creating a floating-point
 * array with natoms * natoms.
 * @param distances double**
 * @param natoms int
 * @return void
 */
void create_distances_array(double **distances, int natoms);
/**
 * Frees the memory allocated for the distances array,
 * @param distances double**
 * @param natoms int
 * @return void
 */
void delete_distances_array(double **distances, int natoms);

/**
 * Allocates memory for a distance mask array for natoms. This just means creating a boolean array with natoms * natoms.
 * @param mask bool**
 * @param natoms int
 * @return void
 */
void create_distmask_array(bool **mask, int natoms);
/**
 * Frees the memory allocated for the distances mask array,
 * @param mask bool**
 * @param natoms int
 * @return void
 */
void delete_distmask_array(bool **mask, int natoms);

/**
 * Allocates memory for a floating point array farray with nelements.
 * @param farray double**
 * @param nelements int
 * @return void
 */
void create_float_array(double **farray, int nelements);

/**
 * Frees the memory allocated for the floating point array farray,
 * @param farray double**
 * @param nelements int
 * @return void
 */
void delete_float_array(double **farray, int nelements);









/**
 * Sets the xyzw pointers in the atom records in the atoms array to the corresponding positions in the coordinates array coords.
 * @param atoms Atom<double>*
 * @param coords float*
 * @param natoms int
 * @return void
 */
void set_atom_coords(Atom<double> *atoms, double *coords, int natoms);


} // namespace leri
#endif // LERI_FOLDMALLOC_H_
