/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 * Contributor: Tian-Jun ShangGuan ()
 * 
 */

#ifndef LERI_CGMD_CONFIG_H_
#define LERI_CGMD_CONFIG_H_

#include "leri/common.h"
#include "leri/util/malloc.h"
#include "leri/util/matrix.h"
#include "leri/util/strutil.h"
#include "leri/util/fileutil.h"
#include "leri/util/hdf5util.h"
#include "leri/protein/sequence.h"

namespace leri {

// template<typename Dtype> using lmap = map<string, vector<Dtype> >;

// TODO: remove
// static int find_dirtype_index(string* dirtype,string direction){
//   int didx;
//   for (int i = 0; i < 2; i++) {
//     if (dirtype[i] == direction) {
//       didx = i;
//     }
//   }
//   return didx;
// }


template <typename Dtype>
class MDConfig {
public:
  MDConfig(string fseq, string output_dirt = "", string job_name = "", string postfix = "_config.h5")
  :output_dirt_(output_dirt),
   job_name_(job_name),
   postfix_(postfix),
   fseq_(fseq) {
    base_name_ = output_dirt_ + job_name_;
  }
  ~MDConfig() {}

public:
  void init(const string file);
  void actuate();
  void finish();

private:
  void read_cfg(const string file);
  void set_init_struct(const string file);
  void set_angle_range();
  void read_torsion_angles(const string fname);
  void read_dist_map(const string file); // the index for the first residue must start from 1
  void write_torsion_angle_energy(const string output_file);
  void write_dist_energy(const string file, int num_aa);
  
  void read_file_ndrd();
  void populate_rama_maps();
  void write_rama_map_pot(const string file);

  void write_dist_spring(const string file);
  void write_angle_spring(const string file);
  void write_dihedral_spring(const string file);
  void write_rama_coord(const string file);
  void write_count_hbond(const string file);
  void write_affine_alignment(const string file);
  void write_backbone_dependent_point(const string file);
  void write_backbone_pair(string file);
  void write_sidechain_radial(string file, int radial_mode);

  //////////////////////////////////////////////////////////////////////////////////////////////////////////
  // TODO: improve them
  hid_t create_data(hid_t file_id,const char* groupname,vector<string> input);
  void construct_equilibrium_structure(Dtype** pos);//, Dtype** rama_map, Dtype** angles, Dtype** blengths);
  void make_tab_matrics(Dtype*** ptr, Dtype* phi, vector<Dtype> theta, vector<Dtype> bond_lengths);
  //////////////////////////////////////////////////////////////////////////////////////////////////////////

public:
  string output_dirt_;    // output directory
  string job_name_;       // job name
  string base_name_;
  string postfix_;
  // string dirt_;
  string fseq_;

  string file_pdb_;
  string file_ndrd_; // Neighbor-dependent Ramachandran Distributions
  string dirt_bdrl_; // Backbone-dependent rotamer library
  // string file_bdrl_; // Backbone-dependent rotamer library
  // string file_rmjp_; // Radial MJ-1996 potential
  string dirt_radial_;  // Radial MJ-1996 potential
  string file_phsi_; // Torsion angles
  string file_dist_; // Distance map

  // string res_; // Amino acid sequence
  // string sec_; // Secondary structure
  // string chn_; // Chain
  // int num_aa_;
  // vector<int> idx_aa_;
  // vector<int> idx_aa_ord_;
  // vector<Dtype> phi_;
  // vector<Dtype> psi_;

  // int** rama_map_freq_;
private:
  int num_aa_;
  int num_atom_;
  string sequence_1L_;
  vector<string> sequence_3L_;
  vector<string> common_aa_;

  int num_dist_pairs_;
  vector<vector<Dtype> > mtx_dist_;


  int nrow_phsi_;
  Dtype** coord_bb_; // position of backbone, N-CA-C
  Dtype** phsi_;
  Dtype** rphi_;
  Dtype** rpsi_;
  Dtype** bias_;
  Dtype** scale_;
  Dtype** pwidth_;
  vector<int> angle_idx_;
  vector<string> angle_type_;
  vector<vector<Dtype> > rphsi_;
  Dtype* bias_phi_;
  Dtype* bias_psi_;


  Dtype hbond_scale_; // weight for h-bond energy
  // Distance map
  int dist_space_cutoff_;
  Dtype dist_centroid_;
  Dtype dist_threshold_;
  Dtype dist_scale_;
  Dtype dist_energy_wgt_;

  /////////////////////////////////////////
  Dtype*** rama_maps_;
  Dtype* rama_bins_;
  int* residue_id_;
  int* rama_map_id_;
  // string restype_[22];
  vector<string> restype_;
  /////////////////////////////////////////
protected:
  DISABLE_COPY_AND_ASSIGN(MDConfig);
}; // class MDConfig

} // namespace leri
#endif // LERI_CGMD_CONFIG_H_
