/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

/**
 * This file contains the implementation of module definitions
 *
 */

#ifndef LERI_MODULES_H_
#define LERI_MODULES_H_
#include "leri/common.h"

#include "leri/folding/energy/dopecb.h"
#include "leri/folding/energy/dopepw.h"
#include "leri/folding/energy/burial.h"
#include "leri/folding/energy/hnce.h"
#include "leri/folding/energy/cnce.h"
#include "leri/folding/energy/tsp.h"
#include "leri/folding/energy/radial.h"

namespace leri {

typedef struct EnergyModule {
  string name_;
  string doc_; 
} Energy_Module;

template <typename Dtype>
class Module {
public:
  // explicit Module(const SolverParameter& param);
  explicit Module(){};
  // explicit Module(const string& param_file){};
  Module(const string& param_file){};
  virtual ~Module() {}

  string get_name(int i) { return energy_modules_[i].name_; }

public:
  DOPE_CB<Dtype> dope_cb;
  DOPE_PW<Dtype> dope_pw;
  BurialE<Dtype> burial;
  HNCE<Dtype> hnce; 
  CNCE<Dtype> cnce;
  Radial<Dtype> radial;
  TSP<Dtype> tsp;
  static Energy_Module energy_modules_[];

protected:
  DISABLE_COPY_AND_ASSIGN(Module);
};

template <typename Dtype>
Energy_Module Module<Dtype>::energy_modules_[] = {
  {"DOPE-CB", "Discrete Optimized Protein Energy function (main-chain + CB atoms)"},
  {"DOPE-PW", "Discrete Optimized Protein Energy function (main-chain + CB atoms)"},
  {"BURIAL",  "Residue depth"},
  {"HNCE",    "Hydrogen-bond constraints"},
  {"CNCE",    "Residue contact bias"},
  {"TSP",     "Torsional statistical potential"},
  {"RADIAL",  "Radial energy function"},
  {"", ""} // Sentinel array element. Don't remove, it is needed to know where the array ends.
};

} // namespace leri
#endif // LERI_MODULES_H_
