/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

/**
 * This file contains the implementation of Protein Folding Simulation
 *
 */

#ifndef LERI_SIMULATION_H_
#define LERI_SIMULATION_H_

#include "leri/common.h"
#include "leri/protein/sequence.h"
#include "leri/folding/foldmalloc.h"
#include "leri/folding/driver.h"

namespace leri {

template <typename Dtype>
class Simulation {
public:
  // explicit Simulation(string name) {
  //   boost::to_upper(name);
  //   if (!name.compare("DSA")) {
  //     selct_driver_ = 0;
  //   }
  // }
  // explicit Simulation(int mod = 0)
  // :selct_driver_(mod) {}
  explicit Simulation(string fasta_name, string output_dirt = "", string job_name = "", int mod = 0)
  :output_dirt_(output_dirt), 
   job_name_(job_name),
   fasta_name_(fasta_name),
   selct_driver_(mod) {
    base_name_ = output_dirt_ + job_name_;
  }
  ~Simulation() {}
  

  /**
   * Initialize simulator
   * @param void
   * @return void
   */
  void init();
  /**
   * Executes a simulation using the currently selected driver module, using cfgname as the
   * configuration file for the driver.
   * @return void
   */
  void actuate();
  void finish();


  
  void set_system(Chain<Dtype>* chain);


  /**
   * Selects the current driver using a name string
   * @param name string
   * @return void
   */
  void select_driver_by_name(string name);
  /**
   * Selects the current driver using an index value
   * @param driver int
   * @return void
   */
  void select_driver_by_index(int driver);
  /**
   * Set structure from FASTA
   * @param protein Protein<Dtype>*
   * @return void
   */
private:
  void init_system();
  /**
   * Initialize hydrogen bond and residue-contact bias energies
   * @param protein Protein<Dtype>*
   * @return void
   */
  void init_bias_energy();

  // Configuration for protein folding
  string cfg_driver(string output_dirt);
  string cfg_dsa(string output_dirt, string job_name);
  // Write DOPE-W configuration file
  string cfg_dopew();
  // Write Burial configuration file
  string cfg_burial();
  // Write HNCE configuration file
  string cfg_hnce(string output_dirt);
  // Write CNCE configuration file
  string cfg_cnce(string output_dirt);
  void write_folding_cfg(string output_dirt, string job_name); 


private:
  string output_dirt_;    // output directory
  string job_name_;       // job name
  string base_name_;
  string fasta_name_;

  int selct_driver_; // TODO: same as the sampling_mod_

  Chain<Dtype>* chain_;
  // Protein<Dtype>* protein_;
  // int num_of_chains_;

protected:
  DISABLE_COPY_AND_ASSIGN(Simulation);
};

} // namespace leri
#endif // LERI_SIMULATION_H_
