/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_FIGURE_H_
#define LERI_FIGURE_H_

#include "leri/common.h"
#include "leri/util/fileutil.h"

namespace leri {

template <typename Dtype>
void progress_bar(string str, const int idx = -1, const int max_num = -1, string postfix = "", Dtype fval = -999.0, int format = 0) {
// void progress_bar (Dtype percentage, string str, int val = -1, string postfix = "", Dtype fval = -999.0) {
  fflush (stdout);
  const static string progress_bar_ = "===========================";
  const static int progress_bar_width_  = 27;
  Dtype percentage = (1.0*idx/max_num);
  Dtype pct = (percentage * 100);
  int lpad = (int) (percentage * progress_bar_width_);
  int rpad = progress_bar_width_ - lpad;
  if (idx == -1) {
    printf ("\r-- %s[%.*s%*s] %.2f%% completed ", str.c_str(), lpad, progress_bar_.c_str(), rpad, "", pct);
  } else if (fval == -999.0) {
    if (!postfix.empty()) {
      printf ("\r-- %s (%d/%d %s) [%.*s%*s] %.2f%% completed ", str.c_str(), idx, max_num, postfix.c_str(), lpad, progress_bar_.c_str(), rpad, "", pct);
    } else {
      printf ("\r-- %s (%d/%d) [%.*s%*s] %.2f%% completed ", str.c_str(), idx, max_num, lpad, progress_bar_.c_str(), rpad, "", pct);
    }
  } else {
    if (format == 0) {
      printf ("\r-- %s %d/%d %s %.3f [%.*s%*s] %.2f%% completed ", str.c_str(), idx, max_num, postfix.c_str(), fval, lpad, progress_bar_.c_str(), rpad, "", pct);
    } else if (format == 1) {
      printf ("\r-- %s %d/%d %s %.3f%% [%.*s%*s] %.2f%% completed ", str.c_str(), idx, max_num, postfix.c_str(), fval*100, lpad, progress_bar_.c_str(), rpad, "", pct);
    } else {
      printf ("\r-- %s %d/%d %s %.2e [%.*s%*s] %.2f%% completed ", str.c_str(), idx, max_num, postfix.c_str(), fval, lpad, progress_bar_.c_str(), rpad, "", pct);
    }
  }
  fflush (stdout);
}


template <typename Dtype>
void hex_color(Dtype val, Dtype vmin, Dtype vmax, vector<Dtype>& color) {
  // COLOUR c = {1.0,1.0,1.0}; // white
  // vector<Dtype> color(3, 1.0); // white
  color = {1.0, 1.0, 1.0}; // white
  Dtype dval;

  if (val < vmin) { val = vmin; }
  if (val > vmax) { val = vmax; }
  dval = vmax - vmin;

  if (val < (vmin + 0.25 * dval)) {
    color[0] = 0;
    color[1] = 4 * (val - vmin) / dval;
  } else if (val < (vmin + 0.5 * dval)) {
    color[0] = 0;
    color[2] = 1 + 4 * (vmin + 0.25 * dval - val) / dval;
  } else if (val < (vmin + 0.75 * dval)) {
    color[0] = 4 * (val - vmin - 0.5 * dval) / dval;
    color[2] = 0;
  } else {
    color[1] = 1 + 4 * (vmin + 0.75 * dval - val) / dval;
    color[2] = 0;
  }
  color[0] *= 255;
  color[1] *= 255;
  color[2] *= 255;
}
// Plot data with lines in Gnuplot
void plot_data_w_lines(string file_name);
/*===== Figure created by Gnuplot =====*/
// void figure_stats(string file_name, int num_aa, const string path, string indicator = "");
// 0: similarity matrix
// 1: relative entropy matrix
// 2: bar plot
void plot_fig_script(string file_name, int num, int flag, string indicator = "", vector<int> idx = vector<int>());
void figure_stats(string file_name, int num, string indicator = "");

// void plot_couplings(string file_name, int num, string indicator = "", vector<int> idx = vector<int>());
// void plot_point_mutations(string file_name, int num_aa, const string path);
// void plot_sequence_energy(string file_name, const string path);
// void plot_matrix(string file_name, int num, string indicator = "");      // Plot figure from a square matrix data
// void col3_matrix_plot(string file_name, int num_aa, const string path, string tname); // Plot figure from a 3 columns data
// void plot_col3_matrix(string file_name, int num, string indicator = ""); // Plot figure from a 3 columns data
// void fig_script_close(string file_name, int num, string indicator = ""){};
} // namespace leri


#endif // LERI_FIGURE_H_
