/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_GNUPLOT_H_
#define LERI_GNUPLOT_H_

#include "leri/common.h"
#include "leri/util/strutil.h"
#include "leri/util/fileutil.h"

namespace leri {

template <typename Dtype> 
class Gnuplot {
public:
  inline Gnuplot(bool persist = false) {
    open(persist);
  }
  inline virtual ~Gnuplot() { close(); }

  void send_cmd_line(const string& text, bool use_buffer = false);
  void send_end_of_data(unsigned repeat_buffer = 1);
  void send_new_data();
  void write_buffer2file(const string& file_name);

  void style(string& header,
             Dtype w  = 1200, Dtype h   = 1200, 
             Dtype tm = 0.80, Dtype bm = 0.15, 
             Dtype lm = 0.15, Dtype rm = 0.85, 
             int font_size = 21, bool fig_type = true);
  void plotting(string fname, 
                string& str, 
                string style, 
                vector<Dtype> nxy, 
                vector<string> xylabel, 
                string type,
                string append = "",
                string palette = "",
                vector<int> idx = vector<int>(),
                vector<int> idx_col = vector<int>());

  void open(bool persist);
  void close();
private:
  Gnuplot(Gnuplot const&) = delete;
  void operator=(Gnuplot const&) = delete;

private:
  string str_style_;
  FILE* fid_;
  vector<string> buffer;
  bool fig_type_;
};

} // namespace leri
#endif // LERI_MATRIX_H_
