/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

/**
 * This file contains the implementation of Javas 
 *
 */

#ifndef LERI_JAVAS_H_
#define LERI_JAVAS_H_

#include "leri/common.h"
#include "leri/util/util.h"
#include "leri/util/malloc.h"
#include "leri/util/matrix.h"
#include "leri/util/fileutil.h"

namespace leri {

template <typename Dtype>
class Javas {
public:
  Javas(string output_dirt = "", string job_name = "")
  :output_dirt_(output_dirt),
   job_name_(job_name) {
    base_name_ = output_dirt_ + job_name_;
    // vstr_ = VERS_INFO("// Java script generated by");
  }
  ~Javas() {}
  void set_param(int m, int n, int* idx_col = NULL);
  void set_postfix(string postfix);

  void plotting(const string plot_type,                              // bar, line, scatter, heatmap etc.
                Dtype** mtx = NULL,                                  
                const vector<string> txylabels = vector<string>(),
                const vector<string> xtickers  = vector<string>(),
                const vector<string> ytickers  = vector<string>(),
                const vector<string> legends   = vector<string>(),
                int* margin = NULL,                                  // top, right, bottom, left, width, height, rotational angle 
                const vector<string> xyaxis    = vector<string>(),
                const vector<string> dlabels   = vector<string>()    // labels for data points
               );

  void plotting(const string csv, 
                Dtype* xyrange = NULL, 
                const vector<string> labels = vector<string>(), 
                const vector<string> tlabels = vector<string>(),
                const string id_name = "");

  
  // TODO: combine with plotting(...)
  void write_mtx_in_javas(const string csv,                                    // data
                          const string plot_type,                              // heatmap etc.
                          const vector<string> xtickers    = vector<string>(), // xtickers
                          const vector<string> xylabels    = vector<string>(), // xlabel, ylabel, title
                          const vector<int>    idx_col     = vector<int>(),     
                          const vector<string> ytickers    = vector<string>(), 
                          int* margin = NULL,                                   
                          Dtype** mtx = NULL,                                   
                          const vector<int> axisp          = {0,0,2,0,0},      // flag (for MSA and others), xlabels direction, xlabels' position, ylabels' direction, ylabels' position  
                          const vector<string> cbar_text = vector<string>(),
                          int ctype = -1,  // color scheme
                          int ndigit = 9   // number of digits after point
                         );
  
  void community2chord(
                       const string id_name = "", int m = -1, int n = -1);
  void mtx2chord(Dtype** mtx, const vector<string>& labels = vector<string>(), 
                 const string id_name = "", int m = -1, int n = -1);

  void mtx2volcano(Dtype** mtx, const vector<string>& labels = vector<string>(), 
                   const string id_name = "");







  // void plotting_in_javas(const string csv,
  //                        const string type, 
  //                        const vector<string> tlabels     = vector<string>(), 
  //                        const vector<string> txylabels   = vector<string>(), 
  //                        const vector<string> legends     = vector<string>(),
  //                        const vector<int>    idx_col     = vector<int>(),
  //                        const vector<string> tick_labels = vector<string>(),
  //                        int* margin = NULL,
  //                        Dtype** mtx = NULL);



  

  // void write_vec_in_javas_bar(const string csv,
  //                             const string type, 
  //                             const vector<string> tlabels     = vector<string>(), 
  //                             const vector<string> txylabels   = vector<string>(), 
  //                             const vector<int>    idx_col     = vector<int>(),
  //                             const vector<string> tick_labels = vector<string>(),
  //                             int* margin = NULL,
  //                             Dtype* vec = NULL);
  // void write_mtx_in_javas_bar(const string csv,
  //                             const string type, 
  //                             const vector<string> tlabels     = vector<string>(), 
  //                             const vector<string> txylabels   = vector<string>(), 
  //                             const vector<int>    idx_col     = vector<int>(),
  //                             const vector<string> tick_labels = vector<string>(),
  //                             int* margin = NULL); 
  
  // void write_mtx_in_javas_histogram(const string csv, 
  //                                   Dtype* xyrange = NULL, 
  //                                   const vector<string> labels = vector<string>(), 
  //                                   const vector<string> glabels = vector<string>(), 
  //                                   const string id_name = "");

  
private:
  string output_dirt_;    // output directory
  string job_name_;       // job name
  string base_name_;
  // string vstr_;

  string file_name_;
  string id_element_;
  int nrow_;
  int ncol_;

  // int axisp_[] = {0, 0, 0}; 
  // {top, right, bottom, left}, x reverse, y reverse
  const string pos_str_[4] = {"top", "right", "bottom", "left"};

protected:
  DISABLE_COPY_AND_ASSIGN(Javas);
}; // class Javas

} // namespace leri
#endif // LERI_JAVAS_H_
