/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

/**
 * This file contains of PyMol
 */

#ifndef LERI_PYMOL_H_
#define LERI_PYMOL_H_

#include "leri/common.h"
#include "leri/util/fileutil.h"

namespace leri {

template <typename Dtype> 
class Pymol {
public:
  explicit Pymol() {}
  virtual ~Pymol() {}
 
  void init();
  void script(string file_name, string pdb, vector<int> blocks, bool flag = false);
  void finish(){};

private:
  string pymol_style();
  string is_pdb(string pdb);
  string block_style(string name, vector<int> blocks, int idx_1, int idx_2);
  string export_style(string file_name, string block_name = "");
  void exec_pymol(string file_name, string str_style, bool rflag = false);


private:
  string str_;
  string str_style_;
protected:
  DISABLE_COPY_AND_ASSIGN(Pymol);
};


// template <typename Dtype>
// void write_pymol_config(string file_name, string pdb, vector<int> blocks, bool flag = false) {
//   FILE* fid;
//   if (flag) { // all blocks
//     fid = fopen((file_name+"_pymol.pml").c_str(), "w");
//     fprintf(fid, "# PyMOL script for the residue blocks inferred by Leri Analytics (v%s).\n\n", VERSION);
//     // Show white cartoon
//     if (!pdb.empty()) {
//       fprintf(fid,"delete all\n");
//       fprintf(fid, "load %s, main\n", pdb.c_str());
//     }
//     fprintf(fid, "reset \nhide all\n");
//     fprintf(fid, "bg_color white\n");  //  background color
//     fprintf(fid, "show cartoon\n");
//     fprintf(fid, "color white\n\n");
//     // Other cosmetics
//     fprintf(fid,"set two_sided_lighting,    1\n");
//     fprintf(fid,"set sphere_quality,        2\n");
//     fprintf(fid,"set sphere_scale,          1\n");
//     fprintf(fid,"set sphere_transparency,   0.8\n");
//     fprintf(fid,"set surface_quality,       2\n");
//     fprintf(fid,"set stick_quality,        10\n");
//     fprintf(fid,"set cartoon_oval_quality, 10\n");
//     fprintf(fid,"set cartoon_loop_quality,  6\n");
//     fprintf(fid,"set cartoon_fancy_helices, 1\n");
//     fprintf(fid,"set cartoon_fancy_sheets,  1\n\n");

//     // fprintf(fid,"set_view (\\\n");
//     // fprintf(fid," -0.655116498,   -0.695889294,   -0.294215441,\\\n");
//     // fprintf(fid,"  0.386854321,    0.025535638,   -0.921788275,\\\n");
//     // fprintf(fid,"  0.648974240,   -0.717696548,    0.252478868,\\\n");
//     // fprintf(fid,"  0.000000000,    0.000000000, -148.182495117,\\\n");
//     // fprintf(fid,"-12.619968414,  -89.585556030,   -3.959166050,\\\n");
//     // fprintf(fid,"122.692764282,  173.672241211,  -20.000000000 )\n");
//     int n = blocks.size();
//     fprintf(fid, "select mutant_sec, resi ");
//     for (int i = 0; i < n; i++) {
//       fprintf(fid, "%i", blocks[i]+1);
//       if (i < n-1) { fprintf(fid, "+"); }
//       else { fprintf(fid, "\n"); }
//     }
//     fprintf(fid, "set sphere_scale, 0.85, (mutant_sec)\n");
//     fprintf(fid, "show spheres, (mutant_sec)\n");
//     fprintf(fid, "color tv_yellow, mutant_sec\n\n");
//     fprintf(fid, "zoom complete=1\n");
//     fprintf(fid, "set depth_cue, 0; set opaque_background, off; ray 800, 800; png ");
//     fprintf(fid, "%s_designed\n", file_name.c_str());
//     fclose(fid);
//   } else {
//     fid = fopen((file_name+"_pymol.pml").c_str(), "w");
//     fprintf(fid, "# PyMOL script for the residue blocks inferred by Leri Analytics (v%s).\n\n", VERSION);
//     // Show white cartoon
//     if (!pdb.empty()) {
//       fprintf(fid,"delete all\n");
//       fprintf(fid, "load %s, main\n", pdb.c_str());
//     }
//     fprintf(fid, "reset \nhide all\n");
//     fprintf(fid, "bg_color white\n");  //  background color
//     fprintf(fid, "show cartoon\n");
//     fprintf(fid, "color white\n\n");
//     // Other cosmetics
//     fprintf(fid,"set two_sided_lighting,   1\n");
//     fprintf(fid,"set sphere_quality,       2\n");
//     fprintf(fid,"set surface_quality,      2\n");
//     fprintf(fid,"set stick_quality,       10\n");
//     fprintf(fid,"set cartoon_oval_quality,10\n");
//     fprintf(fid,"set cartoon_loop_quality, 6\n");
//     fprintf(fid,"set cartoon_fancy_helices,1\n");
//     fprintf(fid,"set cartoon_fancy_sheets, 1\n\n");

//     vector<int>::iterator it;
//     int idx_1 = 0, idx_2 = 0;
//     it = find(blocks.begin(), blocks.end(), -1);
//     idx_1 = distance(blocks.begin(), it);
//     it = find(blocks.begin(), blocks.end(), -2);
//     idx_2 = distance(blocks.begin(), it);
//     fprintf(fid, "select red_sec, resi ");
//     for (int i = 0; i < idx_1; i++) {
//       fprintf(fid, "%i", blocks[i]+1);
//       if (i < idx_1-1) { fprintf(fid, "+"); }
//       else { fprintf(fid, "\n"); }
//     }
//     fprintf(fid, "set sphere_scale, 0.85, (red_sec)\n");
//     fprintf(fid, "show spheres, (red_sec)\n");
//     fprintf(fid, "color tv_red, red_sec\n\n");

//     fprintf(fid, "select green_sec, resi ");
//     for (int i = idx_1+1; i < idx_2; i++) {
//       fprintf(fid, "%i", blocks[i]+1);
//       if (i < idx_2-1) { fprintf(fid, "+"); }
//       else { fprintf(fid, "\n"); }
//     }
//     fprintf(fid, "set sphere_scale, 0.85, (green_sec)\n");
//     fprintf(fid, "show spheres, (green_sec)\n");
//     fprintf(fid, "color tv_green, green_sec\n\n");

//     fprintf(fid, "select blue_sec, resi ");
//     for (int i = idx_2+1; i < blocks.size(); i++) {
//       fprintf(fid, "%i", blocks[i]+1);
//       if (i < blocks.size()-1) { fprintf(fid, "+"); }
//       else { fprintf(fid, "\n"); }
//     }
//     fprintf(fid, "set sphere_scale, 0.85, (blue_sec)\n");
//     fprintf(fid, "show spheres, (blue_sec)\n");
//     fprintf(fid, "color tv_blue, blue_sec\n\n");
//     fprintf(fid, "zoom complete=1\n");
//     fprintf(fid, "set depth_cue, 0; set opaque_background, off; ray 800, 800; png ");
//     fprintf(fid, "%s_blocks\n", file_name.c_str());
//     fclose(fid);

//     //  Red block
//     fid = fopen((file_name+"_pymol_red.pml").c_str(), "w");
//     fprintf(fid, "# PyMOL script for the residue blocks inferred by Leri Analytics (v%s).\n\n", VERSION);
//     // Show white cartoon
//     if (!pdb.empty()) {
//       fprintf(fid,"delete all\n");
//       fprintf(fid, "load %s, main\n", pdb.c_str());
//     }
//     fprintf(fid, "reset \nhide all\n");
//     fprintf(fid, "bg_color white\n");  //  background color
//     fprintf(fid, "show cartoon\n");
//     fprintf(fid, "color white\n\n");
//     // Other cosmetics
//     fprintf(fid,"set two_sided_lighting,   1\n");
//     fprintf(fid,"set sphere_quality,       2\n");
//     fprintf(fid,"set surface_quality,      2\n");
//     fprintf(fid,"set stick_quality,       10\n");
//     fprintf(fid,"set cartoon_oval_quality,10\n");
//     fprintf(fid,"set cartoon_loop_quality, 6\n");
//     fprintf(fid,"set cartoon_fancy_helices,1\n");
//     fprintf(fid,"set cartoon_fancy_sheets, 1\n\n");

//     fprintf(fid, "select red_sec, resi ");
//     for (int i = 0; i < idx_1; i++) {
//       fprintf(fid, "%i", blocks[i]+1);
//       if (i < idx_1-1) { fprintf(fid, "+"); }
//       else { fprintf(fid, "\n"); }
//     }
//     fprintf(fid, "set sphere_scale, 0.85, (red_sec)\n");
//     fprintf(fid, "show spheres, (red_sec)\n");
//     fprintf(fid, "color tv_red, red_sec\n\n");
//     fprintf(fid, "zoom complete=1\n");
//     fprintf(fid, "set depth_cue, 0; set opaque_background, off; ray 800, 800; png ");
//     fprintf(fid, "%s_red_block\n", file_name.c_str());
//     fclose(fid);
//     //  Blue block
//     fid = fopen((file_name+"_pymol_blue.pml").c_str(), "w");
//     fprintf(fid, "# PyMOL script for the residue blocks inferred by Leri Analytics (v%s).\n\n", VERSION);
//     // Show white cartoon
//     if (!pdb.empty()) {
//       fprintf(fid,"delete all\n");
//       fprintf(fid, "load %s, main\n", pdb.c_str());
//     }
//     fprintf(fid, "reset \nhide all\n");
//     fprintf(fid, "bg_color white\n");  //  background color
//     fprintf(fid, "show cartoon\n");
//     fprintf(fid, "color white\n\n");
//     // Other cosmetics
//     fprintf(fid,"set two_sided_lighting,   1\n");
//     fprintf(fid,"set sphere_quality,       2\n");
//     fprintf(fid,"set surface_quality,      2\n");
//     fprintf(fid,"set stick_quality,       10\n");
//     fprintf(fid,"set cartoon_oval_quality,10\n");
//     fprintf(fid,"set cartoon_loop_quality, 6\n");
//     fprintf(fid,"set cartoon_fancy_helices,1\n");
//     fprintf(fid,"set cartoon_fancy_sheets, 1\n\n");

//     fprintf(fid, "select blue_sec, resi ");
//     for (int i = idx_2+1; i < blocks.size(); i++) {
//       fprintf(fid, "%i", blocks[i]+1);
//       if (i < blocks.size()-1) { fprintf(fid, "+"); }
//       else { fprintf(fid, "\n"); }
//     }
//     fprintf(fid, "set sphere_scale, 0.85, (blue_sec)\n");
//     fprintf(fid, "show spheres, (blue_sec)\n");
//     fprintf(fid, "color tv_blue, blue_sec\n\n");
//     fprintf(fid, "zoom complete=1\n");
//     fprintf(fid, "set depth_cue, 0; set opaque_background, off; ray 800, 800; png ");
//     fprintf(fid, "%s_blue_block\n", file_name.c_str());
//     fclose(fid);
//   }
// }



} // namespace leri
#endif // LERI_PYMOL_H_
