/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_PYPLOT_H_
#define LERI_PYPLOT_H_

#include "leri/common.h"
#include "leri/util/matrix.h"
#include "leri/util/fileutil.h"

namespace leri {

template <typename Dtype>
class Pyplot {
public:
  Pyplot(string output_dirt = "", string job_name = "")
  :output_dirt_(output_dirt),
   job_name_(job_name) {
    base_name_ = output_dirt_ + job_name_;
    // vstr_ = VERS_INFO("// Pyplot script generated by");
  }
  ~Pyplot() {}

public:
  void init();
  void plotting(const string plot_type,                              // bar, line, scatter, heatmap etc.
                const string file_name,                                  
                const vector<string> txylabels = vector<string>(),    // title, xlabel, ylabel
                const vector<string> xtickers  = vector<string>(),    // xtick label
                const vector<string> ytickers  = vector<string>(),    // ytick label
                const vector<string> legends   = vector<string>(),
                Dtype* margin = NULL,                                   // top, right, bottom, left, width, height, rotation angle (x), rotation angle (y)  
                const vector<string> xyaxis    = vector<string>(),   
                const vector<string> dlabels   = vector<string>()     // labels for data points
               );
  void finish();
private:
  string heatmap();
  string str_vec(const vector<string> vec);
  void exec_python(string file_name, bool rflag = false);
private:
  string output_dirt_;    // output directory
  string job_name_;       // job name
  string base_name_;

  string file_name_;
  // string id_element_;
  // int nrow_;
  // int ncol_;
  int dpi_;
  string fmt_;

  Dtype* margin_;
  // // int axisp_[] = {0, 0, 0}; 
  // // {top, right, bottom, left}, x reverse, y reverse
  // const string pos_str_[4] = {"top", "right", "bottom", "left"};
  vector<string> txylabels_;
  vector<string> xtickers_;
  vector<string> ytickers_;
  vector<string> legends_;
  vector<string> xyaxis_;
  vector<string> dlabels_;

protected:
  DISABLE_COPY_AND_ASSIGN(Pyplot);
}; // class Pyplot

} // namespace leri
#endif // LERI_PYPLOT_H_