/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

/**
 * This file contains all the internal leri header file.
 */

static const char* MONTHS[] = { "Jan.", "Feb.", "Mar.", "Apr.", "May",  "Jun.",
                                "Jul.", "Aug.", "Sep.", "Oct.", "Nov.", "Dec." };

void about_leri();
void info();
void set_gflag_info();


/////////////////////////////////////////////////////////////////////////////////
void about_leri() {
  printf("\n");
  printf("-- -------------------------------------------------------\n");
  printf("## Leri   :: Learning Engines to Recognize Life\n");
  printf("## Version : LESRIA (LERI v%s)      \n", VERSION);
  printf("## Released: Aug. 3, 2017.                   \n");
  printf("## Author  : Dr Yaan J. Jang                 \n");
  printf("## Email   : %s             \n", string(EMAIL).c_str());
  // printf("# Last Modified    : %s %2d, %2d.        \n", MONTHS[ctm->tm_mon], ctm->tm_mday, (ctm->tm_year+1900));
  printf("## Webpage : %s/leri                         \n", string(LHTTPS).c_str()); 
  // printf("#  Leri Incentive Product - Leri Executable Build                \n");
  printf("## Copyright (C) 2015-%d Leri Ltd.           \n", (ctm->tm_year+1900));
  printf("## Online tutorials: https://app.gitbook.com/@leri/s/doc\n");
  printf("-- -------------------------------------------------------\n");
  // printf("   New Biochemistry Building, Department of Biochemistry,     \n");
  // printf("   University of Oxford, Oxford OX1 3QU, UK.                  \n");
  // printf("   Cavendish Laboratory, Department of Physics,                  \n");
  // printf("   University of Cambridge, Cambridge CB3 0HA, UK.               \n");
  // printf("   James Franck Institute, The University of Chicago             \n");
  // printf("   929 East 57 Street, Chicago, IL 60637, US.                    \n");
  // printf("-----------------------------------------------------------------\n");
  // printf(" Leri Ltd., headquartered in Oxford OX1 3QU, UK, is a  \n");
  // printf(" solution and service provider, specializing in research for     \n");
  // printf(" Genomics, Biophysics, Structural Biology, Epigenetics, and      \n");
  // printf(" Clinical Research. It provides softwares, services and          \n");
  // printf(" solutions that support analyses of large-scale data in life     \n");
  // printf(" science within both industries and academia.                    \n");
  // printf("\n");
  // printf(" Leri Analytics is a computational tool, providing bioinformatics\n");
  // printf(" services for both academic labs and companies. Leri Analytics   \n");
  // printf(" builds better solutions to their specific projects by analysis. \n");
  // printf(" on (a) Protein science; (b) Genomics; (c) Transcriptomics;      \n");
  // printf(" (d) Epigenomics; and (e) Other Omics. The analyses mainly       \n");
  // printf(" include protien sequence analysis, protein folding and design,  \n");
  // printf(" variant calling and discovery, de-novo or reference-guided      \n");
  // printf(" assembly, copy number variation, expression analysis, genome    \n");
  // printf(" annotation, DNA methylation, and biomedical images analysis,    \n");
  // printf(" but Leri Analytics is not limited to those analysis.            \n"); 
  // printf("\n");
  // printf(" Leri supports de novo genome assembly, genome alignment,   \n");
  // printf(" SNP validation control, variant analysis. ChIP-seq analysis, \n");
  // printf(" NGS analysis, RNA-seq analysis, protein folding, protein     \n");
  // printf(" design, GPCR-G protein binding, protein structure prediction \n");
  // printf(" & alignment, cancer genomics and gene expression analysis.   \n");
  // printf("\n");
  // printf(" More details about Leri Analytics softwares can be found at:    \n");
  // printf(" %s/leri              \n", string(HTTPS).c_str());
  // printf("-----------------------------------------------------------------\n");
  printf("\n");
  // return 0;
}

void info() {
  printf("\n");
  printf("-- -------------------------------------------\n");
  printf("## LERI :: Learning Engine to Recognize lIfe\n");
  printf("## LESRIA :: Leri v%s (%s %d)\n", VERSION, MONTHS[ctm->tm_mon], (ctm->tm_year+1900));
  printf("## Copyright %d (C) Leri Ltd.\n", (ctm->tm_year+1900));
  printf("## %s/leri\n", string(LHTTPS).c_str());
  // printf("-- - - - - - - - - - - - - - - - - - - - - - -\n");
  // printf("-- -·-·-·-·-·-·-·-·-·-·-·-·-·-·-·-·-·-·-·-·-·-\n");
  // printf("-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n");
  printf("-- -------------------------------------------\n");
}

void set_gflag_info() {
  gflags::SetVersionString(AS_STRING(LERI_VERSION));  // Set version
  gflags::SetUsageMessage(
    " usage command line\n"
    " leri <command> <args>\n"
    " \n"
    " These are common Ler commands used in various situations:\n"
    " \n"
    " get leri and system info. (see also: leri help tutorial)\n"
    "  about                -about leri\n"
    "  usage                -usage of leri\n"
    "  help                 -leri help\n"
    "  device               -show GPU diagnostic information\n"
    " \n"
    " work on the omics (see also: leri help metaomics)\n"
    "  lipid                -lipid traffic (TODO: working 2019)\n"
    " \n"
    " work on the omics (see also: leri help metaomics)\n"
    "  sff2fastq            -convert an SFF file from the 454 genome sequencer\n"
    "                        to FASTQ including the sequences and quality scores\n"
    "  base_call            -base calling\n"
    "  atgc_stats           -statistics of FASTQ sequence\n"
    "  atgc_quality         -quality control\n"
    "  atgc_correct         -correct errors in sequencing data\n"
    "  atgc_dna2aa          -convert DNA to amino acid\n"
    "  atgc_assembly        -sequence assembly by de Bruijn graph or ant colony optimization\n" // Transcriptome assembly and differential expression analysis for RNA-Seq
    "  read_filter          -filter reads with low quality\n"
    "  kmer_counter         -count kmer\n"
    "  kmer_generator       -generate kmers\n"

    // "  quality_control      -quality control (reserved, worked 2019)\n"
    "  synteny              -(TODO: working 2019)\n"
    "  gene_exp             -gene expression (TODO: working 2019)\n"
    "  cnv                  -copy number variant (TODO: working 2019)\n"
    " \n"
    " work on the proteomics (see also: leri help proteomics)\n"
    "  sequence_convertor   -convert the ouput of Jackhmmer to fasta\n"
    "  sequence_mix         -pairwise-mixture protein sequences (particular for GPCR-G proteins)\n"
    "  sequence_trim        -trim a multiple protein sequence alignment\n"
    "  sequence_stats       -statistical analysis on a protein mutiple sequence alignment\n"
    "  sequence_eca         -infer coupled relationship betwen pairwise residues and function-related blocks\n"
    "  sequence_energy      -calculate energy for a protein sequence or multiple sequences\n"
    "  sequence_design      -design a protein sequence\n"
    "  sequence_ddg         -infer protein stability (ddG = dG_wt - dG_mutant)\n"
    "  point_mutation       -point mutation for a given protein sequence\n"
    "  residue_contact      -estimate contacts between pairwise residues from protein MSA\n"
    "  selectivity_sector   -detect selectivity sectors of determinants in GPCR-G proteins\n"
    " \n"
    " work on protein folding\n"
    "  pdb_parser           -parser a PDB file and write it out in simplified PDB format and/or FASTA sequence file\n"
    "  pdb_voxel            -grid protein structure into voxels\n"
    "  pdb_contact          -calculate residue contacts from a give PDB file\n"
    "  fasta2pdb            -convert protien amino acid sequence into its extended structure\n"
    "  fold                 -predict tertiary structure\n"
    "  fold_energy          -compute energy of a protein tertiary structure\n"
    "  folding              -predict folding pathways & tertiary structure\n"
    "  phasing              -protein folding using a swarm of particle swarms\n"
    " \n"
    " work on statistics (see also: leri help statistics)\n"
    "  stats_ica            -independent component analysis on a given matrix\n"
    // "  stats_ttest          -t-test for two samples (TODO)\n"
    "  stats_pearson        -Pearson correlation\n"
    "  stats_tsne           -t-distributed stochastic neighbor embedding (t-SNE) method\n"
    " \n"
    " work on optimization tools (see also: leri help optimization)\n"
    "  opt_chpso            -the convergent heterogeneous particle swarm optimizer\n"
    "  opt_jde              -the differential evolution optimizer\n"
    // "  opt_lbfgs            -the Broyden–Fletcher–Goldfarb–Shanno (BFGS) algorithm with limited-memory\n"
    // "  opt_gd               -the Gradient Descent method\n"
    " \n"
    " work on sampling (see also: leri help sampling)\n"
    "  nested_sampler       -nested sampler\n"
    " \n"
    " work on machine learning (see also: leri help learning)\n"
    "  senet                -sequentially evolving neural network\n"
    "  phsior               -protein torsion angle predictor based on CNN deep network\n"
    "  nirecor              -intelligent predictor based on swarm intelligence and neural network\n"
    "  optifel              -evolving fuzzy modeling approach\n"
    "  imgor                -deep neural network for images\n"
    " \n"
    " other tools (see also: leri help tools)\n"
    "  extract_feature      -extract features from protein sequences, PSSM, SS, SA\n"
    "  img2ubyte            -image format convertor\n"
    " \n"
    // " 'leri help -a' and 'leri help -g' list available subcommands and some concept guides.\n"
    // "  See 'leri help <command>' or 'leri help <concept>' to read about a specific subcommand or concept."
    " \n");
}