/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_LAYER_CONCATENATION_LAYER_H_
#define LERI_LAYER_CONCATENATION_LAYER_H_

#include "leri/common.h"
#include "leri/layers/layer.h" 

namespace leri {

template <typename Dtype>
class concatenation_layer : public base_layer<Dtype> {
public:
  concatenation_layer(const string layer_name, int w, int h, leri::pad_type p = leri::zero)
  :base_layer<Dtype>(layer_name, w, h) {
    nmaps_    = 0;
    pad_type_ = p;
    this->has_wgts_ = false;
    this->ptr_act_  = NULL;
  }
  virtual ~concatenation_layer() {}
  
  virtual string get_config_string();
  virtual LMatrix<Dtype>* new_connection(base_layer<Dtype>& top, int wgt_mtx_idx);

  virtual void calc_dw(const base_layer<Dtype>& top_layer, LMatrix<Dtype>& dw, const int train = 1) {}
  virtual void accumulate_signal(const base_layer<Dtype>& top, const LMatrix<Dtype>& w, const int train = 0);
  virtual void distribute_delta(base_layer<Dtype>& top, const LMatrix<Dtype>& w, const int train = 1);

private:
  int nmaps_;
  leri::pad_type pad_type_;
  map<const base_layer<Dtype>*, int> layer2channel_;
};

} // namespace leri
#endif // LERI_LAYER_CONCATENATION_LAYER_H_
