/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_LAYER_CONVOLUTION_LAYER_H_
#define LERI_LAYER_CONVOLUTION_LAYER_H_

#include "leri/common.h"
#include "leri/util/matrix.h"
#include "leri/layers/layer.h"

namespace leri {

template <typename Dtype>
class convolution_layer : public base_layer<Dtype> {
public:
  // convolution_layer(const string layer_name, int w, int c, int s, activation_function* p)
  convolution_layer(const string layer_name, int w, int c, int s, Activation<Dtype>* p)
  :base_layer<Dtype>(layer_name, w, w, c) {
    this->ptr_act_   = p; 
    stride_          = s; 
    row_kernel_      = w; 
    col_kernel_      = w; 
    nmaps_           = c;
    kernels_per_map_ = 0; 
    this->col_pad_   = col_kernel_-1; 
    this->row_pad_   = row_kernel_-1;
    this->use_bias_  = true;
    groups_   = 1;
  }
  // convolution_layer(const string layer_name, int w, int c, int s, int g, activation_function* p)
  convolution_layer(const string layer_name, int w, int c, int s, int g, Activation<Dtype>* p)
  : base_layer<Dtype>(layer_name, w, w, c) {
    this->ptr_act_   = p;
    stride_          = s; 
    row_kernel_      = w; 
    col_kernel_      = w;
    nmaps_           = c;
    kernels_per_map_ = 0;
    this->col_pad_   = col_kernel_-1; 
    this->row_pad_   = row_kernel_-1;
    this->use_bias_  = true;
    groups_   = g;
  }
  virtual  ~convolution_layer() {}

  virtual string get_config_string();
  virtual int get_fan_size() { return row_kernel_*col_kernel_*nmaps_*kernels_per_map_; }
  virtual void resize(int w, int h = 1, int c = 1);
 
  virtual LMatrix<Dtype>* new_connection(base_layer<Dtype>& top, int weight_mat_index);
  virtual void activate_nodes();
  
  virtual void calc_dw(const base_layer<Dtype>& top, LMatrix<Dtype>& dw, const int train = 1);
  virtual void accumulate_signal( const base_layer<Dtype>& top, const LMatrix<Dtype>& w, const int train = 0);
  virtual void distribute_delta(base_layer<Dtype>& top, const LMatrix<Dtype>& w, const int train = 1);

public:
  int row_kernel_;
  int col_kernel_;
  int nmaps_;
  int kernels_per_map_;
  int groups_;
private:
  int stride_;
};

} // namespace leri
#endif // LERI_LAYER_CONVOLUTION_LAYER_H_
