/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_LAYER_DROPUOUT_H_
#define LERI_LAYER_DROPUOUT_H_

#include "leri/common.h"
#include "leri/layers/layer.h"

namespace leri {

template <typename Dtype>
class dropout_layer : public base_layer<Dtype> {
public:
  dropout_layer(const string layer_name, Dtype val)
  :base_layer<Dtype>(layer_name, 1) {
    this->has_wgts_     = false;
    this->dropout_rate_ = val;
    this->ptr_act_      = NULL;
  }
  virtual  ~dropout_layer() {}

  virtual string get_config_string();
  virtual void resize(int w, int h = 1, int c = 1);
  virtual LMatrix<Dtype>* new_connection(base_layer<Dtype>& top, int wgt_mtx_idx);

  virtual void calc_dw(const base_layer<Dtype>& top_layer, LMatrix<Dtype>& dw, const int train = 1) {}
  virtual void accumulate_signal(const base_layer<Dtype>& top, const LMatrix<Dtype>& w, const int train = 0);
  virtual void distribute_delta(base_layer<Dtype>& top, const LMatrix<Dtype>& w, const int train = 1);
private:
  Dtype dropout_rate_;
  LMatrix<Dtype> drop_mask_;
};

} // namespace leri
#endif // LERI_LAYER_DROPUOUT_H_
