/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_LAYER_FULLY_CONNECTED_LAYER_H_
#define LERI_LAYER_FULLY_CONNECTED_LAYER_H_

#include "leri/common.h"
#include "leri/layers/layer.h"

namespace leri {

template <typename Dtype>
class fully_connected_layer : public base_layer<Dtype> {
public:
  // fully_connected_layer(const string layer_name, int size, activation_function* p)
  fully_connected_layer(const string layer_name, int size, Activation<Dtype>* p)
  :base_layer<Dtype>(layer_name, size, 1, 1) {
    this->ptr_act_  = p; 
    this->use_bias_ = true;  
    this->bias_ = LMatrix<Dtype>(this->node_.cols, this->node_.rows, this->node_.chans);
    this->bias_.fill(0.);

  }
  virtual  ~fully_connected_layer() {}

  virtual string get_config_string();
  virtual void calc_dw(const base_layer<Dtype>& top_layer, LMatrix<Dtype>& dw, const int train = 1);
  virtual void update_bias(const LMatrix<Dtype>& newbias, Dtype alpha);
  virtual void accumulate_signal(const base_layer<Dtype>& top, const LMatrix<Dtype>& w, const int train = 0);
  virtual void distribute_delta(base_layer<Dtype>& top, const LMatrix<Dtype>& w, const int train = 1);
};

} // namespace leri
#endif // LERI_LAYER_FULLY_CONNECTED_LAYER_H_

