/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_LAYER_H_
#define LERI_LAYER_H_

#include "leri/common.h"
#include "leri/util/strutil.h"
#include "leri/util/matrix.h"
#include "leri/layers/activation.h"

namespace leri {

template <typename Dtype>
class base_layer {
public:
  base_layer(const string layer_name, int w, int h = 1, int c = 1) 
  :name_(layer_name),
   ptr_act_(NULL),
   col_pad_(0),
   row_pad_(0),
   node_(w, h, c),
   delta_(w, h, c,NULL,false),
   has_wgts_(true),
   use_bias_(false),
   lrate_(1.f),
   nthreads_(1) {}
  virtual ~base_layer(){ if(ptr_act_) { delete ptr_act_; } }

  virtual string get_config_string() = 0; 
  inline void set_learning_factor(Dtype val = 1.0f) { lrate_ = val; }
  void set_threading(int n);

  inline bool has_weights() { return has_wgts_; }
  inline bool use_bias() { return use_bias_; }

  virtual int get_fan_size() { return node_.chans*node_.rows*node_.cols; }
  virtual void resize(int _w, int _h = 1, int _c = 1);

  // inline float f(float *in, int i, int size, float bias) {return ptr_act_->f(in, i, size, bias);};
  Dtype df(Dtype* in, int i, int size);
  virtual void activate_nodes();
  virtual LMatrix<Dtype>* new_connection(base_layer &top, int weight_mat_index);

  virtual void calc_dw(const base_layer& top_layer, LMatrix<Dtype>& dw, const int train = 1) = 0;
  virtual void update_bias(const LMatrix<Dtype>& newbias, Dtype alpha) {};
  virtual void accumulate_signal(const base_layer& top_node, const LMatrix<Dtype>& w, const int train = 0) = 0;
  virtual void distribute_delta(base_layer& top, const LMatrix<Dtype>& w, const int train = 1) = 0;

public: 
  string name_;
  // activation_function* ptr_act_;
  Activation<Dtype>* ptr_act_;
  int col_pad_;
  int row_pad_;
  LMatrix<Dtype> node_;
  LMatrix<Dtype> bias_;
  LMatrix<Dtype> delta_;
  vector<pair<int, base_layer*> > linked_layers_forward_;
  vector<pair<int, base_layer*> > linked_layers_backward_;
protected:
  bool has_wgts_;
  bool use_bias_;
  Dtype lrate_;
  int nthreads_;
};

} // namespace
#endif // LERI_LAYER_H_
