/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_LAYER_MAXPOOLING_H_
#define LERI_LAYER_MAXPOOLING_H_

#include "leri/common.h"
#include "leri/layers/layer.h"

namespace leri {

template <typename Dtype>
class max_pooling_layer : public base_layer<Dtype> {
public:
  max_pooling_layer(const string layer_name, int pool_size)
  :base_layer<Dtype>(layer_name, 1) {
    stride_    = pool_size;
    pool_size_ = pool_size;
    this->has_wgts_  = false;
  }
  max_pooling_layer(const string layer_name, int pool_size, int stride )
  :base_layer<Dtype>(layer_name, 1) {
    stride_    = stride;
    pool_size_ = pool_size;
    this->has_wgts_  = false;
  }
  virtual  ~max_pooling_layer(){}

  virtual string get_config_string();
  virtual void resize(int w, int h = 1, int c = 1);

  virtual LMatrix<Dtype>* new_connection(base_layer<Dtype>& top, int wgt_mtx_idx);
  virtual void calc_dw(const base_layer<Dtype>& top_layer, LMatrix<Dtype>& dw, const int train = 1) {}
  virtual void accumulate_signal(const base_layer<Dtype>& top, const LMatrix<Dtype>& w, const int train = 0);
  virtual void distribute_delta(base_layer<Dtype>& top, const LMatrix<Dtype>& w, const int train = 1);

protected:
  int pool_size_;
  int stride_;
  vector<int> max_map_; // connect pooled outputs to top layer
};


} // namespace leri
#endif // LERI_LAYER_MAXPOOLING_H_
