/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

/**
 * This code was modified from mnist_parser.h https://github.com/nyanp/tiny-cnn
 * Copyright (c) 2013, T. Nomi
 * Copyright (c) 2018, Y. Jang
 *
 */

#ifndef LERI_UBYTE_H_
#define LERI_UBYTE_H_

#include "leri/common.h"

namespace leri {
// std::string data_name() {return std::string("MNIST");}

// from tiny_cnn
template<typename T>
T* reverse_endian(T* p) {
  std::reverse(reinterpret_cast<char*>(p), reinterpret_cast<char*>(p) + sizeof(T));
  return p;
}

template <typename Dtype>
class UBYTE {
public:
  UBYTE(){}
  ~UBYTE(){}
public:
  void parse_img_data() {};
  // void parse_mnist_header(std::ifstream& ifs, mnist_header& header);
  // from tiny_cnn (kinda)
  bool parse_mnist_labels(const string& label_file, vector<int> *labels);
  inline bool is_little_endian() {
    int x = 1;
    return *(char*) &x != 0;
  }
  // from tiny_cnn
  struct mnist_header {
    int magic_number;
    int num_items;
    int num_rows;
    int num_cols;
  };

  // from tiny_cnn (kinda)
  bool parse_mnist_images(const string& image_file, 
                          vector<vector<Dtype> > *images,
                          Dtype scale_min = -1.0, Dtype scale_max = 1.0,
                          int x_padding = 0, int y_padding = 0);

  // == load data (MNIST-28x28x1 size, no padding, pixel range -1 to 1)
  void parse_test_data(string data_path, vector<vector<Dtype> > &test_images, vector<int> &test_labels, 
    Dtype min_val=-1.f, Dtype max_val=1.f, int padx = 0, int pady = 0);

  void parse_train_data(string data_path, vector<vector<Dtype> > &train_images, vector<int> &train_labels, 
    Dtype min_val=-1.f, Dtype max_val=1.f, int padx = 0, int pady = 0);


private:

protected:
  DISABLE_COPY_AND_ASSIGN(UBYTE);
}; // class SENET




} // namespace


#endif
