
/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_LAYER_SHUFFLE_LAYER_H_
#define LERI_LAYER_SHUFFLE_LAYER_H_

#include "leri/common.h"
#include "leri/layers/layer.h"

namespace leri {

template <typename Dtype>
class shuffle_layer : public base_layer<Dtype> {
public:
  shuffle_layer(const string layer_name, int g)
  :base_layer<Dtype>(layer_name, 1) {
    // ptr_act_  = new_activation_function("identity");
    this->ptr_act_  = new_activation_func<Dtype>("identity");
    this->has_wgts_ = false;
    groups_   = g;
    if (groups_ < 1) { groups_ = 1; }
  }
  virtual  ~shuffle_layer() {}

  virtual string get_config_string();
  virtual void resize(int w, int h = 1, int c = 1);

  inline Dtype df(Dtype* in, int i, int size) { return 1.; };

  virtual LMatrix<Dtype>* new_connection(base_layer<Dtype>& top, int weight_mat_index);
  virtual void activate_nodes() { return; }
  virtual void calc_dw(const base_layer<Dtype>& top_layer, LMatrix<Dtype>& dw, const int train = 1) {}
  virtual void accumulate_signal(const base_layer<Dtype>& top, const LMatrix<Dtype>& w, const int train = 0);
  virtual void distribute_delta(base_layer<Dtype>& top, const LMatrix<Dtype>& w, const int train = 1);

private:
  int groups_;
};

} // namespace leri
#endif // LERI_LAYER_SHUFFLE_LAYER_H_

