/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_GFF_H_
#define LERI_GFF_H_

#include "leri/common.h"
#include "leri/util/strutil.h"

namespace leri {

// https://www.ensembl.org/info/website/upload/gff.html
// Fields must be tab-separated. Also, all but the final field in each feature line must contain a value; "empty" columns should be denoted with a '.'

// seqname - name of the chromosome or scaffold; chromosome names can be given with or without the 'chr' prefix. Important note: the seqname must be one used within Ensembl, i.e. a standard chromosome name or an Ensembl identifier such as a scaffold ID, without any additional content such as species or assembly. See the example GFF output below.
// source - name of the program that generated this feature, or the data source (database or project name)
// feature - feature type name, e.g. Gene, Variation, Similarity
// start - Start position of the feature, with sequence numbering starting at 1.
// end - End position of the feature, with sequence numbering starting at 1.
// score - A floating point value.
// strand - defined as + (forward) or - (reverse).
// frame - One of '0', '1' or '2'. '0' indicates that the first base of the feature is the first base of a codon, '1' that the second base is the first base of a codon, and so on..
// attribute - A semicolon-separated list of tag-value pairs, providing additional information about each feature.
template <typename Dtype>
class GFF {
public:
  GFF(string output_dirt = "", string job_name = "")
  :output_dirt_(output_dirt),
   job_name_(job_name) {
    base_name_ = output_dirt_ + job_name_;
  }
  ~GFF(){}

  void read_file(const string file, char delimiter = '\t');

private:
  string output_dirt_;    // output directory
  string job_name_;       // job name
  string base_name_;

  // Fields
  vector<string> seqname_;
  vector<string> source_;
  vector<string> feature_;
  vector<int>    start_;
  vector<int>    end_;
  vector<string> score_;
  vector<string> strand_;
  vector<string> frame_;
  vector<string> attribute_;

protected:
  DISABLE_COPY_AND_ASSIGN(GFF);
}; // class GFF

} // namespace leri

#endif // LERI_GFF_H_
