/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_LIPID_H_
#define LERI_LIPID_H_

#include "leri/common.h"
// #include "leri/lipid/yeast_genes.h"
#include "leri/lipid/gff.h"
#include "leri/stats/tsne.h"
// #include "leri/stats/pca.h"
#include "leri/util/fileutil.h"
#include "leri/gio/json.h"
#include "leri/gio/javas.h"
#include "leri/gio/gnuplot.h"

namespace leri {

template <typename Dtype>
class Lipid {
public:
  
  Lipid(string output_dirt = "", string job_name = "", int max_iter = 1000)
  :output_dirt_(output_dirt), job_name_(job_name), max_iter_(max_iter) {
    base_name_ = output_dirt_ + job_name_;
  }
  ~Lipid(){}

public:
  void init(int kdim, Dtype perplexity);
  void actuate(const string dirt);
  void finish() {};
private:
  void read_all_libraries(vector<string>& lib, const string dirt);
private:
  string output_dirt_;    // output directory
  string job_name_;       // job name
  string base_name_;

  int num_library_;
  int num_gene_;
  vector<vector<string> > name_gene_;
  // vector<string> name_gene_;
  vector<vector<Dtype> > mtx_t_; // number of transposons per gene (all)
  vector<vector<Dtype> > mtx_r_; // number of reads per gene (all)


  int max_iter_;
  int kdim_; // the dimensions of the expected
  Dtype perplexity_; // tunable parameter for t-SNE

  int nrow_ ;
  Dtype** mtx_transposon_; // number of transposons per gene
  Dtype** mtx_read_; // number of reads per gene
protected:
  DISABLE_COPY_AND_ASSIGN(Lipid);
}; // class Lipid

} // namespace leri

#endif // LERI_LIPID_H_
