/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_PHSI_NEW_H_
#define LERI_PHSI_NEW_H_

#include "leri/common.h"
#include "leri/gio/figure.h"
#include "leri/gio/html.h"
#include "leri/gio/javas.h"

#include "leri/layers/parser_ubyte.h"
#include "leri/layers/network.h"

namespace leri {

template <typename Dtype>
class Imgor {
public:
  Imgor(string output_dirt = "", string job_name = "", string cfg = "", string solver = "rmsprop", int echo = 20, int mbs = 24, Dtype lrate = 0.04, int nthreads = 1)
  :output_dirt_(output_dirt),
   job_name_(job_name),
   cfg_(cfg),
   solver_(solver), 
   echo_(echo),
   mbs_(mbs), 
   lrate_(lrate), 
   nthreads_(nthreads) {
    base_name_ = output_dirt_ + job_name_;
  }
  ~Imgor(){}

  void set_dirt(string d) { dirt_ = d; }
  // void set_mini_batch_size(int n) { batch_size_ = n; };
  void actuate();
  void finish();
  
  inline void set_info(string task, string user_name, string jobid, string jobcpt, string jobwat, string jobrun) {
    task_       = task;
    user_name_  = user_name;
    job_id_     = jobid;
    job_cpt_    = jobcpt;
    job_wat_    = jobwat;
    job_run_    = jobrun;
    job_tot_    = num2string(stoi(job_cpt_) + stoi(job_wat_) + stoi(job_run_), 0);
  }

private:
  // Dtype train();
  Dtype test(Network<float>& cnn, const vector<vector<float>>& test_images, const vector<int>& test_labels);
  void predict(const string model, const vector<vector<float> >& test_images);
private:
  string output_dirt_;
  string job_name_;
  string base_name_;
  string cfg_;
  string solver_;
  int echo_;
  int mbs_;
  Dtype lrate_;
  int nthreads_;
  string dirt_;

  // int batch_size_;
  string task_;
  string user_name_;
  string job_id_;
  string job_tot_;
  string job_cpt_;
  string job_wat_;
  string job_run_;

  vector<vector<float>> test_images_;
  vector<int> test_labels_;
  vector<vector<float>> train_images_;
  vector<int> train_labels_;
protected:
  DISABLE_COPY_AND_ASSIGN(Imgor);
}; // class Phsior

} // namespace leri
#endif // LERI_PHSI_H_
