/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_NIRECOR_H_
#define LERI_NIRECOR_H_

#include "leri/common.h"
#include "leri/util/strutil.h"
#include "leri/solvers/chpso.h"
#include "leri/stats/pearson.h"

namespace leri {

// Neural Network
// #define NOP 1   // Number of output
#define HLN 5      // Number of nodes in hidden layer: 5

template <typename Dtype>
class NiRecor {
public:
  NiRecor(string dirt, int phase, string output_dirt = "", string job_name = "", int regression = 1, int max_iter = 2e3, int necho = 10, int nout = 1) 
    :dirt_(dirt), 
     output_dirt_(output_dirt), 
     job_name_(job_name),
     phase_(phase),
     max_iter_(max_iter),
     necho_(necho),
     regression_(regression),
     nout_(nout) {
      base_name_ = output_dirt_ + job_name_;
    }
  ~NiRecor() {}

public:
  void init() {};
  void actuate(string param = "", int nout = 1);
  void finish() {};
private:
  void load_data(string param = "");
  void load_data2(string param = "");
  static Dtype fnn_model(const Dtype* xvar, const int nvar, void* instance, Dtype* gvar = NULL, Dtype** out = NULL);
  static void nirecor_fnn(Dtype* out, Dtype* xvar, int n, Dtype* wght, Dtype* bias, int nout = 1);
  void calc_rate(Dtype** mtx, Dtype** mat);
  void result_report(int nrow, Dtype** real_and_pred, int n);

private:
  string dirt_;
  string output_dirt_;
  string job_name_; 
  string base_name_;

  int phase_;
  int max_iter_;
  int necho_;
  int nvar_;
  int regression_;

  Dtype ** feature_;
  Dtype ** out_;
  Dtype *  param_;

  int nrow_;
  int ncol_;
  int nout_;
  int idx_bias_;

  Dtype mse_;
  Dtype acc_; // accuracy
  Dtype prs_; // precision
  Dtype rcl_; // recall
  Dtype sfy_; // specificity
  Dtype f1s_; // F1 score

  vector<string> str_;  // TODO:

protected:
  DISABLE_COPY_AND_ASSIGN(NiRecor);
}; // class NiRecor

} // namespace leri
#endif // LERI_NIRECOR_H_
