/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_OPTIFEL_H_
#define LERI_OPTIFEL_H_

#include "leri/common.h"
#include "leri/solvers/chpso.h"
#include "leri/stats/pearson.h"

namespace leri {

template <typename Dtype>
class Optifel {
public:
  Optifel(string dirt, int phase, string output_dirt = "", string job_name = "", int nrule = 3, int max_iter = 2e3, int necho = 5) 
    :dirt_(dirt), 
     output_dirt_(output_dirt), 
     job_name_(job_name),
     phase_(phase),
     max_iter_(max_iter),
     necho_(necho),
     nrule_(nrule) {
      base_name_ = output_dirt_ + job_name_;
    }
  ~Optifel() {}

public:
  void init(string param = "", int nout = 1);
  void actuate();
  void finish() {};

private:
  void load_data(string param = "");
  static Dtype ts_fuzzy_model(const Dtype* xvar, const int nvar, void* instance, Dtype* gvar = NULL, Dtype** out = NULL);
  void result_report(int nrow, Dtype** real_and_pred, int n);

private:
  string dirt_;
  string output_dirt_;
  string job_name_; 
  string base_name_;

  int phase_;
  int max_iter_;
  int necho_;
  int nvar_;
  int nrule_;
  
  // OptiFel
  Dtype** feature_;
  Dtype** out_;
  Dtype* param_;
  Dtype mse_;
  int nrow_;
  int ncol_;
  int nout_;

  vector<string> str_;  // TODO:

protected:
  DISABLE_COPY_AND_ASSIGN(Optifel);
}; // class OptiFel

} // namespace leri
#endif // LERI_OPTIFEL_H_
