/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_PROTEIN_SECONDARY_STRUCT_H_
#define LERI_PROTEIN_SECONDARY_STRUCT_H_

#include "leri/common.h"
#include "leri/util/matrix.h"
#include "leri/util/fileutil.h"
#include "leri/gio/figure.h"
#include "leri/gio/html.h"
#include "leri/gio/javas.h"

#include "leri/layers/parser_ubyte.h"
#include "leri/layers/network.h"

#include "leri/protein/bioutil.h"

namespace leri {

template <typename Dtype>
class Pross {
public:
  Pross(string output_dirt = "", string job_name = "", string cfg = "", string solver = "rmsprop", int echo = 20, int mbs = 24, Dtype lrate = 0.04, int nthreads = 1)
  :output_dirt_(output_dirt),
   job_name_(job_name),
   cfg_(cfg),
   solver_(solver), 
   echo_(echo),
   mbs_(mbs), 
   lrate_(lrate), 
   nthreads_(nthreads) {
    base_name_ = output_dirt_ + job_name_;
  }
  ~Pross(){}

  // void set_mini_batch_size(int n) { batch_size_ = n; };
  void init();
  void actuate();
  void finish();
  
  inline void set_info(string task, string user_name, string jobid, string jobcpt, string jobwat, string jobrun) {
    task_       = task;
    user_name_  = user_name;
    job_id_     = jobid;
    job_cpt_    = jobcpt;
    job_wat_    = jobwat;
    job_run_    = jobrun;
    job_tot_    = num2string(stoi(job_cpt_) + stoi(job_wat_) + stoi(job_run_), 0);
  }

  void set_phase(bool phase) { phase_ = phase; }
  void set_list(string list) { list_  = list; }
  void set_dirt(string dirt) { dirt_  = dirt; }
  void set_window(int w)     { win_   = w; }
  void set_number(int n)     { num_samples_ = n; }

private:
  void make_data();
  Dtype train();
  Dtype test(Network<Dtype>& cnn, const vector<vector<Dtype>>& test_data, const vector<int>& test_label);
  void predict();

private:
  string output_dirt_;
  string job_name_;
  string base_name_;
  string cfg_;
  string solver_;
  int echo_;
  int mbs_;
  Dtype lrate_;
  int nthreads_;
  string list_;
  string dirt_;

  // int batch_size_;
  bool phase_;
  int num_samples_;
  int win_;
  string task_;
  string user_name_;
  string job_id_;
  string job_tot_;
  string job_cpt_;
  string job_wat_;
  string job_run_;

  vector<vector<Dtype> > train_data_;
  vector<int> train_label_;
  vector<vector<Dtype> > test_data_;
  vector<int> test_label_;
protected:
  DISABLE_COPY_AND_ASSIGN(Pross);
}; // class Pross

} // namespace leri
#endif // LERI_PROTEIN_SECONDARY_STRUCT_H_
