/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_SENET_H_
#define LERI_SENET_H_

#include "leri/common.h"
#include "leri/util/timer.h"
#include "leri/util/strutil.h"
#include "leri/gio/figure.h"
#include "leri/gio/html.h"
#include "leri/gio/javas.h"

#include "leri/layers/parser_ubyte.h"
#include "leri/layers/network.h"

namespace leri {
// ARNDCQEGHILKMFPSTWYV
// QEGHLKWYV
// U: inputs
// F: fully-connected layer
// O: outputs
// C: concate

// R: residue-block
// I: 1x1 conv
// P: pooling
// S: semi-pooling
// D: dropout

// Convolutional kernel size
// H : 1
// N : 3
// C : 5
// O : 7
static map<char, string> AA2NET = {{'U', "input 28 28 1"}, {'F', "semi_stochastic_pool 2 2"}, {'O', "softmax 10"},
                                   {'C', "concate"},
                                   {'I', "convolution 1 16 1 elu"},
                                   {'T', "convolution 3 32 1 elu"},
                                   {'M', "convolution 5 8 1 elu"},
                                   {'N', "convolution 5 48 1 elu"},
                                   {'P', "semi_stochastic_pool 2 2"},
                                   {'S', "semi_stochastic_pool 3 3"}
                                  };

template <typename Dtype>
class SENET {
public:
  SENET(string output_dirt = "", string job_name = "", string cfg = "", string solver = "rmsprop", int echo = 20, int mbs = 24, Dtype lrate = 0.04, int nthreads = 1)
  :output_dirt_(output_dirt),
   job_name_(job_name),
   cfg_(cfg),
   solver_(solver), 
   echo_(echo),
   mbs_(mbs), 
   lrate_(lrate), 
   nthreads_(nthreads) {
    base_name_ = output_dirt_ + job_name_;
  }
  ~SENET(){}
public:
  void init();
  void actuate();
  void finish();
  void set_dirt(string d) { dirt_ = d; }
  inline void set_info(string task, string user_name, string jobid, string jobcpt, string jobwat, string jobrun) {
    task_       = task;
    user_name_  = user_name;
    job_id_     = jobid;
    job_cpt_    = jobcpt;
    job_wat_    = jobwat;
    job_run_    = jobrun;
    job_tot_    = num2string(stoi(job_cpt_) + stoi(job_wat_) + stoi(job_run_), 0);
  }
  void generate_net(); // width is even number, e.g., 1, 3, 5, ...
private:
  void search_best_graph();
  // Dtype train();
  Dtype test(Network<float>& cnn, const vector<vector<float>>& test_images, const vector<int>& test_labels);
  void predict(const string model, const vector<vector<float> >& test_images);
private:
  string str_random(string str, int nlayers);
private:
  string output_dirt_;
  string job_name_;
  string base_name_;
  string cfg_;
  string solver_;
  int echo_;
  int mbs_;
  Dtype lrate_;
  int nthreads_;
  string dirt_;

  // int batch_size_;
  string task_;
  string user_name_;
  string job_id_;
  string job_tot_;
  string job_cpt_;
  string job_wat_;
  string job_run_;

  string str_aa_;
  vector<vector<string> > mtx_str_net_;
  int depth_;
  int width_;


  vector<vector<float>> test_images_;
  vector<int> test_labels_;
  vector<vector<float>> train_images_;
  vector<int> train_labels_;
protected:
  DISABLE_COPY_AND_ASSIGN(SENET);
}; // class SENET

} // namespace leri
#endif // LERI_SENET_H_
