/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

/**
 * This file contains the implementation of calculating 
 *
 */

#ifndef LERI_DE_BRUJIN_GRAPH_H_
#define LERI_DE_BRUJIN_GRAPH_H_

#include "leri/common.h"
#include "leri/omics/de_brujin_node.h"
#include "leri/omics/bloom_filter.h"
#include <map>
#include <utility>
#include <set>

namespace leri {

template <typename Dtype>
class DeBrujinGraph {
public:
  explicit DeBrujinGraph(string file_name, int kmer_size);
  void read_contig();
  void build_graph();
  void euler_path(vector< vector<string> >& path);

  void kmerize(vector<string>& kmers, vector<string> reads, int k);

  void assembled_sequence(string& read);
  ~DeBrujinGraph(){}

  multimap<DeBrujinNode<Dtype>, DeBrujinNode<Dtype>> graph_;
private:
  string file_name_;
  int kmer_size_;
  vector<string> contigs_;
  vector< vector<string> > read_path_;

  map<string, DeBrujinNode<Dtype>> nodes;
  int num_semi_;
  int num_bal_;
  int num_neither_;
  
  DeBrujinNode<Dtype> head_;
  DeBrujinNode<Dtype> tail_;
  bool hasEulerianWalk();
  bool hasEulerianCycle();
  bool isEulerian();
protected:
  DISABLE_COPY_AND_ASSIGN(DeBrujinGraph);
}; // class DeBrujinGraph

} // namespace leri
#endif // LERI_DE_BRUJIN_GRAPH_H_
