/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

/**
 * This file contains the implementation of calculating 
 *
 */

#ifndef LERI_DE_BRUJIN_NODE_H_
#define LERI_DE_BRUJIN_NODE_H_

#include "leri/common.h"

namespace leri {

template <typename Dtype>
class DeBrujinNode {
public:
  /**
   * Constructor that creates a new node and assigns string a to the kmer value of the node 
   */
   DeBrujinNode(); 
  /**
   * Getter function for kmer string
   */
  string getKmer() const;
  void setKmer(string a);
  /** 
   * Overloading of operators < and = so that instances of this class may be placed in STL containers
   */
 DeBrujinNode<Dtype>& operator=(const DeBrujinNode<Dtype> &rhs);
 int operator<(const DeBrujinNode<Dtype> &rhs) const;
  bool isSemiBalanced();
  bool isBalanced();
  int indeg;
  int outdeg;

private:
  /**
   * The string value that the node holds 
   */  
  string kmer;
// protected:
  // DISABLE_COPY_AND_ASSIGN(DeBrujinNode);
}; // class DeBrujinNode

} // namespace leri
#endif // LERI_DE_BRUJIN_NODE_H_
