/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

/**
 * This file contains the implementation of Omics 
 *
 */

#ifndef LERI_OMICS_H_
#define LERI_OMICS_H_

#include "leri/common.h"

// #include "sibe/bsam.h"
#define _A 0
#define _C 1
#define _G 2
#define _T 3
#define _N 4

// TODO:
#define leri_min(a,b) ( (a) < (b) ? (a) : (b) )
#define leri_max(a,b) ( (a) > (b) ? (a) : (b) )

namespace leri {

template <typename Dtype>
class Omics {
public:
  // explicit Omics(){ a = 0; }
  Omics(){}
  virtual ~Omics(){}

  // BamReader<Dtype> reader;
  void statistics();
  // // BAM
  // void read_bam(const string& filename, const string& indexFilename = "", const bool lookForIndex = false, const bool preferStandardIndex = false);
  // // typ: short or long read
  // void read_fastq(string& _fastq, const string fastq_file, bool typ);
  // void fastq_split();

  // void fastq_kmer(int k);

  // Evaluate basic stats for a fasta/fastq file
  //   Output:
  // - Bases: total number of bases found in the file
  // - contigs: total number of contigs/scaffolds/reads found
  // - mean_length: mean lenght of contigs/scaffolds/reads
  // - longest: longest contig/scaffold/read
  // - N50: contig/scaffold/read N50
  // - n: number of contigs/scaffolds/reads contributing to the N50
  // static vector<int> read_len;
  // static vector<string> read_seq;
  // static vector<string> rqual;
  // static vector<string> rname;
  // static vector<string> rcomment;

  // Read stats
  int num_read;
  int len_max;
  unsigned long int total_bp;
  vector<int> read_len;
  vector<string> read_seq;
  vector<string> rqual;
  vector<string> rname;
  vector<string> rcomment;

  // Qual stats
  int phred;
  int min_phred;
  int max_phred;
  Dtype qual_mean;
  Dtype qual_std;
  Dtype qual_median;
  Dtype qual_25th;
  Dtype qual_75th;
  Dtype qual_sum;
  vector<Dtype> per_read_qual;
  vector<unsigned int> per_base_qual;
  vector<unsigned int> per_base_count;

  void fastx_type(int* type, const string& fid);
  void fastx_init();
  void fastx_read(const string fid, int fastx_typ, int saveinfo = 0, int readseq = 0, int minlen = 0, string selctg = "");
  void fastx_calc();
  void get_phred();
  void fastx_trim(const string fid, int fastx_typ);
  void fastx_stat(const string file_name);
  void fastx_quality_transform(string read_qual);
  void calc_percentile(Dtype* val, vector<Dtype> vect, size_t size, Dtype percentile);
  void fastx_finish();


  // SFF to Fastq
  /* This is the overall sff header file */
  // #define SFF_MAGIC   0x2e736666 /* ".sff" */
  // #define VERSION "0.9.2"
  // #define SFF_VERSION "\0\0\0\1"
  // #define PRG_NAME "sff2fastq"
  static const int PADDING_SIZE = 8;
  // #define SFF_VERSION_LEN 4
  static const uint32_t SFF_MAGIC = 0x2e736666;
  static const int SFF_VERSION_LEN = 4;
  static const char* SFF_VERSION;
  // static const string SFF_VERSION;// = "\0\0\0\1";

  typedef struct {
    uint32_t  magic;
    char      version[4];
    uint64_t  index_offset;
    uint32_t  index_len;
    uint32_t  num_of_reads;
    uint16_t  header_len;
    uint16_t  key_len;
    uint16_t  flow_len;
    uint8_t   flowgram_format;
    char     *flow_chars;
    char     *key_seq;
    // uint8_t  *eight_byte_padding;
  } sff_common_header;

/*
 * There is a read_header per "reading" in the SFF archive.
 * It too is padded to an 8-byte boundary.
 */
  typedef struct {
    uint16_t  header_len;
    uint16_t  name_len;
    uint32_t  nbases;
    uint16_t  clip_qual_left;
    uint16_t  clip_qual_right;
    uint16_t  clip_adapter_left;
    uint16_t  clip_adapter_right;
    char     *name;
  } sff_read_header;

/*
 * The read_data section per reading, following the read_header.
 * It is padded to an 8-byte boundary.
 */
  typedef struct {
    uint16_t *flowgram;   /* x 100.0 */
    uint8_t  *flow_index; /* relative to last */
    char     *bases;
    uint8_t  *quality;
  } sff_read_data;
  void sff2fastq(const string sff_file, const string fastq_file, int trim_flag);
  void read_sff_common_header(FILE *fp, sff_common_header *h);
  void verify_sff_common_header(sff_common_header *h);
  void read_padding(FILE *fp, int header_size);
  void read_sff_read_header(FILE *fp, sff_read_header *rh);
  void read_sff_read_data(FILE *fp, sff_read_data *rd, uint16_t nflows, uint32_t nbases);
  void get_clip_values(sff_read_header rh, int trim_flag, int *left_clip, int *right_clip);
  char* get_read_bases(sff_read_data rd, int left_clip, int right_clip);
  uint8_t* get_read_quality_values(sff_read_data rd, int left_clip, int right_clip);




  void construct_fastq_entry(FILE *fp, char *name, char *bases, uint8_t *quality, int nbases);
  void base2idx(unsigned int* idx, unsigned char _aa);


private:
  // int a;  // TODO:
protected:
  DISABLE_COPY_AND_ASSIGN(Omics);
}; // class Omics

} // namespace leri
#endif // LERI_OMICS_H_
