/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_ACO_H_
#define LERI_ACO_H_

#include "leri/common.h"
#include "leri/util/aco_randoms.cc"
// #include "leri/util/random.h"

namespace leri {


template <typename Dtype>
class ACO {

private:
  
  // int ntrial   = 5;
  int max_gen  = 2000;
  int num_ants = 100;
  int num_nodes, start_node;
  Dtype alpha = 0.8;
  Dtype beta  = 1.0;
  Dtype rho   = 0.15;
  Dtype tau   = 3.0;
  Dtype Q     = 4700;
  Dtype best_len;

  vector<int> best_path;
  vector<vector<int> > graph;
  vector<vector<int> > paths;
  vector<vector<Dtype> > pheromones, delta_pheromone, probabilites;
  Randoms* randoms;

  void update_pheromone();
  int valid(int antk, int iteration);
  void route(int antk);
  int node();
  Dtype length(int antk);
  Dtype PHI(int nodei, int nodej, int antk);
  bool visited(int antk, int c);
  bool exists(int nodei, int nodej);
  void create_graph(vector<vector<int> >& adj_mtx, vector<vector<int> > x);
public:
  ACO();
  ~ACO();
  void init(vector<int> config, vector<double>param, vector<vector<int> > overlaps, vector<vector<int> >& adj_mtx);
  // void printPheromones();
  // void print_result();
  void actuation();
  vector<int> get_best_path() { return best_path; }
protected:
  DISABLE_COPY_AND_ASSIGN(ACO);
};


} // namespace leri


#endif // LERI_ACO_H_
