/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_CHPSO_SOLVER_H_
#define LERI_CHPSO_SOLVER_H_

#include "leri/common.h"
#include "leri/gio/javas.h"
#include "leri/gio/gnuplot.h"
#include "leri/gio/figure.h"

namespace leri {

template <typename Dtype>
class CHPSO {
public:
  CHPSO<Dtype>(int type, string output_dirt = "", string job_name = "", int ntrial = 30, int max_iter = 2000) 
    :output_dirt_(output_dirt), 
     job_name_(job_name),
     max_iter_(max_iter),
     ntrial_(ntrial) { 
      base_name_ = output_dirt_ + job_name_;
    }
  CHPSO<Dtype>(string output_dirt = "", string job_name = "", int nvar = 20, int snp = 8, int max_iter = 2e3, int ntrial = 30) 
    :output_dirt_(output_dirt), 
     job_name_(job_name),
     max_iter_(max_iter),
     nvar_(nvar), 
     ntrial_(ntrial) {
     snp_ = snp;
      base_name_ = output_dirt_ + job_name_;
    }
  ~CHPSO(){}

  typedef Dtype (*evaluate_func)(const Dtype* xvar, const int n, void* instance, Dtype* gvar, Dtype** mout);
  void init();
  void actuate(evaluate_func proc_evaluate, Dtype* xvar, void *instance);
  void finish();

private:  
  void search_space();
  void selection(Dtype* a, Dtype* b, int idx);
  // void minimize(Dtype* opt);
  void minimize(evaluate_func proc_evaluate, Dtype* fval, void *instance, Dtype *record = NULL);
  void ichpso_minimize(evaluate_func proc_evaluate, Dtype* fval, void *instance, Dtype *record = NULL);

private:
  string output_dirt_;
  string job_name_; 
  string base_name_;

  int max_iter_;
  int nvar_;
  int ntrial_;

  int snp_;        // Number of populations in sub-group
  Dtype acc_;      // Acceleration factor
  Dtype iwt_max_;  // Maximum inertia weight
  Dtype iwt_min_;  // Minimum inertia weight
  Dtype vel_limt_; // Limit of velocity
  Dtype** ss_;     // Searching space
  Dtype* mv_;      // Moving step
  Dtype rmin_;     // Lower boundary
  Dtype rmax_;     // Upper boundary


  Dtype **pos_a_, **pos_b_, **pos_c_, **pos_d_;
  Dtype **velocity_a_, **velocity_b_, **velocity_c_, **velocity_d_;
  Dtype **localpos_a_, **localpos_b_, **localpos_c_, **localpos_d_;
  Dtype *global_a_, *global_b_, *global_c_, *global_d_, *global_;
  Dtype *grad_;

protected:
  DISABLE_COPY_AND_ASSIGN(CHPSO);
}; // class CHPSO

} // namespace leri
#endif // LERI_CHPSO_SOLVER_H_
