/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_JDE_SOLVER_H_
#define LERI_JDE_SOLVER_H_

#include "leri/common.h"
#include "leri/gio/javas.h"
#include "leri/gio/gnuplot.h"
#include "leri/gio/figure.h"

namespace leri {

template <typename Dtype>
class JDE {
public:
  JDE<Dtype>(int type, string output_dirt = "", string job_name = "", int ntrial = 30, int max_iter = 2000) 
    :output_dirt_(output_dirt), 
     job_name_(job_name),
     max_iter_(max_iter),
     ntrial_(ntrial) { 
      base_name_ = output_dirt_ + job_name_;
    }
  JDE<Dtype>(string output_dirt = "", string job_name = "", int nvar = 20, int np = 12, int max_iter = 2e3, int ntrial = 30) 
    :output_dirt_(output_dirt), 
     job_name_(job_name),
     max_iter_(max_iter),
     nvar_(nvar), 
     ntrial_(ntrial) {
     npop_ = np;
      base_name_ = output_dirt_ + job_name_;
    }
  ~JDE() {}

  typedef Dtype (*evaluate_func)(const Dtype* xvar, const int n, void* instance, Dtype* gvar, Dtype** mout);

  void init();
  void actuate(evaluate_func proc_evaluate, Dtype* xvar, void *instance);
  void finish();
private:
  void search_space();
  void minimize(evaluate_func proc_evaluate, Dtype* fval, void *instance, Dtype *record = NULL);
  void jde_generate_R1R2R3(vector<int>& r1, vector<int>& r2, vector<int>& r3, int np, vector<int> r0, long rnd_uni_init);


private:
  string output_dirt_;
  string job_name_; 
  string base_name_;

  int max_iter_;
  int nvar_;
  int ntrial_;
  int npop_;        // Number of populations in sub-group

  Dtype **popold, **pop, **vi, **ui;
  Dtype *pval, *oval; // Fitness of parents and offsprings
  Dtype **mask;
  int *rows, *cols;
  Dtype *jrandi;
  Dtype* grad_;
  Dtype** ss_;
  
  Dtype TAU_1 = 0.1;
  Dtype TAU_2 = 0.1;
  Dtype Fw    = 0.5;
  Dtype CRw   = 0.9;
protected:
  DISABLE_COPY_AND_ASSIGN(JDE);
}; // class JDE

} // namespace leri
#endif // LERI_JDE_SOLVER_H_
