/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_SOLVER_H_
#define LERI_SOLVER_H_

#include "leri/common.h"
#include "leri/util/malloc.h"
#include "leri/util/matrix.h"

// // hack for VS2010 to handle c++11 for(:)
// #if (_MSC_VER  == 1600)
//  #ifndef __for__
//  #define __for__ for each
//  #define __in__ in
//  #endif
// #else
  // #ifndef __for__
  // #define __for__ for
  // #define __in__ :
  // #endif
// #endif

namespace leri {



// template <typename Dtype>
// class SolverParam {
// public:
//   int m;
//   int past;
//   int max_iterations;
//   int linesearch;
//   int max_linesearch;
//   Dtype epsilon;
//   Dtype delta;
//   Dtype min_step;
//   Dtype max_step;
//   Dtype ftol;
//   Dtype wolfe;
// public:
//   SolverParam() {
//     // m              = 6;            // The number of corrections to approximate the inverse hessian matrix
//     // past           = 0;            // Distance for delta-based conergence
//     // max_iterations = 1e4;
//     // max_linesearch = 5e8;
//     // epsilon        = Dtype(1e-5);  // Tolerance for convergence
//     // delta          = Dtype(0.0);     // Delta for convergence
//     // linesearch     = LBFGS_LINESEARCH_BACKTRACKING;// LBFGS_LINESEARCH_BACKTRACKING_ARMIJO;  // The line search strategy
//     // min_step       = Dtype(1e-20); // The minimum step length allowed in the line search
//     // max_step       = Dtype(1e+20); // The maximum step length allowed in the line search
//     // ftol           = Dtype(1e-4);  // A parameter to control the accuracy of the line search routine, (0, 0.5)
//     // wolfe          = Dtype(0.9);   // A coefficient for the Wolfe condition, (ftol, 1)
//   }
//   ~SolverParam(){}
//   void set_param(int nvar, int npop) {
//     nvar_ = nvar;
//     npop_ = npop;
//   }
//   int get_nvar() { return nvar_; }
//   int get_npop() { return npop_; }
// private:
//   int nvar_;
//   int npop_;
// };


// template <typename Dtype>
// class Solver {
// // public: // TODO
// public:
//   explicit Solver(const SolverParam<Dtype>& param);
//   // explicit Solver(const string& param_file);
//   // ~Solver(){}
//   // virtual void init(const SolverParam<Dtype>& param) {}
// public:
//   // learning rates are 'tweaked' in inc_w function so that they can be similar for all solvers
//   Dtype learning_rate;
//   Solver(): learning_rate(0.01f) {}
//   virtual ~Solver(){}
//   virtual void reset() {}
//   // this increments the weight matrix w, which corresponds to connection index 'g'
//   // bottom is the number of grads coming up from the lower layer
//   // top is the current output node value of the upper layer
//   virtual void increment_w(LMatrix<float> *w,  int g, const LMatrix<float> &dW, const float custom_factor=1.0f){}// LMatrix *top){}
//   virtual void push_back(int w, int h, int c){} 
// private:
// protected:
//   DISABLE_COPY_AND_ASSIGN(Solver);
// };



// template <typename Dtype>
// class sgd: public Solver<Dtype>{
// public:
//   static const char *name() { return "sgd"; }

//   virtual void increment_w(LMatrix<float> *w,  int g, const LMatrix<float> &dW, const float custom_factor = 1.0f)
//   {
//     const Dtype w_decay = 0.01f; // 1;
//     const Dtype lr = custom_factor*this->learning_rate;
//     for(int s=0; s <w->size(); s++)  
//       w->x[s] -= lr*(dW.x[s] + w_decay*w->x[s]);
//   }
// };

// template <typename Dtype>
// class adagrad: public Solver<Dtype> {
//   // persistent variables that mirror size of weight LMatrix
//   vector<LMatrix<float> *> G1;
// public:
//   static const char *name(){return "adagrad";}

//   virtual ~adagrad() {
//     for(int i = 0; i < G1.size(); i++) {
//       delete G1[i];
//     }
//   }

//   virtual void push_back(int w, int h, int c) {
//     G1.push_back(new LMatrix<float>(w, h, c));
//   }// G1[G1.size() - 1]->fill(0);

  
//   virtual void reset() { for(auto g : G1) g->fill(0.f);}
//   virtual void increment_w(LMatrix<float> *w,  int g, const LMatrix<float> &dW, const float custom_factor = 1.0f)
//   {
//     float *g1 = G1[g]->x;
//     // float min, max;
//     // G1[g]->min_max(&min, &max);
//     // cout << "((" << min << "," << max << ")";
//     const float eps = 1.e-8f;
//     // if(G1[g]->size() != w->size()) throw;
//     const float lr = custom_factor*this->learning_rate;
//     for(int s=0; s<w->size(); s++) 
//     {
//       g1[s] += dW.x[s] * dW.x[s];
//       // if(g1[s] < 1) throw;
//       w->x[s] -= lr*dW.x[s]/(sqrt(g1[s]) + eps);
//     } 
//   };
// };
// template <typename Dtype>
// class rmsprop: public Solver<Dtype>
// {
//   // persistent variables that mirror size of weight LMatrix
//   vector<LMatrix<float> *> G1;
// public:
//   static const char *name(){return "rmsprop";}
//   virtual ~rmsprop(){for(auto g : G1) delete g;}

//   virtual void push_back(int w, int h, int c){ G1.push_back(new LMatrix<float>(w,h,c)); G1[G1.size() - 1]->fill(0);}
//   virtual void reset() { for(auto g : G1) g->fill(0.f);}
//   virtual void increment_w(LMatrix<float> *w,  int g, const LMatrix<float> &dW, const float custom_factor = 1.0f)
//   {
//     float *g1 = G1[g]->x;
//     const float eps = 1.e-8f;
//     const float mu = 0.999f;
//     const float lr = 0.01f*custom_factor*this->learning_rate;

//     for(int s=0; s<(int)w->size(); s++) {
//       g1[s] = mu * g1[s]+(1-mu) * dW.x[s] * dW.x[s];
//       w->x[s] -= lr*dW.x[s]/(sqrt(g1[s]) + eps);
//     } 
//   };

// };
// template <typename Dtype>
// class adam: public Solver<Dtype>
// {
//   float b1_t, b2_t;
//   const float b1, b2;
//   // persistent variables that mirror size of weight LMatrix
//   vector<LMatrix<float> *> G1;
//   vector<LMatrix<float> *> G2;
// public:
//   static const char *name(){return "adam";}
//   adam(): Solver<Dtype>(), b1_t(0.9f), b2_t(0.999f), b1(0.9f), b2(0.999f) {}
//   virtual ~adam(){
//     // for(auto g : G1) delete g; 
//     for(int i = 0; i < G1.size(); i++) {
//       delete G1[i];
//     }
//     for(auto g : G2) delete g;
//   }

//   virtual void reset()
//   {
//     b1_t*=b1; b2_t*=b2;
//     for(auto g : G1) g->fill(0.f);
//     for(auto g : G2) g->fill(0.f);
//   }

//   virtual void push_back(int w, int h, int c)
//   {
//     G1.push_back(new LMatrix<float>(w,h,c)); G1[G1.size() - 1]->fill(0); 
//     G2.push_back(new LMatrix<float>(w,h,c)); G2[G2.size() - 1]->fill(0);
//   }

//   virtual void increment_w(LMatrix<float> *w,  int g, const LMatrix<float> &dW, const float custom_factor = 1.0f)
//   {
//     float *g1 = G1[g]->x;
//     float *g2 = G2[g]->x;
//     const float eps = 1.e-8f;
//     const float b1=0.9f, b2=0.999f;
//     const float lr = 0.1f*custom_factor*this->learning_rate;
//     for(int s=0; s<(int)w->size(); s++)
//       {
//         g1[s] = b1* g1[s]+(1-b1) * dW.x[s];
//         g2[s] = b2* g2[s]+(1-b2) * dW.x[s]*dW.x[s];
//         w->x[s] -= lr* (g1[s]/(1.f-b1_t)) / ((float)sqrt(g2[s]/(1.-b2_t)) + eps);
//       } 
//   };
// };

// template <typename Dtype>
// Solver<Dtype>* new_solver(const char *type) {
//   if(type==NULL) return NULL;
//   string act(type);
//   if(act.compare(sgd<Dtype>::name())==0) { return new sgd<Dtype>();}
//   if(act.compare(rmsprop<Dtype>::name())==0) { return new rmsprop<Dtype>();}
//   if(act.compare(adagrad<Dtype>::name())==0) { return new adagrad<Dtype>();}
//   if(act.compare(adam<Dtype>::name())==0) { return new adam<Dtype>();}

//   return NULL;
// }

















// template <typename Dtype>
// class solver {
// public:
//   // learning rates are 'tweaked' in inc_w function so that they can be similar for all solvers
//   Dtype learning_rate_;
//   solver():learning_rate_(0.01f) {}
//   virtual ~solver(){}
//   virtual void reset() {}
//   // this increments the weight LMatrix w, which corresponds to connection index 'g'
//   // bottom is the number of grads coming up from the lower layer
//   // top is the current output node value of the upper layer
//   virtual void increment_w(LMatrix<Dtype> *w,  int g, const LMatrix<Dtype> &dW, const Dtype custom_factor=1.0f){}//, LMatrix *top){}
//   virtual void push_back(int w, int h, int c){} 
// };

// // #ifndef LERI_NO_TRAINING


// template <typename Dtype>
// class SGD: public solver<Dtype> {
// public:
//   static const char *name() { return "sgd"; }

//   virtual void increment_w(LMatrix<Dtype> *w,  int g, const LMatrix<Dtype> &dW, const Dtype custom_factor = 1.0f)
//   {
//     const Dtype w_decay=0.01f;//1;
//     const Dtype lr=custom_factor*this->learning_rate_;
//     for(int s=0; s<w->size(); s++)  
//       w->x[s] -= lr*(dW.x[s] + w_decay*w->x[s]);
//   }
// };

// template <typename Dtype>
// class ADAGRAD : public solver<Dtype> {
//   // persistent variables that mirror size of weight LMatrix<Dtype>
//   vector<LMatrix<Dtype> *> G1;
// public:
//   static const char *name(){return "adagrad";}

//   // ~ADAGRAD() { __for__(auto g __in__ G1) delete g; }
//   virtual void push_back(int w, int h, int c) {
//     G1.push_back(new LMatrix<Dtype>(w, h, c));
//   }// G1[G1.size() - 1]->fill(0);

  
//   virtual void reset() { __for__(auto g __in__ G1) g->fill(0.f);}
//   virtual void increment_w(LMatrix<Dtype> *w,  int g, const LMatrix<Dtype> &dW, const Dtype custom_factor = 1.0f)
//   {
//     Dtype *g1 = G1[g]->x;
//     //Dtype min, max;
//     //G1[g]->min_max(&min, &max);
//     //cout << "((" << min << "," << max << ")";
//     const Dtype eps = 1.e-8f;
//     // if (G1[g]->size() != w->size()) throw;
//     const Dtype lr = custom_factor*this->learning_rate_;
//     for(int s=0; s<w->size(); s++) 
//     {
//       g1[s] += dW.x[s] * dW.x[s];
//       //if (g1[s] < 1) throw;
//       w->x[s] -= lr*dW.x[s]/(sqrt(g1[s]) + eps);
//     } 
//   };
// };

// template <typename Dtype>
// class RMSPROP : public solver<Dtype> {
//   // persistent variables that mirror size of weight LMatrix<Dtype>
//   vector<LMatrix<Dtype> *> G1;
// public:
//   static const char *name(){return "rmsprop";}
//   // ~RMSPROP() {__for__(auto g __in__ G1) delete g;}

//   virtual void push_back(int w, int h, int c){ G1.push_back(new LMatrix<Dtype>(w,h,c)); G1[G1.size() - 1]->fill(0);}
//   virtual void reset() { __for__(auto g __in__ G1) g->fill(0.f);}
//   virtual void increment_w(LMatrix<Dtype> *w,  int g, const LMatrix<Dtype> &dW, const Dtype custom_factor = 1.0f)
//   {
//     Dtype *g1 = G1[g]->x;
//     const Dtype eps = 1.e-8f;
//     const Dtype mu = 0.999f;
//     const Dtype lr = 0.01f*custom_factor*this->learning_rate_;

//     for(int s=0; s<(int)w->size(); s++)
//     {
//       g1[s] = mu * g1[s]+(1-mu) * dW.x[s] * dW.x[s];
//       w->x[s] -= lr*dW.x[s]/(sqrt(g1[s]) + eps);
//     } 
//   };

// };

// template <typename Dtype>
// class ADAM: public solver<Dtype> {
//   const Dtype b1;
//   Dtype b1_t;
//   const Dtype b2;
//   Dtype b2_t;
//   // persistent variables that mirror size of weight LMatrix<Dtype>
//   vector<LMatrix<Dtype> *> G1;
//   vector<LMatrix<Dtype> *> G2;
// public:
//   static const char *name(){return "adam";}
//   ADAM(): solver<Dtype>(), b1(0.9f), b1_t(0.9f), b2(0.999f), b2_t(0.999f) {}
//   // virtual ~ADAM(){__for__(auto g __in__ G1) delete g; __for__(auto g __in__ G2) delete g;}

//   virtual void reset()
//   {
//     b1_t*=b1; b2_t*=b2;
//     __for__(auto g __in__ G1) g->fill(0.f);
//     __for__(auto g __in__ G2) g->fill(0.f);
//   }

//   virtual void push_back(int w, int h, int c)
//   {
//     G1.push_back(new LMatrix<Dtype>(w,h,c)); G1[G1.size() - 1]->fill(0); 
//     G2.push_back(new LMatrix<Dtype>(w,h,c)); G2[G2.size() - 1]->fill(0);
//   }

//   virtual void increment_w(LMatrix<Dtype> *w,  int g, const LMatrix<Dtype> &dW, const Dtype custom_factor = 1.0f)
//   {
//     Dtype *g1 = G1[g]->x;
//     Dtype *g2 = G2[g]->x;
//     const Dtype eps = 1.e-8f;
//     const Dtype b1=0.9f, b2=0.999f;
//     const Dtype lr = 0.1f*custom_factor*this->learning_rate_;
//     for(int s=0; s<(int)w->size(); s++)
//       {
//         g1[s] = b1* g1[s]+(1-b1) * dW.x[s];
//         g2[s] = b2* g2[s]+(1-b2) * dW.x[s]*dW.x[s];
//         w->x[s] -= lr* (g1[s]/(1.f-b1_t)) / ((Dtype)sqrt(g2[s]/(1.-b2_t)) + eps);
//       } 
//   }

// };

// template<typename Dtype>
// solver<Dtype>* new_solver(string type) {
// // solver<Dtype>* new_solver(const char *type) {
//   // if(type == NULL) return NULL;
//   // string act(type);
//   string act = type;
//   if(act.compare(SGD<Dtype>::name())==0) { return new SGD<Dtype>();}
//   if(act.compare(RMSPROP<Dtype>::name())==0) { return new RMSPROP<Dtype>();}
//   if(act.compare(ADAGRAD<Dtype>::name())==0) { return new ADAGRAD<Dtype>();}
//   if(act.compare(ADAM<Dtype>::name())==0) { return new ADAM<Dtype>();}
//   return NULL;
// }
// // template<typename Dtype>
// // solver<Dtype>* new_solver(const char *type) { return NULL; }
// // template<typename Dtype>
// // solver<Dtype>* new_solver(string act) { return NULL; }





// #endif





template<typename Dtype>
class Solver {
public:
  Solver() : learning_rate_(0.01f) {}
  virtual ~Solver() {}
  virtual void increment_w(LMatrix<Dtype>* w,  int g, const LMatrix<Dtype>& dW, const Dtype custom_factor = 1.0f) {}
  virtual void push_back(int w, int h, int c) {} 
  virtual void reset() {}
public:
  Dtype learning_rate_;
};

template<typename Dtype>
class sgd : public Solver<Dtype> {
public:
  static const string name() { return "sgd"; }
  virtual void increment_w(LMatrix<Dtype> *w, int g, const LMatrix<Dtype> &dW, const Dtype custom_factor = 1.0f) {
    const Dtype w_decay = 0.01f;
    const Dtype lr = custom_factor*this->learning_rate_;
    for(int i = 0; i < w->size(); i++) {
      w->x[i] -= lr*(dW.x[i] + w_decay*w->x[i]);
    }
  }
};

template<typename Dtype>
class adagrad : public Solver<Dtype> {
public:
  static const string name() { return "adagrad"; }
  virtual ~adagrad() {
    for(int i = 0; i < G1.size(); i++) { delete G1[i]; }
  }
  virtual void push_back(int w, int h, int c) {
    G1.push_back(new LMatrix<Dtype>(w, h, c));
  }
  virtual void reset() {
    for(int i = 0; i < G1.size(); i++) {
      G1[i]->fill(0.f);
    }
  }
  virtual void increment_w(LMatrix<Dtype>* w, int idx, const LMatrix<Dtype>& dW, const Dtype custom_factor = 1.0f) {
    Dtype* g1 = G1[idx]->x;
    const Dtype eps = 1.e-8f;
    const Dtype lr = custom_factor*this->learning_rate_;
    for(int i = 0; i < w->size(); i++) {
      g1[i] += dW.x[i] * dW.x[i];
      w->x[i] -= lr*dW.x[i]/(sqrt(g1[i]) + eps);
    } 
  }
private:
  vector<LMatrix<Dtype> *> G1;
};

template<typename Dtype>
class rmsprop : public Solver<Dtype> {
public:
  static const string name(){return "rmsprop";}
  virtual ~rmsprop(){
    for(int i = 0; i < G1.size(); i++) { delete G1[i]; }
  }
  virtual void push_back(int w, int h, int c){ G1.push_back(new LMatrix<Dtype>(w,h,c)); G1[G1.size() - 1]->fill(0);}
  virtual void reset() {
    for(int i = 0; i < G1.size(); i++) {
      G1[i]->fill(0.f);
    }
  }
  virtual void increment_w(LMatrix<Dtype> *w,  int g, const LMatrix<Dtype> &dW, const Dtype custom_factor = 1.0f) {
    Dtype* g1 = G1[g]->x;
    const Dtype eps = 1.e-8f;
    const Dtype mu = 0.999f;
    const Dtype lr = 0.01f*custom_factor*this->learning_rate_;
    for(int i = 0; i < (int)w->size(); i++) {
      g1[i] = mu * g1[i]+(1-mu) * dW.x[i] * dW.x[i];
      w->x[i] -= lr*dW.x[i]/(sqrt(g1[i]) + eps);
    } 
  }
private:
  vector<LMatrix<Dtype> *> G1;
};

template<typename Dtype>
class adam : public Solver<Dtype> {
public:
  static const string name() { return "adam"; }
  adam(): Solver<Dtype>(), b1(0.9f), b1_t(0.9f), b2(0.999f), b2_t(0.999f) {}
  virtual ~adam(){ 
    for(int i = 0; i < G1.size(); i++) { delete G1[i]; }
    for(int i = 0; i < G2.size(); i++) { delete G2[i]; }
  }
  virtual void reset() {
    b1_t *= b1;
    b2_t *= b2;
    for(int i = 0; i < G1.size(); i++) { G1[i]->fill(0.f); }
    for(int i = 0; i < G2.size(); i++) { G2[i]->fill(0.f); }
  }
  virtual void push_back(int w, int h, int c) {
    G1.push_back(new LMatrix<Dtype>(w,h,c));
    G1[G1.size() - 1]->fill(0); 
    G2.push_back(new LMatrix<Dtype>(w,h,c));
    G2[G2.size() - 1]->fill(0);
  }
  virtual void increment_w(LMatrix<Dtype>* w, int idx, const LMatrix<Dtype>& dW, const Dtype custom_factor = 1.0f) {
    Dtype *g1 = G1[idx]->x;
    Dtype *g2 = G2[idx]->x;
    const Dtype eps = 1.e-8f;
    const Dtype b1 = 0.9f;
    const Dtype b2 = 0.999f;
    const Dtype lr = 0.1f*custom_factor*this->learning_rate_;
    for(int i = 0; i < (int)w->size(); i++) {
      g1[i] = b1 * g1[i]+(1-b1) * dW.x[i];
      g2[i] = b2 * g2[i]+(1-b2) * dW.x[i]*dW.x[i];
      w->x[i] -= lr * (g1[i]/(1.f-b1_t)) / ((Dtype)sqrt(g2[i]/(1.-b2_t)) + eps);
    } 
  }
private:
  const Dtype b1;
  Dtype b1_t;
  const Dtype b2;
  Dtype b2_t;
  vector<LMatrix<Dtype> *> G1;
  vector<LMatrix<Dtype> *> G2;
};

// Solver<Dtype>* new_solver(string type);
template <typename Dtype>
Solver<Dtype>* new_solver(string type) {
  // if(type == NULL) { return NULL; }
  // string act(type);
  if(type.compare(sgd<Dtype>::name())     == 0) { return new sgd<Dtype>();}
  if(type.compare(rmsprop<Dtype>::name()) == 0) { return new rmsprop<Dtype>();}
  if(type.compare(adagrad<Dtype>::name()) == 0) { return new adagrad<Dtype>();}
  if(type.compare(adam<Dtype>::name())    == 0) { return new adam<Dtype>();}
  return NULL;
}

}  // namespace leri
#endif  // LERI_SOLVER_H_
