/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

/**
 * This file contains the implementation of calculating 
 *
 */

#ifndef LERI_ICA_H_
#define LERI_ICA_H_

#include "leri/common.h"
#include "leri/util/fileutil.h"
#include "leri/util/matrix.h"
#include "leri/stats/svd.h"
// #include "leri/gio/json.h"
#include "leri/gio/javas.h"
#include "leri/gio/gnuplot.h"

namespace leri {

template <typename Dtype>
class ICA {
public:
  ICA(string file, string output_dirt = "", string job_name = "")
  :output_dirt_(output_dirt),
   job_name_(job_name),
   matrix_file_(file) {
   base_name_ = output_dirt_ + job_name_;
   max_iter_  = 1e3;
   tolerance_ = 1e-4;
  }
  ~ICA(){};
  void init();
  void calc();
  void finish();
private:
  /**
   * Main FastICA function. Centers and whitens the input
   * matrix, calls the ICA computation function estimate_unmixing_mat()
   * and computes the output matrixes.
   */
  void fast_ica();
  /**
   * Estimate the mat_eum_ matrix from the
   * preprocessed data.
   */
  void estimate_unmixing_mat(Dtype** Wd, Dtype** X);
  void write_out();

private:
  string output_dirt_;
  string job_name_; 
  string base_name_;
  string matrix_file_;
  int max_iter_;
  Dtype tolerance_;
  int nrow_;
  int ncol_;
  int num_;
  Dtype** mat_raw_;  // pre-processed data matrix [nrow_, ncol_]
  Dtype** mat_eum_;  // pre-whitening matrix that projects data onto the first [num_] principal components
  Dtype** mat_pwm_;  // estimated un-mixing matrix
  Dtype** mat_emm_;  // estimated mixing matrix
  Dtype** mat_esm_;  // estimated source matrix

protected:
  DISABLE_COPY_AND_ASSIGN(ICA);
};


} // namespace leri
#endif // LERI_ICA_H_


