/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

/**
 * Copyright (c) 2014, Laurens van der Maaten (Delft University of Technology)
 * All rights reserved.
 */

#ifndef LERI_STATISTICS_TSNE_H_
#define LERI_STATISTICS_TSNE_H_

#include "leri/common.h"
#include "leri/stats/vptree.h"
#include "leri/stats/sptree.h"
#include "leri/stats/tsne.h"
#include "leri/util/malloc.h"
#include "leri/util/matrix.h"
#include "leri/util/math.h"
#include "leri/gio/figure.h"
#include "leri/gio/gnuplot.h"
#include "leri/gio/javas.h"
#include "leri/lipid/yeast_genes.h"

namespace leri {

static inline double sign(double x) { return (x == .0 ? .0 : (x < .0 ? -1.0 : 1.0)); }

template <typename Dtype>
class TSNE {
public:
  TSNE(string output_dirt = "", string job_name = "", string postfix = "", string caption = "")
  :output_dirt_(output_dirt), job_name_(job_name), postfix_(postfix), caption_(caption) {
    base_name_ = output_dirt_ + job_name_;
  }
  ~TSNE(){}

public:
  void init(const string file, const int max_iter = 1000, char delimiter = ',');
  // void actuate(double* X, int N, int D, double* Y, int no_dims, double perplexity, double theta, int rand_seed,
       // bool skip_random_init, int max_iter=1000, int stop_lying_iter=250, int mom_switch_iter=250);

  void actuate(int kdim, double perplexity = 20, const vector<string> dlabels = vector<string>(), int stop_lying_iter = 250, int mom_switch_iter = 250);
  void finish();
  

  bool load_data(double** data, int* n, int* d, int* no_dims, double* theta, double* perplexity, int* rand_seed, int* max_iter);
  void save_data(double* data, int* landmarks, double* costs, int n, int d);
  void symmetrizeMatrix(unsigned int** row_P, unsigned int** col_P, double** val_P, int N); // should be static!


private:
  void computeGradient(double* P, unsigned int* inp_row_P, unsigned int* inp_col_P, double* inp_val_P, double* Y, int N, int D, double* dC, double theta);
  void computeExactGradient(double* P, double* Y, int N, int D, double* dC);
  double evaluateError(double* P, double* Y, int N, int D);
  double evaluateError(unsigned int* row_P, unsigned int* col_P, double* val_P, double* Y, int N, int D, double theta);
  void zeroMean(double* X, int N, int D);
  void computeGaussianPerplexity(double* X, int N, int D, double* P, double perplexity);
  void computeGaussianPerplexity(double* X, int N, int D, unsigned int** _row_P, unsigned int** _col_P, double** _val_P, double perplexity, int K);
  void computeSquaredEuclideanDistance(double* X, int N, int D, double* DD);
  double randn();

private:
  string output_dirt_;    // output directory
  string job_name_;       // job name
  string base_name_;
  string postfix_;
  string caption_;

  int max_iter_;
  int nrow_;
  int ncol_;

  vector<string> name_row_;
  double* vect_x_;
  double* vect_y_;
  Dtype** mat_;
  double* x_;
  double* y_;
  double perplexity_; // typical values are between 5 and 50.
  double theta_;

  // Dtype** mtx_transposon_; // number of transposons per gene
  // Dtype** mtx_read_; // number of reads per gene
protected:
  DISABLE_COPY_AND_ASSIGN(TSNE);
}; // class tSNE




} // namespace leri
#endif // LERI_STATISTICS_TSNE_H_
