/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */
 
#ifndef LERI_HDF5UTILS_H_
#define LERI_HDF5UTILS_H_

#include <hdf5.h>
#include "leri/common.h"

namespace leri {

template <typename Dtype>
static void hdf5_read(const string file, string path_dset, int dim, hsize_t* dim_array, Dtype* data) {
  // hid_t id_memspace = H5Screate_simple(dim, dim_array, NULL);
  // hid_t id_dset     = H5Dcreate(file_id, path_dset.c_str(), datatype, id_memspace, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  // hid_t id_fspace   = H5Dget_space(id_dset);
  // H5Dwrite(id_dset, datatype, id_memspace, id_fspace, H5P_DEFAULT, data);

  hid_t id_file     = H5Fopen(file.c_str(), H5F_ACC_RDONLY, H5P_DEFAULT);
  hid_t id_memspace = H5Screate_simple(dim, dim_array, NULL);
  hid_t id_dset     = H5Dopen(id_file, path_dset.c_str(), H5P_DEFAULT);
  hid_t id_fspace   = H5Dget_space(id_dset);
  hid_t type_dset   = H5Dget_type(id_dset);
  H5Dread(id_dset, type_dset, id_memspace, id_fspace, H5P_DEFAULT, data);

  H5Dclose(id_dset);
  H5Sclose(id_memspace);
  H5Sclose(id_fspace);
  H5Fclose(id_file);
}

template <typename Dtype>
static void hdf5_write(hid_t file_id, hid_t datatype, string data_path, int dim, hsize_t* dim_array, Dtype* data) {
  hid_t id_memspace = H5Screate_simple(dim, dim_array, NULL);
  hid_t id_dset     = H5Dcreate(file_id, data_path.c_str(), datatype, id_memspace, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  hid_t id_fspace   = H5Dget_space(id_dset);
  H5Dwrite(id_dset, datatype, id_memspace, id_fspace, H5P_DEFAULT, data);

  H5Dclose(id_dset);
  H5Sclose(id_memspace);
  H5Sclose(id_fspace);
  // H5Fclose(file_id);
}

// // template <typename Dtype>
// static void hdf5_write_str(hid_t file_id, string data_path, int dim, hsize_t* dim_array, vector<string> str) {
// // hid_t       file_id;
// hid_t dataset_id, dataspace_id;  /* identifiers */
//    herr_t      status;
//    hid_t       dtype;
//    size_t      size;

//    // file_id = H5Fcreate(FILE, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);

// LOG(INFO)<<"test";
//    hsize_t dims[1] =  {dim};
//    dataspace_id = H5Screate_simple(1, dims, NULL);
// LOG(INFO)<<"test";


//    dtype = H5Tcopy (H5T_C_S1);
// LOG(INFO)<<"test";
//    size = dim * sizeof(char);
// LOG(INFO)<<"test";
//    status = H5Tset_size (dtype, size);
// LOG(INFO)<<"test";

//    int nrow = str.size();
//   char strs[nrow][dim];
// LOG(INFO)<<"test " <<nrow<<":"<<dim;
//   for (int i = 0; i < nrow; i++) {
//     for (int j = 0; j < nrow; j++) {
//       LOG(INFO)<<strs[i][j];
//       LOG(INFO)<<str[i][j];
//       strs[i][j] = (str[i][j]);
//     }
//   }
// LOG(INFO)<<"test";
//     // = {
//     //  "this",
//     //  "this",
//     //  "this",
//     //  "this"};

//    dataset_id = H5Dcreate(file_id, data_path.c_str(), dtype, dataspace_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
// LOG(INFO)<<"test";

//    status = H5Dwrite (dataset_id, dtype, H5S_ALL, H5S_ALL, H5P_DEFAULT, strs);
// LOG(INFO)<<"test";

//    status = H5Dclose(dataset_id);
//    status = H5Sclose(dataspace_id);
//    status = H5Tclose(dtype);
//    // status = H5Fclose(file_id);

// }




















/////////////////////////////////////////////////////////////
class Attr_wr {
public:
  // void Attr_String(hid_t id,const char* name, const char* value, int len,int mode);
  // void Attr_Int(hid_t id, const char* name, int value);
  // void Attr_Float64(hid_t id, const char* name, double value);

  hid_t datatype;
  hid_t memspace;
  hid_t attribute_id;

void Attr_String(hid_t id,const char* name, const char* value, int len,int mode = 1) {
  datatype = H5Tcopy(H5T_C_S1);
  H5Tset_size(datatype, len);
  switch (mode) {
    case 1: {
      memspace = H5Screate(H5S_SCALAR);
      break;
    }
    case 2: {
      hsize_t dim1[1];
      dim1[0] = 1;
      memspace = H5Screate_simple(1, dim1, NULL);
      break;
    }
  }
  attribute_id = H5Acreate(id, name, datatype, memspace, H5P_DEFAULT, H5P_DEFAULT);
  H5Awrite(attribute_id, datatype, value);

}


void Attr_Int(hid_t id, const char* name, int value) {

  datatype = H5T_NATIVE_INT;
  memspace = H5Screate(H5S_SCALAR);
  attribute_id = H5Acreate(id, name, datatype, memspace, H5P_DEFAULT, H5P_DEFAULT);
  H5Awrite(attribute_id, datatype, &value);
}



void Attr_Float64(hid_t id, const char* name, double value){
  datatype = H5T_NATIVE_DOUBLE;
  memspace = H5Screate(H5S_SCALAR);
  attribute_id = H5Acreate(id, name, datatype, memspace, H5P_DEFAULT, H5P_DEFAULT);
  H5Awrite(attribute_id, datatype, &value);
}
};

} // namespace leri
#endif  // LERI_HDF5UTILS_H_
