/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */
 
/* 
   A C-program for MT19937, with initialization improved 2002/1/26.
   Coded by Takuji Nishimura and Makoto Matsumoto.

   Before using, initialize the state by using init_genrand(seed)  
   or init_by_array(init_key, key_length).

   Copyright (C) 1997 - 2002, Makoto Matsumoto and Takuji Nishimura,
   All rights reserved.                          

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

     1. Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.

     2. Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.

     3. The names of its contributors may not be used to endorse or promote 
        products derived from this software without specific prior written 
        permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


   Any feedback is very welcome.
   http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/emt.html
   email: m-mat @ math.sci.hiroshima-u.ac.jp (remove space)
*/

#ifndef LERI_MT19937_H_
#define LERI_MT19937_H_

#include "leri/common.h"

namespace leri {

template <typename Dtype>
class MT19937 {
public:
  explicit MT19937(void) {
    N = 624;
    M = 397;
    MATRIX_A   = 0x9908b0dfUL;
    UPPER_MASK = 0x80000000UL;
    LOWER_MASK = 0x7fffffffUL;
    mt  = new unsigned long[N];
    mti = N+1;
    idum = 0;   // __091210__
    ival = 0.0; // __091210__
  }

  virtual ~MT19937() {}
  // ---init---
  void init_genrand(unsigned long s);
  void init_by_array(unsigned long init_key[], int key_length);

  // ---process--
  unsigned long genrand_int32(void);
  long genrand_int31(void);
  Dtype genrand_real1(void);
  Dtype genrand_real2(void);
  Dtype genrand_real3(void);
  Dtype genrand_res53(void);

  // ---additional--
  int choose_rand(Dtype *input,int total); // [input] must positive
  Dtype normal_distribution(void);   // sampling from N(0,1) // __091210__
private:
  int N;
  int M;
  unsigned long MATRIX_A;
  unsigned long UPPER_MASK;
  unsigned long LOWER_MASK;
  unsigned long *mt;
  int mti;
  int idum;    // normal_distribution N(0,1) //__091210__
  Dtype ival;  // normal_distribution N(0,1) //__091210__
protected:
  DISABLE_COPY_AND_ASSIGN(MT19937);
}; // class MT19937

} // namespace leri
#endif
