/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */
 
#ifndef LERI_STRUTILS_H_
#define LERI_STRUTILS_H_

#include <iterator>
#include "leri/common.h"

namespace leri {

const string WHITESPACE = " \n\r\t\f\v";

static inline string check_path(string path) {
  string tpath = path;
  int found = tpath.find_last_of("/\\");
  int len = tpath.length();
  if (found != len-1) { tpath += "/"; }
  path = tpath;
  return path;
}

static inline string char2string(char c) {
  string s(1, c);
  return s;
}
static inline void str2lower(string& str) {
  transform (str.begin(), str.end(), str.begin(), ::tolower);
}
static inline void str2upper(string& str) {
  transform (str.begin(), str.end(), str.begin(), ::toupper);
}

static inline bool is_prefix(const string& s1, const string& s2) {
  return s1 == s2.substr(0, s1.size());
}
static inline int find_str_index(string s, const vector<string> str) {
  int idx = 0;
  for (int i = 0; i < str.size(); i++) {
    if (!s.compare(str[i])) {
      idx = i;
      break;
    }
  }
  return idx;
}
static inline bool excluded_str(const string str, const vector<string> vstr) {
  bool mode = false;
  for (int i = 0; i < vstr.size(); i++) {
    if (str == vstr[i]) {
      mode = true;
      break;
    }
  }
  return mode;
}
static inline string str_insert_space(string str, int n, bool lr = true) {
  int m = n-str.size(); 
  for (int k = 0; k < m; k++) { // TODO: bug if not define m
    if (lr) { str = " " + str; }
    if (!lr) { str += " "; }
  }
  return str;
}
static inline string to_str(string str, int n = 0, string delimiter = ".") {
  int pos = str.find_first_of('.');
  if (n != 0) {
    return str.substr(0, pos)+delimiter+str.substr(pos+1, n);
  } else {
    return str.substr(0, pos);
  }
}
template <typename Dtype>
static inline string num2string(Dtype num, int n = 0, int s = -1, bool lr = true) {
  string str = to_string(num);
  str = to_str(str, n);
  if (s != -1) {
    int m = s-(int)str.size();
    for (int k = 0; k < m; k++) { // TODO: bug if not define m
      if (lr) { str = " " + str; }
      else    { str += " "; }
    }
  }
  return str;
}

// TODO: merge the two functions
string replace_space(string str);
void str_replace(string& str, const string& from, const string& to);

vector<string> string_split(const string& str, char delimiter);
void string_print(const string str, int len = 60);
string string_print_seq(const string str, int len = 60, string delimiter = "\n", vector<int> idx = vector<int>());

/**
 * Trims the character c from the left of string str.
 */
static inline void trim_left(char *str, char c) {
  char *end;
  int len,i;
  len = strlen(str);
  while (*str && len) {
    end = str + len - 1;
    if(c == *str) {
      for(i=0;i<len-1;i++) {
        str[i]=str[i+1]; 
      }
      *end = '\0';
    } else {
      break;
    }
    len = strlen(str);
  }
}
/**
 * Trims the character c from the right of string str.
 */
static inline void trim_right(char* str, char c) {
  char *end;
  int len;
  len = strlen(str);
  while (*str && len) {
    end = str + len - 1;
    if(c == *end) {
      *end = '\0';
    } else {
      break;
    }
    len = strlen(str);
  }
}
/**
 * Trims not only spaces, but also tabs and line feeds, from the left and right of string str.
 */
static inline void trim_all(string* str_trim) {
  char* str = NULL;
  str = &(*str_trim->begin());
  // NOTE: trim_left must be passed by reference and trim_right must be passed by value
  trim_left(str, 0x09);   // horizontal tab    
  trim_left(str, 0x0b);   // vertical tab
  trim_left(str, 0x0c);   // form feed
  trim_left(str, 0x20);   // space

  trim_right(str, 0x09);   // horizontal tab
  trim_right(str, 0x0a);   // linefeed
  trim_right(str, 0x0b);   // vertical tab
  trim_right(str, 0x0c);   // form feed
  trim_right(str, 0x0d);   // carriage return
  trim_right(str, 0x20);   // space
  *str_trim = string(str);
}

template <typename Dtype>
void remove_duplicates(vector<Dtype>& vec) {
  sort(vec.begin(), vec.end());
  vec.erase(unique(vec.begin(), vec.end()), vec.end());
}

} // namespace leri
#endif  // LERI_STRUTILS_H_
