 /**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_RMSD_TMSCORE_H_
#define LERI_RMSD_TMSCORE_H_

#include "leri/common.h"

namespace leri {


// TODO: remove
#define SQRT3_ 1.732050807568877
template <class T>
void rmatrix (T ev,T r[9],T u[3][3]){   
// calculate rotation matrix
 
 T a00=(r[0]+r[4]+r[8]);
 T a01=(r[5]-r[7]);
 T a02=(r[6]-r[2]);
 T a03=(r[1]-r[3]);
 T a11=(r[0]-r[4]-r[8]);
 T a12=(r[1]+r[3]);
 T a13=(r[6]+r[2]);
 T a22=(-r[0]+r[4]-r[8]);
 T a23=(r[5]+r[7]);
 T a33=(-r[0]-r[4]+r[8]);

 //from Theobald
 a00-=ev;a11-=ev;a22-=ev;a33-=ev; 
 T a2233_3223 = a22 * a33 - a23 * a23; 
 T a1233_3123 = a12 * a33-a13*a23;
 T a1223_3122 = a12 * a23 - a13 * a22; 
 T a0232_3022 = a02 * a23-a03*a22;
 T a0233_3023 = a02 * a33 - a03 * a23;
 T a0231_3021 = a02 * a13-a03*a12;

 T q[4]={a11*a2233_3223-a12*a1233_3123+a13*a1223_3122, -a01*a2233_3223+a12*a0233_3023-a13*a0232_3022,a01*a1233_3123-a11*a0233_3023+a13*a0231_3021,-a01*a1223_3122+a11*a0232_3022-a12*a0231_3021};
 
 T len2q=q[0]*q[0]+q[1]*q[1]+q[2]*q[2]+q[3]*q[3];
 if(!len2q){
  //return the identity matrix
  u[0][0]= 1.0f; u[0][1]= 0.0f; u[0][2]= 0.0f; 
  u[1][0]= 0.0f; u[1][1]= 1.0f; u[1][2]= 0.0f; 
  u[2][0]= 0.0f; u[2][1]= 0.0f; u[2][2]= 1.0f;
 }
 else{
  T invlen2q=1.0/len2q;
  T aj=q[0]*q[0]*invlen2q;
  T xj=q[1]*q[1]*invlen2q;
  T yj=q[2]*q[2]*invlen2q;
  T zj=q[3]*q[3]*invlen2q;
  T  xy = q[1] * q[2]*invlen2q;
  T  az = q[0] * q[3]*invlen2q;
  T  zx = q[3] * q[1]*invlen2q;
  T  ay = q[0] * q[2]*invlen2q;
  T  yz = q[2] * q[3]*invlen2q;
  T  ax = q[0] * q[1]*invlen2q; 

  u[0][0]= aj + xj - yj - zj; u[0][1]= 2.0 * (xy + az); u[0][2]= 2.0 * (zx - ay); 
  u[1][0]= 2.0 * (xy - az);  u[1][1]=aj - xj + yj - zj; u[1][2]= 2.0 * (yz + ax); 
  u[2][0]= 2.0 * (zx + ay);  u[2][1]= 2.0 * (yz - ax); u[2][2]= aj - xj - yj + zj;
 }
} 

template <typename Dtype>
class RmsdTMscore {
public:
  explicit RmsdTMscore(string output_dirt = "", string job_name = "")
  :output_dirt_(output_dirt),
   job_name_(job_name) {
    base_name_ = output_dirt_ + job_name_;
  }
  ~RmsdTMscore() {}

  void init();
  void calc_rmsd();
  void calc_tmscore();
  void finish() {}

private:
  // Optimized Kabsch routine for calculting eigenvector matrix
  double rmsd_svd(int nat,double *dcoords,double u[3][3], double t[3],bool rmsd_flag);
  // iterative TMscore
  int score_fun_dcoords(int nat, float d0, float d, double R[3][3], double t[3],double *coords1, double *coords2,double *acoords,int *ialign,int *nalign,float *tm_score);
  // TODO:
  float tmscore_rmsd_cpu(int nat,float *coords1,float *coords2, float bR[3][3], float bt[3], float *rmsd);

public:
  string output_dirt_;    // output directory
  string job_name_;       // job name
  string base_name_;      // basic name
private:
  // Dtype SQRT3_;
protected:
  DISABLE_COPY_AND_ASSIGN(RmsdTMscore);
};



}  // namespace leri

#endif  // LERI_RMSD_TMSCORE_H_

