/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

/**
 * This file contains the class of DNA
 *
 */

#ifndef LERI_DNA_H_
#define LERI_DNA_H_

#include "leri/common.h"
#include "leri/util/strutil.h"
#include "leri/util/fileutil.h"
#include "leri/atgc/atgc_constant.h"

namespace leri {

template <typename Dtype>
class DNA {
public:
  DNA(string output_dirt = "", string job_name = "")
  :output_dirt_(output_dirt),
   job_name_(job_name) {
    base_name_ = output_dirt_ + job_name_;
  }
  ~DNA() {}

  void init();
  void read_file(string fid);
  string join_fragments(vector<vector<int> > X);
  void find_overlaps();
  void set_seq_order(vector<int> path) { seq_order.assign(path.begin()+1, path.end()); }
  vector<vector<int> > get_overlaps() { return overlaps; }

  void dna2aminoacid();
  
private:
  string output_dirt_;    // output directory
  string job_name_;       // job name
  string base_name_;


  string seq_str_;
  string fname_;


  string sequence;
  vector<int> seq_order;
  vector<string> fragments;
  vector<vector<int> > overlaps;
  int num;

  int LCS(string a, string b);
protected:
  DISABLE_COPY_AND_ASSIGN(DNA);
};

} // namespace leri


#endif // LERI_DNA_H_

