/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_YEAST_GENES_H_
#define LERI_YEAST_GENES_H_

#include "leri/common.h"

namespace leri {

// Yeast genes are listed with functions
static map<string, string> gene_func_map = {
  { "YAL069W", "NULL" },
  { "YAL068W-A", "NULL" },
  { "PAU8", "Protein of unknown function" },
  { "YAL067W-A", "Putative protein of unknown function" },
  { "SEO1", "Putative permease" },
  { "YAL066W", "NULL" },
  { "YAL065C", "Putative protein of unknown function" },
  { "YAL064W-B", "Fungal-specific protein of unknown function" },
  { "TDA8", "Putative protein of unknown function" },
  { "YAL064W", "Protein of unknown function" },
  { "YAL063C-A", "Putative protein of unknown function" },
  { "FLO9", "Lectin-like protein with similarity to Flo1p" },
  { "GDH3", "NADP(+)-dependent glutamate dehydrogenase" },
  { "BDH2", "Putative medium-chain alcohol dehydrogenase with similarity to BDH1" },
  { "BDH1", "NAD-dependent (R,R)-butanediol dehydrogenase" },
  { "YAL059C-A", "NULL" },
  { "ECM1", "Pre-ribosomal factor involved in 60S ribosomal protein subunit export" },
  { "CNE1", "Calnexin" },
  { "YAL056C-A", "NULL" },
  { "GPB2", "Multistep regulator of cAMP-PKA signaling" },
  { "PEX22", "Putative peroxisomal membrane protein" },
  { "ACS1", "Acetyl-coA synthetase isoform" },
  { "FLC2", "Putative calcium channel involved in calcium release under hypotonic" },
  { "OAF1", "Oleate-activated transcription factor" },
  { "AIM2", "Cytoplasmic protein involved in mitochondrial function or organization" },
  { "GEM1", "Outer mitochondrial membrane GTPase, subunit of the ERMES complex" },
  { "YAL047W-A", "NULL" },
  { "SPC72", "Gamma-tubulin small complex (gamma-TuSC) receptor" },
  { "AIM1", "NULL" },
  { "YAL045C", "NULL" },
  { "YAL044W-A", "NULL" },
  { "GCV3", "H subunit of the mitochondrial glycine decarboxylase complex" },
  { "PTA1", "Subunit of holo-CPF" },
  { "YAL042C-A", "NULL" },
  { "ERV46", "Protein localized to COPII-coated vesicles" },
  { "CDC24", "Guanine nucleotide exchange factor (GEF) for Cdc42p" },
  { "CLN3", "G1 cyclin involved in cell cycle progression" },
  { "CYC3", "Cytochrome c heme lyase (holocytochrome c synthase)" },
  { "CDC19", "Pyruvate kinase" },
  { "YAL037C-B", "NULL" },
  { "YAL037C-A", "Putative protein of unknown function" },
  { "YAL037W", "Putative protein of unknown function" },
  { "RBG1", "Member of the DRG family of GTP-binding proteins" },
  { "FUN12", "Translation initiation factor eIF5B" },
  { "YAL034C-B", "NULL" },
  { "MTW1", "Essential component of the MIND kinetochore complex" },
  { "FUN19", "Non-essential protein of unknown function" },
  { "POP5", "Subunit of both RNase MRP and nuclear RNase P" },
  { "PRP45", "Protein required for pre-mRNA splicing" },
  { "YAL031W-A", "NULL" },
  { "GIP4", "Cytoplasmic protein that regulates protein phosphatase 1 Glc7p" },
  { "SNC1", "Vesicle membrane receptor protein (v-SNARE)" },
  { "MYO4", "Type V myosin motor involved in actin-based transport of cargos" },
  { "FRT2", "Tail-anchored ER membrane protein of unknown function" },
  { "SAW1", "5'- and 3'-flap DNA binding protein" },
  { "YAL026C-A", "NULL" },
  { "DRS2", "Trans-golgi network aminophospholipid translocase (flippase)" },
  { "MAK16", "Essential nuclear protein" },
  { "LTE1", "Protein similar to GDP/GTP exchange factors" },
  { "PMT2", "Protein O-mannosyltransferase of the ER membrane" },
  { "FUN26", "High affinity, broad selectivity, nucleoside/nucleobase transporter" },
  { "CCR4", "Component of the CCR4-NOT transcriptional complex" },
  { "ATS1", "Protein required for modification of wobble nucleosides in tRNA" },
  { "YAL019W-A", "NULL" },
  { "FUN30", "Snf2p family member with ATP-dependent chromatin remodeling activity" },
  { "LDS1", "Protein Involved in spore wall assembly" },
  { "PSK1", "PAS domain-containing serine/threonine protein kinase" },
  { "YAL016C-B", "NULL" },
  { "YAL016C-A", "NULL" },
  { "TPD3", "Regulatory subunit A of the heterotrimeric PP2A complex" },
  { "NTG1", "DNA N-glycosylase and apurinic/apyrimidinic (AP) lyase" },
  { "SYN8", "Endosomal SNARE related to mammalian syntaxin 8" },
  { "DEP1", "Component of the Rpd3L histone deacetylase complex" },
  { "CYS3", "Cystathionine gamma-lyase" },
  { "SWC3", "Protein of unknown function" },
  { "MDM10", "Subunit of both the ERMES and the SAM complex" },
  { "SPO7", "Putative regulatory subunit of Nem1p-Spo7p phosphatase holoenzyme" },
  { "FUN14", "Integral mitochondrial outer membrane (MOM) protein" },
  { "ERP2", "Member of the p24 family involved in ER to Golgi transport" },
  { "SSA1", "ATPase involved in protein folding and NLS-directed nuclear transport" },
  { "YAL004W", "NULL" },
  { "EFB1", "Translation elongation factor 1 beta" },
  { "VPS8", "Membrane-binding component of the CORVET complex" },
  { "TFC3", "Subunit of RNA polymerase III transcription initiation factor complex" },
  { "NUP60", "FG-nucleoporin component of central core of the nuclear pore complex" },
  { "ERP1", "Member of the p24 family involved in ER to Golgi transport" },
  { "SWD1", "Subunit of the COMPASS (Set1C) complex" },
  { "RFA1", "Subunit of heterotrimeric Replication Protein A (RPA)" },
  { "SEN34", "Subunit of the tRNA splicing endonuclease" },
  { "BUD14", "Protein involved in bud-site selection" },
  { "ADE1", "N-succinyl-5-aminoimidazole-4-carboxamide ribotide synthetase" },
  { "KIN3", "Nonessential serine/threonine protein kinase" },
  { "CDC15", "Protein kinase of the Mitotic Exit Network" },
  { "YAR019W-A", "NULL" },
  { "PAU7", "Member of the seripauperin multigene family" },
  { "YAR023C", "Putative integral membrane protein" },
  { "UIP3", "Putative integral membrane protein of unknown function" },
  { "YAR028W", "Putative integral membrane protein" },
  { "YAR029W", "Member of DUP240 gene family but contains no transmembrane domains" },
  { "YAR030C", "NULL" },
  { "PRM9", "Pheromone-regulated protein" },
  { "MST28", "Putative integral membrane protein, involved in vesicle formation" },
  { "YAT1", "Outer mitochondrial carnitine acetyltransferase" },
  { "YAR035C-A", "Putative protein of unknown function" },
  { "SWH1", "Protein similar to mammalian oxysterol-binding protein" },
  { "YAR047C", "NULL" },
  { "FLO1", "Lectin-like protein involved in flocculation" },
  { "YAR053W", "NULL" },
  { "YAR060C", "NULL" },
  { "YAR064W", "Pseudogenic fragment with similarity to flocculins" },
  { "YAR066W", "Putative GPI protein" },
  { "YAR068W", "Fungal-specific protein of unknown function" },
  { "YAR069C", "NULL" },
  { "YAR070C", "NULL" },
  { "PHO11", "One of three repressible acid phosphatases" },
  { "IMD1", "NULL" },
  { "YAR075W", "NULL" },
  { "YBL113C", "Helicase-like protein encoded within the telomeric Y' element" },
  { "YBL113W-A", "NULL" },
  { "YBL112C", "Putative protein of unknown function" },
  { "YBL111C", "Helicase-like protein encoded within the telomeric Y' element" },
  { "YBL109W", "NULL" },
  { "PAU9", "Protein of unknown function" },
  { "YBL108W", "NULL" },
  { "YBL107W-A", "NULL" },
  { "MIX23", "Mitochondrial intermembrane space protein of unknown function" },
  { "SRO77", "Protein with roles in exocytosis and cation homeostasis" },
  { "PKC1", "Protein serine/threonine kinase" },
  { "SEA4", "Subunit of SEACAT, a subcomplex of the SEA complex" },
  { "RTG3", "bHLH/Zip transcription factor for retrograde (RTG) and TOR pathways" },
  { "SFT2", "Tetra-spanning membrane protein found mostly in the late Golgi" },
  { "ECM21", "Protein involved in regulating endocytosis of plasma membrane proteins" },
  { "YBL100W-C", "Putative protein of unknown function" },
  { "YBL100C", "NULL" },
  { "ATP1", "Alpha subunit of the F1 sector of mitochondrial F1F0 ATP synthase" },
  { "BNA4", "Kynurenine 3-monooxygenase" },
  { "BRN1", "Subunit of the condensin complex" },
  { "YBL096C", "NULL" },
  { "YBL095W", "NULL" },
  { "YBL094C", "NULL" },
  { "ROX3", "Subunit of the RNA polymerase II mediator complex" },
  { "RPL32", "Ribosomal 60S subunit protein L32" },
  { "SCS22", "Protein involved in regulation of phospholipid metabolism" },
  { "MAP2", "Methionine aminopeptidase" },
  { "MRP21", "Mitochondrial ribosomal protein of the small subunit" },
  { "AVT5", "Putative transporter" },
  { "TEL1", "Protein kinase primarily involved in telomere length regulation" },
  { "RPL23A", "Ribosomal 60S subunit protein L23A" },
  { "YBL086C", "Protein of unknown function" },
  { "BOI1", "Protein implicated in polar growth" },
  { "CDC27", "Subunit of the Anaphase-Promoting Complex/Cyclosome (APC/C)" },
  { "YBL083C", "NULL" },
  { "ALG3", "Dolichol-P-Man dependent alpha(1-3) mannosyltransferase" },
  { "YBL081W", "Non-essential protein of unknown function" },
  { "PET112", "Subunit of the trimeric GatFAB AmidoTransferase(AdT) complex" },
  { "NUP170", "Subunit of inner ring of nuclear pore complex (NPC)" },
  { "ATG8", "Component of autophagosomes and Cvt vesicles" },
  { "YBL077W", "NULL" },
  { "ILS1", "Cytoplasmic isoleucine-tRNA synthetase" },
  { "SSA3", "ATPase involved in protein folding and the response to stress" },
  { "AAR2", "Component of the U5 snRNP complex" },
  { "YBL073W", "NULL" },
  { "RPS8A", "Protein component of the small (40S) ribosomal subunit" },
  { "YBL071C-B", "Putative protein of unknown function" },
  { "KTI11", "Zn-ribbon protein that co-purifies with Dph1 and Dph2" },
  { "YBL071C", "Putative protein of unknown function" },
  { "YBL070C", "NULL" },
  { "AST1", "Lipid raft associated protein" },
  { "YBL068W-A", "NULL" },
  { "PRS4", "5-phospho-ribosyl-1(alpha)-pyrophosphate synthetase, synthesizes PRPP" },
  { "UBP13", "Ubiquitin-specific protease that cleaves Ub-protein fusions" },
  { "SEF1", "Putative transcription factor" },
  { "YBL065W", "NULL" },
  { "PRX1", "Mitochondrial peroxiredoxin with thioredoxin peroxidase activity" },
  { "KIP1", "Kinesin-related motor protein" },
  { "YBL062W", "NULL" },
  { "SKT5", "Activator of Chs3p (chitin synthase III) during vegetative growth" },
  { "YEL1", "Guanine nucleotide exchange factor specific for Arf3p" },
  { "CMC2", "Protein involved in respiratory chain complex assembly or maintenance" },
  { "YBL059W", "NULL" },
  { "SHP1", "UBX domain-containing substrate adaptor for Cdc48p" },
  { "PTH2", "One of two mitochondrially-localized peptidyl-tRNA hydrolases" },
  { "PTC3", "Type 2C protein phosphatase (PP2C)" },
  { "YBL055C", "3'-->5' exonuclease and endonuclease with a possible role in apoptosis" },
  { "TOD6", "PAC motif binding protein involved in rRNA and ribosome biogenesis" },
  { "YBL053W", "NULL" },
  { "SAS3", "Histone acetyltransferase catalytic subunit of NuA3 complex" },
  { "PIN4", "Protein involved in G2/M phase progression and response to DNA damage" },
  { "SEC17", "Alpha-SNAP cochaperone" },
  { "MOH1", "Protein of unknown function, essential for stationary phase survival" },
  { "RRT1", "Putative protein of unknown function" },
  { "EDE1", "Scaffold protein involved in the formation of early endocytic sites" },
  { "PSY4", "Regulatory subunit of protein phosphatase PP4" },
  { "COR1", "Core subunit of the ubiquinol-cytochrome c reductase complex" },
  { "YBL044W", "Putative protein of unknown function" },
  { "ECM13", "Non-essential protein of unknown function" },
  { "FUI1", "High affinity uridine permease, localizes to the plasma membrane" },
  { "PRE7", "Beta 6 subunit of the 20S proteasome" },
  { "ERD2", "HDEL receptor" },
  { "YBL039W-B", "NULL" },
  { "URA7", "Major CTP synthase isozyme (see also URA8)" },
  { "YBL039C-A", "NULL" },
  { "MRPL16", "Mitochondrial ribosomal protein of the large subunit" },
  { "APL3", "Alpha-adaptin" },
  { "YBL036C", "Putative non-specific single-domain racemase" },
  { "POL12", "B subunit of DNA polymerase alpha-primase complex" },
  { "STU1", "Microtubule plus-end-tracking non-motor protein" },
  { "RIB1", "GTP cyclohydrolase II" },
  { "HEK2", "RNA binding protein involved in asymmetric localization of ASH1 mRNA" },
  { "SHE1", "Mitotic spindle protein" },
  { "PET9", "Major ADP/ATP carrier of the mitochondrial inner membrane" },
  { "YBL029C-A", "Protein of unknown function" },
  { "YBL029W", "Non-essential protein of unknown function" },
  { "YBL028C", "Protein of unknown function that may interact with ribosomes" },
  { "RPL19B", "Ribosomal 60S subunit protein L19B" },
  { "LSM2", "Lsm (Like Sm) protein" },
  { "RRN10", "Protein involved in promoting high level transcription of rDNA" },
  { "NCL1", "S-adenosyl-L-methionine-dependent tRNA: m5C-methyltransferase" },
  { "MCM2", "Protein involved in DNA replication" },
  { "PIM1", "ATP-dependent Lon protease" },
  { "HAP3", "Subunit of the Hap2p/3p/4p/5p CCAAT-binding complex" },
  { "RFT1", "Membrane protein required for translocation of Man5GlcNac2-PP-Dol" },
  { "APN2", "Class II abasic (AP) endonuclease involved in repair of DNA damage" },
  { "POP8", "Subunit of both RNase MRP and nuclear RNase P" },
  { "PEP1", "Type I transmembrane sorting receptor for multiple vacuolar hydrolases" },
  { "FUS3", "Mitogen-activated serine/threonine protein kinase involved in mating" },
  { "ACH1", "Protein with CoA transferase activity" },
  { "RRN6", "Component of the core factor (CF) rDNA transcription factor complex" },
  { "FMT1", "Methionyl-tRNA formyltransferase" },
  { "YBL012C", "NULL" },
  { "SCT1", "Glycerol 3-phosphate/dihydroxyacetone phosphate sn-1 acyltransferase" },
  { "YBL010C", "Putative protein of unknown function" },
  { "ALK2", "Protein kinase" },
  { "YBL008W-A", "Putative protein of unknown function" },
  { "HIR1", "Subunit of the HIR complex" },
  { "SLA1", "Cytoskeletal protein binding protein" },
  { "LDB7", "Component of the RSC chromatin remodeling complex" },
  { "YBL006W-A", "NULL" },
  { "PDR3", "Transcriptional activator of the pleiotropic drug resistance network" },
  { "UTP20", "Component of the small-subunit (SSU) processome" },
  { "HTA2", "Histone H2A" },
  { "HTB2", "Histone H2B" },
  { "ECM15", "Non-essential protein of unknown function" },
  { "NTH2", "Putative neutral trehalase, required for thermotolerance" },
  { "RER2", "Forms the dehydrodolichyl diphosphate syntase (DDS) complex with NUS1" },
  { "COQ1", "Hexaprenyl pyrophosphate synthetase" },
  { "GPI18", "Functional ortholog of human PIG-V" },
  { "RCR1", "Protein of the ER membrane involved in cell wall chitin deposition" },
  { "UGA2", "Succinate semialdehyde dehydrogenase" },
  { "DSF2", "Deletion suppressor of mpt5 mutation" },
  { "FLR1", "Plasma membrane transporter of the major facilitator superfamily" },
  { "HHF1", "Histone H4" },
  { "HHT1", "Histone H3" },
  { "IPP1", "Cytoplasmic inorganic pyrophosphatase (PPase)" },
  { "YBR012C", "NULL" },
  { "YBR013C", "Putative protein of unknown function" },
  { "GRX7", "Cis-golgi localized monothiol glutaredoxin" },
  { "MNN2", "Alpha-1,2-mannosyltransferase" },
  { "YBR016W", "Tail-anchored plasma membrane protein with a conserved CYSTM module" },
  { "KAP104", "Transportin or cytosolic karyopherin beta 2" },
  { "GAL7", "Galactose-1-phosphate uridyl transferase" },
  { "GAL10", "UDP-glucose-4-epimerase" },
  { "GAL1", "Galactokinase" },
  { "FUR4", "Plasma membrane localized uracil permease" },
  { "POA1", "Phosphatase that is highly specific for ADP-ribose 1''-phosphate" },
  { "CHS3", "Chitin synthase III" },
  { "SCO2", "Protein anchored to mitochondrial inner membrane" },
  { "OLA1", "P-loop ATPase with similarity to human OLA1 and bacterial YchF" },
  { "ETR1", "2-enoyl thioester reductase" },
  { "YBR027C", "Putative protein of unknown function" },
  { "YPK3", "AGC kinase" },
  { "CDS1", "Phosphatidate cytidylyltransferase (CDP-diglyceride synthetase)" },
  { "RKM3", "Ribosomal lysine methyltransferase" },
  { "RPL4A", "Ribosomal 60S subunit protein L4A" },
  { "YBR032W", "Putative protein of unknown function" },
  { "EDS1", "Putative zinc cluster protein, predicted to be a transcription factor" },
  { "HMT1", "Nuclear SAM-dependent mono- and asymmetric methyltransferase" },
  { "PDX3", "Pyridoxine (pyridoxamine) phosphate oxidase" },
  { "CSG2", "Endoplasmic reticulum membrane protein" },
  { "SCO1", "Copper-binding protein of mitochondrial inner membrane" },
  { "CHS2", "Chitin synthase II" },
  { "ATP3", "Gamma subunit of the F1 sector of mitochondrial F1F0 ATP synthase" },
  { "FIG1", "Integral membrane protein required for efficient mating" },
  { "FAT1", "Very long chain fatty acyl-CoA synthetase and fatty acid transporter" },
  { "CST26", "Acyltransferase" },
  { "QDR3", "Multidrug transporter of the major facilitator superfamily" },
  { "TCM62", "Protein involved in assembly of the succinate dehydrogenase complex" },
  { "GIP1", "Meiosis-specific regulatory subunit of the Glc7p protein phosphatase" },
  { "ZTA1", "NADPH-dependent quinone reductase" },
  { "FMP23", "Putative protein of unknown function" },
  { "RPS11B", "Protein component of the small (40S) ribosomal subunit" },
  { "REB1", "RNA polymerase I enhancer binding protein" },
  { "REG2", "Regulatory subunit of the Glc7p type-1 protein phosphatase" },
  { "YBR051W", "NULL" },
  { "RFS1", "Protein of unknown function" },
  { "YBR053C", "Putative protein of unknown function" },
  { "YRO2", "Protein with a putative role in response to acid stress" },
  { "PRP6", "Splicing factor" },
  { "YBR056W", "Putative glycoside hydrolase of the mitochondrial intermembrane space" },
  { "YBR056W-A", "NULL" },
  { "YBR056C-B", "NULL" },
  { "MUM2", "Protein essential for meiotic DNA replication and sporulation" },
  { "UBP14", "Ubiquitin-specific protease" },
  { "TSC3", "Protein that stimulates the activity of serine palmitoyltransferase" },
  { "AKL1", "Ser/Thr protein kinase" },
  { "ORC2", "Subunit of the origin recognition complex (ORC)" },
  { "TRM7", "2'-O-ribose methyltransferase" },
  { "YBR062C", "Protein of unknown function that interacts with Msb2p" },
  { "YBR063C", "Putative protein of unknown function" },
  { "YBR064W", "NULL" },
  { "ECM2", "Pre-mRNA splicing factor" },
  { "NRG2", "Transcriptional repressor" },
  { "TIP1", "Major cell wall mannoprotein with possible lipase activity" },
  { "BAP2", "High-affinity leucine permease" },
  { "TAT1", "Amino acid transporter for valine, leucine, isoleucine, and tyrosine" },
  { "ALG14", "Component of UDP-GlcNAc transferase" },
  { "YBR071W", "Protein of unknown function found in the cytoplasm and bud neck" },
  { "HSP26", "Small heat shock protein (sHSP) with chaperone activity" },
  { "YBR072C-A", "Putative protein of unknown function" },
  { "RDH54", "DNA-dependent ATPase" },
  { "PFF1", "Multi-spanning vacuolar membrane protease" },
  { "ECM8", "Non-essential protein of unknown function" },
  { "YBR076C-A", "NULL" },
  { "SLM4", "Subunit of the EGO/GSE complex" },
  { "ECM33", "GPI-anchored protein of unknown function" },
  { "RPG1", "eIF3a subunit of the eukaryotic translation initiation factor 3 (eIF3)" },
  { "SEC18", "AAA ATPase and SNARE disassembly chaperone" },
  { "SPT7", "Subunit of the SAGA transcriptional regulatory complex" },
  { "UBC4", "Ubiquitin-conjugating enzyme (E2)" },
  { "TEC1", "Transcription factor targeting filamentation genes and Ty1 expression" },
  { "MIS1", "Mitochondrial C1-tetrahydrofolate synthase" },
  { "RPL19A", "Ribosomal 60S subunit protein L19A" },
  { "AAC3", "Mitochondrial inner membrane ADP/ATP translocator" },
  { "YBR085C-A", "Protein of unknown function" },
  { "IST2", "Cortical ER protein involved in ER-plasma membrane tethering" },
  { "RFC5", "Subunit of heteropentameric Replication factor C (RF-C)" },
  { "POL30", "Proliferating cell nuclear antigen (PCNA)" },
  { "YBR089W", "NULL" },
  { "NHP6B", "High-mobility group (HMG) protein" },
  { "YBR090C", "Putative protein of unknown function" },
  { "TIM12", "Essential protein of the inner mitochondrial membrane" },
  { "PHO3", "Constitutively expressed acid phosphatase similar to Pho5p" },
  { "PHO5", "Repressible acid phosphatase" },
  { "PBY1", "Putative tubulin tyrosine ligase associated with P-bodies" },
  { "RXT2", "Component of the histone deacetylase Rpd3L complex" },
  { "YBR096W", "Putative protein of unknown function" },
  { "VPS15", "Serine/threonine protein kinase involved in vacuolar protein sorting" },
  { "MMS4", "Subunit of structure-specific Mms4p-Mus81p endonuclease" },
  { "YBR099C", "NULL" },
  { "FES1", "Hsp70 (Ssa1p) nucleotide exchange factor" },
  { "EXO84", "Exocyst subunit with dual roles in exocytosis and spliceosome assembly" },
  { "SIF2", "WD40 repeat-containing subunit of Set3C histone deacetylase complex" },
  { "YBR103C-A", "NULL" },
  { "YMC2", "Putative mitochondrial inner membrane transporter" },
  { "VID24", "GID Complex regulatory subunit" },
  { "PHO88", "NULL" },
  { "IML3", "Outer kinetochore protein and component of the Ctf19 complex" },
  { "AIM3", "Protein that inhibits barbed-end actin filament elongation" },
  { "CMD1", "Calmodulin" },
  { "YBR109W-A", "NULL" },
  { "ALG1", "Mannosyltransferase" },
  { "YSA1", "Nudix hydrolase family member with ADP-ribose pyrophosphatase activity" },
  { "SUS1", "Component of both the SAGA histone acetylase and TREX-2 complexes" },
  { "CYC8", "General transcriptional co-repressor" },
  { "YBR113W", "NULL" },
  { "RAD16", "Nucleotide excision repair (NER) protein" },
  { "LYS2", "Alpha aminoadipate reductase" },
  { "YBR116C", "NULL" },
  { "TKL2", "Transketolase" },
  { "TEF2", "Translational elongation factor EF-1 alpha" },
  { "MUD1", "U1 snRNP A protein" },
  { "CBP6", "Mitochondrial protein required for translation of the COB mRNA" },
  { "GRS1", "Cytoplasmic and mitochondrial glycyl-tRNA synthase" },
  { "YBR121C-A", "NULL" },
  { "MRPL36", "Mitochondrial ribosomal protein of the large subunit" },
  { "TFC1", "Subunit of RNA polymerase III transcription initiation factor complex" },
  { "YBR124W", "NULL" },
  { "PTC4", "Cytoplasmic type 2C protein phosphatase (PP2C)" },
  { "TPS1", "Synthase subunit of trehalose-6-P synthase/phosphatase complex" },
  { "YBR126W-A", "NULL" },
  { "YBR126W-B", "NULL" },
  { "VMA2", "Subunit B of V1 peripheral membrane domain of vacuolar H+-ATPase" },
  { "ATG14", "Autophagy-specific subunit of phosphatidylinositol 3-kinase complex I" },
  { "OPY1", "Protein of unknown function" },
  { "SHE3", "Protein adaptor between Myo4p and the She2p-mRNA complex" },
  { "CCZ1", "Subunit of a heterodimeric guanine nucleotide exchange factor (GEF)" },
  { "YBR131C-A", "NULL" },
  { "AGP2", "Plasma membrane regulator of polyamine and carnitine transport" },
  { "HSL7", "Protein arginine N-methyltransferase" },
  { "YBR134W", "NULL" },
  { "CKS1", "Cyclin-dependent protein kinase regulatory subunit and adaptor" },
  { "MEC1", "Genome integrity checkpoint protein and PI kinase superfamily member" },
  { "YBR137W", "Protein with a role in ER delivery of tail-anchored membrane proteins" },
  { "YBR138C", "Cytoplasmic protein of unknown function" },
  { "YBR139W", "NULL" },
  { "IRA1", "GTPase-activating protein" },
  { "BMT2", "Nucleolar S-adenosylmethionine-dependent rRNA methyltransferase" },
  { "YBR141W-A", "NULL" },
  { "MAK5", "Essential nucleolar protein" },
  { "SUP45", "Polypeptide release factor (eRF1) in translation termination" },
  { "YBR144C", "Putative protein of unknown function" },
  { "ADH5", "Alcohol dehydrogenase isoenzyme V" },
  { "MRPS9", "Mitochondrial ribosomal protein of the small subunit" },
  { "RTC2", "Putative vacuolar membrane transporter for cationic amino acids" },
  { "YSW1", "Protein required for normal prospore membrane formation" },
  { "ARA1", "NADP+ dependent arabinose dehydrogenase" },
  { "TBS1", "Protein of unknown function" },
  { "APD1", "Protein of unknown function" },
  { "SPP381", "mRNA splicing factor, component of U4/U6.U5 tri-snRNP" },
  { "RIB7", "Diaminohydroxyphoshoribosylaminopyrimidine deaminase" },
  { "RPB5", "RNA polymerase subunit ABC27" },
  { "CNS1", "TPR-containing co-chaperone" },
  { "SLI15", "Subunit of the conserved chromosomal passenger complex (CPC)" },
  { "ICS2", "Protein of unknown function" },
  { "AMN1", "Modulator of cell separation and mitotic exit" },
  { "IFA38", "Microsomal beta-keto-reductase" },
  { "CDC28", "Cyclin-dependent kinase (CDK) catalytic subunit" },
  { "CSH1", "Mannosylinositol phosphorylceramide (MIPC) synthase catalytic subunit" },
  { "TOS1", "Covalently-bound cell wall protein of unknown function" },
  { "YSY6", "Protein of unknown function" },
  { "EXO5", "Mitochondrial 5'-3' exonuclease and sliding exonuclease" },
  { "ARL1", "Soluble GTPase with a role in regulation of membrane traffic" },
  { "UBS1", "Ubiquitin-conjugating enzyme suppressor that regulates Cdc34p" },
  { "TYR1", "Prephenate dehydrogenase involved in tyrosine biosynthesis" },
  { "POP7", "Subunit of RNase MRP, nuclear RNase P and telomerase" },
  { "PEX32", "Peroxisomal integral membrane protein" },
  { "SSE2", "Member of Hsp110 subclass of the heat shock protein 70 (HSP70) family" },
  { "NPL4", "Substrate-recruiting cofactor of the Cdc48p-Npl4p-Ufd1p segregase" },
  { "SEC66", "Non-essential subunit of Sec63 complex" },
  { "SMY2", "GYF domain protein" },
  { "UMP1", "Chaperone required for correct maturation of the 20S proteasome" },
  { "YBR174C", "NULL" },
  { "SWD3", "Essential subunit of the COMPASS (Set1C) complex" },
  { "ECM31", "Ketopantoate hydroxymethyltransferase" },
  { "EHT1", "Acyl-coenzymeA:ethanol O-acyltransferase" },
  { "YBR178W", "NULL" },
  { "FZO1", "Mitofusin" },
  { "DTR1", "Putative dityrosine transporter of the major facilitator superfamily" },
  { "RPS6B", "Protein component of the small (40S) ribosomal subunit" },
  { "SMP1", "MADS-box transcription factor involved in osmotic stress response" },
  { "YBR182C-A", "Putative protein of unknown function" },
  { "YPC1", "Alkaline ceramidase" },
  { "YBR184W", "Putative protein of unknown function" },
  { "MBA1", "Membrane-associated mitochondrial ribosome receptor" },
  { "PCH2", "Hexameric ring ATPase that remodels chromosome axis protein Hop1p" },
  { "GDT1", "Calcium and manganese transporter with higher affinity for Ca2+" },
  { "NTC20", "Member of the NineTeen Complex (NTC)" },
  { "RPS9B", "Protein component of the small (40S) ribosomal subunit" },
  { "YBR190W", "NULL" },
  { "RPL21A", "Ribosomal 60S subunit protein L21A" },
  { "YBR191W-A", "NULL" },
  { "RIM2", "Mitochondrial pyrimidine nucleotide transporter" },
  { "MED8", "Subunit of the RNA polymerase II mediator complex" },
  { "AIM4", "Protein proposed to be associated with the nuclear pore complex" },
  { "MSI1", "Subunit of chromatin assembly factor I (CAF-1)" },
  { "PGI1", "Glycolytic enzyme phosphoglucose isomerase" },
  { "YBR196C-A", "Putative protein of unknown function" },
  { "YBR196C-B", "Putative protein of unknown function" },
  { "YBR197C", "Protein of unknown function" },
  { "TAF5", "Subunit (90 kDa) of TFIID and SAGA complexes" },
  { "KTR4", "Glycosyltransferase involved in protein glycosylation" },
  { "BEM1", "Protein containing SH3-domains" },
  { "YBR200W-A", "Putative protein of unknown function" },
  { "DER1", "ER membrane protein that promotes export of misfolded polypeptides" },
  { "YBR201C-A", "NULL" },
  { "MCM7", "Component of the Mcm2-7 hexameric helicase complex" },
  { "COS111", "Protein required for antifungal drug ciclopirox olamine resistance" },
  { "LDH1", "Serine hydrolase" },
  { "KTR3", "Putative alpha-1,2-mannosyltransferase" },
  { "YBR206W", "NULL" },
  { "FTH1", "Putative high affinity iron transporter" },
  { "DUR1", "Urea amidolyase" },
  { "YBR209W", "Putative protein of unknown function" },
  { "ERV15", "Protein involved in export of proteins from the endoplasmic reticulum" },
  { "AME1", "Essential kinetochore protein associated with microtubules and SPBs" },
  { "NGR1", "RNA binding protein that negatively regulates growth rate" },
  { "MET8", "Bifunctional dehydrogenase and ferrochelatase" },
  { "SDS24", "Protein involved in cell separation during budding" },
  { "HPC2", "Subunit of the HIR complex" },
  { "YBP1", "Protein involved in cellular response to oxidative stress" },
  { "ATG12", "Ubiquitin-like modifier involved in autophagy and the Cvt pathway" },
  { "PYC2", "Pyruvate carboxylase isoform" },
  { "YBR219C", "Putative protein of unknown function" },
  { "YBR220C", "Putative protein of unknown function" },
  { "PDB1", "E1 beta subunit of the pyruvate dehydrogenase (PDH) complex" },
  { "YBR221W-A", "Putative protein of unknown function" },
  { "PCS60", "Oxalyl-CoA synthetase" },
  { "TDP1", "Tyrosyl-DNA phosphodiesterase I" },
  { "YBR223W-A", "NULL" },
  { "YBR224W", "NULL" },
  { "YBR225W", "Putative protein of unknown function" },
  { "YBR226C", "NULL" },
  { "MCX1", "Non-proteolytic ATPase of the AAA family" },
  { "SLX1", "Endonuclease involved in DNA recombination and repair" },
  { "ROT2", "Glucosidase II catalytic subunit" },
  { "OM14", "Mitochondrial outer membrane receptor for cytosolic ribosomes" },
  { "YBR230W-A", "NULL" },
  { "SWC5", "Component of the SWR1 complex" },
  { "YBR232C", "NULL" },
  { "PBP2", "RNA binding protein" },
  { "DAD3", "Essential subunit of the Dam1 complex (aka DASH complex)" },
  { "ARC40", "Subunit of the ARP2/3 complex" },
  { "VHC1", "Vacuolar membrane cation-chloride cotransporter (CCC)" },
  { "ABD1", "Methyltransferase" },
  { "PRP5", "RNA helicase in the DEAD-box family" },
  { "YBR238C", "Mitochondrial membrane protein" },
  { "ERT1", "Transcriptional regulator" },
  { "THI2", "Transcriptional activator of thiamine biosynthetic genes" },
  { "YBR241C", "Putative transporter, member of the sugar porter family" },
  { "YBR242W", "Putative protein of unknown function" },
  { "ALG7", "UDP-N-acetyl-glucosamine-1-P transferase" },
  { "GPX2", "Phospholipid hydroperoxide glutathione peroxidase" },
  { "ISW1", "ATPase subunit of imitation-switch (ISWI) class chromatin remodelers" },
  { "RRT2", "Methylesterase performing penultimate step of diphthamide biosynthesis" },
  { "ENP1", "Protein associated with U3 and U14 snoRNAs" },
  { "HIS7", "Imidazole glycerol phosphate synthase" },
  { "ARO4", "3-deoxy-D-arabino-heptulosonate-7-phosphate (DAHP) synthase" },
  { "SPO23", "Protein of unknown function" },
  { "MRPS5", "Mitochondrial ribosomal protein of the small subunit" },
  { "DUT1", "Deoxyuridine triphosphate diphosphatase (dUTPase)" },
  { "SRB6", "Subunit of the RNA polymerase II mediator complex" },
  { "TRS20", "Core component of transport protein particle (TRAPP) complexes I-III" },
  { "MTC4", "Protein of unknown function" },
  { "YBR255C-A", "NULL" },
  { "RIB5", "Riboflavin synthase" },
  { "POP4", "Subunit of both RNase MRP and nuclear RNase P" },
  { "SHG1", "Subunit of the COMPASS (Set1C) complex" },
  { "YBR259W", "Protein of unknown function" },
  { "RGD1", "GTPase-activating protein (RhoGAP) for Rho3p and Rho4p" },
  { "TAE1", "AdoMet-dependent proline methyltransferase" },
  { "MIC12", "Component of the MICOS complex" },
  { "SHM1", "Mitochondrial serine hydroxymethyltransferase" },
  { "YPT10", "Rab family GTP-binding protein" },
  { "TSC10", "3-ketosphinganine reductase" },
  { "REI1", "Cytoplasmic pre-60S factor" },
  { "SLM6", "NULL" },
  { "MRPL37", "Mitochondrial ribosomal protein of the large subunit" },
  { "FMP21", "NULL" },
  { "BIT2", "Subunit of TORC2 membrane-associated complex" },
  { "EFM2", "S-adenosylmethionine-dependent methyltransferase" },
  { "HSM3", "Evolutionarily conserved 19S regulatory particle assembly-chaperone" },
  { "UBX7", "UBX (ubiquitin regulatory X) domain-containing protein" },
  { "CHK1", "Serine/threonine kinase and DNA damage checkpoint effector" },
  { "RIF1", "Protein that binds to the Rap1p C-terminus" },
  { "PPS1", "Protein phosphatase" },
  { "YBR277C", "NULL" },
  { "DPB3", "Third-largest subunit of DNA polymerase II (DNA polymerase epsilon)" },
  { "PAF1", "Component of the Paf1p complex involved in transcription elongation" },
  { "SAF1", "F-Box protein involved in proteasome-dependent degradation of Aah1p" },
  { "DUG2", "Component of glutamine amidotransferase (GATase II)" },
  { "MRPL27", "Mitochondrial ribosomal protein of the large subunit" },
  { "SSH1", "Subunit of the Ssh1 translocon complex" },
  { "YBR284W", "Putative metallo-dependent hydrolase superfamily protein" },
  { "YBR285W", "Putative protein of unknown function" },
  { "APE3", "Vacuolar aminopeptidase Y" },
  { "YBR287W", "Protein of unknown function" },
  { "APM3", "Mu3-like subunit of the clathrin associated protein complex (AP-3)" },
  { "SNF5", "Subunit of the SWI/SNF chromatin remodeling complex" },
  { "BSD2", "Heavy metal ion homeostasis protein" },
  { "CTP1", "Mitochondrial inner membrane citrate transporter" },
  { "YBR292C", "Putative protein of unknown function" },
  { "VBA2", "Permease of basic amino acids in the vacuolar membrane" },
  { "SUL1", "High affinity sulfate permease of the SulP anion transporter family" },
  { "PCA1", "Cadmium transporting P-type ATPase" },
  { "PHO89", "Plasma membrane Na+/Pi cotransporter" },
  { "YBR296C-A", "NULL" },
  { "MAL33", "MAL-activator protein" },
  { "MAL31", "Maltose permease" },
  { "YBR298C-A", "Putative protein of unknown function" },
  { "MAL32", "Maltase (alpha-D-glucosidase)" },
  { "YBR300C", "NULL" },
  { "PAU24", "Cell wall mannoprotein" },
  { "COS2", "Endosomal protein involved in turnover of plasma membrane proteins" },
  { "YCL076W", "NULL" },
  { "GEX1", "Proton:glutathione antiporter" },
  { "VBA3", "Permease of basic amino acids in the vacuolar membrane" },
  { "YCL068C", "Putative protein of unknown function" },
  { "HMLALPHA2", "Silenced copy of ALPHA2 at HML" },
  { "HMLALPHA1", "Silenced copy of ALPHA1 at HML" },
  { "YCL065W", "NULL" },
  { "CHA1", "Catabolic L-serine (L-threonine) deaminase" },
  { "VAC17", "Phosphoprotein involved in vacuole inheritance" },
  { "MRC1", "S-phase checkpoint protein required for DNA replication" },
  { "KRR1", "Nucleolar protein required for rRNA synthesis and ribosomal assembly" },
  { "FYV5", "Protein involved in regulation of the mating pathway" },
  { "ADF1", "Transcriptional repressor encoded by the FYV5 antisense strand" },
  { "MIC10", "Conserved component of the MICOS complex" },
  { "PRD1", "Zinc metalloendopeptidase" },
  { "PEX34", "Protein that regulates peroxisome populations" },
  { "KAR4", "Transcription factor required for response to pheromones" },
  { "SPB1", "AdoMet-dependent methyltransferase" },
  { "PBN1", "Component of glycosylphosphatidylinositol-mannosyltransferase I" },
  { "LRE1", "Protein involved in control of cell wall structure and stress response" },
  { "APA1", "AP4A phosphorylase" },
  { "YCL049C", "Protein of unknown function" },
  { "YCL048W-A", "Putative protein of unknown function" },
  { "SPS22", "Protein of unknown function" },
  { "POF1", "Nicotinamide mononucleotide-specific adenylyltransferase (NMNAT)" },
  { "EMC1", "Member of conserved endoplasmic reticulum membrane complex" },
  { "YCL046W", "NULL" },
  { "MGR1", "Subunit of the mitochondrial (mt) i-AAA protease supercomplex" },
  { "PDI1", "Protein disulfide isomerase" },
  { "YCL041C", "NULL" },
  { "YCL042W", "Putative protein of unknown function" },
  { "GLK1", "Glucokinase" },
  { "GID7", "Subunit of GID Complex that binds directly to central component Vid30p" },
  { "ATG22", "Vacuolar integral membrane protein required for efflux of amino acids" },
  { "SRO9", "Cytoplasmic RNA-binding protein" },
  { "GFD2", "Protein of unknown function" },
  { "GRX1", "Glutathione-dependent disulfide oxidoreductase" },
  { "LSB5", "Protein involved in membrane-trafficking events at plasma membrane" },
  { "MXR2", "Methionine-R-sulfoxide reductase" },
  { "STE50", "Adaptor protein for various signaling pathways" },
  { "RRP7", "Essential protein involved in rRNA processing and ribosome biogenesis" },
  { "HIS4", "Multifunctional enzyme containing phosphoribosyl-ATP pyrophosphatase" },
  { "BIK1", "Microtubule-associated protein" },
  { "RNQ1", "[PIN(+)] prion" },
  { "FUS1", "Membrane protein localized to the shmoo tip" },
  { "HBN1", "Protein of unknown function" },
  { "FRM2", "Type II nitroreductase, using NADH as reductant" },
  { "AGP1", "Low-affinity amino acid permease with broad substrate range" },
  { "YCL023C", "NULL" },
  { "KCC4", "Protein kinase of the bud neck involved in the septin checkpoint" },
  { "YCL022C", "NULL" },
  { "YCL021W-A", "Putative protein of unknown function" },
  { "LEU2", "Beta-isopropylmalate dehydrogenase (IMDH)" },
  { "NFS1", "Cysteine desulfurase" },
  { "DCC1", "Subunit of a complex with Ctf8p and Ctf18p" },
  { "BUD3", "Guanine nucleotide exchange factor (GEF) for Cdc42p" },
  { "YCL012C", "Protein of unknown function" },
  { "GBP2", "Poly(A+) RNA-binding protein" },
  { "SGF29", "Component of the HAT/Core module of the SAGA, SLIK, and ADA complexes" },
  { "ILV6", "Regulatory subunit of acetolactate synthase" },
  { "STP22", "Component of the ESCRT-I complex" },
  { "YCL007C", "NULL" },
  { "VMA9", "Vacuolar H+ ATPase subunit e of the V-ATPase V0 subcomplex" },
  { "LDB16", "Protein involved in lipid droplet (LD) assembly" },
  { "PGS1", "Phosphatidylglycerolphosphate synthase" },
  { "YCL002C", "Putative protein of unknown function" },
  { "RER1", "Protein involved in retention of membrane proteins" },
  { "YCL001W-A", "Putative protein of unknown function" },
  { "YCL001W-B", "Putative protein of unknown function" },
  { "YCR001W", "Putative protein of unknown function" },
  { "CDC10", "Component of the septin ring, required for cytokinesis" },
  { "MRPL32", "Mitochondrial ribosomal protein of the large subunit" },
  { "YCP4", "Protein of unknown function" },
  { "CIT2", "Citrate synthase, peroxisomal isozyme involved in glyoxylate cycle" },
  { "YCR006C", "Putative protein of unknown function" },
  { "YCR007C", "Putative integral membrane protein" },
  { "SAT4", "Ser/Thr protein kinase involved in salt tolerance" },
  { "RVS161", "Amphiphysin-like lipid raft protein" },
  { "ADY2", "Acetate transporter required for normal sporulation" },
  { "ADP1", "Putative ATP-dependent permease of the ABC transporter family" },
  { "PGK1", "3-phosphoglycerate kinase" },
  { "YCR013C", "NULL" },
  { "POL4", "DNA polymerase IV" },
  { "YCR015C", "NULL" },
  { "YCR016W", "Putative protein of unknown function" },
  { "CWH43", "GPI lipid remodelase" },
  { "SRD1", "Protein involved in the processing of pre-rRNA to mature rRNA" },
  { "YCR018C-A", "NULL" },
  { "MAK32", "Protein necessary for stability of L-A dsRNA-containing particles" },
  { "PET18", "Protein of unknown function" },
  { "MAK31", "Non-catalytic subunit of N-terminal acetyltransferase of the NatC type" },
  { "HTL1", "Component of the RSC chromatin remodeling complex" },
  { "HSP30", "Negative regulator of the H(+)-ATPase Pma1p" },
  { "YCR022C", "Putative protein of unknown function" },
  { "YCR023C", "Vacuolar membrane protein of unknown function" },
  { "SLM5", "Mitochondrial asparaginyl-tRNA synthetase" },
  { "YCR024C-B", "Putative protein of unknown function" },
  { "PMP1", "Regulatory subunit for the plasma membrane H(+)-ATPase Pma1p" },
  { "YCR025C", "Putative protein of unknown function" },
  { "NPP1", "Nucleotide pyrophosphatase/phosphodiesterase" },
  { "RHB1", "Putative Rheb-related GTPase" },
  { "FEN2", "Plasma membrane H+-pantothenate symporter" },
  { "RIM1", "ssDNA-binding protein essential for mitochondrial genome maintenance" },
  { "SYP1", "Negative regulator of WASP-Arp23 complex" },
  { "RPS14A", "Protein component of the small (40S) ribosomal subunit" },
  { "BPH1", "Protein homologous to Chediak-Higashi syndrome and Beige proteins" },
  { "SNT1", "Subunit of the Set3C deacetylase complex" },
  { "ELO2", "Fatty acid elongase, involved in sphingolipid biosynthesis" },
  { "RRP43", "Exosome non-catalytic core component" },
  { "RBK1", "Putative ribokinase" },
  { "PHO87", "Low-affinity inorganic phosphate (Pi) transporter" },
  { "BUD5", "GTP/GDP exchange factor for Rsr1p (Bud1p)" },
  { "YCR038W-A", "NULL" },
  { "MATALPHA2", "Homeobox-domain protein" },
  { "MATALPHA1", "Transcriptional co-activator that regulates mating-type-specific genes" },
  { "YCR041W", "Protein of unknown function" },
  { "TAF2", "TFIID subunit (150 kDa)" },
  { "YCR043C", "Putative protein of unknown function" },
  { "PER1", "Protein of the endoplasmic reticulum" },
  { "RRT12", "Probable subtilisin-family protease" },
  { "YCR045W-A", "NULL" },
  { "IMG1", "Mitochondrial ribosomal protein of the large subunit" },
  { "BUD23", "Methyltransferase that methylates residue G1575 of 18S rRNA" },
  { "YCR047W-A", "NULL" },
  { "YCR049C", "NULL" },
  { "ARE1", "Acyl-CoA:sterol acyltransferase" },
  { "YCR050C", "Non-essential protein of unknown function" },
  { "YCR051W", "Putative protein of unknown function" },
  { "RSC6", "Component of the RSC chromatin remodeling complex" },
  { "THR4", "Threonine synthase" },
  { "CTR86", "Essential protein of unknown function" },
  { "PWP2", "Conserved 90S pre-ribosomal component" },
  { "YIH1", "Negative regulator of eIF2 kinase Gcn2p" },
  { "TAH1", "Component of conserved R2TP complex (Rvb1-Rvb2-Tah1-Pih1)" },
  { "YCR061W", "Protein of unknown function" },
  { "YCR064C", "NULL" },
  { "BUD31", "Component of the SF3b subcomplex of the U2 snRNP" },
  { "HCM1", "Forkhead transcription factor" },
  { "RAD18", "E3 ubiquitin ligase" },
  { "SED4", "Integral ER membrane protein that stimulates Sar1p GTPase activity" },
  { "ATG15", "Phospholipase" },
  { "CPR4", "Peptidyl-prolyl cis-trans isomerase (cyclophilin)" },
  { "IMG2", "Mitochondrial ribosomal protein of the large subunit" },
  { "RSA4", "Ribosomal large subunit (60S) assembly factor" },
  { "SSK22", "MAP kinase kinase kinase of HOG1 mitogen-activated signaling pathway" },
  { "SOL2", "Protein with a possible role in tRNA export" },
  { "ERS1", "Protein involved in cystine transport" },
  { "YCR075W-A", "NULL" },
  { "FUB1", "Proteasome-binding protein" },
  { "PAT1", "Deadenylation-dependent mRNA-decapping factor" },
  { "PTC6", "Mitochondrial type 2C protein phosphatase (PP2C)" },
  { "SRB8", "Subunit of the RNA polymerase II mediator complex" },
  { "YCR081C-A", "NULL" },
  { "AHC2", "Component of the ADA histone acetyltransferase complex" },
  { "TRX3", "Mitochondrial thioredoxin" },
  { "TUP1", "General repressor of transcription" },
  { "YCR085W", "Putative protein of unknown function" },
  { "CSM1", "Nucleolar protein that mediates homolog segregation during meiosis I" },
  { "YCR087W", "NULL" },
  { "YCR087C-A", "Putative protein of unknown function" },
  { "ABP1", "Actin-binding protein of the cortical actin cytoskeleton" },
  { "FIG2", "Cell wall adhesin, expressed specifically during mating" },
  { "YCR090C", "Putative protein of unknown function" },
  { "KIN82", "Putative serine/threonine protein kinase" },
  { "MSH3", "Mismatch repair protein" },
  { "CDC39", "Subunit of the CCR4-NOT1 core complex" },
  { "CDC50", "Endosomal protein that interacts with phospholipid flippase Drs2p" },
  { "OCA4", "Cytoplasmic protein required for replication of Brome mosaic virus" },
  { "YCR095W-A", "Putative protein of unknown function" },
  { "HMRA2", "Silenced copy of a2 at HMR" },
  { "HMRA1", "Silenced copy of a1 at HMR" },
  { "YCR097W-A", "NULL" },
  { "GIT1", "Plasma membrane permease" },
  { "YCR099C", "Putative protein of unknown function" },
  { "YCR100C", "NULL" },
  { "YCR101C", "Putative protein of unknown function" },
  { "YCR102C", "Putative protein of unknown function" },
  { "YCR102W-A", "NULL" },
  { "PAU3", "Member of the seripauperin multigene family" },
  { "ADH7", "NADPH-dependent medium chain alcohol dehydrogenase" },
  { "RDS1", "Putative zinc cluster transcription factor" },
  { "AAD3", "Putative aryl-alcohol dehydrogenase" },
  { "YCR108C", "Putative protein of unknown function" },
  { "COS7", "Endosomal protein involved in turnover of plasma membrane proteins" },
  { "YDL247W-A", "NULL" },
  { "MPH2", "Alpha-glucoside permease" },
  { "SOR2", "Sorbitol dehydrogenase" },
  { "HXT15", "Putative transmembrane polyol transporter" },
  { "THI13", "Protein involved in synthesis of the thiamine precursor HMP" },
  { "AAD4", "Putative aryl-alcohol dehydrogenase" },
  { "YDL242W", "Putative protein of unknown function" },
  { "YDL241W", "Putative protein of unknown function" },
  { "YDL240C-A", "NULL" },
  { "LRG1", "GTPase-activating protein (GAP)" },
  { "ADY3", "Protein required for spore wall formation" },
  { "GUD1", "Guanine deaminase" },
  { "AIM6", "Protein of unknown function" },
  { "PHO13", "Conserved phosphatase acting as a metabolite repair enzyme" },
  { "YPD1", "Osmotic stress-responsive phosphorelay intermediate sensor protein" },
  { "GYP7", "GTPase-activating protein for yeast Rab family members" },
  { "MFG1", "Regulator of filamentous growth" },
  { "OST4", "Subunit of the oligosaccharyltransferase complex of the ER lumen" },
  { "BRE4", "Zinc finger protein containing five transmembrane domains" },
  { "PTP1", "Phosphotyrosine-specific protein phosphatase" },
  { "SSB1", "Cytoplasmic ATPase that is a ribosome-associated molecular chaperone" },
  { "YDL228C", "NULL" },
  { "HO", "Site-specific endonuclease" },
  { "GCS1", "ADP-ribosylation factor GTPase activating protein (ARF GAP)" },
  { "SHS1", "Component of the septin ring that is required for cytokinesis" },
  { "WHI4", "Putative RNA binding protein" },
  { "HBT1", "Shmoo tip protein, substrate of Hub1p ubiquitin-like protein" },
  { "FMP45", "Integral membrane protein localized to mitochondria" },
  { "YDL221W", "NULL" },
  { "CDC13", "Single stranded DNA-binding protein found at TG1-3 telomere G-tails" },
  { "DTD1", "D-Tyr-tRNA(Tyr) deacylase" },
  { "YDL218W", "Putative protein of unknown function" },
  { "TIM22", "Essential core component of the mitochondrial TIM22 complex" },
  { "RRI1", "Catalytic subunit of the COP9 signalosome (CSN) complex" },
  { "GDH2", "NAD(+)-dependent glutamate dehydrogenase" },
  { "PRR2", "Serine/threonine protein kinase" },
  { "NOP6", "rRNA-binding protein required for 40S ribosomal subunit biogenesis" },
  { "SHR3", "Endoplasmic reticulum packaging chaperone" },
  { "YDL211C", "Protein of unknown function" },
  { "UGA4", "GABA (gamma-aminobutyrate) permease" },
  { "CWC2", "Member of the NineTeen Complex (NTC)" },
  { "NHP2", "Protein related to mammalian high mobility group (HMG) proteins" },
  { "GLE1", "Cytoplasmic nucleoporin required for polyadenylated mRNA export" },
  { "YDL206W", "Putative protein of unknown function" },
  { "HEM3", "Porphobilinogen deaminase" },
  { "RTN2", "Reticulon protein" },
  { "ACK1", "Protein that functions in the cell wall integrity pathway" },
  { "MRPL11", "Mitochondrial ribosomal protein of the large subunit" },
  { "TRM8", "Catalytic subunit of a tRNA methyltransferase complex" },
  { "MGT1", "DNA repair methyltransferase (6-O-methylguanine-DNA methylase)" },
  { "YDL199C", "Putative transporter" },
  { "GGC1", "Mitochondrial GTP/GDP transporter" },
  { "ASF2", "Anti-silencing protein" },
  { "YDL196W", "NULL" },
  { "SEC31", "Component of the Sec13p-Sec31p complex of the COPII vesicle coat" },
  { "SNF3", "Plasma membrane low glucose sensor, regulates glucose transport" },
  { "NUS1", "Forms dehydrodolichyl diphosphate syntase complex with RER2 or SRT1" },
  { "ARF1", "ADP-ribosylation factor" },
  { "RPL35A", "Ribosomal 60S subunit protein L35A" },
  { "UFD2", "Ubiquitin chain assembly factor (E4)" },
  { "RBS1", "Protein involved in assembly of the RNA polymerase III (Pol III) comp" },
  { "PPH22", "Catalytic subunit of protein phosphatase 2A, PP2A" },
  { "YDL187C", "NULL" },
  { "YDL186W", "Putative protein of unknown function" },
  { "YDL185C-A", "NULL" },
  { "VMA1", "Subunit A of the V1 peripheral membrane domain of V-ATPase" },
  { "RPL41A", "Ribosomal 60S subunit protein L41A" },
  { "YDL183C", "Protein that may form an active mitochondrial KHE system" },
  { "LYS20", "Homocitrate synthase isozyme and functions in DNA repair" },
  { "INH1", "Protein that inhibits ATP hydrolysis by the F1F0-ATP synthase" },
  { "YDL180W", "Putative protein of unknown function" },
  { "PCL9", "Cyclin" },
  { "DLD2", "D-2-hydroxyglutarate dehydrogenase, and minor D-lactate dehydrogenase" },
  { "YDL177C", "Putative protein of unknown function" },
  { "YDL176W", "Protein of unknown function" },
  { "AIR2", "RNA-binding subunit of the TRAMP nuclear RNA surveillance complex" },
  { "DLD1", "Major mitochondrial D-lactate dehydrogenase" },
  { "PAR32", "Low complexity protein" },
  { "YDL172C", "NULL" },
  { "GLT1", "NAD(+)-dependent glutamate synthase (GOGAT)" },
  { "UGA3", "Transcriptional activator for GABA-dependent induction of GABA genes" },
  { "UGX2", "Protein of unknown function" },
  { "SFA1", "Bifunctional alcohol dehydrogenase and formaldehyde dehydrogenase" },
  { "NRP1", "Putative RNA binding protein of unknown function" },
  { "FAP7", "Essential NTPase required for small ribosome subunit synthesis" },
  { "CDC36", "Component of the CCR4-NOT core complex, involved in mRNA decapping" },
  { "CDC9", "DNA ligase I found in nucleus and mitochondria" },
  { "YDL163W", "NULL" },
  { "YDL162C", "NULL" },
  { "ENT1", "Epsin-like protein involved in endocytosis and actin patch assembly" },
  { "MHF2", "Component of the heterotetrameric MHF histone-fold complex" },
  { "DHH1", "Cytoplasmic DEAD-box helicase, stimulates mRNA decapping" },
  { "YDL159W-A", "Putative protein of unknown function" },
  { "STE7", "Signal transducing MAP kinase kinase" },
  { "YDL159C-B", "NULL" },
  { "YDL158C", "NULL" },
  { "YDL157C", "Putative protein of unknown function" },
  { "CMR1", "Nuclear protein with a role in protein quality control" },
  { "CLB3", "B-type cyclin involved in cell cycle progression" },
  { "MSH5", "Protein of the MutS family" },
  { "SAS10", "Subunit of U3-containing Small Subunit (SSU) processome complex" },
  { "YDL152W", "NULL" },
  { "BUD30", "NULL" },
  { "RPC53", "RNA polymerase III subunit C53" },
  { "ATG9", "Transmembrane protein involved in forming Cvt and autophagic vesicles" },
  { "NOP14", "Nucleolar protein" },
  { "RPN5", "Subunit of the CSN and 26S proteasome lid complexes" },
  { "LDB17", "Protein involved in the regulation of endocytosis" },
  { "COP1", "Alpha subunit of COPI vesicle coatomer complex" },
  { "YDL144C", "Protein of unknown function" },
  { "CCT4", "Subunit of the cytosolic chaperonin Cct ring complex" },
  { "CRD1", "Cardiolipin synthase" },
  { "BPL1", "Biotin:apoprotein ligase" },
  { "RPO21", "RNA polymerase II largest subunit B220" },
  { "SCM3", "Nonhistone component of centromeric chromatin" },
  { "RGT2", "Plasma membrane high glucose sensor that regulates glucose transport" },
  { "ARF2", "ADP-ribosylation factor" },
  { "RPL35B", "Ribosomal 60S subunit protein L35B" },
  { "RDI1", "Rho GDP dissociation inhibitor" },
  { "PPH21", "Catalytic subunit of protein phosphatase 2A, PP2A" },
  { "RPL41B", "Ribosomal 60S subunit protein L41B" },
  { "SRF1", "Regulator of phospholipase D (Spo14p)" },
  { "CDC53", "Cullin" },
  { "LYS21", "Homocitrate synthase isozyme" },
  { "STF1", "Protein involved in regulation of the mitochondrial F1F0-ATP synthase" },
  { "RPP1B", "Ribosomal protein P1 beta" },
  { "YDL129W", "Protein of unknown function" },
  { "VCX1", "Vacuolar membrane antiporter with Ca2+/H+ and K+/H+ exchange activity" },
  { "PCL2", "Cyclin, interacts with cyclin-dependent kinase Pho85p" },
  { "CDC48", "AAA ATPase" },
  { "HNT1", "Adenosine 5'-monophosphoramidase" },
  { "YDL124W", "NADPH-dependent alpha-keto amide reductase" },
  { "SNA4", "Protein of unknown function" },
  { "UBP1", "Ubiquitin-specific protease" },
  { "YDL121C", "NULL" },
  { "YFH1", "Mitochondrial matrix iron chaperone" },
  { "YDL119C", "NULL" },
  { "YDL118W", "NULL" },
  { "CYK3", "SH3-domain protein located in the bud neck and cytokinetic actin ring" },
  { "NUP84", "Subunit of the Nup84p subcomplex of the nuclear pore complex (NPC)" },
  { "IWR1", "RNA polymerase II transport factor, conserved from yeast to humans" },
  { "YDL114W-A", "NULL" },
  { "YDL114W", "Putative short-chain dehydrogenase/reductase" },
  { "ATG20", "Sorting nexin family member" },
  { "TRM3", "2'-O-ribose methyltransferase" },
  { "RRP42", "Exosome non-catalytic core component" },
  { "TMA17", "ATPase dedicated chaperone that adapts proteasome assembly to stress" },
  { "YDL109C", "Putative lipase" },
  { "KIN28", "Serine/threonine protein kinase, subunit of transcription factor TFIIH" },
  { "MSS2", "Peripherally bound inner membrane protein of the mitochondrial matrix" },
  { "PHO2", "Homeobox transcription factor" },
  { "NSE4", "Component of the SMC5-SMC6 complex" },
  { "QRI7", "Protein involved in threonylcarbamoyl adenosine biosynthesis" },
  { "QRI1", "UDP-N-acetylglucosamine pyrophosphorylase" },
  { "POL3", "Catalytic subunit of DNA polymerase delta" },
  { "DUN1", "Cell-cycle checkpoint S/T protein kinase" },
  { "GET3", "Guanine nucleotide exchange factor for Gpa1p" },
  { "BUG1", "Cis-golgi localized protein involved in ER to Golgi transport" },
  { "SNU23", "Component of the U4/U6.U5 snRNP complex" },
  { "RPN6", "Essential, non-ATPase regulatory subunit of the 26S proteasome lid" },
  { "OPI6", "NULL" },
  { "PMT1", "Protein O-mannosyltransferase of the ER membrane" },
  { "YDL094C", "NULL" },
  { "PMT5", "Protein O-mannosyltransferase" },
  { "SRP14", "Signal recognition particle (SRP) subunit" },
  { "UBX3", "Clathrin-coated vesicle component, regulator of endocytosis" },
  { "RAM1", "Beta subunit of the CAAX farnesyltransferase (FTase)" },
  { "NUR1", "Protein involved in regulation of mitotic exit" },
  { "ASM4", "FG-nucleoporin component of central core of nuclear pore complex (NPC)" },
  { "LUC7", "Essential protein associated with the U1 snRNP complex" },
  { "YDL086W", "Putative carboxymethylenebutenolidase" },
  { "YDL086C-A", "NULL" },
  { "YDL085C-A", "Putative protein of unknown function" },
  { "NDE2", "Mitochondrial external NADH dehydrogenase" },
  { "SUB2", "Component of the TREX complex required for nuclear mRNA export" },
  { "RPS16B", "Protein component of the small (40S) ribosomal subunit" },
  { "RPL13A", "Ribosomal 60S subunit protein L13A" },
  { "RPP1A", "Ribosomal stalk protein P1 alpha" },
  { "THI3", "Regulatory protein that binds Pdc2p and Thi2p transcription factors" },
  { "MRK1", "Glycogen synthase kinase 3 (GSK-3) homolog" },
  { "MDH3", "Peroxisomal malate dehydrogenase" },
  { "VAM6", "Guanine nucleotide exchange factor for the GTPase Gtr1p" },
  { "RXT3", "Component of the Rpd3L histone deacetylase complex" },
  { "RPL31A", "Ribosomal 60S subunit protein L31A" },
  { "BRE1", "E3 ubiquitin ligase" },
  { "YDL073W", "NULL" },
  { "YET3", "Protein of unknown function" },
  { "YDL071C", "NULL" },
  { "BDF2", "Protein involved in transcription initiation" },
  { "CBS1", "Mitochondrial translational activator of the COB mRNA" },
  { "YDL068W", "NULL" },
  { "COX9", "Subunit VIIa of cytochrome c oxidase (Complex IV)" },
  { "IDP1", "Mitochondrial NADP-specific isocitrate dehydrogenase" },
  { "PEX19", "Chaperone and import receptor for newly-synthesized class I PMPs" },
  { "UBC9", "SUMO-conjugating enzyme involved in the Smt3p conjugation pathway" },
  { "SYO1", "Transport adaptor or symportin" },
  { "YDL062W", "NULL" },
  { "RPS29B", "Protein component of the small (40S) ribosomal subunit" },
  { "TSR1", "Protein required for processing of 20S pre-rRNA in the cytoplasm" },
  { "RAD59", "Protein involved DNA double-strand break repair" },
  { "USO1", "Essential protein involved in vesicle-mediated ER to Golgi transport" },
  { "YDL057W", "Putative protein of unknown function" },
  { "MBP1", "Transcription factor" },
  { "PSA1", "GDP-mannose pyrophosphorylase (mannose-1-phosphate guanyltransferase)" },
  { "MCH1", "Protein with similarity to mammalian monocarboxylate permeases" },
  { "PBP4", "Pbp1p binding protein" },
  { "SLC1", "1-acyl-sn-glycerol-3-phosphate acyltransferase" },
  { "LHP1", "RNA binding protein required for maturation of tRNA and U6 snRNA" },
  { "YDL050C", "NULL" },
  { "KNH1", "Protein with similarity to Kre9p" },
  { "STP4", "Protein containing a Kruppel-type zinc-finger domain" },
  { "SIT4", "Ceramide-activated, type 2A-related serine-threonine phosphatase" },
  { "NPC2", "Sterol transport protein and functional homolog of human NPC2/He1" },
  { "MRP10", "Mitochondrial ribosomal protein of the small subunit" },
  { "FAD1", "Flavin adenine dinucleotide (FAD) synthetase" },
  { "MTF2", "Mitochondrial protein that interacts with mitochondrial RNA polymerase" },
  { "PRP11", "Subunit of the SF3a splicing factor complex" },
  { "SIR2", "Conserved NAD+ dependent histone deacetylase of the Sirtuin family" },
  { "YDL041W", "NULL" },
  { "NAT1", "Subunit of protein N-terminal acetyltransferase NatA" },
  { "PRM7", "Pheromone-regulated protein" },
  { "BSC1", "Protein of unconfirmed function" },
  { "PUS9", "Mitochondrial tRNA:pseudouridine synthase" },
  { "GPR1", "Plasma membrane G protein coupled receptor (GPCR)" },
  { "YDL034W", "NULL" },
  { "SLM3", "tRNA-specific 2-thiouridylase" },
  { "YDL032W", "NULL" },
  { "DBP10", "Putative ATP-dependent RNA helicase of the DEAD-box protein family" },
  { "PRP9", "Subunit of the SF3a splicing factor complex" },
  { "ARP2", "Essential component of the Arp2/3 complex" },
  { "MPS1", "Dual-specificity kinase" },
  { "YDL027C", "NULL" },
  { "YDL026W", "NULL" },
  { "RTK1", "Putative protein kinase, potentially phosphorylated by Cdc28p" },
  { "YDL025W-A", "NULL" },
  { "DIA3", "Protein of unknown function" },
  { "YDL022C-A", "Protein of unknown function" },
  { "YDL023C", "NULL" },
  { "GPD1", "NAD-dependent glycerol-3-phosphate dehydrogenase" },
  { "GPM2", "Nonfunctional homolog of Gpm1p phosphoglycerate mutase" },
  { "RPN4", "Transcription factor that stimulates expression of proteasome genes" },
  { "OSH2", "Member of an oxysterol-binding protein family with seven members" },
  { "ERP3", "Protein with similarity to Emp24p and Erv25p" },
  { "CDC7", "Ser/Thr kinase and catalytic subunit of DDK (Dbf4-dependent kinase)" },
  { "YDL016C", "NULL" },
  { "TSC13", "Enoyl reductase" },
  { "NOP1", "Histone glutamine methyltransferase, modifies H2A at Q105 in nucleolus" },
  { "SLX5", "Subunit of the Slx5-Slx8 SUMO-targeted Ub ligase (STUbL) complex" },
  { "YDL012C", "Tail-anchored plasma membrane protein with a conserved CYSTM module" },
  { "YDL011C", "NULL" },
  { "GRX6", "Cis-golgi localized monothiol glutaredoxin, binds Fe-S cluster" },
  { "YDL009C", "Protein of unknown function" },
  { "APC11", "Catalytic core subunit, Anaphase-Promoting Complex/Cyclosome (APC/C)" },
  { "YDL007C-A", "Putative protein of unknown function" },
  { "RPT2", "ATPase of the 19S regulatory particle of the 26S proteasome" },
  { "PTC1", "Type 2C protein phosphatase (PP2C)" },
  { "MED2", "Subunit of the RNA polymerase II mediator complex" },
  { "ATP16", "Delta subunit of the central stalk of mitochondrial F1F0 ATP synthase" },
  { "MCD1", "Essential alpha-kleisin subunit of the cohesin complex" },
  { "NHP10", "Non-essential INO80 chromatin remodeling complex subunit" },
  { "RMD1", "Cytoplasmic protein required for sporulation" },
  { "NTH1", "Neutral trehalase, degrades trehalose" },
  { "YRB1", "Ran GTPase binding protein" },
  { "RCR2", "Vacuolar protein" },
  { "YDR003W-A", "Putative protein of unknown function" },
  { "RAD57", "Protein that stimulates strand exchange" },
  { "MAF1", "Highly conserved negative regulator of RNA polymerase III" },
  { "SOK1", "Protein of unknown function" },
  { "TRP1", "Phosphoribosylanthranilate isomerase" },
  { "YDR008C", "NULL" },
  { "GAL3", "Transcriptional regulator" },
  { "YDR010C", "Putative protein of unknown function" },
  { "SNQ2", "Plasma membrane ATP-binding cassette (ABC) transporter" },
  { "RPL4B", "Ribosomal 60S subunit protein L4B" },
  { "PSF1", "Subunit of the GINS complex (Sld5p, Psf1p, Psf2p, Psf3p)" },
  { "RAD61", "Subunit of a complex that inhibits sister chromatid cohesion" },
  { "HED1", "Meiosis-specific protein" },
  { "YDR015C", "NULL" },
  { "DAD1", "Essential subunit of the Dam1 complex (aka DASH complex)" },
  { "KCS1", "Inositol hexakisphosphate and inositol heptakisphosphate kinase" },
  { "YDR018C", "Probable membrane protein with three predicted transmembrane domains" },
  { "GCV1", "T subunit of the mitochondrial glycine decarboxylase complex" },
  { "DAS2", "Putative protein of unknown function" },
  { "FAL1", "Nucleolar protein required for maturation of 18S rRNA" },
  { "ATG31", "Autophagy-specific protein required for autophagosome formation" },
  { "SES1", "Cytosolic seryl-tRNA synthetase" },
  { "FYV1", "NULL" },
  { "RPS11A", "Protein component of the small (40S) ribosomal subunit" },
  { "NSI1", "RNA polymerase I termination factor" },
  { "VPS54", "Component of the GARP (Golgi-associated retrograde protein) complex" },
  { "REG1", "Regulatory subunit of type 1 protein phosphatase Glc7p" },
  { "YDR029W", "Putative protein of unknown function" },
  { "RAD28", "Protein involved in DNA repair" },
  { "MIX14", "Mitochondrial intermembrane space protein of unknown function" },
  { "PST2", "Protein with similarity to a family of flavodoxin-like proteins" },
  { "MRH1", "Protein that localizes primarily to the plasma membrane" },
  { "LYS14", "Transcriptional activator involved in regulating lysine biosynthesis" },
  { "YDR034C-A", "Putative protein of unknown function" },
  { "YDR034W-B", "Predicted tail-anchored plasma membrane protein" },
  { "ARO3", "3-deoxy-D-arabino-heptulosonate-7-phosphate (DAHP) synthase" },
  { "EHD3", "3-hydroxyisobutyryl-CoA hydrolase" },
  { "KRS1", "Lysyl-tRNA synthetase" },
  { "ENA5", "Protein with similarity to P-type ATPase sodium pumps" },
  { "ENA2", "P-type ATPase sodium pump" },
  { "ENA1", "P-type ATPase sodium pump" },
  { "RSM10", "Mitochondrial ribosomal protein of the small subunit" },
  { "YDR042C", "Putative protein of unknown function" },
  { "NRG1", "Transcriptional repressor" },
  { "HEM13", "Coproporphyrinogen III oxidase" },
  { "RPC11", "RNA polymerase III subunit C11" },
  { "BAP3", "Amino acid permease" },
  { "HEM12", "Uroporphyrinogen decarboxylase" },
  { "YDR048C", "NULL" },
  { "VMS1", "Component of a Cdc48p-complex involved in protein quality control" },
  { "TPI1", "Triose phosphate isomerase, abundant glycolytic enzyme" },
  { "DET1", "Acid phosphatase" },
  { "DBF4", "Regulatory subunit of Cdc7p-Dbf4p kinase complex" },
  { "YDR053W", "NULL" },
  { "CDC34", "Ubiquitin-conjugating enzyme (E2)" },
  { "PST1", "Cell wall protein that contains a putative GPI-attachment site" },
  { "YDR056C", "NULL" },
  { "YOS9", "ER quality-control lectin" },
  { "TGL2", "Triacylglycerol lipase that is localized to the mitochondria" },
  { "UBC5", "Ubiquitin-conjugating enzyme" },
  { "MAK21", "Constituent of 66S pre-ribosomal particles" },
  { "YDR061W", "Protein with similarity to ABC transporter family members" },
  { "LCB2", "Component of serine palmitoyltransferase" },
  { "AIM7", "Protein that interacts with Arp2/3 complex" },
  { "RPS13", "Protein component of the small (40S) ribosomal subunit" },
  { "RRG1", "Protein of unknown function" },
  { "RTR2", "Protein of unknown function" },
  { "OCA6", "Cytoplasmic protein required for replication of Brome mosaic virus" },
  { "DOS2", "Protein of unknown function" },
  { "DOA4", "Ubiquitin hydrolase" },
  { "FMP16", "Protein of unknown function" },
  { "PAA1", "Polyamine acetyltransferase" },
  { "IPT1", "Inositolphosphotransferase" },
  { "SNF11", "Subunit of the SWI/SNF chromatin remodeling complex" },
  { "TPS2", "Phosphatase subunit of the trehalose-6-P synthase/phosphatase complex" },
  { "PPH3", "Catalytic subunit of protein phosphatase PP4 complex" },
  { "RAD55", "Protein that stimulates strand exchange" },
  { "SED1", "Major stress-induced structural GPI-cell wall glycoprotein" },
  { "SHU2", "Component of Shu complex (aka PCSS complex)" },
  { "PET100", "Chaperone that facilitates the assembly of cytochrome c oxidase" },
  { "TFB5", "Component of RNA polymerase II general transcription factor TFIIH" },
  { "VPS41", "Subunit of the HOPS endocytic tethering complex" },
  { "PDC2", "Transcription factor for thiamine-regulated genes" },
  { "STN1", "Telomere end-binding and capping protein" },
  { "RRP8", "Nucleolar S-adenosylmethionine-dependent rRNA methyltransferase" },
  { "TVP23", "Integral membrane protein" },
  { "AFR1", "Protein required for pheromone-induced projection (shmoo) formation" },
  { "SSS1", "Subunit of the Sec61p translocation complex (Sec61p-Sss1p-Sbh1p)" },
  { "RRP1", "Essential evolutionarily conserved nucleolar protein" },
  { "SLU7", "RNA splicing factor" },
  { "YDR089W", "NULL" },
  { "YDR090C", "NULL" },
  { "RLI1", "Essential Fe-S protein" },
  { "UBC13", "E2 ubiquitin-conjugating enzyme" },
  { "DNF2", "Aminophospholipid translocase (flippase)" },
  { "YDR094W", "NULL" },
  { "YDR095C", "NULL" },
  { "GIS1", "Histone demethylase and transcription factor" },
  { "MSH6", "Protein required for mismatch repair in mitosis and meiosis" },
  { "GRX3", "Glutathione-dependent oxidoreductase" },
  { "BMH2", "14-3-3 protein, minor isoform" },
  { "TVP15", "Integral membrane protein" },
  { "ARX1", "Nuclear export factor for the ribosomal pre-60S subunit" },
  { "YDR102C", "Putative protein of unknown function" },
  { "STE5", "Pheromone-responsive MAPK scaffold protein" },
  { "SPO71", "Meiosis-specific protein required for prospore membrane morphogenesis" },
  { "TMS1", "Vacuolar membrane protein of unknown function" },
  { "ARP10", "Component of the dynactin complex" },
  { "TMN2", "Protein with a role in cellular adhesion and filamentous growth" },
  { "TRS85", "Component of transport protein particle (TRAPP) complex III" },
  { "YDR109C", "D-ribulokinase" },
  { "FOB1", "Nucleolar protein that binds the rDNA replication fork barrier site" },
  { "ALT2", "Catalytically inactive alanine transaminase" },
  { "IRC2", "NULL" },
  { "PDS1", "Securin" },
  { "YDR114C", "Putative protein of unknown function" },
  { "YDR115W", "NULL" },
  { "MRPL1", "Mitochondrial ribosomal protein of the large subunit" },
  { "TMA64", "Protein of unknown function that associates with ribosomes" },
  { "APC4", "Subunit of the Anaphase-Promoting Complex/Cyclosome (APC/C)" },
  { "YDR118W-A", "NULL" },
  { "VBA4", "Protein of unknown function" },
  { "COX26", "Stabilizes or regulates formation of respiratory chain supercomplexes" },
  { "TRM1", "tRNA methyltransferase" },
  { "DPB4", "Subunit of DNA pol epsilon and of ISW2 chromatin accessibility complex" },
  { "KIN1", "S/T protein kinase" },
  { "INO2", "Transcription factor" },
  { "YDR124W", "Putative protein of unknown function" },
  { "ECM18", "Protein of unknown function" },
  { "SWF1", "Palmitoyltransferase that acts on transmembrane proteins" },
  { "ARO1", "Pentafunctional arom protein" },
  { "MTC5", "Subunit of SEACAT, a subcomplex of the SEA complex" },
  { "SAC6", "Fimbrin, actin-bundling protein" },
  { "FIN1", "Spindle pole body-related intermediate filament protein" },
  { "YDR131C", "F-box protein subunit of SCF ubiquitin ligase complex" },
  { "YDR132C", "NULL" },
  { "YDR133C", "NULL" },
  { "YCF1", "Vacuolar glutathione S-conjugate transporter" },
  { "VPS61", "NULL" },
  { "RGP1", "Subunit of a Golgi membrane exchange factor (Ric1p-Rgp1p)" },
  { "HPR1", "Subunit of THO/TREX complexes" },
  { "RUB1", "Ubiquitin-like protein with similarity to mammalian NEDD8" },
  { "MTQ2", "S-adenosylmethionine-dependent methyltransferase" },
  { "DOP1", "Trans-Golgi network localized, leucine-zipper domain protein" },
  { "PEX7", "Peroxisomal signal receptor for peroxisomal matrix proteins" },
  { "SAN1", "Ubiquitin-protein ligase" },
  { "MKC7", "GPI-anchored aspartyl protease" },
  { "TAF12", "Subunit (61/68 kDa) of TFIID and SAGA complexes" },
  { "SWI5", "Transcription factor that recruits Mediator and Swi/Snf complexes" },
  { "EKI1", "Ethanolamine kinase" },
  { "KGD2", "Dihydrolipoyl transsuccinylase" },
  { "YDR149C", "NULL" },
  { "NUM1", "Protein required for nuclear migration" },
  { "CTH1", "Member of the CCCH zinc finger family" },
  { "GIR2", "Highly-acidic RWD domain-containing cytoplasmic protein" },
  { "ENT5", "Protein containing an N-terminal epsin-like domain" },
  { "YDR154C", "NULL" },
  { "CPR1", "Cytoplasmic peptidyl-prolyl cis-trans isomerase (cyclophilin)" },
  { "RPA14", "RNA polymerase I subunit A14" },
  { "YDR157W", "Putative protein of unknown function" },
  { "HOM2", "Aspartic beta semi-aldehyde dehydrogenase" },
  { "SAC3", "mRNA export factor" },
  { "SSY1", "Component of the SPS plasma membrane amino acid sensor system" },
  { "YDR161W", "NULL" },
  { "NBP2", "Protein involved in the HOG (high osmolarity glycerol) pathway" },
  { "CWC15", "Non-essential protein involved in pre-mRNA splicing" },
  { "SEC1", "Sm-like protein involved in docking and fusion of exocytic vesicles" },
  { "TRM82", "Noncatalytic subunit of a tRNA methyltransferase complex" },
  { "SEC5", "Essential 107kDa subunit of the exocyst complex" },
  { "TAF10", "Subunit (145 kDa) of TFIID and SAGA complexes" },
  { "CDC37", "Essential Hsp90p co-chaperone" },
  { "STB3", "Ribosomal RNA processing element (RRPE)-binding protein" },
  { "YDR169C-A", "Putative protein of unknown function" },
  { "SEC7", "Guanine nucleotide exchange factor (GEF) for ADP ribosylation factors" },
  { "HSP42", "Small heat shock protein (sHSP) with chaperone activity" },
  { "SUP35", "Translation termination factor eRF3" },
  { "ARG82", "Inositol polyphosphate multikinase (IPMK)" },
  { "HMO1", "Chromatin associated high mobility group (HMG) family member" },
  { "RSM24", "Mitochondrial ribosomal protein of the small subunit" },
  { "NGG1", "Subunit of chromatin modifying histone acetyltransferase complexes" },
  { "UBC1", "Ubiquitin-conjugating enzyme" },
  { "SDH4", "Membrane anchor subunit of succinate dehydrogenase (SDH)" },
  { "CSN9", "Subunit of the Cop9 signalosome" },
  { "YDR179W-A", "NULL" },
  { "SCC2", "Subunit of cohesin loading factor (Scc2p-Scc4p)" },
  { "SAS4", "Subunit of the SAS complex (Sas2p, Sas4p, Sas5p)" },
  { "CDC1", "Putative mannose-ethanolamine phosphate phosphodiesterase" },
  { "YDR182W-A", "Putative protein of unknown function" },
  { "PLP1", "Protein that interacts with CCT (chaperonin containing TCP-1) complex" },
  { "YDR183C-A", "NULL" },
  { "ATC1", "Nuclear protein" },
  { "UPS3", "Mitochondrial protein of unknown function" },
  { "YDR186C", "NULL" },
  { "YDR187C", "NULL" },
  { "CCT6", "Subunit of the cytosolic chaperonin Cct ring complex" },
  { "SLY1", "Hydrophilic protein involved in ER/Golgi vesicle trafficking" },
  { "RVB1", "ATP-dependent DNA helicase, also known as pontin" },
  { "HST4", "NAD(+)-dependent protein deacetylase" },
  { "NUP42", "FG-nucleoporin component of central core of the nuclear pore complex" },
  { "YDR193W", "NULL" },
  { "MSS116", "Mitochondrial transcription elongation factor" },
  { "YDR194W-A", "Putative protein of unknown function" },
  { "REF2", "RNA-binding protein" },
  { "CAB5", "Subunit of the CoA-Synthesizing Protein Complex (CoA-SPC)" },
  { "CBS2", "Mitochondrial translational activator of the COB mRNA" },
  { "RKM2", "Ribosomal protein lysine methyltransferase" },
  { "YDR199W", "NULL" },
  { "VPS64", "Protein required for cytoplasm to vacuole targeting of proteins" },
  { "SPC19", "Essential subunit of the Dam1 complex (aka DASH complex)" },
  { "RAV2", "Subunit of RAVE complex (Rav1p, Rav2p, Skp1p)" },
  { "YDR203W", "NULL" },
  { "COQ4", "Protein with a role in ubiquinone (Coenzyme Q) biosynthesis" },
  { "MSC2", "Endoplasmic reticulum zinc transporter" },
  { "EBS1", "Protein involved in translation inhibition and nonsense-mediated decay" },
  { "UME6", "Rpd3L histone deacetylase complex subunit" },
  { "MSS4", "Phosphatidylinositol-4-phosphate 5-kinase" },
  { "YDR209C", "Putative protein of unknown function" },
  { "YDR210W", "Predicted tail-anchored plasma membrane protein" },
  { "GCD6", "Catalytic epsilon subunit of the translation initiation factor eIF2B" },
  { "TCP1", "Alpha subunit of chaperonin-containing T-complex" },
  { "UPC2", "Sterol regulatory element binding protein" },
  { "AHA1", "Co-chaperone that binds Hsp82p and activates its ATPase activity" },
  { "YDR215C", "Putative protein of unknown function" },
  { "ADR1", "Carbon source-responsive zinc-finger transcription factor" },
  { "RAD9", "DNA damage-dependent checkpoint protein" },
  { "SPR28", "Sporulation-specific homolog of the CDC3/10/11/12 family of genes" },
  { "MFB1", "Mitochondria-associated F-box protein" },
  { "YDR220C", "NULL" },
  { "GTB1", "Glucosidase II beta subunit, forms a complex with alpha subunit Rot2p" },
  { "YDR222W", "Protein of unknown function" },
  { "CRF1", "Transcriptional corepressor" },
  { "HTB1", "Histone H2B" },
  { "HTA1", "Histone H2A" },
  { "ADK1", "Adenylate kinase, required for purine metabolism" },
  { "SIR4", "SIR protein involved in assembly of silent chromatin domains" },
  { "PCF11", "mRNA 3' end processing factor" },
  { "IVY1", "Phospholipid-binding protein that interacts with both Ypt7p and Vps33p" },
  { "YDR230W", "NULL" },
  { "COX20", "Mitochondrial inner membrane protein" },
  { "HEM1", "5-aminolevulinate synthase" },
  { "RTN1", "Reticulon protein" },
  { "LYS4", "Homoaconitase" },
  { "PRP42", "U1 snRNP protein involved in splicing" },
  { "FMN1", "Riboflavin kinase, produces riboflavin monophosphate (FMN)" },
  { "MRPL7", "Mitochondrial ribosomal protein of the large subunit" },
  { "SEC26", "Essential beta-coat protein of the COPI coatomer" },
  { "YDR239C", "Protein of unknown function" },
  { "SNU56", "Component of U1 snRNP required for mRNA splicing via spliceosome" },
  { "BUD26", "NULL" },
  { "AMD2", "Putative amidase" },
  { "PRP28", "RNA binding protein" },
  { "PEX5", "Peroxisomal membrane signal receptor for peroxisomal matrix proteins" },
  { "MNN10", "Subunit of a Golgi mannosyltransferase complex" },
  { "TRS23", "Core component of transport protein particle (TRAPP) complexes I-III" },
  { "YDR246W-A", "Putative protein of unknown function" },
  { "VHS1", "Cytoplasmic serine/threonine protein kinase" },
  { "YDR248C", "Putative gluconokinase" },
  { "YDR249C", "Putative protein of unknown function" },
  { "YDR250C", "NULL" },
  { "PAM1", "Essential protein of unknown function" },
  { "BTT1", "Heterotrimeric nascent polypeptide-associated complex beta3 subunit" },
  { "MET32", "Zinc-finger DNA-binding transcription factor" },
  { "CHL4", "Outer kinetochore protein required for chromosome stability" },
  { "RMD5", "Component of GID Complex that confers ubiquitin ligase (U3) activity" },
  { "CTA1", "Catalase A" },
  { "RKM4", "Ribosomal lysine methyltransferase" },
  { "HSP78", "Oligomeric mitochondrial matrix chaperone" },
  { "YAP6", "Basic leucine zipper (bZIP) transcription factor" },
  { "SWM1", "Subunit of the anaphase-promoting complex (APC)" },
  { "EXG2", "Exo-1,3-beta-glucanase" },
  { "YDR262W", "Putative protein of unknown function" },
  { "DIN7", "Mitochondrial nuclease functioning in DNA repair and replication" },
  { "AKR1", "Palmitoyl transferase involved in protein palmitoylation" },
  { "PEX10", "Peroxisomal membrane E3 ubiquitin ligase" },
  { "HEL2", "RING finger ubiquitin ligase (E3)" },
  { "CIA1", "Component of cytosolic iron-sulfur protein assembly (CIA) machinery" },
  { "MSW1", "Mitochondrial tryptophanyl-tRNA synthetase" },
  { "YDR269C", "NULL" },
  { "CCC2", "Cu(+2)-transporting P-type ATPase" },
  { "YDR271C", "NULL" },
  { "GLO2", "Cytoplasmic glyoxalase II" },
  { "DON1", "Meiosis-specific component of the spindle pole body" },
  { "YDR274C", "Putative protein of unknown function" },
  { "BSC2", "Protein of unknown function" },
  { "PMP3", "Small plasma membrane protein" },
  { "MTH1", "Negative regulator of the glucose-sensing signal transduction pathway" },
  { "YDR278C", "Putative protein of unknown function" },
  { "RNH202", "Ribonuclease H2 subunit" },
  { "RRP45", "Exosome non-catalytic core component" },
  { "PHM6", "Protein of unknown function" },
  { "YDR282C", "NULL" },
  { "GCN2", "Protein kinase" },
  { "DPP1", "Diacylglycerol pyrophosphate (DGPP) phosphatase" },
  { "ZIP1", "Transverse filament protein of the synaptonemal complex" },
  { "YDR286C", "NULL" },
  { "INM2", "Inositol monophosphatase" },
  { "NSE3", "Component of the SMC5-SMC6 complex" },
  { "RTT103", "Protein involved in transcription termination by RNA polymerase II" },
  { "YDR290W", "NULL" },
  { "HRQ1", "3'-5' DNA helicase that has DNA strand annealing activity" },
  { "SRP101", "Signal recognition particle (SRP) receptor alpha subunit" },
  { "SSD1", "Translational repressor with a role in polar growth and wall integrity" },
  { "DPL1", "Dihydrosphingosine phosphate lyase" },
  { "HDA2", "Subunit of the HDA1 histone deacetylase complex" },
  { "MHR1", "Mitochondrial ribosomal protein of the large subunit" },
  { "SUR2", "Sphinganine C4-hydroxylase" },
  { "ATP5", "Subunit 5 of the stator stalk of mitochondrial F1F0 ATP synthase" },
  { "BFR2", "Component of the SSU and 90S preribosomes" },
  { "PRO1", "Gamma-glutamyl kinase" },
  { "CFT1", "RNA-binding subunit of the mRNA cleavage and polyadenylation factor" },
  { "GPI11", "ER membrane protein involved in a late step of GPI anchor assembly" },
  { "RSC3", "Component of the RSC chromatin remodeling complex" },
  { "CPR5", "Peptidyl-prolyl cis-trans isomerase (cyclophilin) of the ER" },
  { "HNT2", "Dinucleoside triphosphate hydrolase" },
  { "YDR306C", "NULL" },
  { "PMT7", "Putative protein mannosyltransferase similar to Pmt1p" },
  { "SRB7", "Subunit of the RNA polymerase II mediator complex" },
  { "GIC2", "Redundant rho-like GTPase Cdc42p effector" },
  { "SUM1", "Transcriptional repressor that regulates middle-sporulation genes" },
  { "TFB1", "Subunit of TFIIH and nucleotide excision repair factor 3 complexes" },
  { "SSF2", "Protein required for ribosomal large subunit maturation" },
  { "PIB1", "RING-type ubiquitin ligase of the endosomal and vacuolar membranes" },
  { "RAD34", "Protein involved in nucleotide excision repair (NER)" },
  { "IPK1", "Inositol 1,3,4,5,6-pentakisphosphate 2-kinase" },
  { "OMS1", "Protein integral to the mitochondrial membrane" },
  { "HIM1", "Protein of unknown function involved in DNA repair" },
  { "MCM21", "Component of the kinetochore sub-complex COMA" },
  { "YFT2", "Protein required for normal ER membrane biosynthesis" },
  { "SWA2", "Auxilin-like protein involved in vesicular transport" },
  { "DAD4", "Essential subunit of the Dam1 complex (aka DASH complex)" },
  { "YDR320W-B", "NULL" },
  { "ASP1", "Cytosolic L-asparaginase, involved in asparagine catabolism" },
  { "MRPL35", "Mitochondrial ribosomal protein of the large subunit" },
  { "TIM11", "Subunit e of mitochondrial F1F0-ATPase" },
  { "PEP7", "Adaptor protein involved in vesicle-mediated vacuolar protein sorting" },
  { "UTP4", "Subunit of U3-containing 90S preribosome and SSU processome complexes" },
  { "YCG1", "Subunit of the condensin complex" },
  { "YSP2", "Sterol-binding protein" },
  { "YDR327W", "NULL" },
  { "SKP1", "Evolutionarily conserved kinetochore protein" },
  { "PEX3", "Peroxisomal membrane protein (PMP)" },
  { "UBX5", "UBX domain-containing protein that interacts with Cdc48p" },
  { "GPI8", "ER membrane glycoprotein subunit of the GPI transamidase complex" },
  { "IRC3", "Double-stranded DNA-dependent helicase of the DExH/D-box family" },
  { "RQC1", "Component of the ribosome quality control complex (RQC)" },
  { "SWR1", "Swi2/Snf2-related ATPase" },
  { "MSN5", "Karyopherin" },
  { "YDR336W", "NULL" },
  { "MRPS28", "Mitochondrial ribosomal protein of the small subunit" },
  { "YDR338C", "Putative protein of unknown function" },
  { "FCF1", "Putative PINc domain nuclease" },
  { "YDR340W", "NULL" },
  { "YDR341C", "Arginyl-tRNA synthetase" },
  { "HXT7", "High-affinity glucose transporter" },
  { "HXT6", "High-affinity glucose transporter" },
  { "YDR344C", "Putative protein of unknown function" },
  { "HXT3", "Low affinity glucose transporter of the major facilitator superfamily" },
  { "SVF1", "Protein with a potential role in cell survival pathways" },
  { "MRP1", "Mitochondrial ribosomal protein of the small subunit" },
  { "PAL1", "Protein of unknown function thought to be involved in endocytosis" },
  { "YPS7", "Putative GPI-anchored aspartic protease" },
  { "ATP22", "Specific translational activator for the mitochondrial ATP6 mRNA" },
  { "SBE2", "Protein required for bud growth" },
  { "YPQ2", "Putative vacuolar membrane transporter for cationic amino acids" },
  { "TRR1", "Cytoplasmic thioredoxin reductase" },
  { "TRP4", "Anthranilate phosphoribosyl transferase" },
  { "YDR354C-A", "NULL" },
  { "YDR355C", "NULL" },
  { "SPC110", "Inner plaque spindle pole body (SPB) component" },
  { "CNL1", "Subunit of the BLOC-1 complex involved in endosomal maturation" },
  { "GGA1", "Golgi-localized protein with homology to gamma-adaptin" },
  { "EAF1", "Component of the NuA4 histone acetyltransferase complex" },
  { "OPI7", "NULL" },
  { "BCP1", "Essential protein involved in nuclear export of Mss4p" },
  { "TFC6", "Subunit of RNA polymerase III transcription initiation factor complex" },
  { "ESC2", "Sumo-like domain protein" },
  { "SEM1", "19S proteasome regulatory particle lid subcomplex component" },
  { "CDC40", "Pre-mRNA splicing factor" },
  { "ESF1", "Nucleolar protein involved in pre-rRNA processing" },
  { "YDR366C", "Putative protein of unknown function" },
  { "KEI1", "Component of inositol phosphorylceramide (IPC) synthase" },
  { "YPR1", "NADPH-dependent aldo-keto reductase" },
  { "XRS2", "Protein required for DNA repair" },
  { "DXO1", "mRNA 5'-end-capping quality-control protein" },
  { "CTS2", "Putative chitinase" },
  { "YDR371C-A", "NULL" },
  { "VPS74", "Golgi phosphatidylinositol-4-kinase effector and PtdIns4P sensor" },
  { "FRQ1", "N-myristoylated calcium-binding protein" },
  { "PHO92", "Posttranscriptional regulator of phosphate metabolism" },
  { "WIP1", "Kinetochore localized protein of unknown function" },
  { "BCS1", "Protein translocase and chaperone required for Complex III assembly" },
  { "ARH1", "Oxidoreductase of the mitochondrial inner membrane" },
  { "ATP17", "Subunit f of the F0 sector of mitochondrial F1F0 ATP synthase" },
  { "LSM6", "Lsm (Like Sm) protein" },
  { "RGA2", "GTPase-activating protein for polarity-establishment protein Cdc42p" },
  { "SDH6", "Mitochondrial protein involved in assembly of succinate dehydrogenase" },
  { "ARO10", "Phenylpyruvate decarboxylase" },
  { "YRA1", "Nuclear polyadenylated RNA-binding protein" },
  { "YDR381C-A", "NULL" },
  { "RPP2B", "Ribosomal protein P2 beta" },
  { "NKP1", "Central kinetochore protein and subunit of the Ctf19 complex" },
  { "ATO3", "Plasma membrane protein, putative ammonium transporter" },
  { "EFT2", "Elongation factor 2 (EF-2), also encoded by EFT1" },
  { "MUS81", "Subunit of structure-specific Mms4p-Mus81p endonuclease" },
  { "YDR387C", "NULL" },
  { "RVS167", "Calmodulin-binding actin-associated protein" },
  { "SAC7", "GTPase activating protein (GAP) for Rho1p" },
  { "UBA2", "Subunit of heterodimeric nuclear SUMO activating enzyme E1 with Aos1p" },
  { "YDR391C", "Putative protein of unknown function" },
  { "SPT3", "Subunit of the SAGA and SAGA-like transcriptional regulatory complexes" },
  { "SHE9", "Protein required for normal mitochondrial morphology" },
  { "RPT3", "ATPase of the 19S regulatory particle of the 26S proteasome" },
  { "SXM1", "Nuclear transport factor (karyopherin)" },
  { "YDR396W", "NULL" },
  { "NCB2", "Subunit of a heterodimeric NC2 transcription regulator complex" },
  { "UTP5", "Subunit of U3-containing Small Subunit (SSU) processome complex" },
  { "HPT1", "Dimeric hypoxanthine-guanine phosphoribosyltransferase" },
  { "URH1", "Uridine nucleosidase (uridine-cytidine N-ribohydrolase)" },
  { "YDR401W", "NULL" },
  { "DIT2", "N-formyltyrosine oxidase" },
  { "DIT1", "Sporulation-specific enzyme required for spore wall maturation" },
  { "RPB7", "RNA polymerase II subunit B16" },
  { "MRP20", "Mitochondrial ribosomal protein of the large subunit" },
  { "PDR15", "Plasma membrane ATP binding cassette (ABC) transporter" },
  { "YDR406W-A", "NULL" },
  { "TRS120", "Component of transport protein particle (TRAPP) complex II" },
  { "ADE8", "Phosphoribosyl-glycinamide transformylase" },
  { "SIZ1", "SUMO E3 ligase" },
  { "STE14", "Farnesyl cysteine-carboxyl methyltransferase" },
  { "DFM1", "Endoplasmic reticulum (ER) localized protein" },
  { "RRP17", "Component of the pre-60S pre-ribosomal particle" },
  { "YDR413C", "NULL" },
  { "ERD1", "Predicted membrane protein required for lumenal ER protein retention" },
  { "YDR415C", "Putative aminopeptidase" },
  { "SYF1", "Member of the NineTeen Complex (NTC)" },
  { "YDR417C", "NULL" },
  { "RPL12B", "Ribosomal 60S subunit protein L12B" },
  { "RAD30", "DNA polymerase eta" },
  { "HKR1", "Mucin family member that functions as an osmosensor in the HOG pathway" },
  { "ARO80", "Zinc finger transcriptional activator of the Zn2Cys6 family" },
  { "SIP1", "Alternate beta-subunit of the Snf1p kinase complex" },
  { "CAD1", "AP-1-like basic leucine zipper (bZIP) transcriptional activator" },
  { "DYN2", "Cytoplasmic light chain dynein, microtubule motor protein" },
  { "SNX41", "Sorting nexin" },
  { "YDR426C", "NULL" },
  { "RPN9", "Non-ATPase regulatory subunit of the 26S proteasome" },
  { "BNA7", "Formylkynurenine formamidase" },
  { "TIF35", "eIF3g subunit of the eukaryotic translation initiation factor 3 (eIF3)" },
  { "CYM1", "Lysine-specific metalloprotease of the pitrilysin family" },
  { "YDR431W", "NULL" },
  { "NPL3", "RNA-binding protein" },
  { "YDR433W", "NULL" },
  { "GPI17", "Transmembrane protein" },
  { "PPM1", "Carboxyl methyltransferase" },
  { "PPZ2", "Serine/threonine protein phosphatase Z, isoform of Ppz1p" },
  { "GPI19", "Subunit of GPI-GlcNAc transferase involved in synthesis of GlcNAc-PI" },
  { "THI74", "Mitochondrial transporter repressible by thiamine" },
  { "LRS4", "Nucleolar protein that forms a complex with Csm1p" },
  { "DOT1", "Nucleosomal histone H3-Lys79 methylase" },
  { "APT2", "Potential adenine phosphoribosyltransferase" },
  { "YDR442W", "NULL" },
  { "SSN2", "Subunit of the RNA polymerase II mediator complex" },
  { "YDR444W", "Putative hydrolase acting on ester bonds" },
  { "YDR445C", "NULL" },
  { "ECM11", "Meiosis-specific protein" },
  { "RPS17B", "Ribosomal protein 51 (rp51) of the small (40s) subunit" },
  { "ADA2", "Transcription coactivator" },
  { "UTP6", "Nucleolar protein" },
  { "RPS18A", "Protein component of the small (40S) ribosomal subunit" },
  { "YHP1", "Homeobox transcriptional repressor" },
  { "PPN1", "Dual endo- and exopolyphosphatase with a role in phosphate metabolism" },
  { "TSA2", "Stress inducible cytoplasmic thioredoxin peroxidase" },
  { "GUK1", "Guanylate kinase" },
  { "YDR455C", "NULL" },
  { "NHX1", "Na+/H+ and K+/H+ exchanger" },
  { "TOM1", "E3 ubiquitin ligase of the hect-domain class" },
  { "HEH2", "Inner nuclear membrane (INM) protein" },
  { "PFA5", "Palmitoyltransferase with autoacylation activity" },
  { "TFB3", "Subunit of TFIIH and nucleotide excision repair factor 3 complexes" },
  { "MFA1", "Mating pheromone a-factor" },
  { "YDR461C-A", "NULL" },
  { "MRPL28", "Mitochondrial ribosomal protein of the large subunit" },
  { "STP1", "Transcription factor" },
  { "SPP41", "Protein of unknown function" },
  { "YDR464C-A", "NULL" },
  { "RMT2", "Arginine N5 methyltransferase" },
  { "PKH3", "Protein kinase with similarity to mammalian PDK1 and yeast Pkh1p/Phk2p" },
  { "YDR467C", "NULL" },
  { "TLG1", "Essential t-SNARE that mediates fusion of vesicles with the late Golgi" },
  { "SDC1", "Subunit of the COMPASS (Set1C) complex" },
  { "UGO1", "Outer membrane component of the mitochondrial fusion machinery" },
  { "RPL27B", "Ribosomal 60S subunit protein L27B" },
  { "TRS31", "Core component of transport protein particle (TRAPP) complexes I-III" },
  { "PRP3", "Splicing factor" },
  { "JIP4", "Protein of unknown function" },
  { "YDR476C", "Putative protein of unknown function" },
  { "SNF1", "AMP-activated S/T protein kinase" },
  { "SNM1", "Ribonuclease MRP complex subunit" },
  { "PEX29", "ER-resident protein involved in peroxisomal biogenesis" },
  { "DIG2", "MAP kinase-responsive inhibitor of the Ste12p transcription factor" },
  { "PHO8", "Repressible vacuolar alkaline phosphatase" },
  { "CWC21", "Protein involved in RNA splicing by the spliceosome" },
  { "KRE2", "Alpha1,2-mannosyltransferase of the Golgi" },
  { "VPS52", "Component of the GARP (Golgi-associated retrograde protein) complex" },
  { "VPS72", "Htz1p-binding component of the SWR1 complex" },
  { "VPS60", "Protein involved in late endosome to vacuole transport" },
  { "RIB3", "3,4-dihydroxy-2-butanone-4-phosphate synthase (DHBP synthase)" },
  { "PAC11", "Dynein intermediate chain, microtubule motor protein" },
  { "SLD5", "Subunit of the GINS complex (Sld5p, Psf1p, Psf2p, Psf3p)" },
  { "PKH1", "Serine/threonine protein kinase" },
  { "YDR491C", "NULL" },
  { "IZH1", "Membrane protein involved in zinc ion homeostasis" },
  { "MZM1", "Protein required for assembly of the cytochrome bc(1) complex" },
  { "RSM28", "Mitochondrial ribosomal protein of the small subunit" },
  { "VPS3", "Component of CORVET membrane tethering complex" },
  { "PUF6", "Pumilio-homology domain protein" },
  { "ITR1", "Myo-inositol transporter" },
  { "SEC20", "Membrane glycoprotein v-SNARE" },
  { "LCD1", "Essential protein required for the DNA integrity checkpoint pathways" },
  { "RPL37B", "Ribosomal 60S subunit protein L37B" },
  { "PLM2", "Putative transcription factor, contains Forkhead Associated domain" },
  { "SAM2", "S-adenosylmethionine synthetase" },
  { "LPP1", "Lipid phosphate phosphatase" },
  { "SPG3", "Protein required for high temperature survival during stationary phase" },
  { "PSP1", "Asn and gln rich protein of unknown function" },
  { "GMC1", "Protein involved in meiotic progression" },
  { "GIN4", "Protein kinase involved in bud growth and assembly of the septin ring" },
  { "GNP1", "High-affinity glutamine permease" },
  { "YDR509W", "NULL" },
  { "SMT3", "Ubiquitin-like protein of the SUMO family" },
  { "YDR510C-A", "NULL" },
  { "SDH7", "Mitochondrial protein involved in assembly of succinate dehydrogenase" },
  { "EMI1", "Non-essential protein of unknown function" },
  { "GRX2", "Cytoplasmic glutaredoxin" },
  { "YDR514C", "Protein of unknown function that localizes to mitochondria" },
  { "SLF1", "RNA binding protein that associates with polysomes" },
  { "EMI2", "Non-essential protein of unknown function" },
  { "GRH1", "Acetylated cis-Golgi protein, homolog of human GRASP65" },
  { "EUG1", "Protein disulfide isomerase of the endoplasmic reticulum lumen" },
  { "FPR2", "Membrane-bound peptidyl-prolyl cis-trans isomerase (PPIase)" },
  { "URC2", "Putative Zn(II)2Cys6 motif containing transcription factor" },
  { "YDR521W", "NULL" },
  { "SPS2", "Protein expressed during sporulation" },
  { "SPS1", "Putative protein serine/threonine kinase" },
  { "AGE1", "ADP-ribosylation factor (ARF) GTPase activating protein (GAP) effector" },
  { "YDR524W-C", "Putative protein of unknown function" },
  { "YDR524C-B", "Putative protein of unknown function" },
  { "YDR524C-A", "NULL" },
  { "API2", "Putative protein of unknown function" },
  { "SNA2", "Protein of unknown function" },
  { "YDR526C", "NULL" },
  { "RBA50", "Protein involved in transcription" },
  { "HLR1", "Protein involved in regulation of cell wall composition and integrity" },
  { "QCR7", "Subunit 7 of ubiquinol cytochrome-c reductase (Complex III)" },
  { "APA2", "Diadenosine 5',5'''-P1,P4-tetraphosphate phosphorylase II" },
  { "CAB1", "Pantothenate kinase, ATP:D-pantothenate 4'-phosphotransferase" },
  { "KRE28", "Subunit of a kinetochore-microtubule binding complex" },
  { "HSP31", "Methylglyoxalase that converts methylglyoxal to D-lactate" },
  { "FIT1", "Mannoprotein that is incorporated into the cell wall" },
  { "YDR535C", "NULL" },
  { "STL1", "Glycerol proton symporter of the plasma membrane" },
  { "YDR537C", "NULL" },
  { "PAD1", "Phenylacrylic acid decarboxylase" },
  { "FDC1", "Ferulic acid decarboxylase" },
  { "IRC4", "Protein of unknown function" },
  { "YDR541C", "Aldehyde reductase" },
  { "PAU10", "Protein of unknown function" },
  { "YDR543C", "NULL" },
  { "YDR544C", "NULL" },
  { "YRF1-1", "Helicase encoded by the Y' element of subtelomeric regions" },
  { "YDR545C-A", "NULL" },
  { "YEL077C", "Helicase-like protein encoded within the telomeric Y' element" },
  { "YEL077W-A", "NULL" },
  { "YEL076C-A", "Putative protein of unknown function" },
  { "YEL076C", "Putative protein of unknown function" },
  { "YEL075W-A", "NULL" },
  { "YEL075C", "Putative protein of unknown function" },
  { "YEL074W", "NULL" },
  { "YEL073C", "Putative protein of unknown function" },
  { "RMD6", "Protein required for sporulation" },
  { "DLD3", "2-hydroxyglutarate transhydrogenase, and minor D-lactate dehydrogenase" },
  { "DSF1", "Mannitol dehydrogenase" },
  { "HXT13", "Putative transmembrane polyol transporter" },
  { "YEL068C", "Protein of unknown function" },
  { "YEL067C", "Putative protein of unknown function" },
  { "HPA3", "D-Amino acid N-acetyltransferase that detoxifies D-amino acids" },
  { "SIT1", "Ferrioxamine B transporter" },
  { "AVT2", "Putative transporter" },
  { "CAN1", "Plasma membrane arginine permease" },
  { "NPR2", "Subunit of the Iml1p/SEACIT complex" },
  { "CIN8", "Kinesin motor protein" },
  { "PRB1", "Vacuolar proteinase B (yscB) with H3 N-terminal endopeptidase activity" },
  { "SOM1", "Subunit of the mitochondrial inner membrane peptidase (IMP)" },
  { "HHY1", "NULL" },
  { "PCM1", "Essential N-acetylglucosamine-phosphate mutase" },
  { "YEL057C", "NULL" },
  { "HAT2", "Subunit of the Hat1p-Hat2p histone acetyltransferase complex" },
  { "POL5", "DNA Polymerase phi" },
  { "RPL12A", "Ribosomal 60S subunit protein L12A" },
  { "YEL053W-A", "NULL" },
  { "MAK10", "Non-catalytic subunit of the NatC N-terminal acetyltransferase" },
  { "AFG1", "Protein that may act as a chaperone for cytochrome c oxidase subunits" },
  { "VMA8", "Subunit D of the V1 peripheral membrane domain of V-ATPase" },
  { "YEL050W-A", "NULL" },
  { "RML2", "Mitochondrial ribosomal protein of the large subunit (L2)" },
  { "PAU2", "Member of the seripauperin multigene family" },
  { "TCA17", "Component of transport protein particle (TRAPP) complex II" },
  { "FRD1", "Soluble fumarate reductase" },
  { "GLY1", "Threonine aldolase" },
  { "YEL045C", "NULL" },
  { "IES6", "Component of the INO80 chromatin remodeling complex" },
  { "YEL043W", "Predicted cytoskeleton protein involved in intracellular signaling" },
  { "GDA1", "Guanosine diphosphatase located in the Golgi" },
  { "YEF1", "ATP-NADH kinase" },
  { "UTR2", "Chitin transglycosylase" },
  { "CYC7", "Cytochrome c isoform 2, expressed under hypoxic conditions" },
  { "UTR4", "Protein with sequence similarity to acireductone synthases" },
  { "RAD23", "Protein with ubiquitin-like N terminus" },
  { "ANP1", "Subunit of the alpha-1,6 mannosyltransferase complex" },
  { "UTR5", "Protein of unknown function" },
  { "YEL034C-A", "NULL" },
  { "HYP2", "Translation elongation factor eIF-5A" },
  { "MTC7", "Protein of unknown function" },
  { "MCM3", "Protein involved in DNA replication" },
  { "YEL032C-A", "NULL" },
  { "SPF1", "P-type ATPase, ion transporter of the ER membrane" },
  { "YEL030C-A", "NULL" },
  { "ECM10", "Heat shock protein of the Hsp70 family" },
  { "BUD16", "Putative pyridoxal kinase" },
  { "YEL028W", "Putative protein of unknown function" },
  { "VMA3", "Proteolipid subunit c of the V0 domain of vacuolar H(+)-ATPase" },
  { "SNU13", "RNA binding protein" },
  { "YEL025C", "Putative protein of unknown function" },
  { "RIP1", "Ubiquinol-cytochrome-c reductase" },
  { "YEL023C", "Putative protein of unknown function" },
  { "GEA2", "Guanine nucleotide exchange factor for ADP ribosylation factors (ARFs)" },
  { "URA3", "Orotidine-5'-phosphate (OMP) decarboxylase" },
  { "YEL020C-B", "NULL" },
  { "TIM9", "Essential protein of the mitochondrial intermembrane space" },
  { "YEL020C", "NULL" },
  { "MMS21", "Highly conserved SUMO E3 ligase subunit of SMC5-SMC6 complex" },
  { "YEL018C-A", "NULL" },
  { "EAF5", "Non-essential subunit of the NuA4 acetyltransferase complex" },
  { "PMP2", "Proteolipid associated with plasma membrane H(+)-ATPase (Pma1p)" },
  { "GTT3", "Protein of unknown function may be involved in glutathione metabolism" },
  { "NPP2", "Nucleotide pyrophosphatase/phosphodiesterase" },
  { "EDC3", "Non-essential conserved protein with a role in mRNA decapping" },
  { "YEL014C", "Putative protein of unknown function" },
  { "VAC8", "Vacuolar membrane protein" },
  { "UBC8", "Ubiquitin-conjugating enzyme that regulates gluconeogenesis" },
  { "GLC3", "Glycogen branching enzyme, involved in glycogen accumulation" },
  { "YEL010W", "NULL" },
  { "YEL009C-A", "Putative protein of unknown function" },
  { "GCN4", "bZIP transcriptional activator of amino acid biosynthetic genes" },
  { "YEL008W", "Putative protein of unknown function" },
  { "YEL008C-A", "NULL" },
  { "MIT1", "Transcriptional regulator of pseudohyphal growth" },
  { "YEA6", "Putative mitochondrial NAD+ transporter" },
  { "VAB2", "Subunit of the BLOC-1 complex involved in endosomal maturation" },
  { "YEA4", "Uridine diphosphate-N-acetylglucosamine (UDP-GlcNAc) transporter" },
  { "GIM4", "Subunit of the heterohexameric cochaperone prefoldin complex" },
  { "WBP1", "Beta subunit of the oligosaccharyl transferase glycoprotein complex" },
  { "IRC22", "Protein of unknown function" },
  { "MNN1", "Alpha-1,3-mannosyltransferase" },
  { "NOP16", "Constituent of 66S pre-ribosomal particles" },
  { "PMI40", "Mannose-6-phosphate isomerase" },
  { "FMP52", "Protein of unknown function" },
  { "YND1", "Apyrase with wide substrate specificity" },
  { "NUG1", "GTPase that associates with nuclear 60S pre-ribosomes" },
  { "YER006C-A", "NULL" },
  { "PAC2", "Microtubule effector required for tubulin heterodimer formation" },
  { "TMA20", "Protein of unknown function that associates with ribosomes" },
  { "SEC3", "Subunit of the exocyst complex" },
  { "NTF2", "Nuclear envelope protein" },
  { "YER010C", "Bifunctional HMG aldolase/oxaloacetate decarboxylase" },
  { "TIR1", "Cell wall mannoprotein" },
  { "PRE1", "Beta 4 subunit of the 20S proteasome" },
  { "PRP22", "DEAH-box RNA-dependent ATPase/ATP-dependent RNA helicase" },
  { "HEM14", "Protoporphyrinogen oxidase" },
  { "BUD25", "Protein involved in bud-site selection" },
  { "FAA2", "Medium chain fatty acyl-CoA synthetase" },
  { "BIM1", "Microtubule plus end-tracking protein" },
  { "AFG3", "Mitochondrial inner membrane m-AAA protease component" },
  { "SPC25", "Component of the kinetochore-associated Ndc80 complex" },
  { "ISC1", "Inositol phosphosphingolipid phospholipase C" },
  { "SBH2", "Ssh1p-Sss1p-Sbh2p complex component" },
  { "GPA2", "Nucleotide binding alpha subunit of the heterotrimeric G protein" },
  { "RPN3", "Essential non-ATPase regulatory subunit of the 26S proteasome lid" },
  { "SRB4", "Subunit of the RNA polymerase II mediator complex" },
  { "PRO3", "Delta 1-pyrroline-5-carboxylate reductase" },
  { "YER023C-A", "NULL" },
  { "YAT2", "Carnitine acetyltransferase" },
  { "GCD11", "Gamma subunit of the translation initiation factor eIF2" },
  { "CHO1", "Phosphatidylserine synthase" },
  { "GAL83", "One of three possible beta-subunits of the Snf1 kinase complex" },
  { "MIG3", "Transcriptional regulator" },
  { "SMB1", "Core Sm protein Sm B" },
  { "CHZ1", "Histone chaperone for Htz1p/H2A-H2B dimer" },
  { "YPT31", "Rab family GTPase" },
  { "FIR1", "Protein involved in 3' mRNA processing" },
  { "ZRG8", "Protein of unknown function" },
  { "YER034W", "Protein of unknown function" },
  { "EDC2", "RNA-binding protein that directly activates mRNA decapping" },
  { "ARB1", "ATPase of the ATP-binding cassette (ABC) family" },
  { "PHM8", "Lysophosphatidic acid (LPA) phosphatase, nucleotidase" },
  { "KRE29", "Subunit of the SMC5-SMC6 complex" },
  { "YER038W-A", "NULL" },
  { "HVG1", "Protein of unknown function" },
  { "YER039C-A", "Putative protein of unknown function" },
  { "GLN3", "Transcriptional activator of genes regulated by nitrogen catabolite r" },
  { "YEN1", "Holliday junction resolvase" },
  { "MXR1", "Methionine-S-sulfoxide reductase" },
  { "SAH1", "S-adenosyl-L-homocysteine hydrolase" },
  { "ERG28", "Endoplasmic reticulum membrane protein" },
  { "MEI4", "Meiosis-specific protein involved in forming DSBs" },
  { "ACA1", "ATF/CREB family basic leucine zipper (bZIP) transcription factor" },
  { "SPO73", "Meiosis-specific protein required for prospore membrane morphogenesis" },
  { "YER046W-A", "NULL" },
  { "SAP1", "Putative ATPase of the AAA family" },
  { "CAJ1", "Nuclear type II J heat shock protein of the E. coli dnaJ family" },
  { "ISD11", "Cysteine desulfurase (Nfs1p) activator" },
  { "TPA1", "Fe(II)/2-oxoglutarate-dependent dioxygenase family member" },
  { "RSM18", "Mitochondrial ribosomal protein of the small subunit" },
  { "JHD1", "JmjC domain family histone demethylase specific for H3-K36" },
  { "HOM3", "Aspartate kinase (L-aspartate 4-P-transferase)" },
  { "PIC2", "Mitochondrial copper and phosphate carrier" },
  { "YER053C-A", "Protein of unknown function" },
  { "GIP2", "Putative regulatory subunit of protein phosphatase Glc7p" },
  { "HIS1", "ATP phosphoribosyltransferase" },
  { "FCY2", "Purine-cytosine permease" },
  { "RPL34A", "Ribosomal 60S subunit protein L34A" },
  { "HMF1", "Member of the p14.5 protein family" },
  { "PET117", "Assembly factor that couples heme a synthesis to complex IV assembly" },
  { "PCL6", "Pho85p cyclin of the Pho80p subfamily" },
  { "FCY21", "Putative purine-cytosine permease" },
  { "FCY22", "Putative purine-cytosine permease" },
  { "CEM1", "Mitochondrial beta-keto-acyl synthase" },
  { "GPP2", "DL-glycerol-3-phosphate phosphatase involved in glycerol biosynthesis" },
  { "THO1", "Conserved nuclear RNA-binding protein" },
  { "VHR2", "Non-essential nuclear protein" },
  { "ICL1", "Isocitrate lyase" },
  { "RRT13", "Putative protein of unknown function" },
  { "YER066C-A", "NULL" },
  { "RGI1", "Protein of unknown function" },
  { "YER067C-A", "NULL" },
  { "MOT2", "Ubiquitin-protein ligase subunit of the CCR4-NOT complex" },
  { "YER068C-A", "NULL" },
  { "ARG5", "Acetylglutamate kinase and N-acetyl-gamma-glutamyl-phosphate reductase" },
  { "RNR1", "Major isoform of large subunit of ribonucleotide-diphosphate reductase" },
  { "TDA2", "Subunit of a complex that associates with actin filaments" },
  { "VTC1", "Regulatory subunit of the vacuolar transporter chaperone (VTC) complex" },
  { "ALD5", "Mitochondrial aldehyde dehydrogenase" },
  { "RPS24A", "Protein component of the small (40S) ribosomal subunit" },
  { "YOS1", "Integral membrane protein required for ER to Golgi transport" },
  { "PTP3", "Phosphotyrosine-specific protein phosphatase" },
  { "YER076C", "Putative protein of unknown function" },
  { "YER076W-A", "NULL" },
  { "YER077C", "NULL" },
  { "ICP55", "Mitochondrial aminopeptidase" },
  { "YER078W-A", "Putative protein of unknown function" },
  { "YER079W", "Putative protein of unknown function" },
  { "YER079C-A", "NULL" },
  { "AIM9", "Protein of unknown function" },
  { "SER3", "3-phosphoglycerate dehydrogenase and alpha-ketoglutarate reductase" },
  { "UTP7", "Nucleolar protein" },
  { "GET2", "Subunit of the GET complex" },
  { "YER084W", "Protein of unknown function" },
  { "YER084W-A", "NULL" },
  { "YER085C", "Putative protein of unknown function" },
  { "ILV1", "Threonine deaminase, catalyzes first step in isoleucine biosynthesis" },
  { "AIM10", "Protein with similarity to tRNA synthetases" },
  { "YER087C-A", "NULL" },
  { "SBH1", "Beta subunit of Sec61p ER translocation complex (Sec61p-Sss1p-Sbh1p)" },
  { "DOT6", "Protein involved in rRNA and ribosome biogenesis" },
  { "YER088C-A", "NULL" },
  { "YER088W-B", "NULL" },
  { "PTC2", "Type 2C protein phosphatase (PP2C)" },
  { "TRP2", "Anthranilate synthase" },
  { "YER090C-A", "NULL" },
  { "MET6", "Cobalamin-independent methionine synthase" },
  { "YER091C-A", "NULL" },
  { "IES5", "Non-essential INO80 chromatin remodeling complex subunit" },
  { "TSC11", "Subunit of TORC2 (Tor2p-Lst8p-Avo1-Avo2-Tsc11p-Bit61p)" },
  { "AIM11", "Protein of unknown function" },
  { "PUP3", "Beta 3 subunit of the 20S proteasome" },
  { "RAD51", "Strand exchange protein" },
  { "SHC1", "Sporulation-specific activator of Chs3p (chitin synthase III)" },
  { "YER097W", "NULL" },
  { "UBP9", "Ubiquitin-specific protease that cleaves ubiquitin-protein fusions" },
  { "PRS2", "5-phospho-ribosyl-1(alpha)-pyrophosphate synthetase" },
  { "UBC6", "Ubiquitin-conjugating enzyme involved in ERAD" },
  { "AST2", "Lipid raft associated protein" },
  { "RPS8B", "Protein component of the small (40S) ribosomal subunit" },
  { "SSA4", "Heat shock protein that is highly induced upon stress" },
  { "RTT105", "Protein with a role in regulation of Ty1 transposition" },
  { "NUP157", "Subunit of the inner ring of the nuclear pore complex (NPC)" },
  { "MAM1", "Monopolin" },
  { "GLE2", "RNA export factor associated with the nuclear pore complex (NPC)" },
  { "YER107W-A", "NULL" },
  { "FLO8", "Transcription factor" },
  { "KAP123", "Karyopherin beta" },
  { "SWI4", "DNA binding component of the SBF complex (Swi4p-Swi6p)" },
  { "LSM4", "Lsm (Like Sm) protein" },
  { "TMN3", "Protein with a role in cellular adhesion and filamentous growth" },
  { "BOI2", "Protein implicated in polar growth, functionally redundant with Boi1p" },
  { "SPR6", "Protein of unknown function" },
  { "SLX8", "Subunit of Slx5-Slx8 SUMO-targeted ubiquitin ligase (STUbL) complex" },
  { "RPL23B", "Ribosomal 60S subunit protein L23B" },
  { "SHO1", "Transmembrane osmosensor for filamentous growth and HOG pathways" },
  { "AVT6", "Vacuolar aspartate and glutamate exporter" },
  { "YER119C-A", "NULL" },
  { "SCS2", "Integral ER membrane protein, regulates phospholipid metabolism" },
  { "YER121W", "Putative protein of unknown function" },
  { "GLO3", "ADP-ribosylation factor GTPase activating protein (ARF GAP)" },
  { "YCK3", "Palmitoylated vacuolar membrane-localized casein kinase I isoform" },
  { "DSE1", "Daughter cell-specific protein" },
  { "RSP5", "NEDD4 family E3 ubiquitin ligase" },
  { "NSA2", "Protein constituent of 66S pre-ribosomal particles" },
  { "LCP5", "Essential protein involved in maturation of 18S rRNA" },
  { "VFA1", "Protein that interacts with Vps4p and has a role in vacuolar sorting" },
  { "SAK1", "Upstream serine/threonine kinase for the SNF1 complex" },
  { "COM2", "Transcription factor that binds IME1 Upstream Activation Signal (UAS)" },
  { "RPS26B", "Protein component of the small (40S) ribosomal subunit" },
  { "PMD1", "Protein with an N-terminal kelch-like domain" },
  { "GLC7", "Type 1 S/T protein phosphatase (PP1) catalytic subunit" },
  { "YER133W-A", "NULL" },
  { "YER134C", "Magnesium-dependent acid phosphatase" },
  { "YER135C", "Putative protein of unknown function" },
  { "GDI1", "GDP dissociation inhibitor" },
  { "YER137C", "Putative protein of unknown function" },
  { "YER137W-A", "NULL" },
  { "YER138W-A", "Putative protein of unknown function" },
  { "RTR1", "CTD phosphatase" },
  { "EMP65", "Integral membrane protein of the ER" },
  { "COX15", "Heme a synthase" },
  { "MAG1", "3-methyl-adenine DNA glycosylase" },
  { "DDI1", "DNA damage-inducible v-SNARE binding protein" },
  { "UBP5", "Putative ubiquitin-specific protease" },
  { "FTR1", "High affinity iron permease" },
  { "YER145C-A", "Putative protein of unknown function" },
  { "LSM5", "Lsm (Like Sm) protein" },
  { "SCC4", "Subunit of cohesin loading factor (Scc2p-Scc4p)" },
  { "YER147C-A", "NULL" },
  { "SPT15", "TATA-binding protein (TBP)" },
  { "YER148W-A", "NULL" },
  { "PEA2", "Coiled-coil 12S polarisome subunit" },
  { "SPI1", "GPI-anchored cell wall protein involved in weak acid resistance" },
  { "UBP3", "Ubiquitin-specific protease involved in transport and osmotic response" },
  { "YER152C", "Protein with 2-aminoadipate transaminase activity" },
  { "YER152W-A", "NULL" },
  { "PET122", "Mitochondrial translational activator specific for the COX3 mRNA" },
  { "OXA1", "Mitochondrial inner membrane insertase" },
  { "BEM2", "Rho GTPase activating protein (RhoGAP)" },
  { "YER156C", "Putative protein of unknown function" },
  { "COG3", "Essential component of the conserved oligomeric Golgi complex" },
  { "YER158C", "Protein of unknown function" },
  { "YER158W-A", "NULL" },
  { "BUR6", "Subunit of a heterodimeric NC2 transcription regulator complex" },
  { "SPT2", "Protein involved in negative regulation of transcription" },
  { "RAD4", "Protein that recognizes and binds damaged DNA (with Rad23p) during NER" },
  { "GCG1", "Gamma-glutamyl cyclotransferase" },
  { "CHD1", "Chromatin remodeler that regulates various aspects of transcription" },
  { "PAB1", "Poly(A) binding protein" },
  { "YER165C-A", "NULL" },
  { "DNF1", "Aminophospholipid translocase (flippase)" },
  { "BCK2", "Serine/threonine-rich protein involved in PKC1 signaling pathway" },
  { "CCA1", "ATP (CTP):tRNA-specific tRNA nucleotidyltransferase" },
  { "RPH1", "JmjC domain-containing histone demethylase" },
  { "ADK2", "Mitochondrial adenylate kinase" },
  { "RAD3", "5' to 3' DNA helicase" },
  { "BRR2", "RNA-dependent ATPase RNA helicase (DEIH box)" },
  { "YER172C-A", "NULL" },
  { "RAD24", "Checkpoint protein" },
  { "GRX4", "Glutathione-dependent oxidoreductase" },
  { "TMT1", "Trans-aconitate methyltransferase" },
  { "YER175W-A", "Putative protein of unknown function" },
  { "ECM32", "DNA dependent ATPase/DNA helicase" },
  { "BMH1", "14-3-3 protein, major isoform" },
  { "PDA1", "E1 alpha subunit of the pyruvate dehydrogenase (PDH) complex" },
  { "DMC1", "Meiosis-specific recombinase required for double-strand break repair" },
  { "ISC10", "Protein required for sporulation" },
  { "SLO1", "Protein interacting with Arl3p" },
  { "YER181C", "Mitochondrial protein of unknown function" },
  { "FMP10", "Putative protein of unknown function" },
  { "FAU1", "5,10-methenyltetrahydrofolate synthetase" },
  { "TOG1", "Transcriptional activator of oleate genes" },
  { "PUG1", "Plasma membrane protein involved in protoprophyrin and heme transport" },
  { "YER186C", "Putative protein of unknown function" },
  { "YER187W", "Putative protein of unknown function" },
  { "YER188W", "Putative protein of unknown function" },
  { "YER188C-A", "Putative protein of unknown function" },
  { "YER189W", "Putative protein of unknown function" },
  { "YRF1-2", "Helicase encoded by the Y' element of subtelomeric regions" },
  { "YER190C-A", "NULL" },
  { "YER190C-B", "Putative protein of unknown function" },
  { "YFL068W", "Putative protein of unknown function" },
  { "YFL067W", "Protein of unknown function" },
  { "YFL066C", "Helicase-like protein encoded within the telomeric Y' element" },
  { "YFL065C", "Putative protein of unknown function" },
  { "YFL064C", "Putative protein of unknown function" },
  { "YFL063W", "NULL" },
  { "COS4", "Endosomal protein involved in turnover of plasma membrane proteins" },
  { "DDI2", "Cyanamide hydratase that detoxifies cyanamide" },
  { "SNO3", "Protein of unknown function" },
  { "SNZ3", "Pyridoxal-5’-phosphate synthase" },
  { "THI5", "Protein involved in synthesis of the thiamine precursor HMP" },
  { "AAD16", "Putative aryl alcohol dehydrogenase" },
  { "AAD6", "Putative aryl-alcohol dehydrogenase" },
  { "AGP3", "Low-affinity amino acid permease" },
  { "YFL054C", "NULL" },
  { "DAK2", "Dihydroxyacetone kinase" },
  { "YFL052W", "NULL" },
  { "YFL051C", "Putative protein of unknown function" },
  { "ALR2", "Probable Mg(2+) transporter" },
  { "SWP82", "Member of the SWI/SNF chromatin remodeling complex" },
  { "EMP47", "Integral membrane component of ER-derived COPII-coated vesicles" },
  { "RGD2", "GTPase-activating protein (RhoGAP) for Cdc42p and Rho5p" },
  { "FMP32", "Putative assembly factor for cytochrome c oxidase" },
  { "SEC53", "Phosphomannomutase" },
  { "OTU1", "Deubiquitylation enzyme that binds to the chaperone-ATPase Cdc48p" },
  { "YFL042C", "NULL" },
  { "YFL041W-A", "Putative protein of unknown function" },
  { "FET5", "Multicopper oxidase" },
  { "YFL040W", "Putative transporter" },
  { "ACT1", "Actin" },
  { "YPT1", "Rab family GTPase" },
  { "TUB2", "Beta-tubulin" },
  { "RPO41", "Mitochondrial RNA polymerase" },
  { "MOB2", "Activator of Cbk1p kinase" },
  { "RPL22B", "Ribosomal 60S subunit protein L22A" },
  { "YFL034W", "NULL" },
  { "RIM15", "Protein kinase involved in cell proliferation in response to nutrients" },
  { "YFL032W", "NULL" },
  { "HAC1", "Basic leucine zipper (bZIP) transcription factor (ATF/CREB1 homolog)" },
  { "YFL031C-A", "NULL" },
  { "AGX1", "Alanine:glyoxylate aminotransferase (AGT)" },
  { "CAK1", "Cyclin-dependent kinase-activating kinase" },
  { "CAF16", "Part of evolutionarily-conserved CCR4-NOT regulatory complex" },
  { "GYP8", "GTPase-activating protein for yeast Rab family members" },
  { "STE2", "Receptor for alpha-factor pheromone" },
  { "BST1", "GPI inositol deacylase of the endoplasmic reticulum (ER)" },
  { "EPL1", "Subunit of NuA4, an essential histone H4/H2A acetyltransferase complex" },
  { "BUD27", "Unconventional prefoldin protein involved in translation initiation" },
  { "FRS2", "Alpha subunit of cytoplasmic phenylalanyl-tRNA synthetase" },
  { "YFL021C-A", "NULL" },
  { "GAT1", "Transcriptional activator of nitrogen catabolite repression genes" },
  { "PAU5", "Member of the seripauperin multigene family" },
  { "YFL019C", "Putative protein of unknown function" },
  { "LPD1", "Dihydrolipoamide dehydrogenase" },
  { "SMX2", "Core Sm protein Sm G" },
  { "GNA1", "Glucosamine-6-phosphate acetyltransferase" },
  { "MDJ1", "Co-chaperone that stimulates HSP70 protein Ssc1p ATPase activity" },
  { "YFL015W-A", "NULL" },
  { "YFL015C", "Putative protein of unknown function" },
  { "HSP12", "Plasma membrane protein involved in maintaining membrane organization" },
  { "YFL013W-A", "NULL" },
  { "IES1", "Subunit of the INO80 chromatin remodeling complex" },
  { "YFL012W-A", "NULL" },
  { "YFL012W", "Putative protein of unknown function" },
  { "HXT10", "Putative hexose transporter" },
  { "AUA1", "Protein required for the negative regulation by ammonia of Gap1p" },
  { "WWM1", "WW domain containing protein of unknown function" },
  { "CDC4", "F-box protein required for both the G1/S and G2/M phase transitions" },
  { "SMC1", "Subunit of the multiprotein cohesin complex" },
  { "BLM10", "Proteasome activator" },
  { "SEC4", "Rab family GTPase" },
  { "VTC2", "Regulatory subunit of the vacuolar transporter chaperone (VTC) complex" },
  { "MSH4", "Protein involved in meiotic recombination" },
  { "SPB4", "Putative ATP-dependent RNA helicase" },
  { "DEG1", "tRNA:pseudouridine synthase" },
  { "LOC1", "Nucleolar protein involved in the asymmetric localization of ASH1 mRNA" },
  { "NIC96", "Linker nucleoporin component of the nuclear pore complex (NPC)" },
  { "YPI1", "Regulatory subunit of the type I protein phosphatase (PP1) Glc7p" },
  { "RPN11", "Metalloprotease subunit of 19S regulatory particle" },
  { "SAD1", "Conserved zinc-finger domain protein involved in pre-mRNA splicing" },
  { "YFR006W", "Putative X-Pro aminopeptidase" },
  { "YFH7", "Putative kinase with similarity to the PRK/URK/PANK kinase subfamily" },
  { "FAR7", "Protein involved in recovery from pheromone-induced cell cycle arrest" },
  { "GCN20", "Positive regulator of the Gcn2p kinase activity" },
  { "YFR009W-A", "NULL" },
  { "UBP6", "Ubiquitin-specific protease" },
  { "YFR010W-A", "NULL" },
  { "MIC19", "Component of the MICOS complex" },
  { "DCV1", "Protein of unknown function" },
  { "YFR012W-A", "Putative protein of unknown function" },
  { "IOC3", "Subunit of the Isw1a complex" },
  { "CMK1", "Calmodulin-dependent protein kinase" },
  { "GSY1", "Glycogen synthase" },
  { "YFR016C", "NULL" },
  { "IGD1", "Cytoplasmic protein that inhibits Gdb1p glycogen debranching activity" },
  { "YFR018C", "Putative protein of unknown function" },
  { "FAB1", "1-phosphatidylinositol-3-phosphate 5-kinase" },
  { "YFR020W", "NULL" },
  { "ATG18", "Phosphoinositide binding protein" },
  { "ROG3", "Alpha-arrestin involved in ubiquitin-dependent endocytosis" },
  { "PES4", "Poly(A) binding protein, suppressor of DNA polymerase epsilon mutation" },
  { "LSB3", "Protein containing a C-terminal SH3 domain" },
  { "HIS2", "Histidinolphosphatase" },
  { "ULI1", "Protein of unknown function" },
  { "ECO1", "Acetyltransferase" },
  { "CDC14", "Protein phosphatase required for mitotic exit" },
  { "PTR3", "Component of the SPS plasma membrane amino acid sensor system" },
  { "MET10", "Subunit alpha of assimilatory sulfite reductase" },
  { "SMC2", "Subunit of the condensin complex" },
  { "RPL2A", "Ribosomal 60S subunit protein L2A" },
  { "RRT5", "Putative protein of unknown function" },
  { "RPL29", "Ribosomal 60S subunit protein L29" },
  { "YFR032C-B", "NULL" },
  { "QCR6", "Subunit 6 of the ubiquinol cytochrome-c reductase complex" },
  { "PHO4", "Basic helix-loop-helix (bHLH) transcription factor of the myc-family" },
  { "YFR034W-A", "NULL" },
  { "YFR035C", "Putative protein of unknown function" },
  { "CDC26", "Subunit of the Anaphase-Promoting Complex/Cyclosome (APC/C)" },
  { "YFR036W-A", "NULL" },
  { "RSC8", "Component of the RSC chromatin remodeling complex" },
  { "IRC5", "Putative ATPase containing the DEAD/H helicase-related sequence motif" },
  { "OSW7", "Protein involved in outer spore wall assembly" },
  { "SAP155", "Protein required for function of the Sit4p protein phosphatase" },
  { "ERJ5", "Type I membrane protein with a J domain" },
  { "KEG1", "Integral membrane protein of the ER" },
  { "IRC6", "Clathrin coat accessory factor" },
  { "DUG1", "Cys-Gly metallo-di-peptidase" },
  { "YFR045W", "Putative mitochondrial transport protein" },
  { "CNN1", "Kinetochore protein" },
  { "BNA6", "Quinolinate phosphoribosyl transferase" },
  { "RMD8", "Cytosolic protein required for sporulation" },
  { "YMR31", "Subunit of the mitochondrial alpha-ketoglutarate dehydrogenase" },
  { "PRE4", "Beta 7 subunit of the 20S proteasome" },
  { "RET2", "Delta subunit of the coatomer complex (COPI)" },
  { "RPN12", "Subunit of the 19S regulatory particle of the 26S proteasome lid" },
  { "YFR052C-A", "NULL" },
  { "HXK1", "Hexokinase isoenzyme 1" },
  { "YFR054C", "Putative protein of unknown function" },
  { "YFR056C", "NULL" },
  { "IRC7", "Beta-lyase involved in the production of thiols" },
  { "YFR057W", "Putative protein of unknown function" },
  { "COS12", "Endosomal protein involved in turnover of plasma membrane proteins" },
  { "YGL262W", "Putative protein of unknown function" },
  { "PAU11", "Putative protein of unknown function" },
  { "YGL260W", "Putative protein of unknown function" },
  { "YPS5", "Protein with similarity to GPI-anchored aspartic proteases" },
  { "YGL258W-A", "Putative protein of unknown function" },
  { "VEL1", "Protein of unknown function" },
  { "MNT2", "Mannosyltransferase" },
  { "ADH4", "Alcohol dehydrogenase isoenzyme type IV" },
  { "ZRT1", "High-affinity zinc transporter of the plasma membrane" },
  { "FZF1", "Transcription factor involved in sulfite metabolism" },
  { "HXK2", "Hexokinase isoenzyme 2" },
  { "RTG2", "Sensor of mitochondrial dysfunction" },
  { "HFM1", "Meiosis specific DNA helicase" },
  { "RMR1", "Protein required for meiotic recombination and gene conversion" },
  { "ZIP2", "Meiosis-specific protein" },
  { "PDE1", "Low-affinity cyclic AMP phosphodiesterase" },
  { "BRR6", "Essential nuclear envelope integral membrane protein" },
  { "RAI1", "Nuclear protein with decapping endonuclease activity" },
  { "GUS1", "Glutamyl-tRNA synthetase (GluRS)" },
  { "RTF1", "Subunit of RNAPII-associated chromatin remodeling Paf1 complex" },
  { "TAD1", "tRNA-specific adenosine deaminase" },
  { "YGL242C", "Protein of unknown function" },
  { "KAP114", "Karyopherin, responsible for nuclear import of specific proteins" },
  { "DOC1", "Processivity factor" },
  { "YGL239C", "NULL" },
  { "CSE1", "Nuclear envelope protein that acts as a recycling factor" },
  { "HAP2", "Subunit of the Hap2p/3p/4p/5p CCAAT-binding complex" },
  { "MTO1", "Mitochondrial protein" },
  { "YGL235W", "Putative protein of unknown function" },
  { "ADE5", "Enzyme of the 'de novo' purine nucleotide biosynthetic pathway" },
  { "SEC15", "Essential 113 kDa subunit of the exocyst complex" },
  { "TAN1", "Putative tRNA acetyltransferase" },
  { "EMC4", "Member of conserved ER transmembrane complex" },
  { "YGL230C", "Putative protein of unknown function" },
  { "SAP4", "Protein required for function of the Sit4p protein phosphatase" },
  { "SHE10", "Protein involved in outer spore wall assembly" },
  { "VID30", "Central component of GID Complex, involved in FBPase degradation" },
  { "OST5", "Zeta subunit of the oligosaccharyltransferase complex of the ER lumen" },
  { "MTC3", "Protein of unknown function" },
  { "VRG4", "Golgi GDP-mannose transporter" },
  { "SDT1", "Pyrimidine nucleotidase" },
  { "COG1", "Essential component of the conserved oligomeric Golgi complex" },
  { "EDC1", "RNA-binding protein that activates mRNA decapping directly" },
  { "NIF3", "Protein of unknown function" },
  { "FRA2", "NULL" },
  { "MDM34", "Mitochondrial component of the ERMES complex" },
  { "YGL218W", "NULL" },
  { "YGL217C", "NULL" },
  { "KIP3", "Kinesin-related antiparallel sliding motor protein" },
  { "CLG1", "Cyclin-like protein that interacts with Pho85p" },
  { "YGL214W", "NULL" },
  { "SKI8", "Ski complex component and WD-repeat protein" },
  { "VAM7", "Vacuolar SNARE protein" },
  { "NCS6", "Protein required for uridine thiolation of Gln, Lys, and Glu tRNAs" },
  { "YPT32", "Rab family GTPase involved in the exocytic pathway" },
  { "MIG2", "Zinc finger transcriptional repressor" },
  { "SIP2", "One of three beta subunits of the Snf1 kinase complex" },
  { "SPT16", "Subunit of the heterodimeric FACT complex (Spt16p-Pob3p)" },
  { "CHC1", "Clathrin heavy chain" },
  { "POX1", "Fatty-acyl coenzyme A oxidase" },
  { "YGL204C", "Protein of unknown function" },
  { "KEX1", "Cell death protease essential for hypochlorite-induced apoptosis" },
  { "ARO8", "Aromatic aminotransferase I" },
  { "MCM6", "Protein involved in DNA replication" },
  { "EMP24", "Component of the p24 complex" },
  { "YGL199C", "NULL" },
  { "YIP4", "Protein that interacts with Rab GTPases" },
  { "MDS3", "Putative component of the TOR regulatory pathway" },
  { "DSD1", "D-serine dehydratase (aka D-serine ammonia-lyase)" },
  { "GCN1", "Positive regulator of the Gcn2p kinase activity" },
  { "YGL194C-A", "Putative protein of unknown function" },
  { "HOS2", "Histone deacetylase and subunit of Set3 and Rpd3L complexes" },
  { "YGL193C", "Haploid-specific gene repressed by a1-alpha2" },
  { "IME4", "mRNA N6-adenosine methyltransferase required for entry into meiosis" },
  { "COX13", "Subunit VIa of cytochrome c oxidase" },
  { "CDC55", "Regulatory subunit B of protein phosphatase 2A (PP2A)" },
  { "RPS26A", "Protein component of the small (40S) ribosomal subunit" },
  { "YGL188C-A", "Putative protein of unknown function" },
  { "YGL188C", "NULL" },
  { "COX4", "Subunit IV of cytochrome c oxidase" },
  { "TPN1", "Plasma membrane pyridoxine (vitamin B6) transporter" },
  { "YGL185C", "Putative protein with sequence similar to hydroxyacid dehydrogenases" },
  { "STR3", "Peroxisomal cystathionine beta-lyase" },
  { "MND1", "Protein required for recombination and meiotic nuclear division" },
  { "YGL182C", "NULL" },
  { "GTS1", "Protein involved in Arf3p regulation and in transcription regulation" },
  { "ATG1", "Protein serine/threonine kinase" },
  { "TOS3", "Protein kinase" },
  { "MPT5", "mRNA-binding protein of the PUF family" },
  { "YGL177W", "NULL" },
  { "YGL176C", "Putative protein of unknown function" },
  { "SAE2", "Endonuclease required for telomere elongation" },
  { "BUD13", "Subunit of the RES complex" },
  { "XRN1", "Evolutionarily-conserved 5'-3' exonuclease" },
  { "NUP49", "FG-nucleoporin component of central core of the nuclear pore complex" },
  { "ROK1", "RNA-dependent ATPase" },
  { "SPO74", "Component of the meiotic outer plaque of the spindle pole body" },
  { "SUA5", "Protein involved in threonylcarbamoyl adenosine biosynthesis" },
  { "HUR1", "Protein of unknown function" },
  { "PMR1", "High affinity Ca2+/Mn2+ P-type ATPase" },
  { "CUP2", "Copper-binding transcription factor" },
  { "YGL165C", "NULL" },
  { "YRB30", "RanGTP-binding protein" },
  { "RAD54", "DNA-dependent ATPase that stimulates strand exchange" },
  { "SUT1", "Zn(II)2Cys6 family transcription factor" },
  { "YIP5", "Protein that interacts with Rab GTPases" },
  { "AIM14", "NADPH oxidase localized to the perinuclear ER" },
  { "YGL159W", "Putative protein of unknown function" },
  { "RCK1", "Protein kinase involved in oxidative stress response" },
  { "ARI1", "NADPH-dependent aldehyde reductase" },
  { "AMS1", "Vacuolar alpha mannosidase" },
  { "CDC43", "Beta subunit of geranylgeranyltransferase type I" },
  { "LYS5", "Phosphopantetheinyl transferase involved in lysine biosynthesis" },
  { "PEX14", "Central component of the peroxisomal importomer complex" },
  { "YGL152C", "NULL" },
  { "NUT1", "Component of the RNA polymerase II mediator complex" },
  { "INO80", "ATPase and nucleosome spacing factor" },
  { "YGL149W", "Putative protein of unknown function" },
  { "ARO2", "Bifunctional chorismate synthase and flavin reductase" },
  { "RPL9A", "Ribosomal 60S subunit protein L9A" },
  { "RRT6", "Putative protein of unknown function" },
  { "TIP20", "Peripheral membrane protein required for COPI vesicle fusion to the ER" },
  { "ROG1", "Lipase with specificity for monoacylglycerol" },
  { "MRF1", "Mitochondrial translation release factor" },
  { "GPI10", "Integral membrane protein involved in GPI anchor synthesis" },
  { "HUL5", "Multiubiquitin chain assembly factor (E4)" },
  { "YGL140C", "Putative protein of unknown function" },
  { "FLC3", "Putative FAD transporter, similar to Flc1p and Flc2p" },
  { "YGL138C", "Putative protein of unknown function" },
  { "SEC27", "Essential beta'-coat protein of the COPI coatomer" },
  { "MRM2", "Mitochondrial 2' O-ribose methyltransferase" },
  { "RPL1B", "Ribosomal 60S subunit protein L1B" },
  { "PCL10", "Pho85p cyclin" },
  { "ITC1", "Subunit of ATP-dependent Isw2p-Itc1p chromatin remodeling complex" },
  { "YGL132W", "NULL" },
  { "SNT2", "Subunit of Snt2C complex, RING finger ubiquitin ligase (E3)" },
  { "CEG1", "Guanylyltransferase involved in mRNA 5' capping" },
  { "RSM23", "Mitochondrial ribosomal protein of the small subunit" },
  { "CWC23", "Component of a complex containing Cef1p" },
  { "SOH1", "Subunit of the RNA polymerase II mediator complex" },
  { "SCS3", "Protein required for inositol prototrophy" },
  { "MET13", "Major isozyme of methylenetetrahydrofolate reductase" },
  { "MON1", "Subunit of a heterodimeric guanine nucleotide exchange factor (GEF)" },
  { "RPS2", "Protein component of the small (40S) subunit" },
  { "YGL123C-A", "NULL" },
  { "NAB2", "Nuclear polyadenylated RNA-binding protein" },
  { "GPG1", "Proposed gamma subunit of the heterotrimeric G protein" },
  { "PRP43", "RNA helicase in the DEAH-box family" },
  { "COQ8", "ATPase required for ubiquinone biosynthesis and respiratory growth" },
  { "YGL118C", "Putative protein of unknown function" },
  { "YGL117W", "Putative protein of unknown function" },
  { "CDC20", "Activator of anaphase-promoting complex/cyclosome (APC/C)" },
  { "SNF4", "Activating gamma subunit of the AMP-activated Snf1p kinase complex" },
  { "YGL114W", "Putative protein of unknown function" },
  { "SLD3", "Protein involved in the initiation of DNA replication" },
  { "TAF6", "Subunit (60 kDa) of TFIID and SAGA complexes" },
  { "NSA1", "Constituent of 66S pre-ribosomal particles" },
  { "CUE3", "Subunit of ribosome-associated quality control trigger complex (RQT)" },
  { "YGL109W", "NULL" },
  { "YGL108C", "Protein of unknown function, predicted to be palmitoylated" },
  { "RMD9", "Mitochondrial protein required for respiratory growth" },
  { "MLC1", "Essential light chain for Myo1p" },
  { "ARC1", "Protein that binds tRNA and methionyl- and glutamyl-tRNA synthetases" },
  { "VPS73", "Mitochondrial protein" },
  { "RPL28", "Ribosomal 60S subunit protein L28" },
  { "YGL102C", "NULL" },
  { "YGL101W", "NULL" },
  { "SEH1", "Subunit of the Nup84 nuclear pore and SEACAT subcomplexes" },
  { "LSG1", "Putative GTPase involved in 60S ribosomal subunit biogenesis" },
  { "USE1", "Essential SNARE protein localized to the ER" },
  { "SRM1", "Nucleotide exchange factor for Gsp1p" },
  { "TOS8", "Homeodomain-containing protein and putative transcription factor" },
  { "VPS45", "Protein of the Sec1p/Munc-18 family" },
  { "PAN2", "Catalytic subunit of the Pan2p-Pan3p poly(A)-ribonuclease complex" },
  { "SPC105", "Subunit of a kinetochore-microtubule binding complex" },
  { "NUP145", "Essential protein with distinct roles in two nuclear pore subcomplexes" },
  { "NBP35", "Essential cytoplasmic iron-sulfur cluster binding protein" },
  { "LIF1", "Component of the DNA ligase IV complex" },
  { "MF", "NULL" },
  { "YGL088W", "Putative protein of unknown function" },
  { "MMS2", "Ubiquitin-conjugating enzyme variant" },
  { "MAD1", "Coiled-coil protein involved in spindle-assembly checkpoint" },
  { "LCL3", "Putative protein of unknown function" },
  { "GUP1", "Plasma membrane protein involved in remodeling GPI anchors" },
  { "SCY1", "Putative kinase" },
  { "YGL082W", "Putative protein of unknown function" },
  { "YGL081W", "Putative protein of unknown function" },
  { "FMP37", "NULL" },
  { "KXD1", "Subunit of the BLOC-1 complex involved in endosomal maturation" },
  { "DBP3", "RNA-Dependent ATPase, member of DExD/H-box family" },
  { "HNM1", "Plasma membrane transporter for choline, ethanolamine, and carnitine" },
  { "RPL7A", "Ribosomal 60S subunit protein L7A" },
  { "MPS2", "Essential membrane protein localized at nuclear envelope and SPBs" },
  { "YGL074C", "NULL" },
  { "HSF1", "Trimeric heat shock transcription factor" },
  { "YGL072C", "NULL" },
  { "AFT1", "Transcription factor involved in iron utilization and homeostasis" },
  { "RPB9", "RNA polymerase II subunit B12.6" },
  { "YGL069C", "NULL" },
  { "MNP1", "Mitochondrial ribosomal protein of the large subunit" },
  { "NPY1", "NADH diphosphatase (pyrophosphatase)" },
  { "SGF73", "Subunit of DUBm module of SAGA and SLIK" },
  { "ALG2", "Mannosyltransferase in the N-linked glycosylation pathway" },
  { "MRH4", "Mitochondrial ATP-dependent RNA helicase of the DEAD-box family" },
  { "PUS2", "Mitochondrial tRNA:pseudouridine synthase" },
  { "YGL063C-A", "NULL" },
  { "PYC1", "Pyruvate carboxylase isoform" },
  { "DUO1", "Essential subunit of the Dam1 complex (aka DASH complex)" },
  { "YBP2", "Central kinetochore associated protein" },
  { "PKP2", "Mitochondrial protein kinase" },
  { "RAD6", "Ubiquitin-conjugating enzyme (E2)" },
  { "GEP7", "Protein of unknown function" },
  { "SDS23", "Protein involved in cell separation during budding" },
  { "OLE1", "Delta(9) fatty acid desaturase" },
  { "ERV14", "COPII-coated vesicle protein" },
  { "PRM8", "Pheromone-regulated protein" },
  { "YGL052W", "NULL" },
  { "MST27", "Putative integral membrane protein, involved in vesicle formation" },
  { "TYW3", "tRNA methyltransferase required for synthesis of wybutosine" },
  { "TIF4632", "Translation initiation factor eIF4G" },
  { "RPT6", "ATPase of the 19S regulatory particle of the 26S proteasome" },
  { "ALG13", "Catalytic component of UDP-GlcNAc transferase" },
  { "RIM8", "Protein involved in proteolytic activation of Rim101p" },
  { "RNA15", "Component of the cleavage and polyadenylation factor I (CF I)" },
  { "DST1", "General transcription elongation factor TFIIS" },
  { "YGL042C", "NULL" },
  { "YGL041C-B", "Putative protein of unknown function" },
  { "YGL041W-A", "NULL" },
  { "YGL041C", "NULL" },
  { "HEM2", "Aminolevulinate dehydratase" },
  { "YGL039W", "Aldehyde reductase" },
  { "OCH1", "Mannosyltransferase of the cis-Golgi apparatus" },
  { "PNC1", "Nicotinamidase that converts nicotinamide to nicotinic acid" },
  { "YGL036W", "Putative protein of unknown function" },
  { "MIG1", "Transcription factor involved in glucose repression" },
  { "YGL034C", "Putative protein of unknown function" },
  { "HOP2", "Meiosis-specific protein that localizes to chromosomes" },
  { "AGA2", "Adhesion subunit of a-agglutinin of a-cells" },
  { "RPL24A", "Ribosomal 60S subunit protein L24A" },
  { "RPL30", "Ribosomal 60S subunit protein L30" },
  { "CGR1", "Protein involved in nucleolar integrity and processing of pre-rRNA" },
  { "SCW11", "Cell wall protein with similarity to glucanases" },
  { "CWH41", "Processing alpha glucosidase I" },
  { "TRP5", "Tryptophan synthase" },
  { "PGD1", "Subunit of the RNA polymerase II mediator complex" },
  { "YGL024W", "NULL" },
  { "PIB2", "Protein of unknown function" },
  { "STT3", "Subunit of the oligosaccharyltransferase complex of the ER lumen" },
  { "ALK1", "Protein kinase" },
  { "GET1", "Subunit of the GET complex" },
  { "CKB1", "Beta regulatory subunit of casein kinase 2 (CK2)" },
  { "JAC1", "Specialized J-protein that functions in Fe-S cluster biogenesis" },
  { "ATE1", "Arginyl-tRNA-protein transferase" },
  { "KAP122", "Karyopherin beta" },
  { "YGL015C", "Putative protein of unknown function" },
  { "PUF4", "Member of the PUF protein family" },
  { "YGL014C-A", "NULL" },
  { "PDR1", "Transcription factor that regulates the pleiotropic drug response" },
  { "ERG4", "C-24(28) sterol reductase" },
  { "SCL1", "Alpha 1 subunit of the 20S proteasome" },
  { "YGL010W", "NULL" },
  { "LEU1", "Isopropylmalate isomerase" },
  { "PMA1", "Plasma membrane P2-type H+-ATPase" },
  { "YGL007C-A", "Putative protein of unknown function" },
  { "BRP1", "Putative protein of unknown function" },
  { "YGL006W-A", "Putative protein of unknown function" },
  { "PMC1", "Vacuolar Ca2+ ATPase involved in depleting cytosol of Ca2+ ions" },
  { "COG7", "Component of the conserved oligomeric Golgi complex" },
  { "RPN14", "Evolutionarily conserved 19S regulatory particle assembly-chaperone" },
  { "CDH1", "Activator of anaphase-promoting complex/cyclosome (APC/C)" },
  { "ERP6", "Member of the p24 family involved in ER to Golgi transport" },
  { "ERG26", "C-3 sterol dehydrogenase" },
  { "AML1", "NULL" },
  { "SWC4", "Component of the Swr1p complex that incorporates Htz1p into chromatin" },
  { "CUL3", "Ubiquitin-protein ligase" },
  { "PEX31", "Peroxisomal integral membrane protein" },
  { "TFG2", "TFIIF (Transcription Factor II) middle subunit" },
  { "PRP18", "Splicing factor and component of snRNP U5" },
  { "ECT1", "Ethanolamine-phosphate cytidylyltransferase" },
  { "STF2", "Protein involved in resistance to desiccation stress" },
  { "SEC9", "t-SNARE protein required for secretory vesicle-plasma membrane fusion" },
  { "NMA2", "Nicotinic acid mononucleotide adenylyltransferase" },
  { "YGR011W", "NULL" },
  { "YGR012W", "NULL" },
  { "SNU71", "Component of U1 snRNP required for mRNA splicing via spliceosome" },
  { "MSB2", "Mucin family member involved in various signaling pathways" },
  { "YGR015C", "NULL" },
  { "YGR016W", "Putative protein of unknown function" },
  { "YGR017W", "Putative protein of unknown function" },
  { "YGR018C", "Protein of unknown function" },
  { "UGA1", "Gamma-aminobutyrate (GABA) transaminase" },
  { "VMA7", "Subunit F of the V1 peripheral membrane domain of V-ATPase" },
  { "YGR021W", "NULL" },
  { "YGR022C", "NULL" },
  { "MTL1", "Putative plasma membrane sensor" },
  { "THG1", "tRNAHis guanylyltransferase" },
  { "YGR025W", "Putative protein of unknown function" },
  { "YGR026W", "Putative protein of unknown function" },
  { "RPS25A", "Protein component of the small (40S) ribosomal subunit" },
  { "MSP1", "Highly-conserved N-terminally anchored AAA-ATPase" },
  { "ERV1", "Flavin-linked sulfhydryl oxidase of the mitochondrial IMS" },
  { "POP6", "Subunit of RNase MRP, nuclear RNase P and telomerase" },
  { "IMO32", "Conserved mitochondrial protein of unknown function" },
  { "NAG1", "Protein involved in yeast cell wall biogenesis" },
  { "GSC2", "Catalytic subunit of 1,3-beta-glucan synthase" },
  { "TIM21", "Nonessential component of the TIM23 complex" },
  { "RPL26B", "Ribosomal 60S subunit protein L26B" },
  { "YGR035C", "Putative protein of unknown function, potential Cdc28p substrate" },
  { "YGR035W-A", "Putative protein of unknown function" },
  { "CAX4", "Dolichyl pyrophosphate (Dol-P-P) phosphatase" },
  { "ACB1", "Acyl-CoA-binding protein" },
  { "ORM1", "Protein that mediates sphingolipid homeostasis" },
  { "YGR039W", "Putative protein of unknown function" },
  { "KSS1", "Mitogen-activated protein kinase (MAPK)" },
  { "BUD9", "Protein involved in bud-site selection" },
  { "YGR042W", "NULL" },
  { "NQM1", "Transaldolase of unknown function" },
  { "RME1", "Zinc finger protein involved in control of meiosis" },
  { "YGR045C", "Putative protein of unknown function" },
  { "TAM41", "Mitochondrial phosphatidate cytidylyltransferase (CDP-DAG synthase)" },
  { "TFC4", "Subunit of RNA polymerase III transcription initiation factor complex" },
  { "UFD1", "Substrate-recruiting cofactor of the Cdc48p-Npl4p-Ufd1p segregase" },
  { "SCM4", "Mitochondrial outer membrane protein of unknown function" },
  { "YGR050C", "Protein of unknown function" },
  { "YGR051C", "NULL" },
  { "FMP48", "Putative protein of unknown function" },
  { "YGR053C", "NULL" },
  { "YGR054W", "Eukaryotic initiation factor eIF2A" },
  { "MUP1", "High affinity methionine permease" },
  { "RSC1", "Component of the RSC chromatin remodeling complex" },
  { "LST7", "Subunit of the Lst4p-Lst7p GTPase activating protein complex for Gtr2p" },
  { "PEF1", "Penta-EF-hand protein" },
  { "SPR3", "Sporulation-specific homolog of the CDC3/10/11/12 family of genes" },
  { "ERG25", "C-4 methyl sterol oxidase" },
  { "ADE6", "Formylglycinamidine-ribonucleotide (FGAM)-synthetase" },
  { "COX18", "Protein required for membrane insertion of C-terminus of Cox2p" },
  { "SPT4", "Spt4p/5p (DSIF) transcription elongation factor complex subunit" },
  { "YGR064W", "NULL" },
  { "VHT1", "High-affinity plasma membrane H+-biotin (vitamin H) symporter" },
  { "YGR066C", "NULL" },
  { "YGR067C", "Putative protein of unknown function" },
  { "ART5", "Protein proposed to regulate endocytosis of plasma membrane proteins" },
  { "YGR068W-A", "NULL" },
  { "YGR069W", "NULL" },
  { "ROM1", "Guanine nucleotide exchange factor (GEF) for Rho1p" },
  { "ENV11", "Protein proposed to be involved in vacuolar functions" },
  { "UPF3", "Component of the nonsense-mediated mRNA decay (NMD) pathway" },
  { "YGR073C", "NULL" },
  { "SMD1", "Core Sm protein Sm D1" },
  { "PRP38", "Unique component of the U4/U6.U5 tri-snRNP particle" },
  { "MRPL25", "Mitochondrial ribosomal protein of the large subunit" },
  { "PEX8", "Intraperoxisomal organizer of the peroxisomal import machinery" },
  { "PAC10", "Part of the heteromeric co-chaperone GimC/prefoldin complex" },
  { "YGR079W", "Putative protein of unknown function" },
  { "TWF1", "Twinfilin" },
  { "SLX9", "Protein required for pre-rRNA processing" },
  { "TOM20", "Component of the TOM (translocase of outer membrane) complex" },
  { "GCD2", "Delta subunit of the translation initiation factor eIF2B" },
  { "MRP13", "Mitochondrial ribosomal protein of the small subunit" },
  { "RPL11B", "Ribosomal 60S subunit protein L11B" },
  { "PIL1", "Eisosome core component" },
  { "PDC6", "Minor isoform of pyruvate decarboxylase" },
  { "CTT1", "Cytosolic catalase T" },
  { "NNF2", "Protein that exhibits physical and genetic interactions with Rpb8p" },
  { "UTP22", "Component of the small-subunit processome" },
  { "PRP31", "Splicing factor" },
  { "DBF2", "Ser/Thr kinase involved in transcription and stress response" },
  { "DRN1", "Splicing factor that modulates turnover of branched RNAs by Dbr1p" },
  { "VAS1", "Mitochondrial and cytoplasmic valyl-tRNA synthetase" },
  { "RRP46", "Exosome non-catalytic core component" },
  { "TPC1", "Mitochondrial membrane transporter" },
  { "ASK10", "Regulator of the Fps1p glycerol channel" },
  { "ESP1", "Separase/separin, a caspase-like cysteine protease" },
  { "TEL2", "Subunit of the ASTRA complex, involved in chromatin remodeling" },
  { "MDR1", "Cytoplasmic GTPase-activating protein" },
  { "PCP1", "Mitochondrial serine protease" },
  { "GTF1", "Subunit of the trimeric GatFAB AmidoTransferase(AdT) complex" },
  { "NOP7", "Component of several different pre-ribosomal particles" },
  { "SRB5", "Subunit of the RNA polymerase II mediator complex" },
  { "VMA21", "Integral membrane protein required for V-ATPase function" },
  { "VOA1", "ER protein that functions in assembly of the V0 sector of V-ATPase" },
  { "YGR107W", "NULL" },
  { "CLB1", "B-type cyclin involved in cell cycle progression" },
  { "CLB6", "B-type cyclin involved in DNA replication during S phase" },
  { "CLD1", "Mitochondrial cardiolipin-specific phospholipase" },
  { "YGR111W", "Putative protein of unknown function" },
  { "SHY1", "Mitochondrial inner membrane protein required for complex IV assembly" },
  { "DAM1", "Essential subunit of the Dam1 complex (aka DASH complex)" },
  { "YGR114C", "NULL" },
  { "YGR115C", "NULL" },
  { "SPT6", "Nucleosome remodeling protein" },
  { "YGR117C", "Putative protein of unknown function" },
  { "RPS23A", "Ribosomal protein 28 (rp28) of the small (40S) ribosomal subunit" },
  { "NUP57", "FG-nucleoporin component of central core of the nuclear pore complex" },
  { "COG2", "Essential component of the conserved oligomeric Golgi complex" },
  { "MEP1", "Ammonium permease" },
  { "YGR121W-A", "Putative protein of unknown function" },
  { "YGR122W", "Protein that may be involved in pH regulation" },
  { "YGR122C-A", "NULL" },
  { "PPT1", "Protein serine/threonine phosphatase" },
  { "ASN2", "Asparagine synthetase" },
  { "YGR125W", "Putative protein of unknown function" },
  { "YGR126W", "Putative protein of unknown function" },
  { "YGR127W", "Putative protein of unknown function" },
  { "UTP8", "Nucleolar protein required for export of tRNAs from the nucleus" },
  { "SYF2", "Member of the NineTeen Complex (NTC)" },
  { "YGR130C", "Component of the eisosome with unknown function" },
  { "FHN1", "Protein of unknown function" },
  { "PHB1", "Subunit of the prohibitin complex (Phb1p-Phb2p)" },
  { "PEX4", "Peroxisomal ubiquitin conjugating enzyme" },
  { "CAF130", "Subunit of the CCR4-NOT transcriptional regulatory complex" },
  { "PRE9", "Alpha 3 subunit of the 20S proteasome" },
  { "LSB1", "Negative regulator of actin nucleation-promoting factor activity" },
  { "YGR137W", "NULL" },
  { "TPO2", "Polyamine transporter of the major facilitator superfamily" },
  { "YGR139W", "NULL" },
  { "CBF2", "Essential kinetochore protein" },
  { "VPS62", "Vacuolar protein sorting (VPS) protein" },
  { "BTN2", "v-SNARE binding protein" },
  { "SKN1", "Protein involved in sphingolipid biosynthesis" },
  { "THI4", "Thiazole synthase" },
  { "ENP2", "Component of the SSU" },
  { "ECL1", "Protein of unknown function" },
  { "YGR146C-A", "Putative protein of unknown function" },
  { "NAT2", "Protein of unknown function" },
  { "RPL24B", "Ribosomal 60S subunit protein L24B" },
  { "YGR149W", "NULL" },
  { "CCM1", "Mitochondrial 15S rRNA-binding protein" },
  { "YGR151C", "NULL" },
  { "RSR1", "GTP-binding protein of the Ras superfamily" },
  { "YGR153W", "Putative protein of unknown function" },
  { "GTO1", "Omega-class glutathione transferase" },
  { "CYS4", "Cystathionine beta-synthase" },
  { "PTI1", "Essential component of CPF (cleavage and polyadenylation factor)" },
  { "CHO2", "Phosphatidylethanolamine methyltransferase (PEMT)" },
  { "MTR3", "Exosome non-catalytic core component" },
  { "NSR1", "Nucleolar protein that binds nuclear localization sequences" },
  { "YGR160W", "NULL" },
  { "RTS3", "Putative component of the protein phosphatase type 2A complex" },
  { "YGR161W-C", "Putative protein of unknown function" },
  { "TIF4631", "Translation initiation factor eIF4G" },
  { "GTR2", "Subunit of a TORC1-stimulating GTPase and the EGO/GSE complex" },
  { "YGR164W", "Putative protein of unknown function" },
  { "MRPS35", "Mitochondrial ribosomal protein of the small subunit" },
  { "TRS65", "Component of transport protein particle (TRAPP) complex II" },
  { "CLC1", "Clathrin light chain" },
  { "YGR168C", "NULL" },
  { "PUS6", "tRNA:pseudouridine synthase" },
  { "YGR169C-A", "NULL" },
  { "PSD2", "Phosphatidylserine decarboxylase of the Golgi and vacuolar membranes" },
  { "MSM1", "Mitochondrial methionyl-tRNA synthetase (MetRS)" },
  { "YIP1", "Integral membrane protein" },
  { "RBG2", "Protein with a role in translation" },
  { "CBP4", "Mitochondrial protein required for assembly of cytochrome bc1 complex" },
  { "YGR174W-A", "Putative protein of unknown function" },
  { "ERG1", "Squalene epoxidase" },
  { "YGR176W", "NULL" },
  { "ATF2", "Alcohol acetyltransferase" },
  { "PBP1", "Component of glucose deprivation induced stress granules" },
  { "OKP1", "Outer kinetochore protein required for accurate chromosome segregation" },
  { "RNR4", "Ribonucleotide-diphosphate reductase (RNR) small subunit" },
  { "TIM13", "Mitochondrial intermembrane space protein" },
  { "YGR182C", "NULL" },
  { "QCR9", "Subunit 9 of ubiquinol cytochrome-c reductase (Complex III)" },
  { "UBR1", "E3 ubiquitin ligase (N-recognin)" },
  { "TYS1", "Cytoplasmic tyrosyl-tRNA synthetase" },
  { "TFG1", "TFIIF (Transcription Factor II) largest subunit" },
  { "HGH1", "Nonessential protein of unknown function" },
  { "BUB1", "Protein kinase involved in the cell cycle checkpoint into anaphase" },
  { "CRH1", "Chitin transglycosylase" },
  { "YGR190C", "NULL" },
  { "HIP1", "High-affinity histidine permease" },
  { "TDH3", "Glyceraldehyde-3-phosphate dehydrogenase (GAPDH), isozyme 3" },
  { "PDX1", "E3-binding protein of the mitochondrial pyruvate dehydrogenase complex" },
  { "XKS1", "Xylulokinase" },
  { "SKI6", "Exosome non-catalytic core component" },
  { "FYV8", "Protein of unknown function" },
  { "SNG1", "Protein involved in resistance to nitrosoguanidine and 6-azauracil" },
  { "YPP1", "Cargo-transport protein involved in endocytosis" },
  { "PMT6", "Protein O-mannosyltransferase" },
  { "ELP2", "Subunit of Elongator complex" },
  { "YGR201C", "Putative protein of unknown function" },
  { "PCT1", "Cholinephosphate cytidylyltransferase" },
  { "YCH1", "Phosphatase with sequence similarity to Mih1p" },
  { "ADE3", "Cytoplasmic trifunctional enzyme C1-tetrahydrofolate synthase" },
  { "YGR204C-A", "Putative protein of unknown function" },
  { "TDA10", "ATP-binding protein of unknown function" },
  { "MVB12", "ESCRT-I subunit required to stabilize ESCRT-I core complex oligomers" },
  { "CIR1", "Mitochondrial protein that interacts with frataxin (Yfh1p)" },
  { "SER2", "Phosphoserine phosphatase of the phosphoglycerate pathway" },
  { "TRX2", "Cytoplasmic thioredoxin isoenzyme" },
  { "YGR210C", "Putative protein of unknown function" },
  { "ZPR1", "Essential protein with two zinc fingers" },
  { "SLI1", "N-acetyltransferase" },
  { "RTA1", "Protein involved in 7-aminocholesterol resistance" },
  { "RPS0A", "Ribosomal 40S subunit protein S0A" },
  { "RSM27", "Mitochondrial ribosomal protein of the small subunit" },
  { "GPI1", "Membrane protein involved in the synthesis of GlcNAc-PI" },
  { "CCH1", "Voltage-gated high-affinity calcium channel" },
  { "CRM1", "Major karyopherin" },
  { "YGR219W", "NULL" },
  { "MRPL9", "Mitochondrial ribosomal protein of the large subunit" },
  { "TOS2", "Protein involved in localization of Cdc24p to the site of bud growth" },
  { "PET54", "Mitochondrial inner membrane protein" },
  { "HSV2", "Phosphatidylinositol 3,5-bisphosphate-binding protein" },
  { "AZR1", "Plasma membrane transporter of the major facilitator superfamily" },
  { "AMA1", "Activator of meiotic anaphase promoting complex (APC/C)" },
  { "YGR226C", "NULL" },
  { "DIE2", "Dolichyl-phosphoglucose-dependent alpha-1,2-glucosyltransferase" },
  { "YGR228W", "NULL" },
  { "SMI1", "Protein involved in the regulation of cell wall synthesis" },
  { "BNS1", "Protein of unknown function" },
  { "PHB2", "Subunit of the prohibitin complex (Phb1p-Phb2p)" },
  { "NAS6", "Evolutionarily conserved 19S regulatory particle assembly-chaperone" },
  { "PHO81", "Cyclin-dependent kinase (CDK) inhibitor" },
  { "YHB1", "Nitric oxide oxidoreductase" },
  { "MIC26", "Component of the MICOS complex" },
  { "SPG1", "Protein required for high temperature survival during stationary phase" },
  { "YGR237C", "Putative protein of unknown function" },
  { "KEL2", "Protein that negatively regulates mitotic exit" },
  { "PEX21", "Peroxin required for peroxisomal matrix protein targeting" },
  { "PFK1", "Alpha subunit of heterooctameric phosphofructokinase" },
  { "YGR240C-A", "Putative protein of unknown function" },
  { "YAP1802", "Protein of the AP180 family, involved in clathrin cage assembly" },
  { "YGR242W", "NULL" },
  { "FMP43", "NULL" },
  { "LSC2", "Beta subunit of succinyl-CoA ligase" },
  { "SDA1", "Protein required for actin organization and passage through Start" },
  { "BRF1", "TFIIIB B-related factor" },
  { "CPD1", "Cyclic nucleotide phosphodiesterase" },
  { "SOL4", "6-phosphogluconolactonase" },
  { "MGA1", "Protein similar to heat shock transcription factor" },
  { "YGR250C", "NULL" },
  { "NOP19", "Ribosome biogenesis factor" },
  { "GCN5", "Catalytic subunit of ADA and SAGA histone acetyltransferase complexes" },
  { "PUP2", "Alpha 5 subunit of the 20S proteasome" },
  { "ENO1", "Enolase I, a phosphopyruvate hydratase" },
  { "COQ6", "Flavin-dependent monooxygenase involved in ubiquinone biosynthesis" },
  { "GND2", "6-phosphogluconate dehydrogenase (decarboxylating)" },
  { "MTM1", "Mitochondrial protein of the mitochondrial carrier family" },
  { "RAD2", "Single-stranded DNA endonuclease" },
  { "YGR259C", "NULL" },
  { "TNA1", "High affinity nicotinic acid plasma membrane permease" },
  { "APL6", "Beta3-like subunit of the yeast AP-3 complex" },
  { "BUD32", "Protein kinase" },
  { "SAY1", "Sterol deacetylase" },
  { "MES1", "Methionyl-tRNA synthetase" },
  { "YGR265W", "NULL" },
  { "YGR266W", "Protein of unknown function" },
  { "FOL2", "GTP-cyclohydrolase I, catalyzes first step in folic acid biosynthesis" },
  { "HUA1", "Cytoplasmic protein containing a zinc finger domain" },
  { "YGR269W", "NULL" },
  { "YTA7", "Protein that localizes to chromatin" },
  { "YGR270C-A", "NULL" },
  { "SLH1", "Putative RNA helicase related to Ski2p" },
  { "EFG1", "Essential protein required for maturation of 18S rRNA" },
  { "YGR273C", "Putative protein of unknown function" },
  { "TAF1", "TFIID subunit, involved in RNA pol II transcription initiation" },
  { "RTT102", "Component of both the SWI/SNF and RSC chromatin remodeling complexes" },
  { "RNH70", "3'-5' exoribonuclease" },
  { "CAB4", "Subunit of the CoA-Synthesizing Protein Complex (CoA-SPC)" },
  { "CWC22", "Spliceosome-associated protein that is required for pre-mRNA splicing" },
  { "SCW4", "Cell wall protein with similarity to glucanases" },
  { "PXR1", "Essential protein involved in rRNA and snoRNA maturation" },
  { "YOR1", "Plasma membrane ATP-binding cassette (ABC) transporter" },
  { "BGL2", "Endo-beta-1,3-glucanase" },
  { "YGR283C", "Putative methyltransferase" },
  { "ERV29", "Protein localized to COPII-coated vesicles" },
  { "ZUO1", "Ribosome-associated chaperone" },
  { "BIO2", "Biotin synthase" },
  { "IMA1", "Major isomaltase (alpha-1,6-glucosidase/alpha-methylglucosidase)" },
  { "MAL13", "MAL-activator protein" },
  { "MAL11", "High-affinity maltose transporter (alpha-glucoside transporter)" },
  { "YGR290W", "NULL" },
  { "YGR291C", "NULL" },
  { "MAL12", "Maltase (alpha-D-glucosidase)" },
  { "YGR293C", "NULL" },
  { "PAU12", "Protein of unknown function" },
  { "COS6", "Endosomal protein involved in turnover of plasma membrane proteins" },
  { "YRF1-3", "Helicase encoded by the Y' element of subtelomeric regions" },
  { "YGR296C-A", "NULL" },
  { "YGR296C-B", "NULL" },
  { "YHL050C", "Putative protein of unknown function" },
  { "YHL050W-A", "NULL" },
  { "YHL049C", "Putative protein of unknown function" },
  { "YHL048C-A", "Putative protein of unknown function" },
  { "COS8", "Endosomal protein involved in turnover of plasma membrane proteins" },
  { "ARN2", "Transporter" },
  { "YHL046W-A", "NULL" },
  { "PAU13", "Protein of unknown function" },
  { "YHL045W", "NULL" },
  { "YHL044W", "Putative integral membrane protein" },
  { "ECM34", "Protein of unknown function" },
  { "YHL042W", "Protein of unknown function" },
  { "YHL041W", "Putative protein of unknown function" },
  { "ARN1", "ARN family transporter for siderophore-iron chelates" },
  { "EFM1", "Lysine methyltransferase" },
  { "CBP2", "Required for splicing of the group I intron bI5 of the COB pre-mRNA" },
  { "YHL037C", "Putative protein of unknown function" },
  { "MUP3", "Low affinity methionine permease" },
  { "VMR1", "Vacuolar membrane protein" },
  { "YHL034W-A", "NULL" },
  { "SBP1", "Protein that binds eIF4G and has a role in repression of translation" },
  { "RPL8A", "Ribosomal 60S subunit protein L8A" },
  { "GUT1", "Glycerol kinase" },
  { "GOS1", "v-SNARE protein involved in Golgi transport" },
  { "YHL030W-A", "NULL" },
  { "ECM29", "Scaffold protein" },
  { "OCA5", "Cytoplasmic protein required for replication of Brome mosaic virus" },
  { "WSC4", "Endoplasmic reticulum (ER) membrane protein" },
  { "RIM101", "Cys2His2 zinc-finger transcriptional repressor" },
  { "YHL026C", "Putative protein of unknown function" },
  { "SNF6", "Subunit of the SWI/SNF chromatin remodeling complex" },
  { "RIM4", "Putative RNA-binding protein" },
  { "NPR3", "Subunit of the Iml1p/SEACIT complex" },
  { "SPO11", "Meiosis-specific protein that initiates meiotic recombination" },
  { "AIM17", "Protein of unknown function" },
  { "OPI1", "Transcriptional regulator of a variety of genes" },
  { "YHL019W-A", "NULL" },
  { "APM2", "Protein of unknown function" },
  { "YHL018W", "NULL" },
  { "YHL017W", "Putative protein of unknown function" },
  { "DUR3", "Plasma membrane transporter for both urea and polyamines" },
  { "YHL015W-A", "Putative protein of unknown function" },
  { "RPS20", "Protein component of the small (40S) ribosomal subunit" },
  { "YLF2", "Protein of unknown function" },
  { "OTU2", "Protein of unknown function" },
  { "YHL012W", "Putative UTP glucose-1-phosphate uridylyltransferase" },
  { "PRS3", "5-phospho-ribosyl-1(alpha)-pyrophosphate synthetase" },
  { "ETP1", "Protein of unknown function required for growth on ethanol" },
  { "YAP3", "Basic leucine zipper (bZIP) transcription factor" },
  { "YHL008C", "Putative protein of unknown function" },
  { "STE20", "Cdc42p-activated signal transducing kinase" },
  { "YHL006W-A", "NULL" },
  { "SHU1", "Component of Shu complex (aka PCSS complex)" },
  { "YHL005C", "Putative protein of unknown function" },
  { "MRP4", "Mitochondrial ribosomal protein of the small subunit" },
  { "LAG1", "Ceramide synthase component" },
  { "YHL002C-A", "NULL" },
  { "HSE1", "Subunit of the endosomal Vps27p-Hse1p complex" },
  { "RPL14B", "Ribosomal 60S subunit protein L14B" },
  { "OSH7", "Oxysterol-binding protein" },
  { "QCR10", "Subunit of the ubiqunol-cytochrome c oxidoreductase complex" },
  { "LEU5", "Mitochondrial carrier protein" },
  { "TCD1", "tRNA threonylcarbamoyladenosine dehydratase" },
  { "NEM1", "Probable catalytic subunit of Nem1p-Spo7p phosphatase holoenzyme" },
  { "GPA1", "Subunit of the G protein involved in pheromone response" },
  { "TIM10", "Essential protein of the mitochondrial intermembrane space" },
  { "STP2", "Transcription factor" },
  { "ERG11", "Lanosterol 14-alpha-demethylase" },
  { "YHR007C-A", "Putative protein of unknown function" },
  { "SOD2", "Mitochondrial manganese superoxide dismutase" },
  { "TDA3", "Putative oxidoreductase involved in late endosome to Golgi transport" },
  { "RPL27A", "Ribosomal 60S subunit protein L27A" },
  { "DIA4", "Probable mitochondrial seryl-tRNA synthetase" },
  { "VPS29", "Subunit of the membrane-associated retromer complex" },
  { "ARD1", "Subunit of protein N-terminal acetyltransferase NatA" },
  { "SPO13", "Meiotic regulator" },
  { "MIP6", "Putative RNA-binding protein" },
  { "YSC84", "Actin-binding protein" },
  { "YSC83", "Non-essential mitochondrial protein of unknown function" },
  { "ARG4", "Argininosuccinate lyase" },
  { "DED81", "Cytosolic asparaginyl-tRNA synthetase" },
  { "YHR020W", "Prolyl-tRNA synthetase" },
  { "RPS27B", "Protein component of the small (40S) ribosomal subunit" },
  { "ECM12", "Putative protein of unknown function" },
  { "YHR022C", "Putative protein of unknown function" },
  { "YHR022C-A", "Putative protein of unknown function" },
  { "MYO1", "Type II myosin heavy chain" },
  { "MAS2", "Alpha subunit of the mitochondrial processing protease (MPP)" },
  { "THR1", "Homoserine kinase" },
  { "VMA16", "Subunit c'' of the vacuolar ATPase" },
  { "RPN1", "Non-ATPase base subunit of the 19S RP of the 26S proteasome" },
  { "DAP2", "Dipeptidyl aminopeptidase" },
  { "YHR028W-A", "NULL" },
  { "YHI9", "Protein of unknown function" },
  { "SLT2", "Serine/threonine MAP kinase" },
  { "RRM3", "DNA helicase involved in rDNA replication and Ty1 transposition" },
  { "ERC1", "Member of the multi-drug and toxin extrusion (MATE) family" },
  { "YHR032W-A", "NULL" },
  { "YHR032C-A", "NULL" },
  { "YHR033W", "Putative protein of unknown function" },
  { "PIH1", "Component of the conserved R2TP complex (Rvb1-Rvb2-Tah1-Pih1)" },
  { "NEL1", "Activator of Sar1p GTPase activity" },
  { "BRL1", "Essential nuclear envelope/ER integral membrane protein" },
  { "PUT2", "Delta-1-pyrroline-5-carboxylate dehydrogenase" },
  { "RRF1", "Mitochondrial ribosome recycling factor" },
  { "MSC7", "Protein of unknown function" },
  { "VMA10", "Subunit G of the V1 peripheral membrane domain of V-ATPase" },
  { "BCD1", "Essential protein required for the accumulation of box C/D snoRNA" },
  { "SRB2", "Subunit of the RNA polymerase II mediator complex" },
  { "NCP1", "NADP-cytochrome P450 reductase" },
  { "DOG2", "2-deoxyglucose-6-phosphate phosphatase" },
  { "DOG1", "2-deoxyglucose-6-phosphate phosphatase" },
  { "YHR045W", "Putative protein of unknown function" },
  { "INM1", "Inositol monophosphatase" },
  { "AAP1", "Arginine/alanine amino peptidase" },
  { "YHK8", "Presumed antiporter of the major facilitator superfamily" },
  { "FSH1", "Putative serine hydrolase" },
  { "YHR049C-A", "NULL" },
  { "SMF2", "Divalent metal ion transporter involved in manganese homeostasis" },
  { "YHR050W-A", "Protein of unknown function" },
  { "COX6", "Subunit VI of cytochrome c oxidase (Complex IV)" },
  { "CIC1", "Essential protein that interacts with proteasome components" },
  { "YHR052W-A", "NULL" },
  { "CUP1-1", "Metallothionein" },
  { "YHR054C", "Putative protein of unknown function" },
  { "YHR054W-A", "NULL" },
  { "CUP1-2", "Metallothionein" },
  { "RSC30", "Component of the RSC chromatin remodeling complex" },
  { "YHR056W-A", "NULL" },
  { "CPR2", "Peptidyl-prolyl cis-trans isomerase (cyclophilin)" },
  { "MED6", "Subunit of the RNA polymerase II mediator complex" },
  { "FYV4", "Protein of unknown function" },
  { "VMA22", "Protein that is required for vacuolar H+-ATPase (V-ATPase) function" },
  { "GIC1", "Protein involved in initiation of budding and cellular polarization" },
  { "RPP1", "Subunit of both RNase MRP and nuclear RNase P" },
  { "PAN5", "2-dehydropantoate 2-reductase" },
  { "YHR063W-A", "NULL" },
  { "SSZ1", "Hsp70 protein that interacts with Zuo1p (a DnaJ homolog)" },
  { "RRP3", "Protein involved in rRNA processing" },
  { "SSF1", "Constituent of 66S pre-ribosomal particles" },
  { "HTD2", "Mitochondrial 3-hydroxyacyl-thioester dehydratase" },
  { "DYS1", "Deoxyhypusine synthase" },
  { "RRP4", "Exosome non-catalytic core component" },
  { "YHR069C-A", "NULL" },
  { "TRM5", "tRNA(m(1)G37)methyltransferase" },
  { "YHR070C-A", "NULL" },
  { "PCL5", "Cyclin" },
  { "YHR071C-A", "NULL" },
  { "ERG7", "Lanosterol synthase" },
  { "NOP10", "Subunit of box H/ACA snoRNP complex" },
  { "OSH3", "Member of an oxysterol-binding protein family" },
  { "YHR073W-A", "NULL" },
  { "YHR073C-B", "NULL" },
  { "QNS1", "Glutamine-dependent NAD(+) synthetase" },
  { "PPE1", "Protein with carboxyl methyl esterase activity" },
  { "PTC7", "Type 2C serine/threonine protein phosphatase (PP2C)" },
  { "NMD2", "Protein involved in the nonsense-mediated mRNA decay (NMD) pathway" },
  { "YHR078W", "High osmolarity-regulated gene of unknown function" },
  { "IRE1", "Serine-threonine kinase and endoribonuclease" },
  { "SAE3", "Meiosis-specific protein involved in meiotic recombination" },
  { "YHR080C", "NULL" },
  { "LRP1", "Nuclear exosome-associated nucleic acid binding protein" },
  { "KSP1", "Serine/threonine protein kinase" },
  { "SAM35", "Component of the sorting and assembly machinery (SAM) complex" },
  { "STE12", "Transcription factor that is activated by a MAPK signaling cascade" },
  { "IPI1", "Component of the Rix1 complex and possibly pre-replicative complexes" },
  { "NAM8", "RNA binding protein, component of the U1 snRNP protein" },
  { "YHR086W-A", "Putative protein of unknown function" },
  { "RTC3", "Protein of unknown function involved in RNA metabolism" },
  { "RPF1", "Protein involved in assembly and export of the large ribosomal subunit" },
  { "GAR1", "Protein component of the H/ACA snoRNP pseudouridylase complex" },
  { "YNG2", "Subunit of NuA4, an essential histone acetyltransferase complex" },
  { "MSR1", "Mitochondrial arginyl-tRNA synthetase" },
  { "HXT4", "High-affinity glucose transporter" },
  { "AHT1", "Putative protein of unknown function" },
  { "HXT1", "Low-affinity glucose transporter of the major facilitator superfamily" },
  { "YHR095W", "NULL" },
  { "HXT5", "Hexose transporter with moderate affinity for glucose" },
  { "YHR097C", "NULL" },
  { "SFB3", "Component of the Sec23p-Sfb3p heterodimer of the COPII vesicle coat" },
  { "TRA1", "Subunit of SAGA and NuA4 histone acetyltransferase complexes" },
  { "GEP4", "Mitochondrial phosphatidylglycerophosphatase (PGP phosphatase)" },
  { "BIG1", "Integral membrane protein of the endoplasmic reticulum" },
  { "KIC1", "Protein kinase of the PAK/Ste20 family, required for cell integrity" },
  { "SBE22", "Protein involved in bud growth" },
  { "GRE3", "Aldose reductase" },
  { "YPT35", "Endosomal protein of unknown function" },
  { "TRR2", "Mitochondrial thioredoxin reductase" },
  { "CDC12", "Component of the septin ring that is required for cytokinesis" },
  { "GGA2", "Protein that regulates Arf1p, Arf2p to facilitate Golgi trafficking" },
  { "CTM1", "Cytochrome c lysine methyltransferase" },
  { "ERP5", "Protein with similarity to Emp24p and Erv25p" },
  { "UBA4", "E1-like protein that activates Urm1p before urmylation" },
  { "YHR112C", "Protein of unknown function" },
  { "APE4", "Cytoplasmic aspartyl aminopeptidase with possible vacuole function" },
  { "BZZ1", "SH3 domain protein implicated in regulating actin polymerization" },
  { "DMA1", "Ubiquitin-protein ligase (E3)" },
  { "COX23", "Protein that functions in mitochondrial copper homeostasis" },
  { "TOM71", "Mitochondrial outer membrane protein" },
  { "ORC6", "Subunit of the origin recognition complex (ORC)" },
  { "SET1", "Histone methyltransferase, subunit of the COMPASS (Set1C) complex" },
  { "MSH1", "DNA-binding protein of the mitochondria" },
  { "LSM12", "Protein of unknown function that may function in RNA processing" },
  { "CIA2", "Component of cytosolic iron-sulfur protein assembly (CIA) machinery" },
  { "EPT1", "sn-1,2-diacylglycerol ethanolamine- and cholinephosphotranferase" },
  { "NDT80", "Meiosis-specific transcription factor" },
  { "YHR125W", "Putative protein of unknown function" },
  { "ANS1", "Putative GPI protein" },
  { "YHR127W", "Protein of unknown function" },
  { "FUR1", "Uracil phosphoribosyltransferase" },
  { "ARP1", "Actin-related protein of the dynactin complex" },
  { "YHR130C", "Putative protein of unknown function" },
  { "YHR131C", "Putative protein of unknown function" },
  { "YHR131W-A", "NULL" },
  { "ECM14", "Putative metalloprotease with similarity to zinc carboxypeptidases" },
  { "IGO2", "Protein required for initiation of G0 program" },
  { "NSG1", "Protein involved in regulation of sterol biosynthesis" },
  { "WSS1", "SUMO-ligase and SUMO-targeted metalloprotease" },
  { "YCK1", "Palmitoylated plasma membrane-bound casein kinase I (CK1) isoform" },
  { "SPL2", "Protein with similarity to cyclin-dependent kinase inhibitors" },
  { "ARO9", "Aromatic aminotransferase II" },
  { "YHR137C-A", "NULL" },
  { "YHR138C", "Protein of unknown function" },
  { "SPS100", "Protein required for spore wall maturation" },
  { "YHR139C-A", "Putative protein of unknown function" },
  { "YHR140W", "Putative integral membrane protein of unknown function" },
  { "RPL42B", "Ribosomal 60S subunit protein L42B" },
  { "CHS7", "Protein of unknown function" },
  { "DSE2", "Daughter cell-specific secreted protein with similarity to glucanases" },
  { "RPC10", "RNA polymerase subunit ABC10-alpha, found in RNA pol I, II, and III" },
  { "DCD1", "Deoxycytidine monophosphate (dCMP) deaminase" },
  { "YHR145C", "NULL" },
  { "CRP1", "Protein that binds to cruciform DNA structures" },
  { "MRPL6", "Mitochondrial ribosomal protein of the large subunit" },
  { "IMP3", "Component of the SSU processome" },
  { "SKG6", "Integral membrane protein" },
  { "PEX28", "Peroxisomal integral membrane peroxin" },
  { "MTC6", "Protein of unknown function" },
  { "SPO12", "Nucleolar protein of unknown function" },
  { "SPO16", "Meiosis-specific protein involved in synaptonemal complex assembly" },
  { "RTT107", "Protein implicated in Mms22-dependent DNA repair during S phase" },
  { "YSP1", "NULL" },
  { "LIN1", "Non-essential component of U5 snRNP" },
  { "REC104", "Protein involved in early stages of meiotic recombination" },
  { "KEL1", "Protein required for proper cell fusion and cell morphology" },
  { "TDA11", "Putative protein of unknown function" },
  { "PEX18", "Peroxin" },
  { "YAP1801", "Protein of the AP180 family, involved in clathrin cage assembly" },
  { "MPC2", "Highly conserved subunit of the mitochondrial pyruvate carrier (MPC)" },
  { "SOL3", "6-phosphogluconolactonase" },
  { "DNA2", "Tripartite DNA replication factor" },
  { "PRP8", "Component of U4/U6-U5 snRNP complex" },
  { "YHR165W-A", "NULL" },
  { "CDC23", "Subunit of the Anaphase-Promoting Complex/Cyclosome (APC/C)" },
  { "THP2", "Subunit of the THO and TREX complexes" },
  { "MTG2", "Putative GTPase" },
  { "DBP8", "ATPase, putative RNA helicase of the DEAD-box family" },
  { "NMD3", "Protein involved in nuclear export of the large ribosomal subunit" },
  { "ATG7", "Autophagy-related protein and dual specificity member of the E1 family" },
  { "SPC97", "Component of the microtubule-nucleating Tub4p (gamma-tubulin) complex" },
  { "YHR173C", "Protein of unknown function" },
  { "ENO2", "Enolase II, a phosphopyruvate hydratase" },
  { "CTR2", "Low-affinity copper transporter of the vacuolar membrane" },
  { "YHR175W-A", "Putative protein of unknown function" },
  { "FMO1", "Flavin-containing monooxygenase" },
  { "YHR177W", "NULL" },
  { "STB5", "Transcription factor" },
  { "OYE2", "Conserved NADPH oxidoreductase containing flavin mononucleotide (FMN)" },
  { "YHR180W", "Putative protein of unknown function" },
  { "YHR180W-A", "NULL" },
  { "YHR180C-B", "NULL" },
  { "SVP26", "Integral membrane protein of the early Golgi apparatus and ER" },
  { "YHR182W", "Protein of unknown function" },
  { "YHR182C-A", "NULL" },
  { "GND1", "6-phosphogluconate dehydrogenase (decarboxylating)" },
  { "SSP1", "Protein involved in the control of meiotic nuclear division" },
  { "PFS1", "Sporulation protein required for prospore membrane formation" },
  { "KOG1", "Subunit of TORC1" },
  { "IKI1", "Subunit of hexameric RecA-like ATPase Elp456 Elongator subcomplex" },
  { "GPI16", "Subunit of the glycosylphosphatidylinositol transamidase complex" },
  { "PTH1", "One of two mitochondrially-localized peptidyl-tRNA hydrolases" },
  { "ERG9", "Farnesyl-diphosphate farnesyl transferase (squalene synthase)" },
  { "CTF8", "Subunit of a complex with Ctf18p" },
  { "LNP1", "Lunapark family member involved in ER network formation" },
  { "EGD2", "Alpha subunit of the nascent polypeptide-associated complex (NAC)" },
  { "YHR193C-A", "NULL" },
  { "MDM31", "Mitochondrial protein that may have a role in phospholipid metabolism" },
  { "NVJ1", "Nuclear envelope protein" },
  { "UTP9", "Nucleolar protein" },
  { "RIX1", "Component of the Rix1 complex and possibly pre-replicative complexes" },
  { "AIM18", "Protein of unknown function" },
  { "AIM46", "Protein of unknown function" },
  { "NBL1", "Subunit of the conserved chromosomal passenger complex (CPC)" },
  { "RPN10", "Proteasome polyubiquitin receptor" },
  { "PPX1", "Exopolyphosphatase" },
  { "YHR202W", "Putative protein of unknown function" },
  { "RPS4B", "Protein component of the small (40S) ribosomal subunit" },
  { "MNL1", "Alpha-1,2-specific exomannosidase of the endoplasmic reticulum" },
  { "SCH9", "AGC family protein kinase" },
  { "SKN7", "Nuclear response regulator and transcription factor" },
  { "SET5", "Methyltransferase involved in methylation of histone H4 Lys5, -8, -12" },
  { "BAT1", "Mitochondrial branched-chain amino acid (BCAA) aminotransferase" },
  { "CRG1", "S-AdoMet-dependent methyltransferase involved in lipid homeostasis" },
  { "YHR210C", "Putative aldose 1-epimerase superfamily protein" },
  { "FLO5", "Lectin-like cell wall protein (flocculin) involved in flocculation" },
  { "YHR212C", "NULL" },
  { "YHR212W-A", "Pseudogenic fragment with similarity to flocculins" },
  { "YHR213W", "Pseudogenic fragment with similarity to flocculins" },
  { "YHR213W-A", "Putative protein of unknown function" },
  { "YHR213W-B", "Pseudogenic fragment with similarity to flocculins" },
  { "YHR214W", "Putative protein of unknown function" },
  { "YHR214W-A", "NULL" },
  { "YHR214C-D", "Putative protein of unknown function" },
  { "YHR214C-E", "Putative protein of unknown function" },
  { "PHO12", "One of three repressible acid phosphatases" },
  { "IMD2", "Inosine monophosphate dehydrogenase" },
  { "YHR217C", "NULL" },
  { "YHR218W", "Helicase-like protein encoded within the telomeric Y' element" },
  { "YHR218W-A", "NULL" },
  { "YHR219W", "Putative protein of unknown function with similarity to helicases" },
  { "YHR219C-A", "NULL" },
  { "YIL177C", "Putative Y' element ATP-dependent helicase" },
  { "YIL177W-A", "NULL" },
  { "PAU14", "Protein of unknown function" },
  { "VTH1", "Putative membrane glycoprotein" },
  { "IMA3", "Alpha-glucosidase" },
  { "YIL171W-A", "NULL" },
  { "YIL169C", "NULL" },
  { "YIL166C", "NULL" },
  { "YIL165C", "Putative protein of unknown function" },
  { "NIT1", "Nitrilase" },
  { "YIL163C", "Protein of unknown function" },
  { "SUC2", "Invertase" },
  { "YIL161W", "Putative protein of unknown function" },
  { "POT1", "3-ketoacyl-CoA thiolase with broad chain length specificity" },
  { "BNR1", "Formin" },
  { "AIM20", "Protein of unknown function" },
  { "COA1", "Mitochondrial inner membrane protein" },
  { "YIL156W-A", "NULL" },
  { "YIL156W-B", "NULL" },
  { "UBP7", "Ubiquitin-specific protease that cleaves ubiquitin-protein fusions" },
  { "GUT2", "Mitochondrial glycerol-3-phosphate dehydrogenase" },
  { "IMP2", "Catalytic subunit of mitochondrial inner membrane peptidase complex" },
  { "RRD1", "Peptidyl-prolyl cis/trans-isomerase" },
  { "YIL152W", "Putative protein of unknown function" },
  { "ESL1", "hEST1A/B (SMG5/6)-like protein" },
  { "MCM10", "Essential chromatin-associated protein" },
  { "MLP2", "Myosin-like protein associated with the nuclear envelope" },
  { "RPL40A", "Ubiquitin-ribosomal 60S subunit protein L40A fusion protein" },
  { "SLN1", "Transmembrane histidine phosphotransfer kinase and osmosensor" },
  { "ATG32", "Mitochondrial outer membrane protein required to initiate mitophagy" },
  { "PAN6", "Pantothenate synthase" },
  { "NDC80", "Component of the kinetochore-associated Ndc80 complex" },
  { "SSL2", "Component of RNA polymerase transcription factor TFIIH holoenzyme" },
  { "YIL142C-A", "NULL" },
  { "CCT2", "Subunit beta of the cytosolic chaperonin Cct ring complex" },
  { "YIL141W", "NULL" },
  { "AXL2", "Integral plasma membrane protein" },
  { "REV7", "Accessory subunit of DNA polymerase zeta" },
  { "TPM2", "Minor isoform of tropomyosin" },
  { "TMA108", "Ribosome-associated, nascent chain binding factor" },
  { "OM45", "Mitochondrial outer membrane protein of unknown function" },
  { "VHS2", "Regulator of septin dynamics" },
  { "YIL134C-A", "Putative protein of unknown function" },
  { "FLX1", "Mitochondrial flavin adenine dinucleotide transporter" },
  { "RPL16A", "Ribosomal 60S subunit protein L16A" },
  { "CSM2", "Component of Shu complex (aka PCSS complex)" },
  { "FKH1", "Forkhead family transcription factor" },
  { "ASG1", "Zinc cluster protein proposed to be a transcriptional regulator" },
  { "TAO3", "Component of the RAM signaling network" },
  { "MET18", "Component of cytosolic iron-sulfur protein assembly (CIA) machinery" },
  { "RRT14", "Putative protein of unknown function" },
  { "STH1", "ATPase component of the RSC chromatin remodeling complex" },
  { "KGD1", "Subunit of the mitochondrial alpha-ketoglutarate dehydrogenase complex" },
  { "AYR1", "Bifunctional triacylglycerol lipase and 1-acyl DHAP reductase" },
  { "SIM1", "Protein of the SUN family (Sim1p, Uth1p, Nca3p, Sun4p)" },
  { "POG1", "DNA-binding transcriptional activator" },
  { "QDR2", "Plasma membrane transporter of the major facilitator superfamily" },
  { "QDR1", "Multidrug transporter of the major facilitator superfamily" },
  { "RPI1", "Transcription factor, allelic differences between S288C and Sigma1278b" },
  { "RHO3", "Non-essential small GTPase of the Rho/Rac family of Ras-like proteins" },
  { "PRM5", "Pheromone-regulated protein, predicted to have 1 transmembrane segment" },
  { "HIS5", "Histidinol-phosphate aminotransferase" },
  { "YIL115W-A", "NULL" },
  { "NUP159", "FG-nucleoporin component of central core of the nuclear pore complex" },
  { "POR2", "Putative mitochondrial porin (voltage-dependent anion channel)" },
  { "SDP1", "Stress-inducible dual-specificity MAP kinase phosphatase" },
  { "HOS4", "Subunit of the Set3 complex" },
  { "COX5B", "Subunit Vb of cytochrome c oxidase" },
  { "HPM1", "AdoMet-dependent methyltransferase" },
  { "SEC24", "Component of the Sec23p-Sec24p heterodimer of the COPII vesicle coat" },
  { "YIL108W", "Putative metalloendopeptidase" },
  { "PFK26", "6-phosphofructo-2-kinase" },
  { "MOB1", "Component of the mitotic exit network" },
  { "SLM1", "Phosphoinositide PI4,5P(2) binding protein, forms a complex with Slm2p" },
  { "YIL105W-A", "Protein of unknown function" },
  { "SHQ1", "Chaperone protein" },
  { "DPH1", "Protein required for synthesis of diphthamide" },
  { "YIL102C-A", "Putative protein of unknown function" },
  { "YIL102C", "Putative protein of unknown function" },
  { "XBP1", "Transcriptional repressor" },
  { "YIL100C-A", "NULL" },
  { "YIL100W", "NULL" },
  { "SGA1", "Intracellular sporulation-specific glucoamylase" },
  { "FMC1", "Mitochondrial matrix protein" },
  { "FYV10", "Subunit of GID complex" },
  { "BMT5", "Methyltransferase required for m3U2634 methylation of the 25S rRNA" },
  { "PRK1", "Ser/Thr protein kinase" },
  { "LYS12", "Homo-isocitrate dehydrogenase" },
  { "RSM25", "Mitochondrial ribosomal protein of the small subunit" },
  { "YIL092W", "Putative protein of unknown function" },
  { "UTP25", "Nucleolar protein" },
  { "ICE2", "Integral ER membrane protein with type-III transmembrane domains" },
  { "YIL089W", "Protein of unknown function found in the ER and vacuole lumen" },
  { "AVT7", "Putative transporter" },
  { "AIM19", "Protein of unknown function" },
  { "YIL086C", "Putative protein of unknown function" },
  { "KTR7", "Putative mannosyltransferase involved in protein glycosylation" },
  { "SDS3", "Component of the Rpd3L histone deacetylase complex" },
  { "CAB2", "Phosphopantothenoylcysteine synthetase (PPCS)" },
  { "YIL082W", "NULL" },
  { "AIR1", "Zinc knuckle protein" },
  { "THS1", "Threonyl-tRNA synthetase" },
  { "YIL077C", "NULL" },
  { "SEC28", "Epsilon-COP subunit of the coatomer" },
  { "RPN2", "Subunit of the 26S proteasome" },
  { "SER33", "3-phosphoglycerate dehydrogenase and alpha-ketoglutarate reductase" },
  { "SPO22", "Meiosis-specific protein essential for chromosome synapsis" },
  { "HOP1", "Meiosis-specific protein required for chromosome synapsis" },
  { "YIL071W-A", "NULL" },
  { "PCI8", "Possible shared subunit of Cop9 signalosome (CSN) and eIF3" },
  { "MAM33", "Specific translational activator for the mitochondrial COX1 mRNA" },
  { "RPS24B", "Protein component of the small (40S) ribosomal subunit" },
  { "YIL068W-A", "NULL" },
  { "SEC6", "Essential 88kDa subunit of the exocyst complex" },
  { "YIL067C", "Uncharacterized protein of unknown function" },
  { "YIL066W-A", "NULL" },
  { "RNR3", "Minor isoform of large subunit of ribonucleotide-diphosphate reductase" },
  { "FIS1", "Protein involved in mitochondrial fission and peroxisome abundance" },
  { "SEE1", "NULL" },
  { "YRB2", "Protein of unknown function" },
  { "ARC15", "Subunit of the ARP2/3 complex" },
  { "SNP1", "Component of U1 snRNP required for mRNA splicing via spliceosome" },
  { "YIL060W", "Mitochondrial protein of unknown function" },
  { "YIL059C", "NULL" },
  { "YIL058W", "NULL" },
  { "RGI2", "Protein of unknown function" },
  { "VHR1", "Transcriptional activator" },
  { "YIL055C", "Putative protein of unknown function" },
  { "YIL054W", "Protein of unknown function" },
  { "GPP1", "Constitutively expressed DL-glycerol-3-phosphate phosphatase" },
  { "RPL34B", "Ribosomal 60S subunit protein L34B" },
  { "MMF1", "Mitochondrial protein required for transamination of isoleucine" },
  { "PCL7", "Pho85p cyclin of the Pho80p subfamily" },
  { "DFG10", "Probable polyprenol reductase" },
  { "NEO1", "Phospholipid translocase (flippase), role in phospholipid asymmetry o" },
  { "YIL047C-A", "NULL" },
  { "SYG1", "Plasma membrane protein of unknown function" },
  { "YIL046W-A", "Putative protein of unknown function" },
  { "MET30", "F-box protein containing five copies of the WD40 motif" },
  { "PIG2", "Putative type-1 protein phosphatase targeting subunit" },
  { "AGE2", "ADP-ribosylation factor (ARF) GTPase activating protein (GAP) effector" },
  { "CBR1", "Cytochrome b reductase" },
  { "PKP1", "Mitochondrial protein kinase" },
  { "GVP36", "BAR domain protein that localizes to early and late Golgi vesicles" },
  { "APQ12", "Nuclear envelope/ER integral membrane protein" },
  { "TED1", "GPI-glycan remodelase" },
  { "NOT3", "Component of the CCR4-NOT core complex, involved in mRNA decapping" },
  { "PRM2", "Pheromone-regulated protein" },
  { "CST6", "Basic leucine zipper (bZIP) transcription factor from ATF/CREB family" },
  { "CKA1", "Alpha catalytic subunit of casein kinase 2 (CK2)" },
  { "CAP2", "Beta subunit of the capping protein heterodimer (Cap1p and Cap2p)" },
  { "BCY1", "Regulatory subunit of the cyclic AMP-dependent protein kinase (PKA)" },
  { "YIL032C", "NULL" },
  { "ULP2", "Peptidase that deconjugates Smt3/SUMO-1 peptides from proteins" },
  { "YIL030W-A", "NULL" },
  { "SSM4", "Membrane-embedded ubiquitin-protein ligase" },
  { "YIL029W-A", "NULL" },
  { "YIL029C", "NULL" },
  { "YIL028W", "NULL" },
  { "EMC5", "Member of conserved ER transmembrane complex" },
  { "IRR1", "Subunit of the cohesin complex" },
  { "YIL025C", "NULL" },
  { "YIL024C", "Putative protein of unknown function" },
  { "YKE4", "Zinc transporter" },
  { "TIM44", "Essential component of the TIM23 complex" },
  { "RPB3", "RNA polymerase II third largest subunit B44" },
  { "YIL021C-A", "NULL" },
  { "YIL020C-A", "NULL" },
  { "HIS6", "Enzyme that catalyzes the fourth step in the histidine pathway" },
  { "FAF1", "Protein required for pre-rRNA processing" },
  { "RPL2B", "Ribosomal 60S subunit protein L2B" },
  { "VID28", "GID Complex subunit, serves as adaptor for regulatory subunit Vid24p" },
  { "SNL1", "Ribosome-associated protein" },
  { "BAR1", "Aspartyl protease" },
  { "YIL014C-A", "Putative protein of unknown function" },
  { "MNT3", "Alpha-1,3-mannosyltransferase" },
  { "PDR11", "ATP-binding cassette (ABC) transporter" },
  { "YIL012W", "Putative protein of unknown function" },
  { "TIR3", "Cell wall mannoprotein" },
  { "DOT5", "Nuclear thiol peroxidase" },
  { "EST3", "Component of the telomerase holoenzyme" },
  { "FAA3", "Long chain fatty acyl-CoA synthetase" },
  { "URM1", "Ubiquitin-like protein involved in thiolation of cytoplasmic tRNAs" },
  { "NAS2", "Evolutionarily conserved 19S regulatory particle assembly-chaperone" },
  { "YIA6", "Mitochondrial NAD+ transporter" },
  { "EPS1", "ER protein with chaperone and co-chaperone activity" },
  { "BET1", "Type II membrane protein required for vesicular transport" },
  { "CFD1", "Highly conserved iron-sulfur cluster binding protein" },
  { "YIL002W-A", "NULL" },
  { "INP51", "Phosphatidylinositol 4,5-bisphosphate 5-phosphatase" },
  { "YIL001W", "Putative protein of unknown function" },
  { "SGN1", "Cytoplasmic RNA-binding protein" },
  { "MPH1", "3'-5' DNA helicase involved in error-free bypass of DNA lesions" },
  { "AIM21", "Subunit of a complex that associates with actin filaments" },
  { "DJP1", "Cytosolic J-domain-containing protein" },
  { "IST3", "Component of the U2 snRNP" },
  { "PAN1", "Part of actin cytoskeleton-regulatory complex Pan1p-Sla1p-End3p" },
  { "YIR007W", "NULL" },
  { "PRI1", "Subunit of DNA primase" },
  { "MSL1", "U2B component of U2 snRNP" },
  { "DSN1", "Essential component of the outer kinetochore MIND complex" },
  { "STS1", "Protein required for localizing proteasomes to the nucleus" },
  { "SQT1", "Specific chaperone for ribosomal protein Rpl10p" },
  { "GAT4", "Protein containing GATA family zinc finger motifs" },
  { "YIR014W", "NULL" },
  { "RPR2", "Subunit of nuclear RNase P" },
  { "YIR016W", "Putative protein of unknown function" },
  { "MET28", "bZIP transcriptional activator in the Cbf1p-Met4p-Met28p complex" },
  { "YIR017W-A", "NULL" },
  { "YAP5", "Basic leucine zipper (bZIP) iron-sensing transcription factor" },
  { "YIR018C-A", "Putative protein of unknown function" },
  { "FLO11", "GPI-anchored cell surface glycoprotein (flocculin)" },
  { "YIR020C", "Protein of unknown function" },
  { "YIR020W-A", "NULL" },
  { "YIR020C-B", "NULL" },
  { "MRS1", "Splicing protein" },
  { "YIR021W-A", "Putative protein of unknown function" },
  { "SEC11", "18kDa catalytic subunit of the Signal Peptidase Complex (SPC)" },
  { "DAL81", "Positive regulator of genes in multiple nitrogen degradation pathways" },
  { "YIR023C-A", "NULL" },
  { "INA22", "F1F0 ATP synthase peripheral stalk assembly factor" },
  { "MND2", "Subunit of the Anaphase-Promoting Complex/Cyclosome (APC/C)" },
  { "YVH1", "Dual specificity protein phosphatase" },
  { "DAL1", "Allantoinase" },
  { "DAL4", "Allantoin permease" },
  { "DAL2", "Allantoicase" },
  { "DCG1", "Protein of unknown function" },
  { "YIR030W-A", "NULL" },
  { "DAL7", "Malate synthase" },
  { "DAL3", "Ureidoglycolate lyase" },
  { "MGA2", "ER membrane protein involved in regulation of OLE1 transcription" },
  { "LYS1", "Saccharopine dehydrogenase (NAD+, L-lysine-forming)" },
  { "YIR035C", "NULL" },
  { "IRC24", "Putative benzil reductase" },
  { "YIR036W-A", "NULL" },
  { "HYR1", "Thiol peroxidase" },
  { "GTT1", "ER associated glutathione S-transferase" },
  { "YPS6", "Putative GPI-anchored aspartic protease" },
  { "YIR040C", "NULL" },
  { "PAU15", "Protein of unknown function" },
  { "YIR042C", "Putative protein of unknown function" },
  { "YJL225C", "Putative Y' element ATP-dependent helicase" },
  { "YJL225W-A", "NULL" },
  { "PAU1", "Member of the seripauperin multigene family" },
  { "YJL222W-B", "NULL" },
  { "YJL222W-A", "NULL" },
  { "VTH2", "Putative membrane glycoprotein" },
  { "IMA4", "Alpha-glucosidase" },
  { "YJL220W", "NULL" },
  { "HXT9", "Putative hexose transporter that is nearly identical to Hxt11p" },
  { "YJL218W", "Putative acetyltransferase" },
  { "REE1", "Cytoplasmic protein involved in the regulation of enolase (ENO1)" },
  { "IMA5", "Alpha-glucosidase" },
  { "YJL215C", "NULL" },
  { "HXT8", "Protein of unknown function with similarity to hexose transporters" },
  { "YJL213W", "Protein of unknown function that may interact with ribosomes" },
  { "OPT1", "Proton-coupled oligopeptide transporter of the plasma membrane" },
  { "YJL211C", "NULL" },
  { "PEX2", "RING-finger peroxin and E3 ubiquitin ligase" },
  { "CBP1", "Mitochondrial protein, regulator of COB mRNA stability and translation" },
  { "NUC1", "Major mitochondrial nuclease" },
  { "LAA1", "AP-1 accessory protein" },
  { "YJL206C", "Putative protein of unknown function" },
  { "NCE101", "Protein of unknown function" },
  { "RCY1", "F-box protein involved in recycling endocytosed proteins" },
  { "PRP21", "Subunit of the SF3a splicing factor complex" },
  { "YJL202C", "NULL" },
  { "ECM25", "Non-essential protein of unknown function" },
  { "ACO2", "Putative mitochondrial aconitase isozyme" },
  { "MBB1", "Putative protein of unknown function" },
  { "PHO90", "Low-affinity phosphate transporter" },
  { "UBP12", "Ubiquitin-specific protease" },
  { "YJL197C-A", "NULL" },
  { "ELO1", "Elongase I, medium-chain acyl elongase" },
  { "YJL195C", "NULL" },
  { "CDC6", "Essential ATP-binding protein required for DNA replication" },
  { "YJL193W", "Putative protein of unknown function" },
  { "SOP4", "ER-membrane protein" },
  { "RPS14B", "Protein component of the small (40S) ribosomal subunit" },
  { "RPS22A", "Protein component of the small (40S) ribosomal subunit" },
  { "RPL39", "Ribosomal 60S subunit protein L39" },
  { "BUD19", "NULL" },
  { "SWE1", "Protein kinase that regulates the G2/M transition" },
  { "MNN5", "Alpha-1,2-mannosyltransferase" },
  { "ATG36", "Pex3p interacting protein, required for pexophagy" },
  { "GON7", "Component of the EKC/KEOPS protein complex" },
  { "MNN11", "Subunit of a Golgi mannosyltransferase complex" },
  { "YJL182C", "NULL" },
  { "YJL181W", "NULL" },
  { "ATP12", "Assembly factor for F1 sector of mitochondrial F1F0 ATP synthase" },
  { "PFD1", "Subunit of heterohexameric prefoldin" },
  { "ATG27", "Type I membrane protein involved in autophagy and the Cvt pathway" },
  { "RPL17B", "Ribosomal 60S subunit protein L17B" },
  { "SWI3", "Subunit of the SWI/SNF chromatin remodeling complex" },
  { "YJL175W", "NULL" },
  { "KRE9", "Glycoprotein involved in cell wall beta-glucan assembly" },
  { "RFA3", "Subunit of heterotrimeric Replication Protein A (RPA)" },
  { "CPS1", "Vacuolar carboxypeptidase S" },
  { "YJL171C", "NULL" },
  { "ASG7", "Protein that regulates signaling from G protein beta subunit Ste4p" },
  { "YJL169W", "NULL" },
  { "SET2", "Histone methyltransferase with a role in transcriptional elongation" },
  { "ERG20", "Farnesyl pyrophosphate synthetase" },
  { "QCR8", "Subunit 8 of ubiquinol cytochrome-c reductase (Complex III)" },
  { "HAL5", "Putative protein kinase" },
  { "TPK1", "cAMP-dependent protein kinase catalytic subunit" },
  { "YJL163C", "Putative protein of unknown function" },
  { "JJJ2", "Protein of unknown function" },
  { "FMP33", "Putative protein of unknown function" },
  { "YJL160C", "NULL" },
  { "HSP150", "O-mannosylated heat shock protein" },
  { "CIS3", "Mannose-containing glycoprotein constituent of the cell wall" },
  { "FAR1", "CDK inhibitor and nuclear anchor" },
  { "YJL156W-A", "NULL" },
  { "SSY5", "Serine protease of SPS plasma membrane amino acid sensor system" },
  { "FBP26", "Fructose-2,6-bisphosphatase, required for glucose metabolism" },
  { "VPS35", "Endosomal subunit of membrane-associated retromer complex" },
  { "INO1", "Inositol-3-phosphate synthase" },
  { "YJL152W", "NULL" },
  { "SNA3", "Protein involved in efficient MVB sorting of proteins to the vacuole" },
  { "YJL150W", "NULL" },
  { "DAS1", "Putative SCF ubiquitin ligase F-box protein" },
  { "RPA34", "RNA polymerase I subunit A34.5" },
  { "YJL147C", "NULL" },
  { "IDS2", "Protein involved in modulation of Ime2p activity during meiosis" },
  { "SFH5", "Non-classical phosphatidylinositol transfer protein (PITP)" },
  { "YJL144W", "NULL" },
  { "TIM17", "Essential component of the TIM23 complex" },
  { "IRC9", "NULL" },
  { "YAK1", "Serine-threonine protein kinase" },
  { "RPB4", "RNA polymerase II subunit B32" },
  { "YUR1", "Mannosyltransferase involved in protein N-glycosylation" },
  { "TIF2", "Translation initiation factor eIF4A" },
  { "GLG2", "Glycogenin glucosyltransferase" },
  { "YJL136W-A", "Putative protein of unknown function" },
  { "RPS21B", "Protein component of the small (40S) ribosomal subunit" },
  { "YJL135W", "NULL" },
  { "LCB3", "Long-chain base-1-phosphate phosphatase" },
  { "YJL133C-A", "NULL" },
  { "MRS3", "Iron transporter, mediates Fe2+ transport across inner mito membrane" },
  { "YJL132W", "Putative protein of unknown function" },
  { "AIM23", "Mitochondrial translation initiation factor 3 (IF3, mIF3)" },
  { "URA2", "Bifunctional carbamoylphosphate synthetase/aspartate transcarbamylase" },
  { "TRK1", "Component of the Trk1p-Trk2p potassium transport system" },
  { "PBS2", "MAP kinase kinase of the HOG signaling pathway" },
  { "YJL127W-A", "NULL" },
  { "YJL127C-B", "NULL" },
  { "SPT10", "Histone H3 acetylase with a role in transcriptional regulation" },
  { "NIT2", "Nit protein" },
  { "GCD14", "Subunit of tRNA (1-methyladenosine) methyltransferase" },
  { "LSM1", "Lsm (Like Sm) protein" },
  { "MTC1", "Protein of unknown function that may interact with ribosomes" },
  { "ALB1", "Shuttling pre-60S factor" },
  { "RPE1", "D-ribulose-5-phosphate 3-epimerase" },
  { "YJL120W", "NULL" },
  { "YJL119C", "NULL" },
  { "YJL118W", "Protein of unknown function" },
  { "PHO86", "Endoplasmic reticulum (ER) resident protein" },
  { "NCA3", "Protein involved in mitochondrion organization" },
  { "ASF1", "Nucleosome assembly factor" },
  { "MDV1", "Peripheral protein of cytosolic face of mitochondrial outer membrane" },
  { "CCT7", "Subunit of the cytosolic chaperonin Cct ring complex" },
  { "GZF3", "GATA zinc finger protein" },
  { "UTP10", "Nucleolar protein" },
  { "PRM10", "Pheromone-regulated protein" },
  { "YJL107C", "Putative protein of unknown function" },
  { "IME2", "Serine/threonine protein kinase involved in activation of meiosis" },
  { "SET4", "Protein of unknown function, contains a SET domain" },
  { "PAM16", "Subunit of the import motor (PAM complex)" },
  { "GSM1", "Putative zinc cluster protein of unknown function" },
  { "MEF2", "Mitochondrial elongation factor involved in translational elongation" },
  { "GSH1", "Gamma glutamylcysteine synthetase" },
  { "LSB6", "Type II phosphatidylinositol 4-kinase" },
  { "CHS6", "Member of the ChAPs (Chs5p-Arf1p-binding proteins) family" },
  { "SAP185", "Protein that forms a complex with the Sit4p protein phosphatase" },
  { "PHS1", "Essential 3-hydroxyacyl-CoA dehydratase of the ER membrane" },
  { "MRPL49", "Mitochondrial ribosomal protein of the large subunit" },
  { "BCK1", "MAPKKK acting in the protein kinase C signaling pathway" },
  { "KHA1", "Putative K+/H+ antiporter" },
  { "TOK1", "Outward-rectifier potassium channel of the plasma membrane" },
  { "SRS2", "DNA helicase and DNA-dependent ATPase" },
  { "GWT1", "Protein involved in the inositol acylation of GlcN-PI" },
  { "DPB11", "DNA replication initiation protein" },
  { "SIP4", "C6 zinc cluster transcriptional activator" },
  { "ARG3", "Ornithine carbamoyltransferase" },
  { "TRL1", "tRNA ligase" },
  { "YJL086C", "NULL" },
  { "EXO70", "Subunit of the exocyst complex" },
  { "ALY2", "Alpha arrestin" },
  { "TAX4", "EH domain-containing protein" },
  { "IML2", "Protein required for clearance of inclusion bodies" },
  { "ARP4", "Nuclear actin-related protein involved in chromatin remodeling" },
  { "SCP160", "Essential RNA-binding G protein effector of mating response pathway" },
  { "PRY1", "Sterol binding protein involved in the export of acetylated sterols" },
  { "PRY3", "Cell wall-associated protein involved in export of acetylated sterols" },
  { "YJL077W-B", "Putative protein of unknown function" },
  { "ICS3", "Protein with a role in copper homeostasis" },
  { "YJL077W-A", "Protein of unknown function" },
  { "NET1", "Core subunit of the RENT complex" },
  { "APQ13", "NULL" },
  { "SMC3", "Subunit of the multiprotein cohesin complex" },
  { "JEM1", "DnaJ-like chaperone required for nuclear membrane fusion during mating" },
  { "PSF2", "Subunit of the GINS complex (Sld5p, Psf1p, Psf2p, Psf3p)" },
  { "ARG2", "Acetylglutamate synthase (glutamate N-acetyltransferase)" },
  { "YJL070C", "Putative metallo-dependent hydrolase superfamily protein" },
  { "UTP18", "Small-subunit processome protein involved in pre-18S rRNA maturation" },
  { "YJL068C", "Esterase that can function as an S-formylglutathione hydrolase" },
  { "YJL067W", "NULL" },
  { "MPM1", "Mitochondrial intermembrane space protein of unknown function" },
  { "DLS1", "Subunit of ISW2/yCHRAC chromatin accessibility complex" },
  { "YJL064W", "NULL" },
  { "MRPL8", "Mitochondrial ribosomal protein of the large subunit" },
  { "COA3", "Mitochondrial protein required for cytochrome c oxidase assembly" },
  { "LAS21", "Integral plasma membrane protein" },
  { "NUP82", "Linker nucleoporin component of the nuclear pore complex (NPC)" },
  { "BNA3", "Kynurenine aminotransferase" },
  { "YHC3", "Protein required for the ATP-dependent transport of arginine" },
  { "BIT61", "Subunit of TORC2 membrane-associated complex" },
  { "IKS1", "Protein kinase of unknown cellular role" },
  { "ZAP1", "Zinc-regulated transcription factor" },
  { "YJL055W", "NULL" },
  { "TIM54", "Component of the mitochondrial TIM22 complex" },
  { "PEP8", "Vacuolar protein component of the retromer" },
  { "YJL052C-A", "Putative protein of unknown function" },
  { "TDH1", "Glyceraldehyde-3-phosphate dehydrogenase (GAPDH), isozyme 1" },
  { "IRC8", "Bud tip localized protein of unknown function" },
  { "MTR4", "ATP-dependent 3'-5' RNA helicase of the DExD/H family" },
  { "YJL049W", "NULL" },
  { "UBX6", "UBX (ubiquitin regulatory X) domain-containing protein" },
  { "YJL047C-A", "Putative protein of unknown function" },
  { "RTT101", "Cullin subunit of a Roc1p-dependent E3 ubiquitin ligase complex" },
  { "AIM22", "Putative lipoate-protein ligase" },
  { "YJL045W", "Minor succinate dehydrogenase isozyme" },
  { "GYP6", "GTPase-activating protein (GAP) for yeast Rab family member Ypt6p" },
  { "YJL043W", "Putative protein of unknown function" },
  { "MHP1", "Microtubule-associated protein involved in microtubule organization" },
  { "NSP1", "FG-nucleoporin component of central core of the nuclear pore complex" },
  { "NUP192", "Essential subunit of inner ring of nuclear pore complex (NPC)" },
  { "LOH1", "Protein involved in outer spore wall assembly" },
  { "IRC18", "Protein involved in outer spore wall assembly" },
  { "SNX4", "Sorting nexin" },
  { "TAD2", "Subunit of tRNA-specific adenosine-34 deaminase" },
  { "KAR2", "ATPase involved in protein import into the ER" },
  { "HCA4", "DEAD box RNA helicase" },
  { "YJL032W", "NULL" },
  { "BET4", "Alpha subunit of Type II geranylgeranyltransferase" },
  { "MAD2", "Component of the spindle-assembly checkpoint complex" },
  { "VPS53", "Component of the GARP (Golgi-associated retrograde protein) complex" },
  { "YJL028W", "Protein of unknown function" },
  { "YJL027C", "Putative protein of unknown function" },
  { "RNR2", "Ribonucleotide-diphosphate reductase (RNR), small subunit" },
  { "YJL026C-A", "NULL" },
  { "RRN7", "Component of the core factor (CF) rDNA transcription factor complex" },
  { "APS3", "Small subunit of the clathrin-associated adaptor complex AP-3" },
  { "PET130", "Protein of unknown function" },
  { "YJL022W", "NULL" },
  { "BBC1", "Protein possibly involved in assembly of actin patches" },
  { "YJL020W-A", "NULL" },
  { "MPS3", "Nuclear envelope protein" },
  { "YJL016W", "NULL" },
  { "YJL015C", "NULL" },
  { "CCT3", "Subunit of the cytosolic chaperonin Cct ring complex" },
  { "MAD3", "Subunit of spindle-assembly checkpoint complex" },
  { "VTC4", "Vacuolar membrane polyphosphate polymerase" },
  { "RPC17", "RNA polymerase III subunit C17" },
  { "NOP9", "Essential subunit of U3-containing 90S preribosome" },
  { "YJL009W", "NULL" },
  { "CCT8", "Subunit of the cytosolic chaperonin Cct ring complex" },
  { "YJL007C", "Putative protein of unknown function" },
  { "CTK2", "Beta subunit of C-terminal domain kinase I (CTDK-I)" },
  { "CYR1", "Adenylate cyclase" },
  { "SYS1", "Integral membrane protein of the Golgi" },
  { "COX16", "Mitochondrial inner membrane protein" },
  { "OST1", "Alpha subunit of the oligosaccharyltransferase complex of the ER lumen" },
  { "PRE3", "Beta 1 subunit of the 20S proteasome" },
  { "AVT1", "Vacuolar transporter" },
  { "MPP10", "Component of the SSU processome and 90S preribosome" },
  { "YJR003C", "NULL" },
  { "SAG1", "Alpha-agglutinin of alpha-cells" },
  { "APL1", "Beta-adaptin" },
  { "YJR005C-A", "NULL" },
  { "POL31", "Subunit of DNA polymerase delta (polymerase III)" },
  { "SUI2", "Alpha subunit of the translation initiation factor eIF2" },
  { "MHO1", "Protein of unknown function" },
  { "TDH2", "Glyceraldehyde-3-phosphate dehydrogenase (GAPDH), isozyme 2" },
  { "MET3", "ATP sulfurylase" },
  { "SPC1", "Subunit of the signal peptidase complex (SPC)" },
  { "YJR011C", "Putative protein of unknown function" },
  { "YJR012C", "Essential protein of unknown function" },
  { "GPI14", "Glycosylphosphatidylinositol-alpha 1,4 mannosyltransferase I" },
  { "TMA22", "Protein of unknown function" },
  { "YJR015W", "Putative protein of unknown function" },
  { "ILV3", "Dihydroxyacid dehydratase" },
  { "ESS1", "Peptidylprolyl-cis/trans-isomerase (PPIase)" },
  { "YJR018W", "NULL" },
  { "TES1", "Peroxisomal acyl-CoA thioesterase" },
  { "YJR020W", "NULL" },
  { "REC107", "Protein involved in early stages of meiotic recombination" },
  { "LSM8", "Lsm (Like Sm) protein" },
  { "YJR023C", "NULL" },
  { "MDE1", "5'-methylthioribulose-1-phosphate dehydratase" },
  { "BNA1", "3-hydroxyanthranilic acid dioxygenase" },
  { "YJR030C", "NULL" },
  { "GEA1", "Guanine nucleotide exchange factor for ADP ribosylation factors (ARFs)" },
  { "CPR7", "Peptidyl-prolyl cis-trans isomerase (cyclophilin)" },
  { "RAV1", "Subunit of RAVE complex (Rav1p, Rav2p, Skp1p)" },
  { "PET191", "Protein required for assembly of cytochrome c oxidase" },
  { "RAD26", "Protein involved in transcription-coupled nucleotide excision repair" },
  { "HUL4", "Protein with similarity to hect domain E3 ubiquitin-protein ligases" },
  { "YJR037W", "NULL" },
  { "YJR038C", "NULL" },
  { "YJR039W", "NULL" },
  { "GEF1", "Voltage-gated chloride channel" },
  { "URB2", "Protein required for normal metabolism of the rRNA primary transcript" },
  { "NUP85", "Subunit of the Nup84p subcomplex of the nuclear pore complex (NPC)" },
  { "POL32", "Third subunit of DNA polymerase delta" },
  { "VPS55", "Late endosomal protein involved in late endosome to vacuole transport" },
  { "SSC1", "Hsp70 family ATPase" },
  { "TAH11", "DNA replication licensing factor" },
  { "ANB1", "Translation elongation factor eIF-5A" },
  { "CYC1", "Cytochrome c, isoform 1" },
  { "UTR1", "ATP-NADH kinase" },
  { "ISY1", "Member of the NineTeen Complex (NTC)" },
  { "OSM1", "Fumarate reductase, catalyzes the reduction of fumarate to succinate" },
  { "RAD7", "Nucleotide excision repair (NER) protein" },
  { "BFA1", "Subunit of a two-component GTPase-activating protein, Bfa1p-Bub2p" },
  { "KCH1", "Potassium transporter that mediates K+ influx" },
  { "HIT1", "Protein involved in C/D snoRNP assembly" },
  { "YJR056C", "Protein of unknown function" },
  { "CDC8", "Nucleoside monophosphate and nucleoside diphosphate kinase" },
  { "APS2", "Small subunit of the clathrin-associated adaptor complex AP-2" },
  { "PTK2", "Serine/threonine protein kinase" },
  { "CBF1", "Basic helix-loop-helix (bHLH) protein" },
  { "YJR061W", "NULL" },
  { "NTA1", "Amidase" },
  { "RPA12", "RNA polymerase I subunit A12.2" },
  { "CCT5", "Subunit of the cytosolic chaperonin Cct ring complex" },
  { "ARP3", "Essential component of the Arp2/3 complex" },
  { "TOR1", "PIK-related protein kinase and rapamycin target" },
  { "YAE1", "Substrate-specific adaptor protein involved in apo-Rli1p maturation" },
  { "RFC2", "Subunit of heteropentameric Replication factor C (RF-C)" },
  { "HAM1", "Nucleoside triphosphate pyrophosphohydrolase" },
  { "LIA1", "Deoxyhypusine hydroxylase" },
  { "YJR071W", "NULL" },
  { "NPA3", "Member of the conserved GPN-loop GTPase family" },
  { "OPI3", "Methylene-fatty-acyl-phospholipid synthase" },
  { "MOG1", "Conserved nuclear protein that interacts with GTP-Gsp1p" },
  { "HOC1", "Alpha-1,6-mannosyltransferase" },
  { "CDC11", "Component of the septin ring that is required for cytokinesis" },
  { "MIR1", "Mitochondrial phosphate carrier" },
  { "BNA2", "Tryptophan 2,3-dioxygenase or indoleamine 2,3-dioxygenase" },
  { "YJR079W", "Putative protein of unknown function" },
  { "AIM24", "Protein with a role in determining mitochondrial architecture" },
  { "EAF6", "Subunit of the NuA4 acetyltransferase complex" },
  { "ACF4", "Protein of unknown function" },
  { "YJR084W", "Protein that forms a complex with Thp3p" },
  { "YJR085C", "NULL" },
  { "STE18", "G protein gamma subunit" },
  { "YJR087W", "NULL" },
  { "EMC2", "Member of conserved ER transmembrane complex" },
  { "BIR1", "Subunit of chromosomal passenger complex (CPC)" },
  { "GRR1", "F-box protein component of an SCF ubiquitin-ligase complex" },
  { "JSN1", "Member of the Puf family of RNA-binding proteins" },
  { "BUD4", "Anillin-like protein involved in bud-site selection" },
  { "FIP1", "Subunit of cleavage polyadenylation factor (CPF)" },
  { "IME1", "Master regulator of meiosis that is active only during meiotic events" },
  { "RPL43B", "Ribosomal 60S subunit protein L43B" },
  { "SFC1", "Mitochondrial succinate-fumarate transporter" },
  { "YJR096W", "Xylose and arabinose reductase" },
  { "JJJ3", "Protein of unknown function" },
  { "YJR098C", "Putative protein of unknown function" },
  { "YUH1", "Ubiquitin C-terminal hydrolase" },
  { "AIM25", "Mitochondria protein of unknown function" },
  { "RSM26", "Mitochondrial ribosomal protein of the small subunit" },
  { "VPS25", "Component of the ESCRT-II complex" },
  { "URA8", "Minor CTP synthase isozyme (see also URA7)" },
  { "SOD1", "Cytosolic copper-zinc superoxide dismutase" },
  { "ADO1", "Adenosine kinase" },
  { "ECM27", "Protein involved in calcium homeostasis and exit from quiescence" },
  { "YJR107W", "NULL" },
  { "ABM1", "Protein of unknown function" },
  { "CPA2", "Large subunit of carbamoyl phosphate synthetase" },
  { "YMR1", "Phosphatidylinositol 3-phosphate (PI3P) phosphatase" },
  { "YJR111C", "NULL" },
  { "NNF1", "Essential component of the outer kinetochore MIND complex" },
  { "YJR112W-A", "Putative protein of unknown function" },
  { "RSM7", "Mitochondrial ribosomal protein of the small subunit" },
  { "YJR114W", "NULL" },
  { "YJR115W", "Putative protein of unknown function" },
  { "TDA4", "Putative protein of unknown function" },
  { "STE24", "Highly conserved zinc metalloprotease" },
  { "ILM1", "Protein of unknown function" },
  { "JHD2", "JmjC domain family histone demethylase" },
  { "YJR120W", "Protein of unknown function" },
  { "ATP2", "Beta subunit of the F1 sector of mitochondrial F1F0 ATP synthase" },
  { "IBA57", "Protein involved in incorporating iron-sulfur clusters into proteins" },
  { "RPS5", "Protein component of the small (40S) ribosomal subunit" },
  { "YJR124C", "Putative protein of unknown function" },
  { "ENT3", "Protein containing an N-terminal epsin-like domain" },
  { "VPS70", "Protein of unknown function involved in vacuolar protein sorting" },
  { "RSF2", "Zinc-finger protein" },
  { "YJR128W", "Putative protein of unknown function" },
  { "YJR129C", "NULL" },
  { "STR2", "Cystathionine gamma-synthase, converts cysteine into cystathionine" },
  { "MNS1", "Alpha-1,2-mannosidase" },
  { "NMD5", "Karyopherin" },
  { "XPT1", "Xanthine-guanine phosphoribosyl transferase" },
  { "SGM1", "Protein of unknown function" },
  { "MCM22", "Outer kinetochore protein and component of the Ctf3 subcomplex" },
  { "TIM8", "Mitochondrial intermembrane space protein" },
  { "TTI2", "Subunit of the ASTRA complex, involved in chromatin remodeling" },
  { "MET5", "Sulfite reductase beta subunit" },
  { "IML1", "GTPase-activating protein (GAP) subunit of the Iml1p/SEACIT complex" },
  { "HOM6", "Homoserine dehydrogenase (L-homoserine:NADP oxidoreductase)" },
  { "HIR3", "Subunit of the HIR complex" },
  { "YJR140W-A", "NULL" },
  { "YJR141W", "NULL" },
  { "YJR142W", "8-oxo-dGTP diphosphatase of the Nudix hydrolase family" },
  { "PMT4", "Protein O-mannosyltransferase" },
  { "MGM101", "Protein with a role in mitochondrial DNA recombinational repair" },
  { "RPS4A", "Protein component of the small (40S) ribosomal subunit" },
  { "YJR146W", "Protein of unknown function" },
  { "HMS2", "Protein with similarity to heat shock transcription factors" },
  { "BAT2", "Cytosolic branched-chain amino acid (BCAA) aminotransferase" },
  { "YJR149W", "Putative protein of unknown function" },
  { "DAN1", "Cell wall mannoprotein" },
  { "DAN4", "Cell wall mannoprotein" },
  { "YJR151W-A", "Putative protein of unknown function" },
  { "DAL5", "Allantoate permease" },
  { "PGU1", "Endo-polygalacturonase" },
  { "YJR154W", "Putative protein of unknown function" },
  { "AAD10", "Putative aryl-alcohol dehydrogenase" },
  { "THI11", "Protein involved in synthesis of the thiamine precursor HMP" },
  { "YJR157W", "NULL" },
  { "HXT16", "Putative transmembrane polyol transporter" },
  { "SOR1", "Sorbitol dehydrogenase" },
  { "MPH3", "Alpha-glucoside permease" },
  { "COS5", "Endosomal protein involved in turnover of plasma membrane proteins" },
  { "YJR162C", "NULL" },
  { "YKL225W", "NULL" },
  { "PAU16", "Protein of unknown function" },
  { "YKL223W", "NULL" },
  { "YKL222C", "Protein of unknown function" },
  { "MCH2", "Protein with similarity to mammalian monocarboxylate permeases" },
  { "FRE2", "Ferric reductase and cupric reductase" },
  { "COS9", "Endosomal protein involved in turnover of plasma membrane proteins" },
  { "SRY1", "3-hydroxyaspartate dehydratase" },
  { "JEN1", "Monocarboxylate/proton symporter of the plasma membrane" },
  { "URA1", "Dihydroorotate dehydrogenase" },
  { "OXP1", "5-oxoprolinase" },
  { "YRA2", "Member of the REF (RNA and export factor binding proteins) family" },
  { "DOA1", "WD-repeat protein involved in ubiquitin-mediated protein degradation" },
  { "SAC1", "Phosphatidylinositol phosphate (PtdInsP) phosphatase" },
  { "TRP3", "Indole-3-glycerol-phosphate synthase" },
  { "UBA1", "Ubiquitin activating enzyme (E1)" },
  { "STE6", "Plasma membrane ATP-binding cassette (ABC) transporter" },
  { "CBT1", "Protein involved in 5' RNA end processing" },
  { "EMC3", "Member of conserved ER transmembrane complex" },
  { "ADD66", "Protein involved in 20S proteasome assembly" },
  { "LOS1", "Nuclear pore protein" },
  { "EAP1", "eIF4E-associated protein, competes with eIF4G for binding to eIF4E" },
  { "TOR2", "PIK-related protein kinase and rapamycin target" },
  { "YKL202W", "NULL" },
  { "MNN4", "Putative positive regulator of mannosylphosphate transferase Mnn6p" },
  { "PTK1", "Putative serine/threonine protein kinase" },
  { "PEX1", "AAA-peroxin" },
  { "YKT6", "Vesicle membrane protein (v-SNARE) with acyltransferase activity" },
  { "MIA40", "Import and assembly protein in mitochondrial intermembrane space" },
  { "MST1", "Mitochondrial threonyl-tRNA synthetase" },
  { "SDS22", "Regulatory subunit of the type 1 protein phosphatase (PP1) Glc7p" },
  { "ACP1", "Mitochondrial matrix acyl carrier protein" },
  { "DPH2", "Protein required for synthesis of diphthamide" },
  { "CNB1", "Calcineurin B" },
  { "HYM1", "Component of the RAM signaling network" },
  { "PXA2", "Subunit of heterodimeric peroxisomal ABC transport complex, with Pxa1p" },
  { "FAT3", "Protein required for fatty acid uptake" },
  { "MTR2", "mRNA transport regulator" },
  { "ASH1", "Component of the Rpd3L histone deacetylase complex" },
  { "SPE1", "Ornithine decarboxylase" },
  { "YKL183C-A", "Putative protein of unknown function" },
  { "LOT5", "Protein of unknown function" },
  { "FAS1", "Beta subunit of fatty acid synthetase" },
  { "PRS1", "5-phospho-ribosyl-1(alpha)-pyrophosphate synthetase" },
  { "RPL17A", "Ribosomal 60S subunit protein L17A" },
  { "COY1", "Golgi membrane protein with similarity to mammalian CASP" },
  { "STE3", "Receptor for a factor pheromone" },
  { "YKL177W", "NULL" },
  { "LST4", "Subunit of the Lst4p-Lst7p GTPase activating protein complex for Gtr2p" },
  { "ZRT3", "Vacuolar membrane zinc transporter" },
  { "TPO5", "Protein involved in excretion of putrescine and spermidine" },
  { "SNU114", "GTPase component of U5 snRNP involved in mRNA splicing via spliceosome" },
  { "EBP2", "Required for 25S rRNA maturation and 60S ribosomal subunit assembly" },
  { "NNK1", "Protein kinase" },
  { "MRPL38", "Mitochondrial ribosomal protein of the large subunit" },
  { "YKL169C", "NULL" },
  { "KKQ8", "Putative serine/threonine protein kinase with unknown cellular role" },
  { "MRP49", "Mitochondrial ribosomal protein of the large subunit" },
  { "TPK3", "cAMP-dependent protein kinase catalytic subunit" },
  { "YKL165C-A", "NULL" },
  { "MCD4", "Protein involved in GPI anchor synthesis" },
  { "PIR1", "O-glycosylated protein required for cell wall stability" },
  { "PIR3", "O-glycosylated covalently-bound cell wall protein" },
  { "YKL162C-A", "NULL" },
  { "YKL162C", "Putative protein of unknown function" },
  { "KDX1", "Protein kinase" },
  { "ELF1", "Transcription elongation factor with a conserved zinc finger domain" },
  { "RCN1", "Protein involved in calcineurin regulation during calcium signaling" },
  { "APE2", "Aminopeptidase yscII" },
  { "RPS27A", "Protein component of the small (40S) ribosomal subunit" },
  { "YKL156C-A", "NULL" },
  { "RSM22", "Mitochondrial ribosomal protein of the small subunit" },
  { "SRP102", "Signal recognition particle (SRP) receptor beta subunit" },
  { "YKL153W", "NULL" },
  { "GPM1", "Tetrameric phosphoglycerate mutase" },
  { "YKL151C", "NULL" },
  { "MCR1", "Mitochondrial NADH-cytochrome b5 reductase" },
  { "DBR1", "RNA lariat debranching enzyme" },
  { "SDH1", "Flavoprotein subunit of succinate dehydrogenase" },
  { "YKL147C", "NULL" },
  { "AVT3", "Vacuolar transporter" },
  { "RPT1", "ATPase of the 19S regulatory particle of the 26S proteasome" },
  { "YKL145W-A", "NULL" },
  { "RPC25", "RNA polymerase III subunit C25" },
  { "LTV1", "Subunit of the EGO/GSE complex" },
  { "MRP8", "Protein of unknown function" },
  { "SDH3", "Subunit of succinate dehydrogenase and of TIM22 translocase" },
  { "TGL1", "Steryl ester hydrolase" },
  { "CTK1", "Catalytic (alpha) subunit of C-terminal domain kinase I (CTDK-I)" },
  { "HSK3", "Essential subunit of the Dam1 complex (aka DASH complex)" },
  { "MRPL31", "Mitochondrial ribosomal protein of the large subunit" },
  { "CMC1", "Copper-binding protein of the mitochondrial intermembrane space" },
  { "YKL136W", "NULL" },
  { "APL2", "Beta-adaptin subunit of the clathrin-associated protein (AP-1) complex" },
  { "OCT1", "Mitochondrial intermediate peptidase" },
  { "YKL133C", "NULL" },
  { "RMA1", "Putative dihydrofolate synthetase" },
  { "YKL131W", "NULL" },
  { "SHE2", "RNA-binding protein that binds specific mRNAs and interacts with She3p" },
  { "MYO3", "One of two type I myosins" },
  { "PMU1", "Putative phosphomutase" },
  { "PGM1", "Phosphoglucomutase, minor isoform" },
  { "YPK1", "S/T protein kinase" },
  { "RRN3", "Protein required for transcription of rDNA by RNA polymerase I" },
  { "SSH4", "Specificity factor required for Rsp5p-dependent ubiquitination" },
  { "YKL123W", "NULL" },
  { "SRP21", "Subunit of the signal recognition particle (SRP)" },
  { "DGR2", "Protein of unknown function" },
  { "OAC1", "Mitochondrial inner membrane transporter" },
  { "VPH2", "Integral membrane protein required for V-ATPase function" },
  { "YKL118W", "NULL" },
  { "SBA1", "Co-chaperone that binds and regulates Hsp90 family chaperones" },
  { "PRR1", "Serine/threonine protein kinase" },
  { "YKL115C", "NULL" },
  { "APN1", "Major apurinic/apyrimidinic endonuclease" },
  { "RAD27", "5' to 3' exonuclease, 5' flap endonuclease" },
  { "ABF1", "DNA binding protein with possible chromatin-reorganizing activity" },
  { "YKL111C", "NULL" },
  { "KTI12", "Protein that plays a role in modification of tRNA wobble nucleosides" },
  { "HAP4", "Transcription factor" },
  { "SLD2", "Single-stranded DNA origin-binding and annealing protein" },
  { "YKL107W", "Putative short-chain dehydrogenase/reductase" },
  { "YKL106C-A", "Putative protein of unknown function" },
  { "AAT1", "Mitochondrial aspartate aminotransferase" },
  { "SEG2", "Eisosome component" },
  { "GFA1", "Glutamine-fructose-6-phosphate amidotransferase" },
  { "APE1", "Vacuolar aminopeptidase yscI" },
  { "YKL102C", "Putative protein of unknown function" },
  { "HSL1", "Nim1p-related protein kinase" },
  { "YKL100C", "NULL" },
  { "YKL100W-A", "NULL" },
  { "UTP11", "Subunit of U3-containing Small Subunit (SSU) processome complex" },
  { "MTC2", "Protein of unknown function" },
  { "YKL097C", "Putative protein of unknown function" },
  { "YKL096C-B", "Putative protein of unknown function" },
  { "CWP2", "Covalently linked cell wall mannoprotein" },
  { "CWP1", "Cell wall mannoprotein that localizes to birth scars of daughter cells" },
  { "YJU2", "Essential protein required for pre-mRNA splicing" },
  { "YJU3", "Monoglyceride lipase (MGL)" },
  { "MBR1", "Protein involved in mitochondrial functions and stress response" },
  { "BUD2", "GTPase activating factor for Rsr1p/Bud1p" },
  { "YKL091C", "Putative phosphatidylinositol/phosphatidylcholine transfer protein" },
  { "CUE2", "Ubiquitin-binding endonuclease involved in mRNA decay" },
  { "MIF2", "Centromeric CDEIII region binding protein" },
  { "CAB3", "Subunit of PPCDC and CoA-SPC complexes involved in CoA biosynthesis" },
  { "CYT2", "Cytochrome c1 heme lyase" },
  { "SRX1", "Sulfiredoxin" },
  { "MDH1", "Mitochondrial malate dehydrogenase" },
  { "HOT13", "Zinc-binding mitochondrial intermembrane space (IMS) protein" },
  { "YKL083W", "NULL" },
  { "RRP14", "Essential protein, constituent of 66S pre-ribosomal particles" },
  { "TEF4", "Gamma subunit of translational elongation factor eEF1B" },
  { "VMA5", "Subunit C of the V1 peripheral membrane domain of V-ATPase" },
  { "SMY1", "Kinesin-like myosin passenger-protein" },
  { "DHR2", "Predominantly nucleolar DEAH-box ATP-dependent RNA helicase" },
  { "YKL077W", "NULL" },
  { "PSY1", "NULL" },
  { "YKL075C", "Putative protein of unknown function" },
  { "MUD2", "Protein involved in early pre-mRNA splicing" },
  { "LHS1", "Molecular chaperone of the endoplasmic reticulum lumen" },
  { "STB6", "Protein that binds Sin3p in a two-hybrid assay" },
  { "YKL071W", "NADH-dependent aldehyde reductase" },
  { "YKL070W", "Putative protein of unknown function" },
  { "YKL069W", "Methionine-R-sulfoxide reductase" },
  { "YKL068W-A", "Putative protein of unknown function" },
  { "NUP100", "FG-nucleoporin component of central core of the nuclear pore complex" },
  { "YNK1", "Nucleoside diphosphate kinase" },
  { "YKL066W", "NULL" },
  { "YKL065W-A", "NULL" },
  { "YET1", "Endoplasmic reticulum transmembrane protein" },
  { "MNR2", "Vacuolar membrane protein required for magnesium homeostasis" },
  { "YKL063C", "Putative protein of unknown function" },
  { "MSN4", "Stress-responsive transcriptional activator" },
  { "BLI1", "Subunit of the BLOC-1 complex involved in endosomal maturation" },
  { "FBA1", "Fructose 1,6-bisphosphate aldolase" },
  { "MPE1", "Essential conserved subunit of CPF cleavage and polyadenylation factor" },
  { "TOA2", "TFIIA small subunit" },
  { "NUP120", "Subunit of the Nup84p subcomplex of the nuclear pore complex (NPC)" },
  { "TMA19", "Protein that associates with ribosomes" },
  { "OAR1", "Mitochondrial 3-oxoacyl-[acyl-carrier-protein] reductase" },
  { "DEF1", "RNAPII degradation factor" },
  { "MDM35", "Mitochondrial intermembrane space protein" },
  { "YKL053W", "NULL" },
  { "ASK1", "Essential subunit of the Dam1 complex (aka DASH complex)" },
  { "SFK1", "Plasma membrane protein that may act to generate normal levels of PI4P" },
  { "YKL050C", "Protein of unknown function" },
  { "CSE4", "Centromeric histone H3-like protein" },
  { "ELM1", "Serine/threonine protein kinase" },
  { "ANR2", "Protein of unknown function" },
  { "DCW1", "Putative mannosidase" },
  { "PRI2", "Subunit of DNA primase" },
  { "YKL044W", "NULL" },
  { "PHD1", "Transcriptional activator that enhances pseudohyphal growth" },
  { "SPC42", "Central plaque component of spindle pole body (SPB)" },
  { "VPS24", "One of four subunits of the ESCRT-III complex" },
  { "NFU1", "Protein involved in Fe-S cluster transfer to mitochondrial clients" },
  { "PTM1", "Protein of unknown function" },
  { "RGT1", "Glucose-responsive transcription factor" },
  { "AIM26", "Protein of unknown function" },
  { "YKL036C", "NULL" },
  { "UGP1", "UDP-glucose pyrophosphorylase (UGPase)" },
  { "TUL1", "Subunit of the DSC ubiquitin ligase complex" },
  { "YKL033W-A", "Putative protein of unknown function" },
  { "TTI1", "Subunit of the ASTRA complex, involved in chromatin remodeling" },
  { "IXR1", "Transcriptional repressor that regulates hypoxic genes during normoxia" },
  { "YKL031W", "NULL" },
  { "YKL030W", "NULL" },
  { "MAE1", "Mitochondrial malic enzyme" },
  { "TFA1", "TFIIE large subunit" },
  { "TCD2", "tRNA threonylcarbamoyladenosine dehydratase" },
  { "GPX1", "Phospholipid hydroperoxide glutathione peroxidase" },
  { "PAN3", "Essential subunit of the Pan2p-Pan3p poly(A)-ribonuclease complex" },
  { "URA6", "Uridylate kinase" },
  { "YKL023C-A", "NULL" },
  { "YKL023W", "NULL" },
  { "CDC16", "Subunit of the anaphase-promoting complex/cyclosome (APC/C)" },
  { "MAK11", "Protein involved in an early step of 60S ribosomal subunit biogenesis" },
  { "SPT23", "ER membrane protein involved in regulation of OLE1 transcription" },
  { "RAM2", "Alpha subunit of farnesyltransferase and geranylgeranyltransferase-I" },
  { "YKL018C-A", "NULL" },
  { "SWD2", "Subunit of the COMPASS (Set1C) histone H3K4 methyltransferase complex" },
  { "HCS1", "Hexameric DNA polymerase alpha-associated DNA helicase A" },
  { "ATP7", "Subunit d of the stator stalk of mitochondrial F1F0 ATP synthase" },
  { "PUT3", "Transcriptional activator" },
  { "URB1", "Protein required for the normal accumulation of 25S and 5.8S rRNAs" },
  { "ARC19", "Subunit of the ARP2/3 complex" },
  { "PRP40", "U1 snRNP protein involved in splicing" },
  { "CCE1", "Mitochondrial cruciform cutting endonuclease" },
  { "UFD4", "Ubiquitin-protein ligase (E3)" },
  { "MRT4", "Protein involved in mRNA turnover and ribosome assembly" },
  { "LAC1", "Ceramide synthase component" },
  { "CAP1", "Alpha subunit of the capping protein heterodimer (Cap1p and Cap2p)" },
  { "SFT1", "Intra-Golgi v-SNARE" },
  { "RPL14A", "Ribosomal 60S subunit protein L14A" },
  { "BYE1", "Negative regulator of transcription elongation" },
  { "AUR1", "Phosphatidylinositol:ceramide phosphoinositol transferase" },
  { "MRP17", "Mitochondrial ribosomal protein of the small subunit" },
  { "DID4", "Class E Vps protein of the ESCRT-III complex" },
  { "MET14", "Adenylylsulfate kinase" },
  { "VPS1", "Dynamin-like GTPase required for vacuolar sorting" },
  { "PAP1", "Poly(A) polymerase" },
  { "OSH6", "Member of an oxysterol-binding protein family" },
  { "ECM9", "Non-essential protein of unknown function" },
  { "YKR005C", "Putative protein of unknown function" },
  { "MRPL13", "Mitochondrial ribosomal protein of the large subunit" },
  { "MEH1", "Subunit of the EGO/GSE complex" },
  { "RSC4", "Component of the RSC chromatin remodeling complex" },
  { "FOX2", "3-hydroxyacyl-CoA dehydrogenase and enoyl-CoA hydratase" },
  { "TOF2", "Protein required for rDNA silencing and mitotic rDNA condensation" },
  { "YKR011C", "Protein of unknown function" },
  { "YKR012C", "NULL" },
  { "PRY2", "Sterol binding protein involved in the export of acetylated sterols" },
  { "YPT52", "Endosomal Rab family GTPase" },
  { "YKR015C", "Putative protein of unknown function" },
  { "MIC60", "Component of the MICOS complex" },
  { "HEL1", "RING finger ubiquitin ligase (E3)" },
  { "YKR018C", "Protein of unknown function" },
  { "IRS4", "EH domain-containing protein" },
  { "VPS51", "Component of the GARP (Golgi-associated retrograde protein) complex" },
  { "ALY1", "Alpha arrestin, substrate of calcineurin" },
  { "NTR2", "Essential protein that forms a dimer with Ntr1p" },
  { "YKR023W", "NULL" },
  { "DBP7", "Putative ATP-dependent RNA helicase of the DEAD-box family" },
  { "RPC37", "RNA polymerase III subunit C37" },
  { "GCN3", "Alpha subunit of translation initiation factor eIF2B" },
  { "BCH2", "Member of the ChAPs (Chs5p-Arf1p-binding proteins) family" },
  { "SAP190", "Protein that forms a complex with the Sit4p protein phosphatase" },
  { "SET3", "Defining member of the SET3 histone deacetylase complex" },
  { "GMH1", "Golgi membrane protein of unknown function" },
  { "SPO14", "Phospholipase D" },
  { "YKR032W", "Putative protein of unknown function" },
  { "YKR033C", "NULL" },
  { "DAL80", "Negative regulator of genes in multiple nitrogen degradation pathways" },
  { "OPI8", "NULL" },
  { "DID2", "Class E protein of the vacuolar protein-sorting (Vps) pathway" },
  { "CAF4", "WD40 repeat-containing protein associated with the CCR4-NOT complex" },
  { "SPC34", "Essential subunit of the Dam1 complex (aka DASH complex)" },
  { "KAE1", "Highly conserved ATPase of HSP70/DnaK family" },
  { "GAP1", "General amino acid permease" },
  { "YKR040C", "NULL" },
  { "YKR041W", "Protein of unknown function" },
  { "UTH1", "Mitochondrial inner membrane protein" },
  { "SHB17", "Sedoheptulose bisphosphatase involved in riboneogenesis" },
  { "UIP5", "Protein of unknown function that interacts with Ulp1p" },
  { "YKR045C", "Putative protein of unknown function" },
  { "PET10", "NULL" },
  { "YKR047W", "NULL" },
  { "NAP1", "Histone chaperone" },
  { "FMP46", "Putative redox protein containing a thioredoxin fold" },
  { "TRK2", "Component of the Trk1p-Trk2p potassium transport system" },
  { "YKR051W", "NULL" },
  { "MRS4", "Iron transporter of the mitochondrial carrier family" },
  { "YSR3", "Dihydrosphingosine 1-phosphate phosphatase" },
  { "DYN1", "Cytoplasmic heavy chain dynein" },
  { "RHO4", "Non-essential small GTPase" },
  { "TRM2", "tRNA methyltransferase" },
  { "RPS21A", "Protein component of the small (40S) ribosomal subunit" },
  { "GLG1", "Glycogenin glucosyltransferase" },
  { "TIF1", "Translation initiation factor eIF4A" },
  { "UTP30", "Putative subunit of U3-containing 90S preribosome complex" },
  { "KTR2", "Mannosyltransferase involved in N-linked protein glycosylation" },
  { "TFA2", "TFIIE small subunit" },
  { "LAS1", "Endonuclease involved in pre-rRNA processing at both ends of ITS2" },
  { "OAF3", "Putative transcriptional repressor with Zn(2)-Cys(6) finger" },
  { "PAM17", "Constituent of the TIM23 complex" },
  { "CCP1", "Mitochondrial cytochrome-c peroxidase" },
  { "GPT2", "Glycerol-3-phosphate/dihydroxyacetone phosphate sn-1 acyltransferase" },
  { "BET3", "Core component of transport protein particle (TRAPP) complexes I-III" },
  { "MET1", "S-adenosyl-L-methionine uroporphyrinogen III transmethylase" },
  { "YKR070W", "Putative protein of unknown function" },
  { "DRE2", "Component of the cytosolic Fe-S protein assembly (CIA) machinery" },
  { "SIS2", "Negative regulatory subunit of protein phosphatase 1 (Ppz1p)" },
  { "YKR073C", "Putative protein of unknown function" },
  { "AIM29", "Protein of unknown function" },
  { "YKR075C", "Protein of unknown function" },
  { "YKR075W-A", "NULL" },
  { "ECM4", "S-glutathionyl-(chloro)hydroquinone reductase (GS-HQR)" },
  { "MSA2", "Putative transcriptional activator" },
  { "YKR078W", "Cytoplasmic protein of unknown function" },
  { "TRZ1", "tRNA 3'-end processing endonuclease tRNase Z" },
  { "MTD1", "NAD-dependent 5,10-methylenetetrahydrafolate dehydrogenase" },
  { "RPF2", "Essential protein involved in rRNA maturation and ribosomal assembly" },
  { "NUP133", "Subunit of Nup84p subcomplex of nuclear pore complex (NPC)" },
  { "DAD2", "Essential subunit of the Dam1 complex (aka DASH complex)" },
  { "HBS1", "GTPase with similarity to translation release factors" },
  { "MRPL20", "Mitochondrial ribosomal protein of the large subunit" },
  { "PRP16", "DEAH-box RNA helicase involved in second catalytic step of splicing a" },
  { "OMA1", "Metalloendopeptidase of the mitochondrial inner membrane" },
  { "TVP38", "Integral membrane protein" },
  { "TGL4", "Multifunctional lipase/hydrolase/phospholipase" },
  { "PXL1", "Protein that localizes to sites of polarized growth" },
  { "SRL3", "GTB motif (G1/S transcription factor binding) containing protein" },
  { "SRP40", "Nucleolar serine-rich protein" },
  { "PTR2", "Integral membrane peptide transporter" },
  { "RPL40B", "Ubiquitin-ribosomal 60S subunit protein L40B fusion protein" },
  { "MLP1", "Myosin-like protein associated with the nuclear envelope" },
  { "PCC1", "Component of the EKC/KEOPS protein complex" },
  { "ESL2", "hEST1A/B (SMG5/6)-like protein" },
  { "PCK1", "Phosphoenolpyruvate carboxykinase" },
  { "UBP11", "Ubiquitin-specific protease" },
  { "BAS1", "Myb-related transcription factor" },
  { "SKG1", "Transmembrane protein with a role in cell wall polymer composition" },
  { "SIR1", "Protein involved in silencing at mating-type loci HML and HMR" },
  { "FLO10", "Member of the FLO family of cell wall flocculation proteins" },
  { "NFT1", "Putative transporter of the MRP subfamily" },
  { "YKR104W", "Putative transporter of the MRP subfamily" },
  { "VBA5", "Plasma membrane protein of the Major Facilitator Superfamily (MFS)" },
  { "GEX2", "Proton:glutathione antiporter" },
  { "YLL067C", "Putative Y' element ATP-dependent helicase" },
  { "YLL067W-A", "NULL" },
  { "YLL066W-B", "Putative protein of unknown function" },
  { "YLL066C", "Putative Y' element ATP-dependent helicase" },
  { "YLL066W-A", "NULL" },
  { "YLL065W", "NULL" },
  { "PAU18", "Protein of unknown function" },
  { "AYT1", "Acetyltransferase" },
  { "MHT1", "S-methylmethionine-homocysteine methyltransferase" },
  { "MMP1", "High-affinity S-methylmethionine permease" },
  { "GTT2", "Glutathione S-transferase capable of homodimerization" },
  { "YLL059C", "NULL" },
  { "YLL058W", "Putative protein of unknown function with similarity to Str2p" },
  { "JLP1", "Fe(II)-dependent sulfonate/alpha-ketoglutarate dioxygenase" },
  { "YLL056C", "Putative protein of unknown function" },
  { "YCT1", "High-affinity cysteine-specific transporter" },
  { "YLL054C", "Putative protein of unknown function with similarity to Pip2p" },
  { "YLL053C", "Putative protein" },
  { "AQY2", "Water channel that mediates water transport across cell membranes" },
  { "FRE6", "Putative ferric reductase with similarity to Fre2p" },
  { "COF1", "Cofilin, involved in pH-dependent actin filament depolarization" },
  { "LDB18", "Component of the dynactin complex" },
  { "YBT1", "Transporter of the ATP-binding cassette (ABC) family" },
  { "YLL047W", "NULL" },
  { "RNP1", "Ribonucleoprotein that contains two RNA recognition motifs (RRM)" },
  { "RPL8B", "Ribosomal 60S subunit protein L8B" },
  { "YLL044W", "NULL" },
  { "FPS1", "Aquaglyceroporin, plasma membrane channel" },
  { "ATG10", "Conserved E2-like conjugating enzyme" },
  { "SDH2", "Iron-sulfur protein subunit of succinate dehydrogenase" },
  { "VPS13", "Protein involved in prospore membrane morphogenesis" },
  { "UBI4", "Ubiquitin" },
  { "ENT4", "Protein of unknown function" },
  { "YLL037W", "NULL" },
  { "PRP19", "Splicing factor associated with the spliceosome" },
  { "GRC3", "Polynucleotide kinase present on rDNA" },
  { "RIX7", "Putative ATPase of the AAA family" },
  { "IRC19", "Protein of unknown function" },
  { "YLL032C", "Protein of unknown function" },
  { "GPI13", "ER membrane localized phosphoryltransferase" },
  { "RRT7", "Putative protein of unknown function" },
  { "FRA1", "Protein involved in negative regulation of iron regulon transcription" },
  { "TPO1", "Polyamine transporter of the major facilitator superfamily" },
  { "ISA1", "Protein required for maturation of mitochondrial [4Fe-4S] proteins" },
  { "HSP104", "Disaggregase" },
  { "PAU17", "Protein of unknown function" },
  { "SSA2", "HSP70 family ATP-binding protein" },
  { "POM33", "Transmembrane nucleoporin" },
  { "HIF1", "Non-essential component of the HAT-B histone acetyltransferase complex" },
  { "SPA2", "Component of the polarisome" },
  { "YLL020C", "NULL" },
  { "KNS1", "Protein kinase involved in negative regulation of PolIII transcription" },
  { "YLL019W-A", "NULL" },
  { "COX19", "Protein required for cytochrome c oxidase assembly" },
  { "DPS1", "Aspartyl-tRNA synthetase, primarily cytoplasmic" },
  { "BPT1", "ABC type transmembrane transporter of MRP/CFTR family" },
  { "EMC6", "Member of conserved ER transmembrane complex" },
  { "PUF3", "Protein of the mitochondrial outer surface" },
  { "YEH1", "Steryl ester hydrolase" },
  { "SOF1", "Protein required for biogenesis of 40S (small) ribosomal subunit" },
  { "PSR1", "Plasma membrane-associated protein phosphatase" },
  { "COX17", "Copper metallochaperone that transfers copper to Sco1p and Cox11p" },
  { "DRS1", "Nucleolar DEAD-box protein required for ribosome assembly and function" },
  { "YLL007C", "NULL" },
  { "YLL006W-A", "Putative protein of unknown function" },
  { "MMM1", "ER integral membrane protein, ERMES complex subunit" },
  { "SPO75", "Meiosis-specific protein of unknown function" },
  { "ORC3", "Subunit of the origin recognition complex (ORC)" },
  { "SFI1", "Centrin (Cdc31p)-binding protein required for SPB duplication" },
  { "RTT109", "Histone acetyltransferase" },
  { "DNM1", "Dynamin-related GTPase involved in mitochondrial organization" },
  { "YLR001C", "Putative protein of unknown function" },
  { "NOC3", "Subunit of a nuclear complex with Noc2p and pre-replicative complexes" },
  { "CMS1", "Putative subunit of the 90S preribosome processome complex" },
  { "THI73", "Putative plasma membrane permease" },
  { "SSL1", "Subunit of the core form of RNA polymerase transcription factor TFIIH" },
  { "SSK1", "Cytoplasmic phosphorelay intermediate osmosensor and regulator" },
  { "NSE1", "Component of the SMC5-SMC6 complex" },
  { "PAM18", "Subunit of the import motor (PAM complex)" },
  { "RLP24", "Essential protein required for ribosomal large subunit biogenesis" },
  { "TEN1", "Protein that regulates telomeric length" },
  { "LOT6", "FMN-dependent NAD(P)H:quinone reductase" },
  { "YLR012C", "Putative protein of unknown function" },
  { "GAT3", "Protein containing GATA family zinc finger motifs" },
  { "PPR1", "Zinc finger transcription factor" },
  { "BRE2", "Subunit of COMPASS (Set1C) complex" },
  { "PML1", "Subunit of the RES complex" },
  { "MEU1", "Methylthioadenosine phosphorylase (MTAP)" },
  { "POM34", "Subunit of the transmembrane ring of the nuclear pore complex (NPC)" },
  { "PSR2", "Putative plasma membrane phosphatase" },
  { "YEH2", "Steryl ester hydrolase" },
  { "IRC25", "Component of a heterodimeric Poc4p-Irc25p chaperone" },
  { "SDO1", "Guanine nucleotide exchange factor (GEF) for Ria1p" },
  { "IZH3", "Membrane protein involved in zinc ion homeostasis" },
  { "UBR2", "Cytoplasmic ubiquitin-protein ligase (E3)" },
  { "SNF7", "One of four subunits of the ESCRT-III complex" },
  { "SED5", "cis-Golgi t-SNARE syntaxin" },
  { "AAT2", "Cytosolic aspartate aminotransferase involved in nitrogen metabolism" },
  { "ADE16", "Enzyme of 'de novo' purine biosynthesis" },
  { "RPL15A", "Ribosomal 60S subunit protein L15A" },
  { "YLR030W", "Putative protein of unknown function" },
  { "YLR031W", "Putative protein of unknown function" },
  { "RAD5", "DNA helicase/Ubiquitin ligase" },
  { "RSC58", "Component of the RSC chromatin remodeling complex" },
  { "SMF3", "Putative divalent metal ion transporter involved in iron homeostasis" },
  { "MLH2", "Protein involved in mismatch repair and meiotic recombination" },
  { "YLR036C", "Putative protein predicted to have transmembrane domains" },
  { "PAU23", "Cell wall mannoprotein" },
  { "COX12", "Subunit VIb of cytochrome c oxidase" },
  { "RIC1", "Protein involved in retrograde transport to the cis-Golgi network" },
  { "AFB1", "MATalpha-specific a-factor blocker" },
  { "YLR041W", "NULL" },
  { "YLR042C", "Cell wall protein of unknown function" },
  { "TRX1", "Cytoplasmic thioredoxin isoenzyme" },
  { "PDC1", "Major of three pyruvate decarboxylase isozymes" },
  { "STU2", "Microtubule-associated protein (MAP) of the XMAP215/Dis1 family" },
  { "YLR046C", "Putative membrane protein" },
  { "FRE8", "Protein with sequence similarity to iron/copper reductases" },
  { "RPS0B", "Protein component of the small (40S) ribosomal subunit" },
  { "YLR049C", "NULL" },
  { "YLR050C", "NULL" },
  { "FCF2", "Nucleolar protein involved in the early steps of 35S rRNA processing" },
  { "IES3", "Subunit of the INO80 chromatin remodeling complex" },
  { "YLR053C", "Putative protein of unknown function" },
  { "OSW2", "Protein of unknown function reputedly involved in spore wall assembly" },
  { "SPT8", "Subunit of the SAGA transcriptional regulatory complex" },
  { "ERG3", "C-5 sterol desaturase" },
  { "MNL2", "Putative mannosidase involved in ER-associated protein degradation" },
  { "SHM2", "Cytosolic serine hydroxymethyltransferase" },
  { "REX2", "3'-5' RNA exonuclease" },
  { "FRS1", "Beta subunit of cytoplasmic phenylalanyl-tRNA synthetase" },
  { "RPL22A", "Ribosomal 60S subunit protein L22A" },
  { "BUD28", "NULL" },
  { "BMT6", "Methyltransferase required for m3U2843 methylation of the 25S rRNA" },
  { "PER33", "Protein that localizes to the endoplasmic reticulum" },
  { "ENV10", "NULL" },
  { "SPC3", "Subunit of signal peptidase complex" },
  { "PET309", "Specific translational activator for the COX1 mRNA" },
  { "FYV7", "Essential protein required for maturation of 18S rRNA" },
  { "MEF1", "Mitochondrial elongation factor involved in translational elongation" },
  { "XYL2", "Xylitol dehydrogenase" },
  { "RGR1", "Subunit of the RNA polymerase II mediator complex" },
  { "YLR072W", "NULL" },
  { "RFU1", "Protein that inhibits Doa4p deubiquitinating activity" },
  { "BUD20", "C2H2-type zinc finger protein required for ribosome assembly" },
  { "RPL10", "Ribosomal 60S subunit protein L10" },
  { "YLR076C", "NULL" },
  { "FMP25", "Protein required for assembly of respiratory complex III" },
  { "BOS1", "v-SNARE (vesicle specific SNAP receptor)" },
  { "SIC1", "Cyclin-dependent kinase inhibitor (CKI)" },
  { "EMP46", "Integral membrane component of ER-derived COPII-coated vesicles" },
  { "GAL2", "Galactose permease" },
  { "SRL2", "Protein of unknown function" },
  { "EMP70", "Protein with a role in cellular adhesion and filamentous growth" },
  { "RAX2", "N-glycosylated protein" },
  { "ARP6", "Actin-related protein that binds nucleosomes" },
  { "SMC4", "Subunit of the condensin complex" },
  { "CSF1", "Protein required for fermentation at low temperature" },
  { "GAA1", "Subunit of the GPI:protein transamidase complex" },
  { "ALT1", "Alanine transaminase (glutamic pyruvic transaminase)" },
  { "XDJ1", "Chaperone with a role in facilitating mitochondrial protein import" },
  { "GEP5", "Protein of unknown function" },
  { "SUL2", "High affinity sulfate permease" },
  { "NYV1", "v-SNARE component of the vacuolar SNARE complex" },
  { "GIS3", "Protein of unknown function" },
  { "IOC2", "Subunit of the Isw1b complex" },
  { "KIN2", "S/T protein kinase" },
  { "HRT3", "Putative SCF-ubiquitin ligase F-box protein" },
  { "CHA4", "DNA binding transcriptional activator" },
  { "ICT1", "Lysophosphatidic acid acyltransferase" },
  { "MIM2", "Mitochondrial protein required for outer membrane protein import" },
  { "ERG27", "3-keto sterol reductase" },
  { "YLR101C", "NULL" },
  { "APC9", "Subunit of the Anaphase-Promoting Complex/Cyclosome (APC/C)" },
  { "CDC45", "DNA replication initiation factor" },
  { "LCL2", "Putative protein of unknown function" },
  { "SEN2", "Subunit of the tRNA splicing endonuclease" },
  { "MDN1", "NULL" },
  { "REX3", "RNA exonuclease" },
  { "YLR108C", "Protein of unknown function" },
  { "AHP1", "Thiol-specific peroxiredoxin" },
  { "CCW12", "Cell wall mannoprotein" },
  { "YLR111W", "Putative protein of unknown function" },
  { "YLR112W", "Putative protein of unknown function" },
  { "HOG1", "Mitogen-activated protein kinase involved in osmoregulation" },
  { "AVL9", "Conserved protein involved in exocytic transport from the Golgi" },
  { "CFT2", "Subunit of the mRNA cleavage and polyadenlylation factor (CPF)" },
  { "MSL5", "Component of commitment complex" },
  { "CLF1", "Member of the NineTeen Complex (NTC)" },
  { "YLR118C", "NULL" },
  { "SRN2", "Component of the ESCRT-I complex" },
  { "YPS1", "Aspartic protease" },
  { "YLR120W-A", "NULL" },
  { "YPS3", "Aspartic protease" },
  { "YLR122C", "Putative protein of unknown function" },
  { "YLR123C", "NULL" },
  { "YLR124W", "NULL" },
  { "YLR125W", "Putative protein of unknown function" },
  { "YLR126C", "Putative glutamine amidotransferase" },
  { "APC2", "Subunit of the Anaphase-Promoting Complex/Cyclosome (APC/C)" },
  { "DCN1", "Scaffold-type E3 ligase" },
  { "DIP2", "Nucleolar protein" },
  { "ZRT2", "Low-affinity zinc transporter of the plasma membrane" },
  { "ACE2", "Transcription factor required for septum destruction after cytokinesis" },
  { "USB1", "Putative poly(U)-specific 3'-to-5' RNA exonuclease" },
  { "CKI1", "Choline kinase" },
  { "PDC5", "Minor isoform of pyruvate decarboxylase" },
  { "SLX4", "Endonuclease involved in processing DNA" },
  { "TIS11", "mRNA-binding protein expressed during iron starvation" },
  { "RKM5", "Protein lysine methyltransferase" },
  { "NHA1", "Na+/H+ antiporter" },
  { "SLS1", "Mitochondrial membrane protein" },
  { "YLR140W", "NULL" },
  { "RRN5", "Protein involved in transcription of rDNA by RNA polymerase I" },
  { "PUT1", "Proline oxidase" },
  { "DPH6", "Diphthamide synthetase" },
  { "ACF2", "Intracellular beta-1,3-endoglucanase" },
  { "RMP1", "Subunit of RNase MRP" },
  { "SPE4", "Spermine synthase" },
  { "YLR146W-A", "Putative protein of unknown function" },
  { "SMD3", "Core Sm protein Sm D3" },
  { "PEP3", "Component of CORVET membrane tethering complex" },
  { "YLR149C", "Protein of unknown function" },
  { "YLR149C-A", "NULL" },
  { "STM1", "Protein required for optimal translation under nutrient stress" },
  { "PCD1", "8-oxo-dGTP diphosphatase" },
  { "YLR152C", "Putative protein of unknown function" },
  { "ACS2", "Acetyl-coA synthetase isoform" },
  { "RNH203", "Ribonuclease H2 subunit" },
  { "YLR154W-A", "NULL" },
  { "YLR154W-B", "NULL" },
  { "TAR1", "Protein potentially involved in regulation of respiratory metabolism" },
  { "YLR154W-E", "NULL" },
  { "YLR154W-F", "NULL" },
  { "YLR154C-G", "Putative protein of unknown function" },
  { "YLR154C-H", "Putative protein of unknown function" },
  { "ASP3-1", "Cell-wall L-asparaginase II involved in asparagine catabolism" },
  { "YLR156W", "Putative protein of unknown function" },
  { "YLR156C-A", "Putative protein of unknown function" },
  { "ASP3-2", "Cell-wall L-asparaginase II involved in asparagine catabolism" },
  { "YLR157W-D", "Putative protein of unknown function" },
  { "YLR157W-E", "Putative protein of unknown function identified by gene-trapping" },
  { "YLR157C-C", "Putative protein of unknown function" },
  { "ASP3-3", "Cell-wall L-asparaginase II involved in asparagine catabolism" },
  { "YLR159W", "Putative protein of unknown function" },
  { "YLR159C-A", "Putative protein of unknown function" },
  { "ASP3-4", "Cell-wall L-asparaginase II involved in asparagine catabolism" },
  { "YLR161W", "Putative protein of unknown function" },
  { "YLR162W", "Protein of unknown function" },
  { "RRT15", "Putative protein of unknown function" },
  { "MAS1", "Beta subunit of the mitochondrial processing protease (MPP)" },
  { "YLR163W-A", "NULL" },
  { "SHH4", "Putative alternate subunit of succinate dehydrogenase (SDH)" },
  { "PUS5", "Pseudouridine synthase" },
  { "SEC10", "Essential 100kDa subunit of the exocyst complex" },
  { "RPS31", "Fusion protein cleaved to yield ribosomal protein S31 and ubiquitin" },
  { "UPS2", "Mitochondrial intermembrane space protein" },
  { "YLR169W", "NULL" },
  { "APS1", "Small subunit of the clathrin-associated adaptor complex AP-1" },
  { "YLR171W", "NULL" },
  { "DPH5", "Methyltransferase required for synthesis of diphthamide" },
  { "YLR173W", "Putative protein of unknown function" },
  { "IDP2", "Cytosolic NADP-specific isocitrate dehydrogenase" },
  { "CBF5", "Pseudouridine synthase catalytic subunit of box H/ACA snoRNPs" },
  { "RFX1", "Major transcriptional repressor of DNA-damage-regulated genes" },
  { "YLR177W", "Putative protein of unknown function" },
  { "TFS1", "Inhibitor of carboxypeptidase Y (Prc1p), and Ras GAP (Ira2p)" },
  { "YLR179C", "Protein of unknown function with similarity to Tfs1p" },
  { "SAM1", "S-adenosylmethionine synthetase" },
  { "VTA1", "Multivesicular body (MVB) protein" },
  { "SWI6", "Transcription cofactor" },
  { "TOS4", "Putative transcription factor, contains Forkhead Associated domain" },
  { "YLR184W", "NULL" },
  { "RPL37A", "Ribosomal 60S subunit protein L37A" },
  { "EMG1", "Methyltransferase for rRNA" },
  { "SKG3", "Protein of unknown function" },
  { "MDL1", "Mitochondrial inner membrane half-type ABC transporter" },
  { "ATG26", "UDP-glucose:sterol glucosyltransferase" },
  { "MMR1", "Phosphorylated protein of the mitochondrial outer membrane" },
  { "PEX13", "Peroxisomal importomer complex component" },
  { "HCR1", "eIF3j component of translation initiation factor 3 (eIF3)" },
  { "UPS1", "Phosphatidic acid transfer protein" },
  { "YLR194C", "NULL" },
  { "NMT1", "N-myristoyl transferase" },
  { "PWP1", "Protein with WD-40 repeats involved in rRNA processing" },
  { "NOP56", "Essential evolutionarily-conserved nucleolar protein" },
  { "YLR198C", "NULL" },
  { "PBA1", "Protein involved in 20S proteasome assembly" },
  { "YKE2", "Subunit of the heterohexameric Gim/prefoldin protein complex" },
  { "COQ9", "Protein required for ubiquinone biosynthesis and respiratory growth" },
  { "YLR202C", "NULL" },
  { "MSS51", "Specific translational activator for the mitochondrial COX1 mRNA" },
  { "QRI5", "Mitochondrial inner membrane protein" },
  { "HMX1", "ER localized heme oxygenase" },
  { "ENT2", "Epsin-like protein required for endocytosis and actin patch assembly" },
  { "HRD3", "ER membrane protein that plays a central role in ERAD" },
  { "SEC13", "Structural component of 3 complexes" },
  { "PNP1", "Purine nucleoside phosphorylase" },
  { "CLB4", "B-type cyclin involved in cell cycle progression" },
  { "ATG38", "Homodimeric subunit of autophagy-specific PtdIns-3-kinase complex I" },
  { "TUB4", "Gamma-tubulin" },
  { "CRR1", "Putative glycoside hydrolase of the spore wall envelope" },
  { "FRE1", "Ferric reductase and cupric reductase" },
  { "CDC123", "Assembly factor for the eIF2 translation initiation factor complex" },
  { "CPR6", "Peptidyl-prolyl cis-trans isomerase (cyclophilin)" },
  { "YLR217W", "NULL" },
  { "COA4", "Twin Cx(9)C protein involved in cytochrome c oxidase organization" },
  { "MSC3", "Protein of unknown function" },
  { "CCC1", "Vacuolar Fe2+/Mn2+ transporter" },
  { "RSA3", "Protein with a likely role in ribosomal maturation" },
  { "UTP13", "Nucleolar protein" },
  { "YLR222C-A", "NULL" },
  { "IFH1", "Coactivator, regulates transcription of ribosomal protein (RP) genes" },
  { "YLR224W", "NULL" },
  { "YLR225C", "Putative protein of unknown function" },
  { "BUR2", "Cyclin for the Sgv1p (Bur1p) protein kinase" },
  { "ADY4", "Structural component of the meiotic outer plaque" },
  { "ECM22", "Sterol regulatory element binding protein" },
  { "CDC42", "Small rho-like GTPase" },
  { "YLR230W", "NULL" },
  { "BNA5", "Kynureninase" },
  { "YLR232W", "NULL" },
  { "EST1", "TLC1 RNA-associated factor involved in telomere length regulation" },
  { "TOP3", "DNA Topoisomerase III" },
  { "YLR235C", "NULL" },
  { "YLR236C", "Putative protein of unknown function" },
  { "THI7", "Plasma membrane transporter responsible for the uptake of thiamine" },
  { "FAR10", "Protein involved in recovery from arrest in response to pheromone" },
  { "LIP2", "Lipoyl ligase" },
  { "VPS34", "Phosphatidylinositol (PI) 3-kinase that synthesizes PI-3-phosphate" },
  { "YLR241W", "NULL" },
  { "ARV1", "Cortical ER protein" },
  { "GPN3", "Putative GTPase with a role in biogenesis of RNA pol II and polIII" },
  { "MAP1", "Methionine aminopeptidase" },
  { "CDD1", "Cytidine deaminase" },
  { "ERF2", "Subunit of a palmitoyltransferase" },
  { "IRC20", "E3 ubiquitin ligase and putative helicase" },
  { "RCK2", "Protein kinase involved in response to oxidative and osmotic stress" },
  { "YEF3", "Translation elongation factor 3" },
  { "SSP120", "Protein of unknown function" },
  { "SYM1", "Protein required for ethanol metabolism" },
  { "YLR252W", "NULL" },
  { "MCP2", "Mitochondrial protein of unknown function involved in lipid homeostas" },
  { "NDL1", "Homolog of nuclear distribution factor NudE" },
  { "YLR255C", "Putative protein of unknown function" },
  { "HAP1", "Zinc finger transcription factor" },
  { "YLR257W", "Protein of unknown function" },
  { "GSY2", "Glycogen synthase" },
  { "HSP60", "Tetradecameric mitochondrial chaperonin" },
  { "LCB5", "Minor sphingoid long-chain base kinase" },
  { "VPS63", "Putative protein of unknown function" },
  { "YPT6", "Rab family GTPase" },
  { "TMA7", "Protein of unknown that associates with ribosomes" },
  { "RED1", "Protein component of the synaptonemal complex axial elements" },
  { "RPS28B", "Protein component of the small (40S) ribosomal subunit" },
  { "YLR264C-A", "Putative protein of unknown function" },
  { "NEJ1", "Protein involved in regulation of nonhomologous end joining" },
  { "PDR8", "Transcription factor" },
  { "BOP2", "Protein of unknown function" },
  { "SEC22", "R-SNARE protein" },
  { "YLR269C", "NULL" },
  { "DCS1", "Non-essential hydrolase involved in mRNA decapping" },
  { "YLR271W", "NULL" },
  { "YCS4", "Subunit of the condensin complex" },
  { "PIG1", "Putative targeting subunit for type-1 protein phosphatase Glc7p" },
  { "MCM5", "Component of the Mcm2-7 hexameric helicase complex" },
  { "SMD2", "Core Sm protein Sm D2" },
  { "DBP9", "DEAD-box protein required for 27S rRNA processing" },
  { "YSH1", "Endoribonuclease" },
  { "YLR278C", "Zinc-cluster protein" },
  { "YLR279W", "NULL" },
  { "YLR280C", "NULL" },
  { "YLR281C", "NULL" },
  { "YLR282C", "NULL" },
  { "YLR283W", "Putative protein of unknown function" },
  { "ECI1", "Peroxisomal delta3,delta2-enoyl-CoA isomerase" },
  { "NNT1", "S-adenosylmethionine-dependent methyltransferase" },
  { "YLR285C-A", "Putative protein of unknown function" },
  { "CTS1", "Endochitinase" },
  { "YLR286W-A", "NULL" },
  { "YLR287C", "Putative protein of unknown function" },
  { "RPS30A", "Protein component of the small (40S) ribosomal subunit" },
  { "MEC3", "DNA damage and meiotic pachytene checkpoint protein" },
  { "GUF1", "Mitochondrial matrix GTPase" },
  { "YLR290C", "NULL" },
  { "GCD7", "Beta subunit of the translation initiation factor eIF2B" },
  { "SEC72", "Non-essential subunit of Sec63 complex" },
  { "GSP1", "Ran GTPase" },
  { "YLR294C", "NULL" },
  { "ATP14", "Subunit h of the F0 sector of mitochondrial F1F0 ATP synthase" },
  { "YLR296W", "Putative protein of unknown function" },
  { "YLR297W", "Protein of unknown function" },
  { "YHC1", "Component of the U1 snRNP complex required for pre-mRNA splicing" },
  { "ECM38", "Gamma-glutamyltranspeptidase" },
  { "YLR299C-A", "NULL" },
  { "EXG1", "Major exo-1,3-beta-glucanase of the cell wall" },
  { "HRI1", "Protein of unknown function that interacts with Sec72p and Hrr25p" },
  { "YLR302C", "Putative protein of unknown function" },
  { "MET17", "O-acetyl homoserine-O-acetyl serine sulfhydrylase" },
  { "ACO1", "Aconitase" },
  { "STT4", "Phosphatidylinositol-4-kinase" },
  { "UBC12", "Enzyme that mediates the conjugation of Rub1p" },
  { "CDA1", "Chitin deacetylase" },
  { "YLR307C-A", "NULL" },
  { "CDA2", "Chitin deacetylase" },
  { "IMH1", "Protein involved in vesicular transport" },
  { "CDC25", "Membrane bound guanine nucleotide exchange factor" },
  { "YLR311C", "NULL" },
  { "YLR312C", "NULL" },
  { "MRPL15", "Mitochondrial ribosomal protein of the large subunit" },
  { "SPH1", "Protein involved in shmoo formation and bipolar bud site selection" },
  { "CDC3", "Component of the septin ring that is required for cytokinesis" },
  { "NKP2", "Central kinetochore protein and subunit of the Ctf19 complex" },
  { "TAD3", "Subunit of tRNA-specific adenosine-34 deaminase" },
  { "YLR317W", "NULL" },
  { "EST2", "Reverse transcriptase subunit of the telomerase holoenzyme" },
  { "BUD6", "Actin- and formin-interacting protein" },
  { "MMS22", "Subunit of E3 ubiquitin ligase complex involved in replication repair" },
  { "SFH1", "Component of the RSC chromatin remodeling complex" },
  { "VPS65", "NULL" },
  { "CWC24", "General splicing factor" },
  { "PEX30", "ER-resident protein involved in peroxisomal biogenesis" },
  { "RPL38", "Ribosomal 60S subunit protein L38" },
  { "YLR326W", "Putative protein of unknown function" },
  { "TMA10", "Protein of unknown function that associates with ribosomes" },
  { "NMA1", "Nicotinic acid mononucleotide adenylyltransferase" },
  { "REC102", "Protein involved in early stages of meiotic recombination" },
  { "CHS5", "Component of the exomer complex" },
  { "JIP3", "Putative protein of unknown function" },
  { "MID2", "O-glycosylated plasma membrane protein" },
  { "RPS25B", "Protein component of the small (40S) ribosomal subunit" },
  { "YLR334C", "NULL" },
  { "NUP2", "Nucleoporin involved in nucleocytoplasmic transport" },
  { "SGD1", "Essential nuclear protein" },
  { "VRP1", "Verprolin, proline-rich actin-associated protein" },
  { "OPI9", "NULL" },
  { "YLR339C", "NULL" },
  { "RPP0", "Conserved ribosomal protein P0 of the ribosomal stalk" },
  { "SPO77", "Meiosis-specific protein of unknown function" },
  { "FKS1", "Catalytic subunit of 1,3-beta-D-glucan synthase" },
  { "YLR342W-A", "Putative protein of unknown function" },
  { "GAS2", "1,3-beta-glucanosyltransferase" },
  { "RPL26A", "Ribosomal 60S subunit protein L26A" },
  { "YLR345W", "Similar to 6-phosphofructo-2-kinase enzymes" },
  { "YLR346C", "NULL" },
  { "KAP95", "Karyopherin beta" },
  { "YLR347W-A", "NULL" },
  { "DIC1", "Mitochondrial dicarboxylate carrier" },
  { "YLR349W", "NULL" },
  { "ORM2", "Protein that mediates sphingolipid homeostasis" },
  { "NIT3", "Nit protein" },
  { "YLR352W", "NULL" },
  { "BUD8", "Protein involved in bud-site selection" },
  { "TAL1", "Transaldolase, enzyme in the non-oxidative pentose phosphate pathway" },
  { "ILV5", "Acetohydroxyacid reductoisomerase and mtDNA binding protein" },
  { "ATG33", "Mitochondrial mitophagy-specific protein" },
  { "RSC2", "Component of the RSC chromatin remodeling complex" },
  { "YLR358C", "Protein of unknown function" },
  { "ADE13", "Adenylosuccinate lyase" },
  { "VPS38", "Part of a Vps34p phosphatidylinositol 3-kinase complex" },
  { "DCR2", "Protein phosphatase" },
  { "YLR361C-A", "Putative protein of unknown function" },
  { "STE11", "Signal transducing MEK kinase" },
  { "NMD4", "Protein that may be involved in nonsense-mediated mRNA decay" },
  { "YLR363W-A", "Protein of unknown function" },
  { "GRX8", "Glutaredoxin that employs a dithiol mechanism of catalysis" },
  { "YLR365W", "Putative protein of unknown function" },
  { "YLR364C-A", "NULL" },
  { "YLR366W", "NULL" },
  { "RPS22B", "Protein component of the small (40S) ribosomal subunit" },
  { "MDM30", "F-box component of an SCF ubiquitin protein ligase complex" },
  { "SSQ1", "Mitochondrial hsp70-type molecular chaperone" },
  { "ARC18", "Subunit of the ARP2/3 complex" },
  { "ROM2", "Guanine nucleotide exchange factor (GEF) for Rho1p and Rho2p" },
  { "ELO3", "Elongase" },
  { "VID22", "Glycosylated integral membrane protein localized to plasma membrane" },
  { "YLR374C", "NULL" },
  { "STP3", "Zinc-finger protein of unknown function" },
  { "PSY3", "Component of Shu complex (aka PCSS complex)" },
  { "FBP1", "Fructose-1,6-bisphosphatase" },
  { "SEC61", "Conserved ER protein translocation channel" },
  { "YLR379W", "NULL" },
  { "CSR1", "Phosphatidylinositol transfer protein" },
  { "CTF3", "Outer kinetochore protein that forms a complex with Mcm16p and Mcm22p" },
  { "NAM2", "Mitochondrial leucyl-tRNA synthetase" },
  { "SMC6", "Component of the SMC5-SMC6 complex" },
  { "IKI3", "Subunit of Elongator complex" },
  { "SWC7", "Protein of unknown function" },
  { "VAC14", "Enzyme regulator" },
  { "REH1", "Cytoplasmic 60S subunit biogenesis factor" },
  { "RPS29A", "Protein component of the small (40S) ribosomal subunit" },
  { "STE23", "Metalloprotease" },
  { "ECM19", "Protein of unknown function" },
  { "CCW14", "Covalently linked cell wall glycoprotein" },
  { "ART10", "Protein of unknown function that contains 2 PY motifs" },
  { "ATP10", "Assembly factor for the F0 sector of mitochondrial F1F0 ATP synthase" },
  { "CST9", "SUMO E3 ligase" },
  { "COX8", "Subunit VIII of cytochrome c oxidase (Complex IV)" },
  { "VPS33", "ATP-binding protein that is a subunit of the HOPS and CORVET complexes" },
  { "AFG2", "ATPase of the CDC48/PAS1/SEC18 (AAA) family, forms a hexameric complex" },
  { "SKI2", "Ski complex component and putative RNA helicase" },
  { "BDF1", "Protein involved in transcription initiation" },
  { "YLR399W-A", "NULL" },
  { "YLR400W", "NULL" },
  { "DUS3", "Dihydrouridine synthase" },
  { "YLR402W", "NULL" },
  { "SFP1", "Regulates transcription of ribosomal protein and biogenesis genes" },
  { "FLD1", "NULL" },
  { "DUS4", "Dihydrouridine synthase" },
  { "RPL31B", "Ribosomal 60S subunit protein L31B" },
  { "YLR406C-A", "Putative protein of unknown function" },
  { "YLR407W", "Putative protein of unknown function" },
  { "BLS1", "Subunit of the BLOC-1 complex involved in endosomal maturation" },
  { "UTP21", "Subunit of U3-containing 90S preribosome and SSU processome complexes" },
  { "VIP1", "Bifunctional inositol pyrophosphate kinase and phosphatase" },
  { "CTR3", "High-affinity copper transporter of the plasma membrane" },
  { "BER1", "Protein involved in microtubule-related processes" },
  { "YLR412C-A", "Putative protein of unknown function" },
  { "INA1", "Protein of unknown function" },
  { "PUN1", "Plasma membrane protein with a role in cell wall integrity" },
  { "YLR415C", "Putative protein of unknown function" },
  { "YLR416C", "NULL" },
  { "VPS36", "Component of the ESCRT-II complex" },
  { "CDC73", "Component of the Paf1p complex" },
  { "YLR419W", "Putative helicase with limited sequence similarity to human Rb protein" },
  { "URA4", "Dihydroorotase" },
  { "RPN13", "Subunit of the 19S regulatory particle of the 26S proteasome lid" },
  { "YLR422W", "NULL" },
  { "ATG17", "Scaffold protein responsible for phagophore assembly site organization" },
  { "SPP382", "Essential protein that forms a dimer with Ntr2p" },
  { "TUS1", "Guanine nucleotide exchange factor (GEF) that modulates Rho1p activity" },
  { "TDA5", "Putative protein of unknown function" },
  { "MAG2", "Cytoplasmic protein of unknown function" },
  { "YLR428C", "NULL" },
  { "CRN1", "Coronin" },
  { "SEN1", "ATP-dependent 5' to 3' RNA/DNA and DNA helicase" },
  { "ATG23", "Peripheral membrane protein required for autophagy and CVT" },
  { "IMD3", "Inosine monophosphate dehydrogenase" },
  { "CNA1", "Calcineurin A" },
  { "YLR434C", "NULL" },
  { "TSR2", "Protein with a potential role in pre-rRNA processing" },
  { "ECM30", "Protein of unknown function" },
  { "DIF1", "Protein that regulates nuclear localization of Rnr2p and Rnr4p" },
  { "YLR437C-A", "NULL" },
  { "CAR2", "L-ornithine transaminase (OTAse)" },
  { "LSM3", "Lsm (Like Sm) protein" },
  { "MRPL4", "Mitochondrial ribosomal protein of the large subunit" },
  { "SEC39", "Component of the Dsl1p tethering complex" },
  { "RPS1A", "Ribosomal protein of the small (40S) subunit" },
  { "SIR3", "Silencing protein" },
  { "ECM7", "Putative integral membrane protein with a role in calcium uptake" },
  { "YLR444C", "NULL" },
  { "GMC2", "Protein involved in meiotic crossing over" },
  { "YLR446W", "Putative hexokinase" },
  { "VMA6", "Subunit d of the V0 integral membrane domain of V-ATPase" },
  { "RPL6B", "Ribosomal 60S subunit protein L6B" },
  { "FPR4", "Peptidyl-prolyl cis-trans isomerase (PPIase)" },
  { "HMG2", "HMG-CoA reductase" },
  { "LEU3", "Zinc-knuckle transcription factor, repressor and activator" },
  { "SST2", "GTPase-activating protein for Gpa1p" },
  { "RIF2", "Protein that binds to the Rap1p C-terminus" },
  { "FMP27", "Putative protein of unknown function" },
  { "YLR455W", "NULL" },
  { "YLR456W", "Protein of unknown function" },
  { "NBP1", "Spindle pole body (SPB) component" },
  { "YLR458W", "NULL" },
  { "GAB1", "GPI transamidase subunit" },
  { "YLR460C", "Member of the quinone oxidoreductase family" },
  { "PAU4", "Member of the seripauperin multigene family" },
  { "YLR462W", "Putative protein of unknown function with similarity to helicases" },
  { "YLR463C", "NULL" },
  { "YLR464W", "Putative protein of unknown function" },
  { "BSC3", "NULL" },
  { "YRF1-4", "Helicase encoded by the Y' element of subtelomeric regions" },
  { "YLR466C-A", "NULL" },
  { "YLR466C-B", "NULL" },
  { "YRF1-5", "Helicase encoded by the Y' element of subtelomeric regions" },
  { "YLR467C-A", "NULL" },
  { "YML133C", "Putative Y' element ATP-dependent helicase" },
  { "YML133W-B", "NULL" },
  { "YML133W-A", "NULL" },
  { "COS3", "Endosomal protein involved in turnover of plasma membrane proteins" },
  { "YML131W", "Protein of unknown function" },
  { "ERO1", "Thiol oxidase required for oxidative protein folding in the ER" },
  { "COX14", "Mitochondrial cytochrome c oxidase (complex IV) assembly factor" },
  { "MSC1", "Protein of unknown function" },
  { "RSC9", "Component of the RSC chromatin remodeling complex" },
  { "ERG13", "3-hydroxy-3-methylglutaryl-CoA (HMG-CoA) synthase" },
  { "PGA3", "Putative cytochrome b5 reductase, localized to the plasma membrane" },
  { "TUB3", "Alpha-tubulin" },
  { "PHO84", "High-affinity inorganic phosphate (Pi) transporter" },
  { "YML122C", "Putative protein of unknown function" },
  { "GTR1", "Subunit of a TORC1-stimulating GTPase and the EGO/GSE complex" },
  { "NDI1", "NADH:ubiquinone oxidoreductase" },
  { "YML119W", "Putative protein of unknown function" },
  { "NGL3", "3'-5' exonuclease specific for poly-A RNAs" },
  { "NAB6", "Putative RNA-binding protein" },
  { "YML116W-A", "NULL" },
  { "ATR1", "Multidrug efflux pump of the major facilitator superfamily" },
  { "VAN1", "Component of the mannan polymerase I" },
  { "TAF8", "TFIID subunit (65 kDa)" },
  { "DAT1", "DNA binding protein that recognizes oligo(dA).oligo(dT) tracts" },
  { "CTK3", "Gamma subunit of C-terminal domain kinase I" },
  { "BUL2", "Component of the Rsp5p E3-ubiquitin ligase complex" },
  { "COQ5", "2-hexaprenyl-6-methoxy-1,4-benzoquinone methyltransferase" },
  { "ZDS2", "Protein with a role in regulating Swe1p-dependent polarized growth" },
  { "YML108W", "Protein of unknown function" },
  { "PML39", "Protein required for nuclear retention of unspliced pre-mRNAs" },
  { "URA5", "Major orotate phosphoribosyltransferase (OPRTase) isozyme" },
  { "SEC65", "Subunit of the signal recognition particle (SRP)" },
  { "MDM1", "PtdIns-3-P binding protein that tethers the ER to vacuoles at NVJs" },
  { "NUP188", "Subunit of the inner ring of the nuclear pore complex (NPC)" },
  { "CAC2", "Subunit of chromatin assembly factor I (CAF-1), with Rlf2p and Msi1p" },
  { "YML101C-A", "NULL" },
  { "CUE4", "Protein of unknown function" },
  { "YML100W-A", "Putative protein of unknown function" },
  { "TSL1", "Large subunit of trehalose 6-phosphate synthase/phosphatase complex" },
  { "YML099W-A", "NULL" },
  { "ARG81", "Zinc finger transcription factor involved in arginine-responsive genes" },
  { "TAF13", "TFIID subunit (19 kDa)" },
  { "VPS9", "Guanine nucleotide exchange factor (GEF) and ubiquitin receptor" },
  { "YML096W", "Putative protein with similarity to asparagine synthetases" },
  { "RAD10", "Single-stranded DNA endonuclease (with Rad1p)" },
  { "YML094C-A", "NULL" },
  { "GIM5", "Subunit of the heterohexameric cochaperone prefoldin complex" },
  { "UTP14", "Subunit of U3-containing Small Subunit (SSU) processome complex" },
  { "PRE8", "Alpha 2 subunit of the 20S proteasome" },
  { "RPM2", "Protein subunit of mitochondrial RNase P" },
  { "YML090W", "NULL" },
  { "YML089C", "NULL" },
  { "UFO1", "F-box receptor protein" },
  { "AIM33", "Protein of unknown function, highly conserved across species" },
  { "ALO1", "D-Arabinono-1,4-lactone oxidase" },
  { "TUB1", "Alpha-tubulin" },
  { "YML084W", "NULL" },
  { "YML083C", "Protein of unknown function" },
  { "YML082W", "Putative protein predicted to have carbon-sulfur lyase activity" },
  { "ATP18", "Subunit of the mitochondrial F1F0 ATP synthase" },
  { "TDA9", "Transcription factor that regulates acetate production" },
  { "DUS1", "Dihydrouridine synthase" },
  { "YML079W", "Non-essential protein of unknown function" },
  { "CPR3", "Mitochondrial peptidyl-prolyl cis-trans isomerase (cyclophilin)" },
  { "BET5", "Core component of transport protein particle (TRAPP) complexes I-III" },
  { "WAR1", "Homodimeric Zn2Cys6 zinc finger transcription factor" },
  { "HMG1", "HMG-CoA reductase" },
  { "FPR3", "Nucleolar peptidyl-prolyl cis-trans isomerase (PPIase)" },
  { "RPL6A", "Ribosomal 60S subunit protein L6A" },
  { "TCB3", "Cortical ER protein involved in ER-plasma membrane tethering" },
  { "COG8", "Component of the conserved oligomeric Golgi complex" },
  { "DAK1", "Dihydroxyacetone kinase" },
  { "POB3", "Subunit of the heterodimeric FACT complex (Spt16p-Pob3p)" },
  { "ITT1", "Protein that modulates the efficiency of translation termination" },
  { "ERV41", "Protein localized to COPII-coated vesicles" },
  { "SMA2", "Meiosis-specific prospore membrane protein" },
  { "ORC1", "Largest subunit of the origin recognition complex" },
  { "TEM1", "GTP-binding protein of the Ras superfamily" },
  { "RPS1B", "Ribosomal protein of the small (40S) subunit" },
  { "MFT1", "Subunit of the THO complex" },
  { "PIF1", "DNA helicase, potent G-quadruplex DNA binder/unwinder" },
  { "OGG1", "Nuclear and mitochondrial glycosylase/lyase" },
  { "NTE1", "Serine esterase" },
  { "HUG1", "Ribonucleotide reductase inhibitor" },
  { "SML1", "Ribonucleotide reductase inhibitor" },
  { "YML057C-A", "NULL" },
  { "CMP2", "Calcineurin A" },
  { "IMD4", "Inosine monophosphate dehydrogenase" },
  { "SPC2", "Subunit of signal peptidase complex" },
  { "CYB2", "Cytochrome b2 (L-lactate cytochrome-c oxidoreductase)" },
  { "YML054C-A", "Putative protein of unknown function" },
  { "YML053C", "Putative protein of unknown function" },
  { "SUR7", "Plasma membrane protein, component of eisosomes" },
  { "GAL80", "Transcriptional regulator involved in the repression of GAL genes" },
  { "AIM32", "Protein of unknown function" },
  { "RSE1", "Protein involved in pre-mRNA splicing" },
  { "GSF2", "Endoplasmic reticulum (ER) localized integral membrane protein" },
  { "YML047W-A", "NULL" },
  { "PRM6", "Potassium transporter that mediates K+ influx" },
  { "PRP39", "U1 snRNP protein involved in splicing" },
  { "RRN11", "Component of the core factor (CF) rDNA transcription factor complex" },
  { "CAT2", "Carnitine acetyl-CoA transferase" },
  { "VPS71", "Nucleosome-binding component of the SWR1 complex" },
  { "YMD8", "Putative nucleotide sugar transporter" },
  { "YML037C", "Putative protein of unknown function" },
  { "CGI121", "Component of the EKC/KEOPS complex" },
  { "AMD1", "AMP deaminase" },
  { "YML034C-A", "NULL" },
  { "SRC1", "Inner nuclear membrane protein" },
  { "RAD52", "Protein that stimulates strand exchange" },
  { "YML031C-A", "NULL" },
  { "NDC1", "Subunit of the transmembrane ring of the nuclear pore complex (NPC)" },
  { "RCF1", "Cytochrome c oxidase subunit" },
  { "USA1", "Scaffold subunit of the Hrd1p ubiquitin ligase" },
  { "TSA1", "Thioredoxin peroxidase" },
  { "YOX1", "Homeobox transcriptional repressor" },
  { "RPS18B", "Protein component of the small (40S) ribosomal subunit" },
  { "YML6", "Mitochondrial ribosomal protein of the large subunit" },
  { "RPS17A", "Ribosomal protein 51 (rp51) of the small (40s) subunit" },
  { "NSE5", "Component of the SMC5-SMC6 complex" },
  { "APT1", "Adenine phosphoribosyltransferase" },
  { "UNG1", "Uracil-DNA glycosylase" },
  { "YML020W", "Putative protein of unknown function" },
  { "OST6", "Subunit of the oligosaccharyltransferase complex of the ER lumen" },
  { "YML018C", "Protein of unknown function" },
  { "PSP2", "Asn rich cytoplasmic protein that contains RGG motifs" },
  { "PPZ1", "Serine/threonine protein phosphatase Z, isoform of Ppz2p" },
  { "TAF11", "TFIID subunit (40 kDa)" },
  { "TRM9", "tRNA methyltransferase" },
  { "UBX2", "Bridging factor involved in ER-associated protein degradation (ERAD)" },
  { "YML012C-A", "NULL" },
  { "ERV25", "Member of the p24 family involved in ER to Golgi transport" },
  { "RAD33", "Protein involved in nucleotide excision repair" },
  { "SPT5", "Spt4p/5p (DSIF) transcription elongation factor complex subunit" },
  { "YML009W-B", "NULL" },
  { "YML009C-A", "NULL" },
  { "MRPL39", "Mitochondrial ribosomal protein of the large subunit" },
  { "ERG6", "Delta(24)-sterol C-methyltransferase" },
  { "YML007C-A", "NULL" },
  { "YAP1", "Basic leucine zipper (bZIP) transcription factor" },
  { "GIS4", "CAAX box containing protein of unknown function" },
  { "TRM12", "S-adenosylmethionine-dependent methyltransferase" },
  { "GLO1", "Monomeric glyoxalase I" },
  { "YML003W", "Putative protein of unknown function" },
  { "YML002W", "Putative protein of unknown function" },
  { "YPT7", "Rab family GTPase" },
  { "CDC5", "Polo-like kinase" },
  { "YMR001C-A", "Putative protein of unknown function" },
  { "MIX17", "Mitochondrial intermembrane space protein" },
  { "AIM34", "Protein of unknown function" },
  { "MVP1", "Protein required for sorting proteins to the vacuole" },
  { "TAF4", "TFIID subunit (48 kDa)" },
  { "PLB2", "Phospholipase B (lysophospholipase) involved in lipid metabolism" },
  { "YMR007W", "Putative protein of unknown function" },
  { "PLB1", "Phospholipase B (lysophospholipase) involved in lipid metabolism" },
  { "ADI1", "Acireductone dioxygenease involved in methionine salvage pathway" },
  { "YMR010W", "NULL" },
  { "HXT2", "High-affinity glucose transporter of the major facilitator superfamily" },
  { "CLU1", "Subunit of the eukaryotic translation initiation factor 3 (eIF3)" },
  { "SEC59", "Dolichol kinase" },
  { "YMR013C-A", "NULL" },
  { "YMR013W-A", "NULL" },
  { "BUD22", "Protein required for rRNA maturation and ribosomal subunit biogenesis" },
  { "ERG5", "C-22 sterol desaturase" },
  { "SOK2", "Nuclear protein that negatively regulates pseudohyphal differentiation" },
  { "SPO20", "Meiosis-specific subunit of the t-SNARE complex" },
  { "YMR018W", "NULL" },
  { "STB4", "Putative transcription factor" },
  { "FMS1", "Polyamine oxidase" },
  { "MAC1", "Copper-sensing transcription factor" },
  { "UBC7", "Ubiquitin conjugating enzyme" },
  { "MSS1", "Mitochondrial protein" },
  { "MRPL3", "Mitochondrial ribosomal protein of the large subunit" },
  { "CSI1", "Subunit of the Cop9 signalosome" },
  { "PEX12", "C3HC4-type RING-finger peroxin and E3 ubiquitin ligase" },
  { "YMR027W", "A metal-dependent phosphatase, part of the DUF89 protein family" },
  { "TAP42", "Essential protein involved in the TOR signaling pathway" },
  { "FAR8", "Protein involved in recovery from arrest in response to pheromone" },
  { "RSF1", "Protein required for respiratory growth" },
  { "EIS1", "Component of the eisosome required for proper eisosome assembly" },
  { "YMR031W-A", "NULL" },
  { "HOF1", "Protein that regulates actin cytoskeleton organization" },
  { "YMR030W-A", "Putative protein of unknown function" },
  { "ARP9", "Component of both the SWI/SNF and RSC chromatin remodeling complexes" },
  { "YMR034C", "NULL" },
  { "IMP2'", "Transcriptional activator involved in maintenance of ion homeostasis" },
  { "MIH1", "Protein tyrosine phosphatase involved in cell cycle control" },
  { "MSN2", "Stress-responsive transcriptional activator" },
  { "CCS1", "Copper chaperone for superoxide dismutase Sod1p" },
  { "SUB1", "Transcriptional regulator" },
  { "YET2", "Protein of unknown function that may interact with ribosomes" },
  { "ARA2", "NAD-dependent arabinose dehydrogenase" },
  { "ARG80", "Transcription factor involved in regulating arginine-responsive genes" },
  { "MCM1", "Transcription factor" },
  { "IOC4", "Member of a complex (Isw1b) with Isw1p and Ioc2p" },
  { "YMR046W-A", "NULL" },
  { "NUP116", "FG-nucleoporin component of central core of the nuclear pore complex" },
  { "CSM3", "Replication fork associated factor" },
  { "ERB1", "Constituent of 66S pre-ribosomal particles" },
  { "FAR3", "Protein of unknown function" },
  { "YMR052C-A", "NULL" },
  { "STB2", "Protein that interacts with Sin3p in a two-hybrid assay" },
  { "STV1", "Subunit a of the vacuolar-ATPase V0 domain" },
  { "BUB2", "Mitotic exit network regulator" },
  { "AAC1", "Mitochondrial inner membrane ADP/ATP translocator" },
  { "YMR057C", "NULL" },
  { "FET3", "Ferro-O2-oxidoreductase" },
  { "SEN15", "Subunit of the tRNA splicing endonuclease" },
  { "SAM37", "Component of the Sorting and Assembly Machinery (SAM) complex" },
  { "RNA14", "Component of the cleavage and polyadenylation factor I (CF I)" },
  { "ARG7", "Mitochondrial ornithine acetyltransferase" },
  { "RIM9", "Plasma membrane protein of unknown function" },
  { "AEP1", "Protein required for expression of the mitochondrial OLI1 gene" },
  { "KAR5", "Protein required for nuclear membrane fusion during karyogamy" },
  { "SOV1", "Mitochondrial protein of unknown function" },
  { "UBX4", "UBX domain-containing protein that interacts with Cdc48p" },
  { "AVO2", "Component of a complex containing the Tor2p kinase and other proteins" },
  { "NAT4", "N alpha-acetyl-transferase" },
  { "MOT3", "Transcriptional repressor, activator" },
  { "TVP18", "Integral membrane protein" },
  { "ABF2", "Mitochondrial DNA-binding protein" },
  { "IRC21", "Protein of unknown function" },
  { "YMR074C", "NULL" },
  { "RCO1", "Essential component of the Rpd3S histone deacetylase complex" },
  { "YMR075C-A", "NULL" },
  { "PDS5", "Cohesion maintenance factor" },
  { "VPS20", "Myristoylated subunit of the ESCRT-III complex" },
  { "CTF18", "Subunit of a complex with Ctf8p" },
  { "SEC14", "Phosphatidylinositol/phosphatidylcholine transfer protein" },
  { "NAM7", "ATP-dependent RNA helicase of the SFI superfamily" },
  { "ISF1", "Serine-rich, hydrophilic protein" },
  { "YMR082C", "Putative protein of unknown function" },
  { "ADH3", "Mitochondrial alcohol dehydrogenase isozyme III" },
  { "YMR084W", "Putative protein of unknown function" },
  { "YMR085W", "Putative protein of unknown function" },
  { "SEG1", "Component of eisosome required for proper eisosome assembly" },
  { "YMR086C-A", "NULL" },
  { "YMR087W", "NULL" },
  { "VBA1", "Permease of basic amino acids in the vacuolar membrane" },
  { "YTA12", "Mitochondrial inner membrane m-AAA protease component" },
  { "YMR090W", "Putative protein of unknown function" },
  { "NPL6", "Component of the RSC chromatin remodeling complex" },
  { "AIP1", "Actin cortical patch component" },
  { "UTP15", "Nucleolar protein" },
  { "CTF13", "Subunit of the CBF3 complex" },
  { "SNO1", "Protein of unconfirmed function" },
  { "SNZ1", "Subunit of a two-component pyridoxal-5’-phosphate synthase" },
  { "MTG1", "Putative GTPase peripheral to the mitochondrial inner membrane" },
  { "ATP25", "Mitochondrial protein required for the stability of Oli1p (Atp9p) mRNA" },
  { "YMR099C", "Glucose-6-phosphate 1-epimerase (hexose-6-phosphate mutarotase)" },
  { "MUB1", "MYND domain-containing protein" },
  { "SRT1", "Forms the dehydrodolichyl diphosphate syntase (DDS) complex with NUS1" },
  { "YMR102C", "Protein of unknown function" },
  { "YMR103C", "Putative protein of unknown function" },
  { "YPK2", "Protein kinase similar to S/T protein kinase Ypk1p" },
  { "PGM2", "Phosphoglucomutase" },
  { "YMR105W-A", "Putative protein of unknown function" },
  { "YKU80", "Subunit of telomeric Ku complex (Yku70p-Yku80p)" },
  { "SPG4", "Protein required for high temperature survival during stationary phase" },
  { "ILV2", "Acetolactate synthase" },
  { "MYO5", "One of two type I myosin motors" },
  { "HFD1", "Dehydrogenase involved in ubiquinone and sphingolipid metabolism" },
  { "YMR111C", "NULL" },
  { "MED11", "Subunit of the RNA polymerase II mediator complex" },
  { "FOL3", "Dihydrofolate synthetase, involved in folic acid biosynthesis" },
  { "YMR114C", "Protein of unknown function" },
  { "MGR3", "Subunit of the mitochondrial (mt) i-AAA protease supercomplex" },
  { "ASC1", "G-protein beta subunit and guanine dissociation inhibitor for Gpa2p" },
  { "SPC24", "Component of the kinetochore-associated Ndc80 complex" },
  { "SHH3", "Putative mitochondrial inner membrane protein of unknown function" },
  { "ASI1", "Subunit of the inner nuclear membrane Asi ubiquitin ligase complex" },
  { "YMR119W-A", "NULL" },
  { "ADE17", "Enzyme of 'de novo' purine biosynthesis" },
  { "RPL15B", "Ribosomal 60S subunit protein L15B" },
  { "YMR122C", "Putative protein of unknown function" },
  { "YMR122W-A", "NULL" },
  { "PKR1", "V-ATPase assembly factor" },
  { "EPO1", "Protein involved in septin-ER tethering" },
  { "STO1", "Large subunit of the nuclear mRNA cap-binding protein complex" },
  { "DLT1", "Protein of unknown function" },
  { "SAS2", "Histone acetyltransferase (HAT) catalytic subunit of the SAS complex" },
  { "ECM16", "Essential DEAH-box ATP-dependent RNA helicase specific to U3 snoRNP" },
  { "POM152", "Glycoprotein subunit of transmembrane ring of nuclear pore complex" },
  { "YMR130W", "NULL" },
  { "RRB1", "Specific chaperone for ribosomal protein Rpl3p" },
  { "JLP2", "Protein of unknown function" },
  { "REC114", "Protein involved in early stages of meiotic recombination" },
  { "ERG29", "Protein of unknown function involved in ergosterol biosynthesis" },
  { "GID8", "Subunit of GID Complex, binds strongly to central component Vid30p" },
  { "YMR135W-A", "NULL" },
  { "GAT2", "Protein containing GATA family zinc finger motifs" },
  { "PSO2", "Nuclease required for DNA single- and double-strand break repair" },
  { "CIN4", "GTP-binding protein involved in beta-tubulin (Tub2p) folding" },
  { "RIM11", "Protein kinase" },
  { "SIP5", "Protein of unknown function" },
  { "YMR141C", "Putative protein of unknown function" },
  { "RPL13B", "Ribosomal 60S subunit protein L13B" },
  { "YMR141W-A", "NULL" },
  { "RPS16A", "Protein component of the small (40S) ribosomal subunit" },
  { "YMR144W", "NULL" },
  { "NDE1", "Mitochondrial external NADH dehydrogenase" },
  { "TIF34", "eIF3i subunit of the eukaryotic translation initiation factor 3 (eIF3)" },
  { "YMR147W", "NULL" },
  { "OSW5", "NULL" },
  { "SWP1", "Delta subunit of the oligosaccharyl transferase glycoprotein complex" },
  { "IMP1", "Catalytic subunit of mitochondrial inner membrane peptidase complex" },
  { "YIM2", "NULL" },
  { "YIM1", "Protein of unknown function" },
  { "NUP53", "FG-nucleoporin component of central core of nuclear pore complex (NPC)" },
  { "YMR153C-A", "NULL" },
  { "RIM13", "Calpain-like cysteine protease" },
  { "YMR155W", "Putative protein of unknown function" },
  { "TPP1", "DNA 3'-phosphatase" },
  { "AIM36", "Protein of unknown function" },
  { "MRPS8", "Mitochondrial ribosomal protein of the small subunit" },
  { "YMR158C-A", "Putative protein of unknown function" },
  { "YMR158W-B", "NULL" },
  { "ATG16", "Conserved protein involved in autophagy" },
  { "YMR160W", "Protein of unknown function" },
  { "HLJ1", "Co-chaperone for Hsp40p" },
  { "DNF3", "Trans-golgi network aminophospholipid translocase (flippase)" },
  { "INP2", "Peroxisome-specific receptor important for peroxisome inheritance" },
  { "MSS11", "Transcription factor" },
  { "PAH1", "Mg2+-dependent phosphatidate (PA) phosphatase" },
  { "YMR166C", "NULL" },
  { "MLH1", "Protein required for mismatch repair in mitosis and meiosis" },
  { "CEP3", "Essential kinetochore protein" },
  { "ALD3", "Cytoplasmic aldehyde dehydrogenase" },
  { "ALD2", "Cytoplasmic aldehyde dehydrogenase" },
  { "EAR1", "Specificity factor required for Rsp5p-dependent ubiquitination" },
  { "HOT1", "Transcription factor for glycerol biosynthetic genes" },
  { "YMR172C-A", "NULL" },
  { "DDR48", "DNA damage-responsive protein" },
  { "YMR173W-A", "NULL" },
  { "PAI3", "Cytoplasmic proteinase A (Pep4p) inhibitor" },
  { "SIP18", "Phospholipid-binding hydrophilin" },
  { "YMR175W-A", "Putative protein of unknown function" },
  { "ECM5", "Subunit of the Snt2C complex" },
  { "MMT1", "Putative metal transporter involved in mitochondrial iron accumulation" },
  { "YMR178W", "NULL" },
  { "SPT21", "Protein with a role in transcriptional silencing" },
  { "CTL1", "RNA 5'-triphosphatase, localizes to both the nucleus and cytoplasm" },
  { "YMR181C", "Protein of unknown function" },
  { "RGM1", "Putative zinc finger DNA binding transcription factor" },
  { "YMR182W-A", "NULL" },
  { "SSO2", "Plasma membrane t-SNARE" },
  { "ADD37", "Protein of unknown function" },
  { "RTP1", "Protein required for the nuclear import and biogenesis of RNA pol II" },
  { "HSC82", "Cytoplasmic chaperone of the Hsp90 family" },
  { "YMR187C", "Putative protein of unknown function" },
  { "MRPS17", "Mitochondrial ribosomal protein of the small subunit" },
  { "GCV2", "P subunit of the mitochondrial glycine decarboxylase complex" },
  { "SGS1", "RecQ family nucleolar DNA helicase" },
  { "SPG5", "Protein required for proteasome assembly during quiescence" },
  { "GYL1", "Putative GTPase activating protein (GAP) with a role in exocytosis" },
  { "MRPL24", "Mitochondrial ribosomal protein of the large subunit" },
  { "YMR193C-A", "NULL" },
  { "RPL36A", "Ribosomal 60S subunit protein L36A" },
  { "CMC4", "Protein that localizes to the mitochondrial intermembrane space" },
  { "YMR194C-A", "NULL" },
  { "ICY1", "Protein of unknown function" },
  { "YMR196W", "Putative protein of unknown function" },
  { "VTI1", "Protein involved in cis-Golgi membrane traffic" },
  { "CIK1", "Kinesin-associated protein" },
  { "CLN1", "G1 cyclin involved in regulation of the cell cycle" },
  { "ROT1", "Molecular chaperone involved in protein folding in ER" },
  { "RAD14", "Protein that recognizes and binds damaged DNA during NER" },
  { "ERG2", "C-8 sterol isomerase" },
  { "TOM40", "Component of the TOM (translocase of outer membrane) complex" },
  { "INP1", "Peripheral membrane protein of peroxisomes" },
  { "PFK2", "Beta subunit of heterooctameric phosphofructokinase" },
  { "YMR206W", "Putative protein of unknown function" },
  { "HFA1", "Mitochondrial acetyl-coenzyme A carboxylase" },
  { "ERG12", "Mevalonate kinase" },
  { "YMR209C", "Putative S-adenosylmethionine-dependent methyltransferase" },
  { "YMR210W", "NULL" },
  { "DML1", "Essential protein involved in mtDNA inheritance" },
  { "EFR3", "Protein required for Stt4-containing PI kinase complex localization" },
  { "CEF1", "Essential splicing factor" },
  { "SCJ1", "One of several homologs of bacterial chaperone DnaJ" },
  { "GAS3", "Putative 1,3-beta-glucanosyltransferase" },
  { "SKY1", "SR protein kinase (SRPK)" },
  { "GUA1", "GMP synthase" },
  { "TRS130", "Component of transport protein particle (TRAPP) complex II" },
  { "ESC1", "Protein involved in telomeric silencing" },
  { "ERG8", "Phosphomevalonate kinase" },
  { "YMR221C", "NULL" },
  { "FSH2", "Putative serine hydrolase that localizes to the cytoplasm" },
  { "UBP8", "Ubiquitin-specific protease component of the SAGA acetylation complex" },
  { "MRE11", "Nuclease subunit of the MRX complex with Rad50p and Xrs2p" },
  { "MRPL44", "Mitochondrial ribosomal protein of the large subunit" },
  { "YMR226C", "NADP(+)-dependent serine dehydrogenase and carbonyl reductase" },
  { "TAF7", "TFIID subunit (67 kDa)" },
  { "MTF1", "Mitochondrial RNA polymerase specificity factor" },
  { "RRP5", "RNA binding protein involved in synthesis of 18S and 5.8S rRNAs" },
  { "RPS10B", "Protein component of the small (40S) ribosomal subunit" },
  { "YMR230W-A", "Putative protein of unknown function" },
  { "PEP5", "Histone E3 ligase, component of CORVET membrane tethering complex" },
  { "FUS2", "Cell fusion regulator" },
  { "TRI1", "Non-essential sumoylated protein of unknown function" },
  { "RNH1", "Ribonuclease H1" },
  { "RNA1", "GTPase activating protein (GAP) for Gsp1p" },
  { "TAF9", "Subunit (17 kDa) of TFIID and SAGA complexes" },
  { "BCH1", "Member of the ChAPs family (Chs5p-Arf1p-binding proteins)" },
  { "DFG5", "Putative mannosidase" },
  { "RNT1", "Nuclear dsRNA-specific ribonuclease (RNase III)" },
  { "CUS1", "Protein required for assembly of U2 snRNP into the spliceosome" },
  { "YHM2", "Citrate and oxoglutarate carrier protein" },
  { "RPL20A", "Ribosomal 60S subunit protein L20A" },
  { "YMR242W-A", "Putative protein of unknown function" },
  { "ZRC1", "Vacuolar membrane zinc transporter" },
  { "YMR244W", "Putative protein of unknown function" },
  { "COA6", "Protein involved in cytochrome c oxidase (Complex IV) assembly" },
  { "YMR245W", "NULL" },
  { "FAA4", "Long chain fatty acyl-CoA synthetase" },
  { "RKR1", "RING domain E3 ubiquitin ligase" },
  { "YMR247W-A", "Putative protein of unknown function" },
  { "GAD1", "Glutamate decarboxylase" },
  { "GTO3", "Omega class glutathione transferase" },
  { "HOR7", "Protein of unknown function" },
  { "YMR252C", "NULL" },
  { "YMR253C", "Putative protein of unknown function" },
  { "YMR254C", "Putative protein of unknown function" },
  { "GFD1", "Coiled-coiled protein of unknown function" },
  { "COX7", "Subunit VII of cytochrome c oxidase (Complex IV)" },
  { "PET111", "Mitochondrial translational activator specific for the COX2 mRNA" },
  { "ROY1", "GTPase inhibitor with similarity to F-box proteins" },
  { "TRM732", "Protein involved in 2'-O-methylation of C32 of substrate tRNAs" },
  { "TIF11", "Translation initiation factor eIF1A" },
  { "TPS3", "Regulatory subunit of trehalose-6-phosphate synthase/phosphatase" },
  { "YMR262W", "Protein of unknown function" },
  { "SAP30", "Component of Rpd3L histone deacetylase complex" },
  { "CUE1", "Ubiquitin-binding protein" },
  { "YMR265C", "Putative protein of unknown function" },
  { "RSN1", "Membrane protein of unknown function" },
  { "PPA2", "Mitochondrial inorganic pyrophosphatase" },
  { "PRP24", "Splicing factor that reanneals snRNPs during spliceosome recycling" },
  { "TMA23", "Nucleolar protein implicated in ribosome biogenesis" },
  { "RRN9", "Protein involved in promoting high level transcription of rDNA" },
  { "URA10", "Minor orotate phosphoribosyltransferase (OPRTase) isozyme" },
  { "SCS7", "Sphingolipid alpha-hydroxylase" },
  { "YMR272W-A", "NULL" },
  { "YMR272W-B", "Protein of unknown function" },
  { "ZDS1", "Protein with a role in regulating Swe1p-dependent polarized growth" },
  { "RCE1", "Type II CAAX prenyl protease" },
  { "BUL1", "Ubiquitin-binding component of the Rsp5p E3-ubiquitin ligase complex" },
  { "DSK2", "Nuclear-enriched ubiquitin-like polyubiquitin-binding protein" },
  { "FCP1", "Carboxy-terminal domain (CTD) phosphatase" },
  { "PRM15", "Phosphoribomutase" },
  { "YMR279C", "NULL" },
  { "CAT8", "Zinc cluster transcriptional activator" },
  { "GPI12", "ER membrane protein involved in the second step of GPI anchor assembly" },
  { "AEP2", "Mitochondrial protein" },
  { "RIT1", "Initiator methionine 2'-O-ribosyl phosphate transferase" },
  { "YKU70", "Subunit of the telomeric Ku complex (Yku70p-Yku80p)" },
  { "NGL2", "Protein involved in 5.8S rRNA processing" },
  { "MRPL33", "Mitochondrial ribosomal protein of the large subunit" },
  { "DSS1", "3'-5' exoribonuclease" },
  { "HSH155", "U2-snRNP associated splicing factor" },
  { "ABZ2", "Aminodeoxychorismate lyase (4-amino-4-deoxychorismate lyase)" },
  { "HAS1", "ATP-dependent RNA helicase" },
  { "YMR290W-A", "NULL" },
  { "TDA1", "Protein kinase of unknown cellular role" },
  { "GOT1", "Homodimeric protein that is packaged into COPII vesicles" },
  { "HER2", "Subunit of the trimeric GatFAB AmidoTransferase(AdT) complex" },
  { "JNM1", "Component of the yeast dynactin complex" },
  { "YMR294W-A", "NULL" },
  { "YMR295C", "Protein of unknown function that associates with ribosomes" },
  { "LCB1", "Component of serine palmitoyltransferase" },
  { "PRC1", "Vacuolar carboxypeptidase Y (proteinase C, CPY)" },
  { "LIP1", "Ceramide synthase subunit" },
  { "DYN3", "Dynein light intermediate chain (LIC)" },
  { "ADE4", "Phosphoribosylpyrophosphate amidotransferase (PRPPAT)" },
  { "ATM1", "Mitochondrial inner membrane ATP-binding cassette (ABC) transporter" },
  { "YME2", "Integral inner mitochondrial membrane protein" },
  { "ADH2", "Glucose-repressible alcohol dehydrogenase II" },
  { "UBP15", "Ubiquitin-specific protease involved in protein deubiquitination" },
  { "YMR304C-A", "NULL" },
  { "SCW10", "Cell wall protein with similarity to glucanases" },
  { "FKS3", "Protein involved in spore wall assembly" },
  { "YMR306C-A", "NULL" },
  { "GAS1", "Beta-1,3-glucanosyltransferase" },
  { "YMR307C-A", "NULL" },
  { "PSE1", "Karyopherin/importin that interacts with the nuclear pore complex" },
  { "NIP1", "eIF3c subunit of the eukaryotic translation initiation factor 3 (eIF3)" },
  { "YMR310C", "Putative methyltransferase" },
  { "GLC8", "Regulatory subunit of protein phosphatase 1 (Glc7p)" },
  { "ELP6", "Subunit of hexameric RecA-like ATPase Elp456 Elongator subcomplex" },
  { "TGL3", "Bifunctional triacylglycerol lipase and LPE acyltransferase" },
  { "PRE5", "Alpha 6 subunit of the 20S proteasome" },
  { "YMR315W", "Protein with NADP(H) oxidoreductase activity" },
  { "YMR315W-A", "Putative protein of unknown function" },
  { "DIA1", "Protein of unknown function" },
  { "YMR316C-A", "Protein of unknown function" },
  { "YMR316C-B", "NULL" },
  { "YMR317W", "Putative protein of unknown function" },
  { "ADH6", "NADPH-dependent medium chain alcohol dehydrogenase" },
  { "FET4", "Low-affinity Fe(II) transporter of the plasma membrane" },
  { "YMR320W", "Putative protein of unknown function" },
  { "YMR321C", "Putative protein of unknown function" },
  { "SNO4", "Possible chaperone and cysteine protease" },
  { "ERR3", "Enolase, a phosphopyruvate hydratase" },
  { "YMR324C", "NULL" },
  { "PAU19", "Protein of unknown function" },
  { "YMR326C", "NULL" },
  { "YRF1-6", "Helicase encoded by the Y' element of subtelomeric regions" },
  { "YNL339W-B", "NULL" },
  { "YNL339W-A", "NULL" },
  { "YNL338W", "NULL" },
  { "YNL337W", "NULL" },
  { "COS1", "Endosomal protein involved in turnover of plasma membrane proteins" },
  { "DDI3", "Cyanamide hydratase that detoxifies cyanamide" },
  { "SNO2", "Protein of unknown function" },
  { "SNZ2", "Protein involved in thiamine and pyridoxine biosynthesis" },
  { "THI12", "Protein involved in synthesis of the thiamine precursor HMP" },
  { "AAD14", "Putative aryl-alcohol dehydrogenase" },
  { "RPD3", "Histone deacetylase, component of both the Rpd3S and Rpd3L complexes" },
  { "PEX6", "AAA-peroxin" },
  { "MDJ2", "Constituent of the mitochondrial import motor" },
  { "EGT2", "Glycosylphosphatidylinositol (GPI)-anchored cell wall endoglucanase" },
  { "PFA3", "Palmitoyltransferase for Vac8p" },
  { "FIG4", "Phosphatidylinositol 3,5-bisphosphate (PtdIns[3,5]P) phosphatase" },
  { "YNL324W", "NULL" },
  { "LEM3", "Membrane protein of the plasma membrane and ER" },
  { "KRE1", "Cell wall glycoprotein involved in beta-glucan assembly" },
  { "VNX1", "Calcium/H+ antiporter localized to the endoplasmic reticulum membrane" },
  { "YNL320W", "Putative protein of unknown function" },
  { "YNL319W", "NULL" },
  { "HXT14", "Protein with similarity to hexose transporter family members" },
  { "PFS2", "Integral subunit of the pre-mRNA CPF complex" },
  { "PHA2", "Prephenate dehydratase" },
  { "ATP11", "Molecular chaperone" },
  { "DAL82", "Positive regulator of allophanate inducible genes" },
  { "EMW1", "Essential conserved protein with a role in cell wall integrity" },
  { "RFA2", "Subunit of heterotrimeric Replication Protein A (RPA)" },
  { "SKP2", "F-box protein of unknown function" },
  { "ZIM17", "Protein co-chaperone with a zinc finger motif" },
  { "STB1", "Protein with role in regulation of MBF-specific transcription at Start" },
  { "KRI1", "Essential nucleolar protein required for 40S ribosome biogenesis" },
  { "MCK1", "Dual-specificity ser/thr and tyrosine protein kinase" },
  { "MRPS18", "Mitochondrial ribosomal protein of the small subunit" },
  { "BXI1", "Protein involved in apoptosis" },
  { "YPT11", "Rab GTPase" },
  { "YNL303W", "NULL" },
  { "RPS19B", "Protein component of the small (40S) ribosomal subunit" },
  { "RPL18B", "Ribosomal 60S subunit protein L18B" },
  { "TOS6", "Glycosylphosphatidylinositol-dependent cell wall protein" },
  { "TRF5", "Non-canonical poly(A) polymerase" },
  { "CLA4", "Cdc42p-activated signal transducing kinase" },
  { "MON2", "Protein with a role in endocytosis and vacuole integrity" },
  { "YNL296W", "NULL" },
  { "YNL295W", "NULL" },
  { "RIM21", "pH sensor molecule, component of the RIM101 pathway" },
  { "MSB3", "Rab GTPase-activating protein" },
  { "PUS4", "Pseudouridine synthase" },
  { "MID1", "Stretch-activated Ca2+-permeable cation channel" },
  { "RFC3", "Subunit of heteropentameric Replication factor C (RF-C)" },
  { "PCL1", "Cyclin, interacts with cyclin-dependent kinase Pho85p" },
  { "CAF40", "Component of the CCR4-NOT transcriptional complex" },
  { "SEC21", "Gamma subunit of coatomer" },
  { "CUS2", "Putative checkpoint factor in transcription" },
  { "YNL285W", "NULL" },
  { "MRPL10", "Mitochondrial ribosomal protein of the large subunit" },
  { "WSC2", "Sensor-transducer of the stress-activated PKC1-MPK1 signaling pathway" },
  { "POP3", "Subunit of both RNase MRP and nuclear RNase P" },
  { "HCH1", "Heat shock protein regulator" },
  { "ERG24", "C-14 sterol reductase" },
  { "PRM1", "Pheromone-regulated multispanning membrane protein" },
  { "CAF120", "Part of the CCR4-NOT transcriptional regulatory complex" },
  { "YNL277W-A", "Putative protein of unknown function" },
  { "MET2", "L-homoserine-O-acetyltransferase" },
  { "YNL276C", "NULL" },
  { "BOR1", "Boron efflux transporter of the plasma membrane" },
  { "GOR1", "Glyoxylate reductase" },
  { "TOF1", "Subunit of a replication-pausing checkpoint complex" },
  { "SEC2", "Guanyl-nucleotide exchange factor for the small G-protein Sec4p" },
  { "BNI1", "Formin" },
  { "ALP1", "Arginine transporter" },
  { "BSC4", "Protein of unknown function" },
  { "LYP1", "Lysine permease" },
  { "PIK1", "Phosphatidylinositol 4-kinase" },
  { "YNL266W", "NULL" },
  { "IST1", "Protein with positive role in the multivesicular body sorting pathway" },
  { "PDR17", "Phosphatidylinositol transfer protein (PITP)" },
  { "YIF1", "Integral membrane protein" },
  { "POL2", "Catalytic subunit of DNA polymerase (II) epsilon" },
  { "ORC5", "Subunit of the origin recognition complex (ORC)" },
  { "LTO1", "Substrate-specific adaptor protein involved in apo-Rli1p maturation" },
  { "ATX1", "Cytosolic copper metallochaperone" },
  { "DSL1", "Peripheral membrane protein needed for Golgi-to-ER retrograde traffic" },
  { "SIP3", "Putative sterol transfer protein" },
  { "FOL1", "Multifunctional enzyme of the folic acid biosynthesis pathway" },
  { "GIS2", "Translational activator for mRNAs with internal ribosome entry sites" },
  { "RTC4", "Protein of unknown function" },
  { "TEX1", "Protein involved in mRNA export" },
  { "MRPL17", "Mitochondrial ribosomal protein of the large subunit" },
  { "NRD1", "RNA-binding subunit of Nrd1 complex" },
  { "RAD50", "Subunit of MRX complex with Mre11p and Xrs2p" },
  { "MPA43", "Protein of unknown function" },
  { "RPA49", "RNA polymerase I subunit A49" },
  { "YNL247W", "NULL" },
  { "VPS75", "NAP family histone chaperone" },
  { "CWC25", "Splicing factor required for the first step of pre-mRNA splicing" },
  { "SUI1", "Translation initiation factor eIF1" },
  { "SLA2", "Adaptor protein that links actin to clathrin and endocytosis" },
  { "ATG2", "Peripheral membrane protein required for autophagic vesicle formation" },
  { "ZWF1", "Glucose-6-phosphate dehydrogenase (G6PD)" },
  { "NAR1", "Subunit of the cytosolic iron-sulfur (FeS) protein assembly machinery" },
  { "LAP3", "Cysteine aminopeptidase with homocysteine-thiolactonase activity" },
  { "KEX2", "Kexin, a subtilisin-like protease (proprotein convertase)" },
  { "YTP1", "Probable type-III integral membrane protein of unknown function" },
  { "SIN4", "Subunit of the RNA polymerase II mediator complex" },
  { "YNL235C", "NULL" },
  { "YNL234W", "Protein of unknown function with similarity to globins" },
  { "BNI4", "Targeting subunit for Glc7p protein phosphatase" },
  { "CSL4", "Exosome non-catalytic core component" },
  { "PDR16", "Phosphatidylinositol transfer protein (PITP)" },
  { "ELA1", "Elongin A" },
  { "URE2", "Nitrogen catabolite repression transcriptional regulator" },
  { "YNL228W", "NULL" },
  { "JJJ1", "Co-chaperone that stimulates the ATPase activity of Ssa1p" },
  { "YNL226W", "NULL" },
  { "CNM67", "Component of the spindle pole body outer plaque" },
  { "SQS1", "Protein that stimulates the ATPase and helicase activities of Prp43p" },
  { "ATG4", "Conserved cysteine protease required for autophagy" },
  { "SSU72", "Phosphatase and transcription/RNA-processing factor" },
  { "POP1", "Subunit of RNase MRP, nuclear RNase P and telomerase complexes" },
  { "ADE12", "Adenylosuccinate synthase" },
  { "ALG9", "Mannosyltransferase, involved in N-linked glycosylation" },
  { "MGS1", "Protein with DNA-dependent ATPase and ssDNA annealing activities" },
  { "YNL217W", "NULL" },
  { "RAP1", "Essential DNA-binding transcription regulator that binds many loci" },
  { "IES2", "Protein that associates with the INO80 chromatin remodeling complex" },
  { "PEX17", "Membrane peroxin of the peroxisomal importomer complex" },
  { "RRG9", "Protein of unknown function" },
  { "VID27", "Cytoplasmic protein of unknown function" },
  { "YNL211C", "NULL" },
  { "MER1", "mRNA-binding protein required for meiosis-specific mRNA splicing" },
  { "SSB2", "Cytoplasmic ATPase that is a ribosome-associated molecular chaperone" },
  { "YNL208W", "Protein of unknown function" },
  { "RIO2", "Essential serine kinase involved in the processing of 20S pre-rRNA" },
  { "RTT106", "Histone chaperone" },
  { "YNL205C", "NULL" },
  { "SPS18", "Protein of unknown function, contains a putative zinc-binding domain" },
  { "YNL203C", "NULL" },
  { "SPS19", "Peroxisomal 2,4-dienoyl-CoA reductase" },
  { "PSY2", "Subunit of protein phosphatase PP4 complex" },
  { "YNL200C", "NULL" },
  { "GCR2", "Transcriptional activator of genes involved in glycolysis" },
  { "YNL198C", "NULL" },
  { "WHI3", "RNA binding protein that sequesters CLN3 mRNA in cytoplasmic foci" },
  { "SLZ1", "Sporulation-specific protein with a leucine zipper motif" },
  { "YNL195C", "Protein of unknown function" },
  { "YNL194C", "Integral membrane protein" },
  { "YNL193W", "Putative protein of unknown function" },
  { "CHS1", "Chitin synthase I" },
  { "DUG3", "Component of glutamine amidotransferase (GATase II)" },
  { "YNL190W", "Hydrophilin essential in desiccation-rehydration process" },
  { "SRP1", "Karyopherin alpha homolog" },
  { "KAR1", "Protein involved in karyogamy and spindle pole body duplication" },
  { "SWT21", "Protein involved in mRNA splicing" },
  { "UBP10", "Ubiquitin-specific protease, deubiquitinates Ub-protein moieties" },
  { "MRPL19", "Mitochondrial ribosomal protein of the large subunit" },
  { "YNL184C", "Protein of unknown function" },
  { "NPR1", "Protein kinase" },
  { "IPI3", "Component of the Rix1 complex and pre-replicative complexes (pre-RCs)" },
  { "YNL181W", "NULL" },
  { "RHO5", "Non-essential small GTPase of the Rho/Rac family of Ras-like proteins" },
  { "YNL179C", "NULL" },
  { "RPS3", "Protein component of the small (40S) ribosomal subunit" },
  { "MRPL22", "Mitochondrial ribosomal protein of the large subunit" },
  { "TDA7", "Cell cycle-regulated gene of unknown function" },
  { "NOP13", "Nucleolar protein found in preribosomal complexes" },
  { "YNL174W", "NULL" },
  { "MDG1", "Plasma membrane protein" },
  { "APC1", "Largest subunit of the Anaphase-Promoting Complex/Cyclosome" },
  { "YNL171C", "NULL" },
  { "YNL170W", "NULL" },
  { "PSD1", "Phosphatidylserine decarboxylase of the mitochondrial inner membrane" },
  { "FMP41", "Putative protein of unknown function" },
  { "SKO1", "Basic leucine zipper transcription factor of the ATF/CREB family" },
  { "BNI5", "Linker protein responsible for recruitment of myosin to the bud neck" },
  { "YNL165W", "Putative protein of unknown function" },
  { "IBD2", "Component of the BUB2-dependent spindle checkpoint pathway" },
  { "RIA1", "Cytoplasmic GTPase/eEF2-like factor involved in ribosomal biogenesis" },
  { "YNL162W-A", "Putative protein of unknown function" },
  { "RPL42A", "Ribosomal 60S subunit protein L42A" },
  { "CBK1", "Serine/threonine protein kinase of the the RAM signaling network" },
  { "YGP1", "Cell wall-related secretory glycoprotein" },
  { "ASI2", "Subunit of the inner nuclear membrane Asi ubiquitin ligase complex" },
  { "PGA1", "Essential component of GPI-mannosyltransferase II" },
  { "IGO1", "Protein required for initiation of G0 program" },
  { "NSG2", "Protein involved in regulation of sterol biosynthesis" },
  { "CUZ1", "Protein with a role in the ubiquitin-proteasome pathway" },
  { "YCK2", "Palmitoylated plasma membrane-bound casein kinase I (CK1) isoform" },
  { "GIM3", "Subunit of the heterohexameric cochaperone prefoldin complex" },
  { "INN1", "Essential protein that associates with contractile actomyosin ring" },
  { "RPC31", "RNA polymerase III subunit C31" },
  { "YNL150W", "NULL" },
  { "PGA2", "Essential protein required for maturation of Gas1p and Pho8p" },
  { "ALF1", "Alpha-tubulin folding protein" },
  { "LSM7", "Lsm (Like Sm) protein" },
  { "YNL146C-A", "Putative protein of unknown function" },
  { "YNL146W", "Putative protein of unknown function" },
  { "MFA2", "Mating pheromone a-factor" },
  { "YNL144C", "Putative protein of unknown function" },
  { "YNL144W-A", "NULL" },
  { "YNL143C", "Protein of unknown function" },
  { "MEP2", "Ammonium permease involved in regulation of pseudohyphal growth" },
  { "AAH1", "Adenine deaminase (adenine aminohydrolase)" },
  { "YNL140C", "Protein of unknown function" },
  { "THO2", "Subunit of the THO complex" },
  { "YSF3", "Component of the SF3b subcomplex of the U2 snRNP" },
  { "SRV2", "CAP (cyclase-associated protein)" },
  { "NAM9", "Mitochondrial ribosomal component of the small subunit" },
  { "EAF7", "Subunit of the NuA4 histone acetyltransferase complex" },
  { "FPR1", "Peptidyl-prolyl cis-trans isomerase (PPIase)" },
  { "YNL134C", "NADH-dependent aldehyde reductase, involved in detoxification of furf" },
  { "FYV6", "Protein of unknown function" },
  { "KRE33", "Protein required for biogenesis of the small ribosomal subunit" },
  { "TOM22", "Component of the TOM (Translocase of Outer Membrane) complex" },
  { "CPT1", "Cholinephosphotransferase" },
  { "DGR1", "Protein of unknown function" },
  { "NRK1", "Nicotinamide riboside kinase" },
  { "TEP1", "PTEN homolog with no demonstrated inositol lipid phosphatase activity" },
  { "FAR11", "Protein involved in recovery from cell cycle arrest" },
  { "SPC98", "Component of the microtubule-nucleating Tub4p (gamma-tubulin) complex" },
  { "ESBP6", "Protein with similarity to monocarboxylate permeases" },
  { "NAF1", "RNA-binding protein required for the assembly of box H/ACA snoRNPs" },
  { "NMA111", "Serine protease and general molecular chaperone" },
  { "YNL122C", "Mitochondrial ribosomal protein of the large subunit" },
  { "TOM70", "Component of the TOM (translocase of outer membrane) complex" },
  { "YNL120C", "NULL" },
  { "NCS2", "Protein required for uridine thiolation of Lys(UUU) and Glu(UUC) tRNAs" },
  { "DCP2", "Catalytic subunit of Dcp1p-Dcp2p decapping enzyme complex" },
  { "MLS1", "Malate synthase, enzyme of the glyoxylate cycle" },
  { "DMA2", "Ubiquitin-protein ligase (E3)" },
  { "YNL115C", "Putative protein of unknown function" },
  { "YNL114C", "NULL" },
  { "RPC19", "RNA polymerase subunit AC19" },
  { "DBP2", "ATP-dependent RNA helicase of the DEAD-box protein family" },
  { "CYB5", "Cytochrome b5" },
  { "NOP15", "Constituent of 66S pre-ribosomal particles" },
  { "YNL109W", "NULL" },
  { "YNL108C", "Protein phosphatase" },
  { "YAF9", "Subunit of NuA4 histone H4 acetyltransferase and SWR1 complexes" },
  { "INP52", "Polyphosphatidylinositol phosphatase" },
  { "RRT16", "NULL" },
  { "LEU4", "Alpha-isopropylmalate synthase (2-isopropylmalate synthase)" },
  { "YNL103W-A", "NULL" },
  { "MET4", "Leucine-zipper transcriptional activator" },
  { "POL1", "Catalytic subunit of the DNA polymerase I alpha-primase complex" },
  { "AVT4", "Vacuolar transporter" },
  { "MIC27", "Component of the MICOS complex" },
  { "OCA1", "Putative protein tyrosine phosphatase" },
  { "RAS2", "GTP-binding protein" },
  { "YNL097C-B", "Putative protein of unknown function" },
  { "PHO23", "Component of the Rpd3L histone deacetylase complex" },
  { "YNL097W-A", "NULL" },
  { "RPS7B", "Protein component of the small (40S) ribosomal subunit" },
  { "YNL095C", "Putative protein of unknown function" },
  { "APP1", "Phosphatidate phosphatase, converts phosphatidate to diacylglycerol" },
  { "YPT53", "Stress-induced Rab family GTPase" },
  { "YNL092W", "S-adenosylmethionine-dependent protein methyltransferase" },
  { "NST1", "Protein of unknown function" },
  { "RHO2", "Non-essential small GTPase of the Rho/Rac family of Ras-like proteins" },
  { "YNL089C", "NULL" },
  { "TOP2", "Topoisomerase II" },
  { "TCB2", "ER protein involved in ER-plasma membrane tethering" },
  { "SNN1", "Subunit of the BLOC-1 complex involved in endosomal maturation" },
  { "MKT1", "Protein similar to nucleases that forms a complex with Pbp1p" },
  { "END3", "EH domain-containing protein involved in endocytosis" },
  { "SAL1", "ADP/ATP transporter" },
  { "PMS1", "ATP-binding protein required for mismatch repair" },
  { "SWS2", "Putative mitochondrial ribosomal protein of the small subunit" },
  { "EOS1", "Protein involved in N-glycosylation" },
  { "TPM1", "Major isoform of tropomyosin" },
  { "NIS1", "Protein localized in the bud neck at G2/M phase" },
  { "APJ1", "Chaperone with a role in SUMO-mediated protein degradation" },
  { "MKS1", "Pleiotropic negative transcriptional regulator" },
  { "IMP4", "Component of the SSU processome" },
  { "MLF3", "Serine-rich protein of unknown function" },
  { "MSK1", "Mitochondrial lysine-tRNA synthetase" },
  { "RNH201", "Ribonuclease H2 catalytic subunit" },
  { "LAT1", "Dihydrolipoamide acetyltransferase component (E2) of the PDC" },
  { "TOM7", "Component of the TOM (translocase of outer membrane) complex" },
  { "RPL16B", "Ribosomal 60S subunit protein L16B" },
  { "FKH2", "Forkhead family transcription factor" },
  { "YNL067W-A", "NULL" },
  { "YNL067W-B", "Putative protein of unknown function" },
  { "RPL9B", "Ribosomal 60S subunit protein L9B" },
  { "SUN4", "Cell wall protein related to glucanases" },
  { "AQR1", "Plasma membrane transporter of the major facilitator superfamily" },
  { "YDJ1", "Type I HSP40 co-chaperone" },
  { "MTQ1", "S-adenosylmethionine-dependent methyltransferase" },
  { "GCD10", "Subunit of tRNA (1-methyladenosine) methyltransferase with Gcd14p" },
  { "NOP2", "rRNA m5C methyltransferase" },
  { "ARP5", "Nuclear actin-related protein involved in chromatin remodeling" },
  { "YNL058C", "Putative protein of unknown function" },
  { "YNL057W", "NULL" },
  { "OCA2", "Protein of unknown function" },
  { "POR1", "Mitochondrial porin (voltage-dependent anion channel)" },
  { "VAC7", "Integral vacuolar membrane protein" },
  { "MSG5", "Dual-specificity protein phosphatase" },
  { "COX5A", "Subunit Va of cytochrome c oxidase" },
  { "COG5", "Component of the conserved oligomeric Golgi complex" },
  { "YNL050C", "Putative protein of unknown function" },
  { "SFB2", "Component of the Sec23p-Sfb2p heterodimer of the COPII vesicle coat" },
  { "ALG11", "Alpha-1,2-mannosyltransferase" },
  { "SLM2", "Phosphoinositide PI4,5P(2) binding protein, forms a complex with Slm1p" },
  { "YNL046W", "Putative protein of unknown function" },
  { "LAP2", "Leucyl aminopeptidase yscIV with epoxide hydrolase activity" },
  { "YIP3", "Protein localized to COPII vesicles" },
  { "YNL043C", "NULL" },
  { "YNL042W-B", "Putative protein of unknown function" },
  { "BOP3", "Protein of unknown function" },
  { "COG6", "Component of the conserved oligomeric Golgi complex" },
  { "YNL040W", "Protein of unknown function" },
  { "BDP1", "Essential subunit of RNA polymerase III transcription factor (TFIIIB)" },
  { "GPI15", "Protein involved in the synthesis of GlcNAc-PI" },
  { "IDH1", "Subunit of mitochondrial NAD(+)-dependent isocitrate dehydrogenase" },
  { "NCE103", "Carbonic anhydrase" },
  { "YNL035C", "Nuclear protein of unknown function" },
  { "YNL034W", "Putative protein of unknown function" },
  { "YNL033W", "Putative protein of unknown function" },
  { "SIW14", "Inositol phosphatase involved in inositol pyrophosphate metabolism" },
  { "HHT2", "Histone H3" },
  { "HHF2", "Histone H4" },
  { "KTR5", "Putative mannosyltransferase involved in protein glycosylation" },
  { "YNL028W", "NULL" },
  { "CRZ1", "Transcription factor, activates transcription of stress response genes" },
  { "SAM50", "Component of the Sorting and Assembly Machinery (SAM) complex" },
  { "SSN8", "Cyclin-like component of the RNA polymerase II holoenzyme" },
  { "KSH1", "Essential protein suggested to function early in the secretory pathway" },
  { "YNL024C", "NULL" },
  { "FAP1", "Protein that binds to Fpr1p" },
  { "RCM1", "rRNA m5C methyltransferase" },
  { "HDA1", "Putative catalytic subunit of a class II histone deacetylase complex" },
  { "ARK1", "Ser/Thr protein kinase" },
  { "YNL019C", "Putative protein of unknown function" },
  { "YNL018C", "Putative protein of unknown function" },
  { "YNL017C", "NULL" },
  { "PUB1", "Poly (A)+ RNA-binding protein" },
  { "PBI2", "Cytosolic inhibitor of vacuolar proteinase B (PRB1)" },
  { "HEF3", "Translational elongation factor EF-3" },
  { "YNL013C", "NULL" },
  { "SPO1", "Meiosis-specific prospore protein" },
  { "YNL011C", "Putative protein of unknown function" },
  { "YNL010W", "NULL" },
  { "IDP3", "Peroxisomal NADP-dependent isocitrate dehydrogenase" },
  { "ASI3", "Subunit of the inner nuclear membrane Asi ubiquitin ligase complex" },
  { "SIS1", "Type II HSP40 co-chaperone that interacts with the HSP70 protein Ssa1p" },
  { "LST8", "Protein required for the transport of Gap1p" },
  { "MRP7", "Mitochondrial ribosomal protein of the large subunit" },
  { "HRB1", "Poly(A+) RNA-binding protein" },
  { "PET8", "S-adenosylmethionine transporter of the mitochondrial inner membrane" },
  { "RLP7", "Nucleolar protein similar to large ribosomal subunit L7 proteins" },
  { "DOM34", "Protein that facilitates ribosomal subunit dissociation" },
  { "CIT1", "Citrate synthase" },
  { "YNR001W-A", "NULL" },
  { "ATO2", "Putative transmembrane protein involved in export of ammonia" },
  { "RPC34", "RNA polymerase III subunit C34" },
  { "YNR003W-A", "NULL" },
  { "SWM2", "Protein with a role in snRNA and snoRNA cap trimethylation" },
  { "YNR005C", "NULL" },
  { "VPS27", "Endosomal protein that forms a complex with Hse1p" },
  { "ATG3", "E2-like enzyme" },
  { "LRO1", "Acyltransferase that catalyzes diacylglycerol esterification" },
  { "NRM1", "Transcriptional co-repressor of MBF-regulated gene expression" },
  { "CSE2", "Subunit of the RNA polymerase II mediator complex" },
  { "PRP2", "RNA-dependent DExD/H-box ATPase" },
  { "URK1", "Uridine/cytidine kinase" },
  { "PHO91", "Low-affinity vacuolar phosphate transporter" },
  { "YNR014W", "Putative protein of unknown function" },
  { "SMM1", "Dihydrouridine synthase" },
  { "ACC1", "Acetyl-CoA carboxylase, biotin containing enzyme" },
  { "TIM23", "Essential component of the TIM23 complex" },
  { "RCF2", "Cytochrome c oxidase subunit" },
  { "ARE2", "Acyl-CoA:sterol acyltransferase" },
  { "ATP23", "Putative metalloprotease of the mitochondrial inner membrane" },
  { "YNR021W", "Putative protein of unknown function" },
  { "MRPL50", "Mitochondrial ribosomal protein of the large subunit" },
  { "SNF12", "73 kDa subunit of the SWI/SNF chromatin remodeling complex" },
  { "MPP6", "Nuclear exosome-associated RNA binding protein" },
  { "YNR025C", "NULL" },
  { "SEC12", "Guanine nucleotide exchange factor (GEF)" },
  { "BUD17", "Putative pyridoxal kinase" },
  { "CPR8", "Peptidyl-prolyl cis-trans isomerase (cyclophilin)" },
  { "YNR029C", "Putative protein of unknown function" },
  { "ALG12", "Alpha-1,6-mannosyltransferase localized to the ER" },
  { "SSK2", "MAP kinase kinase kinase of HOG1 mitogen-activated signaling pathway" },
  { "PPG1", "Putative serine/threonine protein phosphatase" },
  { "HUB1", "Ubiquitin-like protein modifier" },
  { "ABZ1", "Para-aminobenzoate (PABA) synthase" },
  { "SOL1", "Protein with a possible role in tRNA export" },
  { "YNR034W-A", "NULL" },
  { "ARC35", "Subunit of the ARP2/3 complex" },
  { "MRPS12", "Mitochondrial protein" },
  { "RSM19", "Mitochondrial ribosomal protein of the small subunit" },
  { "DBP6", "Essential protein involved in ribosome biogenesis" },
  { "ZRG17", "Endoplasmic reticulum zinc transporter" },
  { "YNR040W", "NULL" },
  { "COQ2", "Para hydroxybenzoate polyprenyl transferase" },
  { "YNR042W", "NULL" },
  { "MVD1", "Mevalonate pyrophosphate decarboxylase" },
  { "AGA1", "Anchorage subunit of a-agglutinin of a-cells" },
  { "PET494", "Mitochondrial translational activator specific for the COX3 mRNA" },
  { "TRM112", "Protein involved in methylation of tRNA, rRNA, and translation factors" },
  { "FPK1", "Ser/Thr protein kinase" },
  { "YNR048W", "Potential noncatalytic subunit for phospholipid translocase Dnf3p" },
  { "MSO1", "Lipid-interacting protein in SNARE complex assembly machinery" },
  { "LYS9", "Saccharopine dehydrogenase (NADP+, L-glutamate-forming)" },
  { "BRE5", "Ubiquitin protease cofactor" },
  { "POP2", "RNase of the DEDD superfamily" },
  { "NOG2", "Putative GTPase" },
  { "ESF2", "Essential nucleolar protein involved in pre-18S rRNA processing" },
  { "HOL1", "Putative transporter in the major facilitator superfamily" },
  { "BIO5", "Putative transmembrane protein involved in the biotin biosynthesis" },
  { "BIO4", "Dethiobiotin synthetase" },
  { "BIO3", "7,8-diamino-pelargonic acid aminotransferase (DAPA)" },
  { "MNT4", "Putative alpha-1,3-mannosyltransferase" },
  { "FRE4", "Ferric reductase" },
  { "YNR061C", "Protein of unknown function" },
  { "YNR062C", "NULL" },
  { "YNR063W", "NULL" },
  { "YNR064C", "Epoxide hydrolase" },
  { "YNR065C", "Protein of unknown function" },
  { "YNR066C", "Putative membrane-localized protein of unknown function" },
  { "DSE4", "Daughter cell-specific secreted protein with similarity to glucanases" },
  { "YNR068C", "Putative protein of unknown function" },
  { "BSC5", "Protein of unknown function" },
  { "PDR18", "Putative transporter of the ATP-binding cassette (ABC) family" },
  { "YNR071C", "Putative aldose 1-epimerase" },
  { "HXT17", "Putative transmembrane polyol transporter" },
  { "YNR073C", "NULL" },
  { "AIF1", "Mitochondrial cell death effector" },
  { "COS10", "Endosomal protein involved in turnover of plasma membrane proteins" },
  { "YNR075C-A", "Protein of unknown function" },
  { "PAU6", "Member of the seripauperin multigene family" },
  { "YNR077C", "Protein of unknown function, abundance changes with carbon source" },
  { "YOL166W-A", "Protein of unknown function" },
  { "YOL166C", "NULL" },
  { "AAD15", "Putative aryl-alcohol dehydrogenase" },
  { "YOL164W-A", "Putative protein of unknown function" },
  { "BDS1", "Bacterially-derived sulfatase" },
  { "YOL163W", "Putative protein of unknown function" },
  { "YOL162W", "Putative protein of unknown function" },
  { "PAU20", "Protein of unknown function" },
  { "YOL160W", "Putative protein of unknown function" },
  { "YOL159C-A", "Protein of unknown function" },
  { "YOL159C", "NULL" },
  { "ENB1", "Ferric enterobactin transmembrane transporter" },
  { "IMA2", "Isomaltase (alpha-1,6-glucosidase/alpha-methylglucosidase)" },
  { "HXT11", "Hexose transporter" },
  { "YOL155W-A", "Putative protein of unknown function" },
  { "HPF1", "Haze-protective mannoprotein" },
  { "ZPS1", "Putative GPI-anchored protein" },
  { "FRE7", "Putative ferric reductase with similarity to Fre2p" },
  { "GRE2", "3-methylbutanal reductase and NADPH-dependent methylglyoxal reductase" },
  { "YOL150C", "NULL" },
  { "DCP1", "Subunit of the Dcp1p-Dcp2p decapping enzyme complex" },
  { "SPT20", "Subunit of the SAGA transcriptional regulatory complex" },
  { "PEX11", "Peroxisomal protein required for medium-chain fatty acid oxidation" },
  { "PSF3", "Subunit of the GINS complex (Sld5p, Psf1p, Psf2p, Psf3p)" },
  { "CTR9", "Component of the Paf1p complex involved in transcription elongation" },
  { "NOP8", "Nucleolar protein required for 60S ribosomal subunit biogenesis" },
  { "RIB4", "Lumazine synthase (DMRL synthase)" },
  { "RRP40", "Exosome non-catalytic core component" },
  { "PPM2", "AdoMet-dependent tRNA methyltransferase" },
  { "ARG8", "Acetylornithine aminotransferase" },
  { "CDC33", "mRNA cap binding protein and translation initiation factor eIF4E" },
  { "RTC1", "Subunit of SEACAT, a subcomplex of the SEA complex" },
  { "BSC6", "Protein of unknown function with 8 putative transmembrane segments" },
  { "PFK27", "6-phosphofructo-2-kinase" },
  { "MED7", "Subunit of the RNA polymerase II mediator complex" },
  { "YOL134C", "NULL" },
  { "HRT1", "RING-H2 domain core subunit of multiple ubiquitin ligase complexes" },
  { "GAS4", "1,3-beta-glucanosyltransferase" },
  { "YOL131W", "Putative protein of unknown function" },
  { "ALR1", "Plasma membrane Mg(2+) transporter" },
  { "VPS68", "Vacuolar membrane protein of unknown function" },
  { "YGK3", "Protein kinase related to mammalian GSK-3 glycogen synthase kinases" },
  { "RPL25", "Ribosomal 60S subunit protein L25" },
  { "MDH2", "Cytoplasmic malate dehydrogenase" },
  { "TRM13", "2'-O-methyltransferase" },
  { "TRM11", "Catalytic subunit of adoMet-dependent tRNA methyltransferase complex" },
  { "HRP1", "Subunit of cleavage factor I" },
  { "SMF1", "Divalent metal ion transporter" },
  { "RPS19A", "Protein component of the small (40S) ribosomal subunit" },
  { "RPL18A", "Ribosomal 60S subunit protein L18A" },
  { "MCH4", "Protein with similarity to mammalian monocarboxylate permeases" },
  { "YOL118C", "Putative protein of unknown function" },
  { "RRI2", "Subunit of the COP9 signalosome (CSN) complex" },
  { "MSN1", "Transcriptional activator" },
  { "PAP2", "Non-canonical poly(A) polymerase" },
  { "YOL114C", "NULL" },
  { "SKM1", "Member of the PAK family of serine/threonine protein kinases" },
  { "MSB4", "GTPase-activating protein of the Ras superfamily" },
  { "MDY2", "Protein involved in inserting tail-anchored proteins into ER membranes" },
  { "SHR5", "Palmitoyltransferase subunit" },
  { "ZEO1", "Peripheral membrane protein of the plasma membrane" },
  { "INO4", "Transcription factor involved in phospholipid synthesis" },
  { "YOL107W", "Putative protein of unknown function" },
  { "YOL106W", "NULL" },
  { "WSC3", "Sensor-transducer of the stress-activated PKC1-MPK1 signaling pathway" },
  { "NDJ1", "Protein that regulates meiotic SPB cohesion and telomere clustering" },
  { "ITR2", "Myo-inositol transporter" },
  { "TPT1", "tRNA 2'-phosphotransferase that catalyzes final step in tRNA splicing" },
  { "IZH4", "Membrane protein involved in zinc ion homeostasis" },
  { "PKH2", "Serine/threonine protein kinase" },
  { "YOL099C", "NULL" },
  { "YOL098C", "NULL" },
  { "YOL097W-A", "Putative protein of unknown function" },
  { "WRS1", "Cytoplasmic tryptophanyl-tRNA synthetase" },
  { "COQ3", "O-methyltransferase" },
  { "HMI1", "Mitochondrial inner membrane localized ATP-dependent DNA helicase" },
  { "RFC4", "Subunit of heteropentameric Replication factor C (RF-C)" },
  { "TRM10", "tRNA methyltransferase" },
  { "YPQ1", "Putative vacuolar membrane transporter for cationic amino acids" },
  { "SPO21", "Component of the meiotic outer plaque of the spindle pole body" },
  { "MSH2", "Protein that binds to DNA mismatches" },
  { "HAL9", "Putative transcription factor containing a zinc finger" },
  { "MPD2", "Member of the protein disulfide isomerase (PDI) family" },
  { "DUF1", "Ubiquitin-binding protein of unknown function" },
  { "MHF1", "Component of the heterotetrameric MHF histone-fold complex" },
  { "ADH1", "Alcohol dehydrogenase" },
  { "YOL085W-A", "NULL" },
  { "YOL085C", "Putative protein of unknown function" },
  { "PHM7", "Protein of unknown function" },
  { "ATG34", "Receptor protein involved in selective autophagy during starvation" },
  { "YOL083C-A", "NULL" },
  { "ATG19", "Receptor protein for the cytoplasm-to-vacuole targeting (Cvt) pathway" },
  { "IRA2", "GTPase-activating protein" },
  { "REX4", "Putative RNA exonuclease" },
  { "YOL079W", "NULL" },
  { "AVO1", "Component of a membrane-bound complex containing the Tor2p kinase" },
  { "ATP19", "Subunit k of the mitochondrial F1F0 ATP synthase" },
  { "BRX1", "Nucleolar protein" },
  { "MDM20", "Non-catalytic subunit of the NatB N-terminal acetyltransferase" },
  { "YOL075C", "Putative ABC transporter" },
  { "YOL073C", "NULL" },
  { "THP1", "Nuclear pore-associated protein" },
  { "EMI5", "NULL" },
  { "NBA1", "Protein of unknown function" },
  { "NUF2", "Component of the kinetochore-associated Ndc80 complex" },
  { "HST1", "NAD(+)-dependent histone deacetylase" },
  { "RTG1", "Transcription factor (bHLH) involved in interorganelle communication" },
  { "RIB2", "Bifunctional DRAP deaminase tRNA:pseudouridine synthase" },
  { "INP54", "Phosphatidylinositol 4,5-bisphosphate 5-phosphatase" },
  { "MET22", "Bisphosphate-3'-nucleotidase" },
  { "CRT10", "Protein involved in transcriptional regulation of RNR2 and RNR3" },
  { "APM4", "Cargo-binding mu subunit of AP-2" },
  { "PRS5", "5-phospho-ribosyl-1(alpha)-pyrophosphate synthetase" },
  { "MAM3", "Protein required for normal mitochondrial morphology" },
  { "GPD2", "NAD-dependent glycerol 3-phosphate dehydrogenase" },
  { "ARG1", "Arginosuccinate synthetase" },
  { "YOL057W", "Dipeptidyl-peptidase III" },
  { "GPM3", "Nonfunctional homolog of Gpm1p phosphoglycerate mutase" },
  { "THI20", "Trifunctional enzyme of thiamine biosynthesis, degradation and salvage" },
  { "PSH1", "E3 ubiquitin ligase targeting centromere-binding protein Cse4p" },
  { "AIM39", "Protein of unknown function" },
  { "DDR2", "Multi-stress response protein" },
  { "SPE2", "S-adenosylmethionine decarboxylase" },
  { "GAL11", "Subunit of the RNA polymerase II mediator complex" },
  { "YOL050C", "NULL" },
  { "GSH2", "Glutathione synthetase" },
  { "RRT8", "Protein involved in spore wall assembly" },
  { "LDS2", "Protein Involved in spore wall assembly" },
  { "YOL046C", "NULL" },
  { "PSK2", "PAS-domain containing serine/threonine protein kinase" },
  { "PEX15", "Tail-anchored type II integral peroxisomal membrane protein" },
  { "NTG2", "DNA N-glycosylase and apurinic/apyrimidinic (AP) lyase" },
  { "NGL1", "Putative endonuclease" },
  { "NOP12", "Nucleolar protein involved in pre-25S rRNA processing" },
  { "RPS15", "Protein component of the small (40S) ribosomal subunit" },
  { "RPP2A", "Ribosomal protein P2 alpha" },
  { "YOL038C-A", "Putative protein of unknown function" },
  { "PRE6", "Alpha 4 subunit of the 20S proteasome" },
  { "YOL037C", "NULL" },
  { "YOL036W", "Protein of unknown function" },
  { "YOL035C", "NULL" },
  { "SMC5", "Subunit of the SMC5-SMC6 complex" },
  { "MSE1", "Mitochondrial glutamyl-tRNA synthetase" },
  { "OPI10", "Protein with a possible role in phospholipid biosynthesis" },
  { "SIL1", "Nucleotide exchange factor for the ER lumenal Hsp70 chaperone Kar2p" },
  { "GAS5", "1,3-beta-glucanosyltransferase" },
  { "YOL029C", "Putative protein of unknown function" },
  { "YAP7", "Putative basic leucine zipper (bZIP) transcription factor" },
  { "MDM38", "Membrane-associated mitochondrial ribosome receptor" },
  { "MIM1", "Mitochondrial protein required for outer membrane protein import" },
  { "LAG2", "Protein that negatively regulates the SCF E3-ubiquitin ligase" },
  { "YOL024W", "Putative protein of unknown function" },
  { "IFM1", "Mitochondrial translation initiation factor 2" },
  { "TSR4", "Cytoplasmic protein required for correct processing of 20S pre-rRNA" },
  { "DIS3", "Exosome core complex catalytic subunit" },
  { "TAT2", "High affinity tryptophan and tyrosine permease" },
  { "YOL019W-A", "Protein of unknown function" },
  { "YOL019W", "Protein of unknown function" },
  { "TLG2", "Syntaxin-like t-SNARE" },
  { "ESC8", "Protein involved in telomeric and mating-type locus silencing" },
  { "CMK2", "Calmodulin-dependent protein kinase" },
  { "IRC10", "Protein of unknown function" },
  { "YOL014W", "Putative protein of unknown function" },
  { "YOL013W-B", "NULL" },
  { "YOL013W-A", "Putative protein of unknown function" },
  { "HRD1", "Ubiquitin-protein ligase involved in ER-associated degradation (ERAD)" },
  { "HTZ1", "Histone variant H2AZ" },
  { "PLB3", "Phospholipase B (lysophospholipase) involved in lipid metabolism" },
  { "RCL1", "Endonuclease that cleaves pre-rRNA at site A2 for 18S rRNA biogenesis" },
  { "MDM12", "Mitochondrial outer membrane protein, ERMES complex subunit" },
  { "COQ10", "Coenzyme Q (ubiquinone) binding protein" },
  { "CSI2", "Protein of unknown function" },
  { "TOP1", "Topoisomerase I" },
  { "RPB11", "RNA polymerase II subunit B12.5" },
  { "SIN3", "Component of both the Rpd3S and Rpd3L histone deacetylase complexes" },
  { "PFA4", "Palmitoyltransferase with autoacylation activity" },
  { "IZH2", "Plasma membrane receptor for plant antifungal osmotin" },
  { "PHO80", "Cyclin" },
  { "RRP6", "Nuclear exosome exonuclease component" },
  { "ALG6", "Alpha 1,3 glucosyltransferase" },
  { "YSP3", "Putative precursor of the subtilisin-like protease III" },
  { "UTP23", "Component of the small subunit processome" },
  { "DNL4", "DNA ligase required for nonhomologous end-joining (NHEJ)" },
  { "TSR3", "Protein required for 20S pre-rRNA processing" },
  { "SGT2", "Glutamine-rich cytoplasmic cochaperone" },
  { "SLG1", "Sensor-transducer of the stress-activated PKC1-MPK1 kinase pathway" },
  { "YOR008C-A", "Putative protein of unknown function" },
  { "YOR008W-B", "NULL" },
  { "TIR4", "Cell wall mannoprotein" },
  { "TIR2", "Putative cell wall mannoprotein" },
  { "AUS1", "Plasma membrane sterol transporter of the ATP-binding cassette family" },
  { "YOR011W-A", "Putative protein of unknown function" },
  { "YOR012W", "Putative protein of unknown function" },
  { "IRC11", "NULL" },
  { "RTS1", "B-type regulatory subunit of protein phosphatase 2A (PP2A)" },
  { "YOR015W", "Putative protein of unknown function" },
  { "ERP4", "Member of the p24 family involved in ER to Golgi transport" },
  { "PET127", "Protein with a role in 5'-end processing of mitochondrial RNAs" },
  { "ROD1", "Alpha-arrestin involved in ubiquitin-dependent endocytosis" },
  { "YOR019W", "Protein of unknown function" },
  { "HSP10", "Mitochondrial matrix co-chaperonin" },
  { "YOR020W-A", "NULL" },
  { "SFM1", "SPOUT methyltransferase" },
  { "YOR022C", "NULL" },
  { "AHC1", "Subunit of the Ada histone acetyltransferase complex" },
  { "YOR024W", "NULL" },
  { "HST3", "Member of the Sir2 family of NAD(+)-dependent protein deacetylases" },
  { "BUB3", "Kinetochore checkpoint WD40 repeat protein" },
  { "STI1", "Hsp90 cochaperone" },
  { "CIN5", "Basic leucine zipper (bZIP) transcription factor of the yAP-1 family" },
  { "YOR029W", "Putative protein of unknown function" },
  { "DFG16", "Probable multiple transmembrane protein" },
  { "CRS5", "Copper-binding metallothionein" },
  { "HMS1", "bHLH protein with similarity to myc-family transcription factors" },
  { "YOR032W-A", "Protein of unknown function" },
  { "EXO1", "5'-3' exonuclease and flap-endonuclease" },
  { "AKR2", "Ankyrin repeat-containing protein" },
  { "YOR034C-A", "Putative protein of unknown function" },
  { "SHE4", "Protein containing a UCS (UNC-45/CRO1/SHE4) domain" },
  { "PEP12", "Target membrane receptor (t-SNARE)" },
  { "CYC2", "Mitochondrial peripheral inner membrane protein" },
  { "HIR2", "Subunit of HIR nucleosome assembly complex" },
  { "CKB2", "Beta' regulatory subunit of casein kinase 2 (CK2)" },
  { "GLO4", "Mitochondrial glyoxalase II" },
  { "YOR041C", "NULL" },
  { "CUE5", "Ubiquitin-binding protein" },
  { "WHI2", "Negative regulator of TORC1 in response to limiting leucine" },
  { "IRC23", "Protein of unknown function" },
  { "TOM6", "Component of the TOM (translocase of outer membrane) complex" },
  { "DBP5", "Cytoplasmic ATP-dependent RNA helicase of the DEAD-box family" },
  { "STD1", "Protein involved in control of glucose-regulated gene expression" },
  { "RAT1", "Nuclear 5' to 3' single-stranded RNA exonuclease" },
  { "RSB1", "Putative sphingoid long-chain base (LCB) efflux transporter" },
  { "YOR050C", "NULL" },
  { "ETT1", "Nuclear protein that inhibits replication of Brome mosaic virus" },
  { "TMC1", "AN1-type zinc finger protein, effector of proteotoxic stress response" },
  { "YOR053W", "NULL" },
  { "VHS3", "Negative regulatory subunit of protein phosphatase 1 Ppz1p" },
  { "YOR055W", "NULL" },
  { "NOB1", "Protein involved in proteasomal and 40S ribosomal subunit biogenesis" },
  { "SGT1", "Cochaperone protein" },
  { "ASE1", "Mitotic spindle midzone-localized microtubule bundling protein" },
  { "LPL1", "Phospholipase" },
  { "SLD7", "Protein with a role in chromosomal DNA replication" },
  { "CKA2", "Alpha' catalytic subunit of casein kinase 2 (CK2)" },
  { "YOR062C", "Protein of unknown function" },
  { "RPL3", "Ribosomal 60S subunit protein L3" },
  { "YNG1", "Subunit of the NuA3 histone acetyltransferase complex" },
  { "CYT1", "Cytochrome c1" },
  { "MSA1", "Activator of G1-specific transcription factors MBF and SBF" },
  { "ALG8", "Glucosyl transferase" },
  { "VPS5", "Nexin-1 homolog" },
  { "VAM10", "Protein involved in vacuole morphogenesis" },
  { "GYP1", "Cis-golgi GTPase-activating protein (GAP) for yeast Rabs" },
  { "NRT1", "High-affinity nicotinamide riboside transporter" },
  { "YOR072W", "Putative protein of unknown function" },
  { "YOR072W-A", "NULL" },
  { "YOR072W-B", "Putative protein of unknown function" },
  { "SGO1", "Component of the spindle checkpoint" },
  { "CDC21", "Thymidylate synthase" },
  { "YOR073W-A", "NULL" },
  { "UFE1", "t-SNARE protein required for retrograde vesicular traffic" },
  { "SKI7", "GTP-binding protein that couples the Ski complex and exosome" },
  { "RTS2", "Basic zinc-finger protein" },
  { "BUD21", "Component of small ribosomal subunit (SSU) processosome" },
  { "ATX2", "Golgi membrane protein involved in manganese homeostasis" },
  { "DIA2", "Origin-binding F-box protein" },
  { "TGL5", "Bifunctional triacylglycerol lipase and LPA acyltransferase" },
  { "YOR082C", "NULL" },
  { "WHI5", "Repressor of G1 transcription" },
  { "LPX1", "Peroxisomal matrix-localized lipase" },
  { "OST3", "Gamma subunit of the oligosaccharyltransferase complex of the ER lumen" },
  { "TCB1", "Lipid-binding ER protein involved in ER-plasma membrane tethering" },
  { "YVC1", "Vacuolar cation channel" },
  { "VPS21", "Endosomal Rab family GTPase" },
  { "PTC5", "Mitochondrial type 2C protein phosphatase (PP2C)" },
  { "TMA46", "Protein of unknown function that associates with translating ribosomes" },
  { "ECM3", "Non-essential protein of unknown function" },
  { "YOR093C", "NULL" },
  { "ARF3", "Glucose-repressible ADP-ribosylation factor" },
  { "RKI1", "Ribose-5-phosphate ketol-isomerase" },
  { "RPS7A", "Protein component of the small (40S) ribosomal subunit" },
  { "YOR097C", "Putative protein of unknown function" },
  { "NUP1", "FG-nucleoporin component of central core of the nuclear pore complex" },
  { "KTR1", "Alpha-1,2-mannosyltransferase" },
  { "CRC1", "Mitochondrial inner membrane carnitine transporter" },
  { "RAS1", "GTPase involved in G-protein signaling in adenylate cyclase activation" },
  { "YOR102W", "NULL" },
  { "OST2", "Epsilon subunit of the oligosaccharyltransferase complex" },
  { "PIN2", "Exomer-dependent cargo protein" },
  { "YOR105W", "Protein of unknown function" },
  { "VAM3", "Syntaxin-like vacuolar t-SNARE" },
  { "RGS2", "Negative regulator of glucose-induced cAMP signaling" },
  { "LEU9", "Alpha-isopropylmalate synthase II (2-isopropylmalate synthase)" },
  { "YOR108C-A", "NULL" },
  { "INP53", "Polyphosphatidylinositol phosphatase" },
  { "TFC7", "RNA pol III transcription initiation factor complex (TFIIIC) subunit" },
  { "YOR111W", "Putative protein of unknown function" },
  { "CEX1", "Component of nuclear aminoacylation-dependent tRNA export pathway" },
  { "AZF1", "Zinc-finger transcription factor" },
  { "YOR114W", "NULL" },
  { "TRS33", "Core component of TRAPP complexes I, II and IV" },
  { "RPO31", "RNA polymerase III largest subunit C160" },
  { "RPT5", "ATPase of the 19S regulatory particle of the 26S proteasome" },
  { "RTC5", "Protein of unknown function" },
  { "RIO1", "Serine kinase involved in cell cycle regulation and rDNA integrity" },
  { "GCY1", "Glycerol dehydrogenase" },
  { "YOR121C", "NULL" },
  { "PFY1", "Profilin" },
  { "LEO1", "Component of the Paf1 complex" },
  { "UBP2", "Ubiquitin-specific protease" },
  { "CAT5", "Protein required for ubiquinone (Coenzyme Q) biosynthesis" },
  { "IAH1", "Isoamyl acetate-hydrolyzing esterase" },
  { "RGA1", "GTPase-activating protein for polarity-establishment protein Cdc42p" },
  { "ADE2", "Phosphoribosylaminoimidazole carboxylase" },
  { "AFI1", "Arf3p polarization-specific docking factor" },
  { "ORT1", "Ornithine transporter of the mitochondrial inner membrane" },
  { "YOR131C", "Putative haloacid dehalogenase-like hydrolase" },
  { "VPS17", "Subunit of the membrane-associated retromer complex" },
  { "EFT1", "Elongation factor 2 (EF-2), also encoded by EFT2" },
  { "BAG7", "Rho GTPase activating protein (RhoGAP)" },
  { "IRC14", "NULL" },
  { "IDH2", "Subunit of mitochondrial NAD(+)-dependent isocitrate dehydrogenase" },
  { "SIA1", "Protein of unassigned function" },
  { "RUP1", "Protein that regulates ubiquitination of Rsp5p" },
  { "YOR139C", "NULL" },
  { "SFL1", "Transcriptional repressor and activator" },
  { "ARP8", "Nuclear actin-related protein involved in chromatin remodeling" },
  { "LSC1", "Alpha subunit of succinyl-CoA ligase" },
  { "THI80", "Thiamine pyrophosphokinase" },
  { "ELG1", "Subunit of an alternative replication factor C complex" },
  { "PNO1", "Essential nucleolar protein required for pre-18S rRNA processing" },
  { "YOR146W", "NULL" },
  { "MDM32", "Mitochondrial inner membrane protein with similarity to Mdm31p" },
  { "SPP2", "Essential protein that promotes the first step of splicing" },
  { "SMP3", "Alpha 1,2-mannosyltransferase" },
  { "MRPL23", "Mitochondrial ribosomal protein of the large subunit" },
  { "RPB2", "RNA polymerase II second largest subunit B150" },
  { "YOR152C", "NULL" },
  { "PDR5", "Plasma membrane ATP-binding cassette (ABC) transporter" },
  { "SLP1", "Glycosylated integral ER membrane protein of unknown function" },
  { "ISN1", "Inosine 5'-monophosphate (IMP)-specific 5'-nucleotidase" },
  { "NFI1", "SUMO E3 ligase" },
  { "PUP1", "Beta 2 subunit of the 20S proteasome" },
  { "PET123", "Mitochondrial ribosomal protein of the small subunit" },
  { "SME1", "Core Sm protein Sm E" },
  { "MTR10", "Nuclear import receptor" },
  { "PNS1", "Protein of unknown function" },
  { "YOR161W-A", "NULL" },
  { "YOR161W-B", "NULL" },
  { "YOR161C-C", "Protein of unknown function" },
  { "YRR1", "Zn2-Cys6 zinc-finger transcription factor" },
  { "DDP1", "Polyphosphate phosphatase" },
  { "GET4", "Protein involved in inserting tail-anchored proteins into ER membranes" },
  { "SEY1", "Dynamin-like GTPase that mediates homotypic ER fusion" },
  { "SWT1", "RNA endoribonuclease involved in perinuclear mRNP quality control" },
  { "RPS28A", "Protein component of the small (40S) ribosomal subunit" },
  { "GLN4", "Glutamine tRNA synthetase" },
  { "YOR169C", "NULL" },
  { "YOR170W", "NULL" },
  { "LCB4", "Sphingoid long-chain base kinase" },
  { "YRM1", "Zinc finger transcription factor involved in multidrug resistance" },
  { "DCS2", "m(7)GpppX pyrophosphatase regulator" },
  { "MED4", "Subunit of the RNA polymerase II mediator complex" },
  { "ALE1", "Broad-specificity lysophospholipid acyltransferase" },
  { "HEM15", "Ferrochelatase" },
  { "MPC54", "Component of the meiotic outer plaque" },
  { "GAC1", "Regulatory subunit for Glc7p type-1 protein phosphatase (PP1)" },
  { "SYC1", "Subunit of the APT subcomplex of cleavage and polyadenylation factor" },
  { "DCI1", "Peroxisomal protein" },
  { "LAS17", "Actin assembly factor" },
  { "RPS30B", "Protein component of the small (40S) ribosomal subunit" },
  { "FYV12", "Protein of unknown function" },
  { "SER1", "3-phosphoserine aminotransferase" },
  { "GSP2", "GTP binding protein (mammalian Ranp homolog)" },
  { "YOR186W", "Putative protein of unknown function" },
  { "YOR186C-A", "NULL" },
  { "TUF1", "Mitochondrial translation elongation factor Tu (EF-Tu)" },
  { "MSB1", "Protein of unknown function" },
  { "IES4", "Component of the INO80 chromatiin remodeling complex" },
  { "SPR1", "Sporulation-specific exo-1,3-beta-glucanase" },
  { "ULS1", "Swi2/Snf2-related translocase, SUMO-Targeted Ubiquitin Ligase (STUbL)" },
  { "THI72", "Transporter of thiamine or related compound" },
  { "YOR192C-C", "Putative protein of unknown function" },
  { "PEX27", "Peripheral peroxisomal membrane protein" },
  { "TOA1", "TFIIA large subunit" },
  { "SLK19", "Kinetochore-associated protein" },
  { "LIP5", "Protein involved in biosynthesis of the coenzyme lipoic acid" },
  { "MCA1", "Ca2+-dependent cysteine protease" },
  { "BFR1", "Component of mRNP complexes associated with polyribosomes" },
  { "YOR199W", "NULL" },
  { "YOR200W", "NULL" },
  { "MRM1", "Ribose methyltransferase" },
  { "HIS3", "Imidazoleglycerol-phosphate dehydratase" },
  { "YOR203W", "NULL" },
  { "DED1", "ATP-dependent DEAD (Asp-Glu-Ala-Asp)-box RNA helicase" },
  { "GEP3", "Protein required for mitochondrial ribosome small subunit biogenesis" },
  { "NOC2", "Protein involved in ribosome biogenesis" },
  { "RET1", "Second-largest subunit of RNA polymerase III" },
  { "PTP2", "Phosphotyrosine-specific phosphatase" },
  { "NPT1", "Nicotinate phosphoribosyltransferase" },
  { "RPB10", "RNA polymerase subunit ABC10-beta" },
  { "MGM1", "Mitochondrial GTPase, present in complex with Ugo1p and Fzo1p" },
  { "STE4", "G protein beta subunit" },
  { "SAS5", "Subunit of the SAS complex (Sas2p, Sas4p, Sas5p)" },
  { "SPR2", "Putative spore wall protein" },
  { "AIM41", "Protein of unknown function" },
  { "RUD3", "Golgi matrix protein" },
  { "RFC1", "Subunit of heteropentameric Replication factor C (RF-C)" },
  { "YOR218C", "NULL" },
  { "STE13", "Dipeptidyl aminopeptidase" },
  { "RCN2", "Protein of unknown function" },
  { "MCT1", "Predicted malonyl-CoA:ACP transferase" },
  { "ODC2", "Mitochondrial inner membrane transporter" },
  { "YOR223W", "NULL" },
  { "RPB8", "RNA polymerase subunit ABC14.5" },
  { "YOR225W", "NULL" },
  { "ISU2", "Mitochondrial protein required for iron-sulfur protein synthesis" },
  { "HER1", "Protein of unknown function" },
  { "MCP1", "Mitochondrial protein of unknown function involved in lipid homeostas" },
  { "WTM2", "Transcriptional modulator" },
  { "WTM1", "Transcriptional modulator" },
  { "MKK1", "MAPKK involved in the protein kinase C signaling pathway" },
  { "YOR231C-A", "NULL" },
  { "MGE1", "Mitochondrial matrix cochaperone" },
  { "KIN4", "Serine/threonine protein kinase" },
  { "RPL33B", "Ribosomal 60S subunit protein L33B" },
  { "IRC13", "Putative protein of unknown function" },
  { "DFR1", "Dihydrofolate reductase involved in tetrahydrofolate biosynthesis" },
  { "HES1", "Protein implicated in the regulation of ergosterol biosynthesis" },
  { "YOR238W", "Putative protein of unknown function" },
  { "ABP140", "AdoMet-dependent tRNA methyltransferase and actin binding protein" },
  { "MET7", "Folylpolyglutamate synthetase" },
  { "SSP2", "Sporulation specific protein that localizes to the spore wall" },
  { "PUS7", "Pseudouridine synthase" },
  { "ESA1", "Catalytic subunit of the histone acetyltransferase complex (NuA4)" },
  { "DGA1", "Diacylglycerol acyltransferase" },
  { "ENV9", "Protein proposed to be involved in vacuolar functions" },
  { "SRL1", "Mannoprotein that exhibits a tight association with the cell wall" },
  { "YOR248W", "NULL" },
  { "APC5", "Subunit of the Anaphase-Promoting Complex/Cyclosome (APC/C)" },
  { "CLP1", "Component of the cleavage and polyadenylation factor I (CF I)" },
  { "TUM1", "Rhodanese domain sulfur transferase" },
  { "TMA16", "Protein of unknown function that associates with ribosomes" },
  { "NAT5", "Subunit of protein N-terminal acetyltransferase NatA" },
  { "SEC63", "Essential subunit of Sec63 complex" },
  { "OSW1", "Protein involved in sporulation" },
  { "TRE2", "Transferrin receptor-like protein" },
  { "CDC31", "Calcium-binding component of the spindle pole body (SPB) half-bridge" },
  { "HNT3", "DNA 5' AMP hydrolase involved in DNA repair" },
  { "RPT4", "ATPase of the 19S regulatory particle of the 26S proteasome" },
  { "GCD1", "Gamma subunit of the translation initiation factor eIF2B" },
  { "RPN8", "Essential non-ATPase regulatory subunit of the 26S proteasome" },
  { "GPN2", "Putative GTPase with a role in biogenesis of RNA pol II and polIII" },
  { "YOR263C", "NULL" },
  { "DSE3", "Daughter cell-specific protein, may help establish daughter fate" },
  { "RBL2", "Protein involved in microtubule morphogenesis" },
  { "PNT1", "Mitochondrial integral inner membrane protein" },
  { "HRK1", "Protein kinase" },
  { "YOR268C", "Putative protein of unknown function" },
  { "PAC1", "Involved in nuclear migration, part of the dynein/dynactin pathway" },
  { "VPH1", "Subunit a of vacuolar-ATPase V0 domain" },
  { "FSF1", "Putative protein" },
  { "YTM1", "Ribosomal assembly factor and 66S pre-ribosomal particle constituent" },
  { "TPO4", "Polyamine transporter of the major facilitator superfamily" },
  { "MOD5", "Delta 2-isopentenyl pyrophosphate:tRNA isopentenyl transferase" },
  { "RIM20", "Protein involved in proteolytic activation of Rim101p" },
  { "CAF20", "Phosphoprotein of the mRNA cap-binding complex" },
  { "YOR277C", "NULL" },
  { "HEM4", "Uroporphyrinogen III synthase" },
  { "RFM1", "Component of the Sum1p-Rfm1p-Hst1p complex" },
  { "FSH3", "Putative serine hydrolase" },
  { "PLP2", "Protein that interacts with the CCT complex to stimulate actin folding" },
  { "YOR282W", "NULL" },
  { "YOR283W", "Phosphatase with a broad substrate specificity" },
  { "HUA2", "Cytoplasmic protein of unknown function" },
  { "RDL1", "Thiosulfate sulfurtransferase" },
  { "RDL2", "Protein with rhodanese activity" },
  { "RRP36", "Component of 90S preribosomes" },
  { "MPD1", "Member of the protein disulfide isomerase (PDI) family" },
  { "YOR289W", "Putative protein of unknown function" },
  { "SNF2", "Catalytic subunit of the SWI/SNF chromatin remodeling complex" },
  { "YPK9", "Vacuolar protein with a possible role in sequestering heavy metals" },
  { "YOR292C", "Putative protein of unknown function" },
  { "RPS10A", "Protein component of the small (40S) ribosomal subunit" },
  { "YOR293C-A", "Putative protein of unknown function" },
  { "RRS1", "Essential protein that binds ribosomal protein L11" },
  { "UAF30", "Subunit of UAF (upstream activation factor) complex" },
  { "YOR296W", "Putative protein of unknown function" },
  { "TIM18", "Component of the mitochondrial TIM22 complex" },
  { "MUM3", "Protein of unknown function involved in outer spore wall organization" },
  { "MBF1", "Transcriptional coactivator" },
  { "BUD7", "Member of the ChAPs family (Chs5p-Arf1p-binding proteins)" },
  { "YOR300W", "NULL" },
  { "RAX1", "Protein involved in bud site selection during bipolar budding" },
  { "YOR302W", "CPA1 uORF" },
  { "CPA1", "Small subunit of carbamoyl phosphate synthetase" },
  { "ISW2", "ATP-dependent DNA translocase involved in chromatin remodeling" },
  { "BIL1", "Protein that binds Bud6p and has a role in actin cable assembly" },
  { "RRG7", "Protein of unknown function" },
  { "MCH5", "Plasma membrane riboflavin transporter" },
  { "SLY41", "Protein involved in ER-to-Golgi transport" },
  { "SNU66", "Component of the U4/U6.U5 snRNP complex" },
  { "YOR309C", "NULL" },
  { "NOP58", "Protein involved in producing mature rRNAs and snoRNAs" },
  { "DGK1", "Diacylglycerol kinase" },
  { "RPL20B", "Ribosomal 60S subunit protein L20B" },
  { "SPS4", "Protein whose expression is induced during sporulation" },
  { "YOR314W", "Putative protein of unknown function" },
  { "YOR314W-A", "NULL" },
  { "SFG1", "Nuclear protein putative transcription factor" },
  { "COT1", "Vacuolar transporter that mediates zinc transport into the vacuole" },
  { "YOR316C-A", "Putative protein of unknown function" },
  { "FAA1", "Long chain fatty acyl-CoA synthetase" },
  { "YOR318C", "NULL" },
  { "HSH49", "U2-snRNP associated splicing factor" },
  { "GNT1", "N-acetylglucosaminyltransferase" },
  { "PMT3", "Protein O-mannosyltransferase" },
  { "LDB19", "Alpha-arrestin involved in ubiquitin-dependent endocytosis" },
  { "PRO2", "Gamma-glutamyl phosphate reductase" },
  { "FRT1", "Tail-anchored ER membrane protein of unknown function" },
  { "YOR325W", "NULL" },
  { "MYO2", "Type V myosin motor involved in actin-based transport of cargos" },
  { "SNC2", "Vesicle membrane receptor protein (v-SNARE)" },
  { "PDR10", "ATP-binding cassette (ABC) transporter" },
  { "SCD5", "Protein required for normal actin organization and endocytosis" },
  { "YOR329W-A", "NULL" },
  { "MIP1", "Mitochondrial DNA polymerase gamma" },
  { "YOR331C", "NULL" },
  { "VMA4", "Subunit E of the V1 domain of the vacuolar H+-ATPase (V-ATPase)" },
  { "YOR333C", "NULL" },
  { "MRS2", "Mitochondrial inner membrane Mg(2+) channel" },
  { "ALA1", "Cytoplasmic and mitochondrial alanyl-tRNA synthetase" },
  { "YOR335W-A", "NULL" },
  { "KRE5", "Protein required for beta-1,6 glucan biosynthesis" },
  { "TEA1", "Ty1 enhancer activator involved in Ty enhancer-mediated transcription" },
  { "YOR338W", "Putative protein of unknown function" },
  { "UBC11", "Ubiquitin-conjugating enzyme" },
  { "RPA43", "RNA polymerase I subunit A43" },
  { "RPA190", "RNA polymerase I largest subunit A190" },
  { "YOR342C", "Protein of unknown function" },
  { "YOR343C", "Putative protein of unknown function" },
  { "TYE7", "Serine-rich protein that contains a bHLH DNA binding motif" },
  { "YOR345C", "NULL" },
  { "REV1", "Deoxycytidyl transferase" },
  { "PYK2", "Pyruvate kinase" },
  { "PUT4", "Proline permease" },
  { "CIN1", "Tubulin folding factor D involved in beta-tubulin (Tub2p) folding" },
  { "MNE1", "Protein involved in splicing Group I aI5-beta intron from COX1 mRNA" },
  { "MEK1", "Meiosis-specific serine/threonine protein kinase" },
  { "TFB6", "Subunit of TFIIH complex" },
  { "SOG2", "Key component of the RAM signaling network" },
  { "MSC6", "Multicopy suppressor of HER2 involved in mitochondrial translation" },
  { "GDS1", "Protein of unknown function" },
  { "CIR2", "Putative ortholog of human ETF-dH" },
  { "SNX3", "Sorting nexin for late-Golgi enzymes" },
  { "HAP5", "Subunit of the Hap2p/3p/4p/5p CCAAT-binding complex" },
  { "VTS1", "Flap-structured DNA-binding and RNA-binding protein" },
  { "PDE2", "High-affinity cyclic AMP phosphodiesterase" },
  { "PRT1", "eIF3b subunit of the eukaryotic translation initiation factor 3 (eIF3)" },
  { "PRE10", "Alpha 7 subunit of the 20S proteasome" },
  { "PIP2", "Autoregulatory, oleate-activated transcription factor" },
  { "YOR364W", "NULL" },
  { "YOR365C", "Putative protein of unknown function" },
  { "YOR366W", "NULL" },
  { "SCP1", "Component of yeast cortical actin cytoskeleton" },
  { "RAD17", "Checkpoint protein" },
  { "RPS12", "Protein component of the small (40S) ribosomal subunit" },
  { "MRS6", "Rab escort protein" },
  { "GPB1", "Multistep regulator of cAMP-PKA signaling" },
  { "NDD1", "Transcriptional activator essential for nuclear division" },
  { "NUD1", "Mitotic exit network (MEN) scaffold protein" },
  { "ALD4", "Mitochondrial aldehyde dehydrogenase" },
  { "GDH1", "NADP(+)-dependent glutamate dehydrogenase" },
  { "YOR376W", "Putative protein of unknown function" },
  { "YOR376W-A", "Putative protein of unknown function" },
  { "ATF1", "Alcohol acetyltransferase" },
  { "AMF1", "Low affinity NH4+ transporter" },
  { "YOR379C", "NULL" },
  { "RDR1", "Transcriptional repressor involved in regulating multidrug resistance" },
  { "FRE3", "Ferric reductase" },
  { "YOR381W-A", "Putative protein of unknown function" },
  { "FIT2", "Mannoprotein that is incorporated into the cell wall" },
  { "FIT3", "Mannoprotein that is incorporated into the cell wall" },
  { "FRE5", "Putative ferric reductase with similarity to Fre2p" },
  { "YOR385W", "Putative protein of unknown function" },
  { "PHR1", "DNA photolyase involved in photoreactivation" },
  { "YOR387C", "Putative protein of unknown function" },
  { "FDH1", "NAD(+)-dependent formate dehydrogenase" },
  { "YOR389W", "Putative protein of unknown function" },
  { "FEX1", "Protein involved in fluoride export" },
  { "HSP33", "Possible chaperone and cysteine protease" },
  { "YOR392W", "NULL" },
  { "ERR1", "Putative phosphopyruvate hydratase" },
  { "PAU21", "Protein of unknown function" },
  { "YOR394C-A", "Protein of unknown function" },
  { "YRF1-8", "One of several telomeric Y' element-encoded DNA helicases" },
  { "YOR396C-A", "NULL" },
  { "YRF1-7", "Helicase encoded by the Y' element of subtelomeric regions" },
  { "YPL283W-B", "NULL" },
  { "YPL283W-A", "NULL" },
  { "PAU22", "Protein of unknown function" },
  { "ERR2", "Enolase, a phosphopyruvate hydratase" },
  { "HSP32", "Possible chaperone and cysteine protease" },
  { "FEX2", "Protein involved in fluoride export" },
  { "YPL278C", "Putative protein of unknown function" },
  { "YPL277C", "Putative protein of unknown function" },
  { "SAM3", "High-affinity S-adenosylmethionine permease" },
  { "SAM4", "S-adenosylmethionine-homocysteine methyltransferase" },
  { "PBI1", "Putative protein of unknown function" },
  { "ATP15", "Epsilon subunit of the F1 sector of mitochondrial F1F0 ATP synthase" },
  { "MDL2", "Mitochondrial inner membrane half-type ABC transporter" },
  { "KAR9", "Spindle positioning factor" },
  { "PLC1", "Phospholipase C" },
  { "ACM1", "Pseudosubstrate inhibitor of the APC/C" },
  { "DIM1", "Essential 18S rRNA dimethylase (dimethyladenosine transferase)" },
  { "DIP5", "Dicarboxylic amino acid permease" },
  { "YPL264C", "Putative membrane protein of unknown function" },
  { "KEL3", "Cytoplasmic protein of unknown function" },
  { "FUM1", "Fumarase" },
  { "YPL261C", "Putative protein of unknown function" },
  { "YPL260W", "NULL" },
  { "APM1", "Mu1-like medium subunit of the AP-1 complex" },
  { "THI21", "Hydroxymethylpyrimidine (HMP) and HMP-phosphate kinase" },
  { "YPL257W", "Putative protein of unknown function" },
  { "CLN2", "G1 cyclin involved in regulation of the cell cycle" },
  { "BBP1", "Protein required for the spindle pole body (SPB) duplication" },
  { "HFI1", "Adaptor protein required for structural integrity of the SAGA complex" },
  { "VIK1", "Protein that forms a kinesin-14 heterodimeric motor with Kar3p" },
  { "YAH1", "Ferredoxin of the mitochondrial matrix" },
  { "YPL251W", "NULL" },
  { "ICY2", "NULL" },
  { "RPL36B", "Ribosomal 60S subunit protein L36B" },
  { "YPL250W-A", "NULL" },
  { "GYP5", "GTPase-activating protein (GAP) for yeast Rab family members" },
  { "GAL4", "DNA-binding transcription factor required for activating GAL genes" },
  { "YPL247C", "Putative protein of unknown function" },
  { "RBD2", "Possible rhomboid protease" },
  { "YPL245W", "Putative protein of unknown function" },
  { "HUT1", "Protein with a role in UDP-galactose transport to the Golgi lumen" },
  { "SRP68", "Core component of the signal recognition particle (SRP) complex" },
  { "IQG1", "Essential protein required for determination of budding pattern" },
  { "CIN2", "GTPase-activating protein (GAP) for Cin4p" },
  { "HSP82", "Hsp90 chaperone" },
  { "YAR1", "Ankyrin-repeat containing, nucleocytoplasmic shuttling chaperone" },
  { "YPL238C", "NULL" },
  { "SUI3", "Beta subunit of the translation initiation factor eIF2" },
  { "ENV7", "Vacuolar membrane protein kinase" },
  { "RVB2", "ATP-dependent DNA helicase, also known as reptin" },
  { "VMA11", "Vacuolar ATPase V0 domain subunit c'" },
  { "NSL1", "Essential component of the MIND kinetochore complex" },
  { "SSO1", "Plasma membrane t-SNARE" },
  { "FAS2", "Alpha subunit of fatty acid synthetase" },
  { "USV1", "Putative transcription factor containing a C2H2 zinc finger" },
  { "YPL229W", "Putative protein of unknown function" },
  { "CET1", "RNA 5'-triphosphatase involved in mRNA 5' capping" },
  { "ALG5", "UDP-glucose:dolichyl-phosphate glucosyltransferase" },
  { "NEW1", "ATP binding cassette protein" },
  { "YPL225W", "Protein of unknown function" },
  { "MMT2", "Putative metal transporter involved in mitochondrial iron accumulation" },
  { "GRE1", "Hydrophilin essential in desiccation-rehydration process" },
  { "FMP40", "Putative protein of unknown function" },
  { "YPL222C-A", "NULL" },
  { "FLC1", "Flavin adenine dinucleotide transporter" },
  { "RPL1A", "Ribosomal 60S subunit protein L1A" },
  { "PCL8", "Cyclin" },
  { "SAR1", "ARF family GTPase" },
  { "BMS1", "GTPase required for ribosomal subunit synthesis and rRNA processing" },
  { "YPL216W", "Putative protein of unknown function" },
  { "CBP3", "Mitochondrial protein required for assembly of cytochrome bc1 complex" },
  { "THI6", "Thiamine-phosphate diphosphorylase and hydroxyethylthiazole kinase" },
  { "LEA1", "Component of U2 snRNP complex" },
  { "PUS1", "tRNA:pseudouridine synthase" },
  { "NIP7", "Nucleolar protein required for 60S ribosome subunit biogenesis" },
  { "SRP72", "Core component of the signal recognition particle (SRP)" },
  { "IPL1", "Aurora kinase of chromosomal passenger complex" },
  { "RKM1", "SET-domain lysine-N-methyltransferase" },
  { "TYW1", "Iron-sulfer protein required for synthesis of Wybutosine modified tRNA" },
  { "PGC1", "Phosphatidylglycerol phospholipase C" },
  { "YPL205C", "NULL" },
  { "HRR25", "Conserved casein kinase" },
  { "TPK2", "cAMP-dependent protein kinase catalytic subunit" },
  { "AFT2", "Iron-regulated transcriptional activator" },
  { "YIG1", "Protein that interacts with glycerol 3-phosphatase" },
  { "CSM4", "Protein required for accurate chromosome segregation during meiosis" },
  { "YPL199C", "Putative protein of unknown function" },
  { "RPL7B", "Ribosomal 60S subunit protein L7B" },
  { "YPL197C", "NULL" },
  { "OXR1", "Protein of unknown function required for oxidative damage resistance" },
  { "APL5", "Delta adaptin-like subunit of the clathrin associated protein complex" },
  { "DDC1", "DNA damage checkpoint protein" },
  { "RSA1", "Protein involved in the assembly of 60S ribosomal subunits" },
  { "PRM3", "Protein required for nuclear envelope fusion during karyogamy" },
  { "YPL191C", "K48-specific deubiquitinating (DUB) enzyme" },
  { "NAB3", "RNA-binding protein, subunit of Nrd1 complex (Nrd1p-Nab3p-Sen1p)" },
  { "COA2", "Cytochrome oxidase assembly factor" },
  { "GUP2", "Probable membrane protein" },
  { "POS5", "Mitochondrial NADH kinase" },
  { "MF", "NULL" },
  { "UIP4", "Protein that interacts with Ulp1p" },
  { "YPL185W", "NULL" },
  { "MRN1", "RNA-binding protein that may be involved in translational regulation" },
  { "RTC6", "Protein involved in translation" },
  { "RTT10", "WD40 domain-containing protein involved in endosomal recycling" },
  { "YPL182C", "NULL" },
  { "CTI6", "Component of the Rpd3L histone deacetylase complex" },
  { "TCO89", "Subunit of TORC1 (Tor1p or Tor2p-Kog1p-Lst8p-Tco89p)" },
  { "PPQ1", "Protein phosphatase that regulates the mating response" },
  { "CBC2", "Small subunit of the heterodimeric cap binding complex with Sto1p" },
  { "CUP9", "Homeodomain-containing transcriptional repressor" },
  { "TRE1", "Transferrin receptor-like protein" },
  { "SPT14", "UDP-glycosyltransferase subunit of the GPI-GnT complex" },
  { "NIP100", "Large subunit of the dynactin complex" },
  { "MRPL40", "Mitochondrial ribosomal protein of the large subunit" },
  { "COX10", "Heme A:farnesyltransferase" },
  { "OYE3", "Conserved NADPH oxidoreductase containing flavin mononucleotide (FMN)" },
  { "DAP1", "Heme-binding protein" },
  { "MEX67", "Poly(A)RNA binding protein involved in nuclear mRNA export" },
  { "YPL168W", "NULL" },
  { "REV3", "Catalytic subunit of DNA polymerase zeta" },
  { "ATG29", "Autophagy-specific protein" },
  { "SET6", "SET domain protein of unknown function" },
  { "MLH3", "Protein involved in DNA mismatch repair and meiotic recombination" },
  { "SVS1", "Cell wall and vacuolar protein" },
  { "YPL162C", "Putative protein of unknown function" },
  { "BEM4", "Protein involved in establishment of cell polarity and bud emergence" },
  { "CDC60", "Cytosolic leucyl tRNA synthetase" },
  { "PET20", "Mitochondrial protein" },
  { "AIM44", "Protein that regulates Cdc42p and Rho1p" },
  { "TGS1", "Trimethyl guanosine synthase, conserved nucleolar methyl transferase" },
  { "PRM4", "Pheromone-regulated protein proposed to be involved in mating" },
  { "KIP2", "Kinesin-related motor protein involved in mitotic spindle positioning" },
  { "PEP4", "Vacuolar aspartyl protease (proteinase A)" },
  { "RAD53", "DNA damage response kinase" },
  { "YPL152W-A", "Protein of unknown function" },
  { "RRD2", "Peptidyl-prolyl cis/trans-isomerase" },
  { "PRP46", "Member of the NineTeen Complex (NTC)" },
  { "YPL150W", "Protein kinase of unknown cellular role" },
  { "ATG5", "Conserved protein involved in autophagy and the Cvt pathway" },
  { "PPT2", "Phosphopantetheine:protein transferase (PPTase)" },
  { "PXA1", "Subunit of heterodimeric peroxisomal ABC transport complex, with Pxa2p" },
  { "NOP53", "Nucleolar protein" },
  { "KES1", "One of seven members of the yeast oxysterol binding protein family" },
  { "POC4", "Component of a heterodimeric Poc4p-Irc25p chaperone" },
  { "RPL33A", "Ribosomal 60S subunit protein L33A" },
  { "YPL142C", "NULL" },
  { "FRK1", "Protein kinase of unknown cellular role" },
  { "MKK2", "MAPKK involved in the protein kinase C signaling pathway" },
  { "UME1", "Component of both the Rpd3S and Rpd3L histone deacetylase complexes" },
  { "SPP1", "Subunit of COMPASS (Set1C)" },
  { "GIP3", "Cytoplasmic protein that regulates protein phosphatase 1 Glc7p" },
  { "YPL136W", "NULL" },
  { "ISU1", "Conserved protein of the mitochondrial matrix" },
  { "YPL135C-A", "NULL" },
  { "ODC1", "Mitochondrial inner membrane transporter" },
  { "RDS2", "Transcription factor involved in regulating gluconeogenesis" },
  { "COX11", "Protein required for delivery of copper to Cox1p" },
  { "RPL5", "Ribosomal 60S subunit protein L5" },
  { "SPO19", "Meiosis-specific prospore protein" },
  { "TAF14", "Subunit of TFIID, TFIIF, INO80, SWI/SNF, and NuA3 complexes" },
  { "TBF1", "Telobox-containing general regulatory factor" },
  { "HHO1", "Histone H1, linker histone with roles in meiosis and sporulation" },
  { "NAN1", "U3 snoRNP protein" },
  { "KAP120", "Karyopherin responsible for the nuclear import of Rpf1p" },
  { "SPC29", "Inner plaque spindle pole body (SPB) component" },
  { "RNY1", "Vacuolar RNase of the T(2) family" },
  { "TFB2", "Subunit of TFIIH and nucleotide excision repair factor 3 complexes" },
  { "MEI5", "Meiosis-specific protein involved in meiotic recombination" },
  { "VPS30", "Subunit of phosphatidylinositol (PtdIns) 3-kinase complexes I and II" },
  { "YPL119C-A", "Putative protein of unknown function" },
  { "DBP1", "Putative ATP-dependent RNA helicase of the DEAD-box protein family" },
  { "MRP51", "Mitochondrial ribosomal protein of the small subunit" },
  { "IDI1", "Isopentenyl diphosphate:dimethylallyl diphosphate isomerase" },
  { "HOS3", "Trichostatin A-insensitive homodimeric histone deacetylase (HDAC)" },
  { "BEM3", "Rho GTPase activating protein (RhoGAP)" },
  { "YPL114W", "NULL" },
  { "YPL113C", "Glyoxylate reductase" },
  { "PEX25", "Peripheral peroxisomal membrane peroxin" },
  { "CAR1", "Arginase, catabolizes arginine to ornithine and urea" },
  { "GDE1", "Glycerophosphocholine (GroPCho) phosphodiesterase" },
  { "YPL109C", "NULL" },
  { "YPL108W", "Cytoplasmic protein of unknown function" },
  { "YPL107W", "NULL" },
  { "SSE1", "ATPase component of heat shock protein Hsp90 chaperone complex" },
  { "SYH1", "Protein of unknown function that influences nuclear pore distribution" },
  { "MSD1", "Mitochondrial aspartyl-tRNA synthetase" },
  { "FMP30", "Protein with a role in maintaining mitochondrial morphology" },
  { "YPL102C", "NULL" },
  { "ELP4", "Subunit of hexameric RecA-like ATPase Elp456 Elongator subcomplex" },
  { "ATG21", "Phosphoinositide binding protein" },
  { "INA17", "F1F0 ATPase synthase peripheral stalk assembly factor" },
  { "MGR2", "Subunit of the TIM23 translocase complex" },
  { "MSY1", "Mitochondrial tyrosyl-tRNA synthetase" },
  { "ERI1", "Endoplasmic reticulum membrane protein that binds and inhibits Ras2p" },
  { "PNG1", "Conserved peptide N-glycanase" },
  { "EEB1", "Acyl-coenzymeA:ethanol O-acyltransferase" },
  { "SEC62", "Essential subunit of Sec63 complex" },
  { "NOG1", "Putative GTPase" },
  { "SSU1", "Plasma membrane sulfite pump involved in sulfite metabolism" },
  { "GLR1", "Cytosolic and mitochondrial glutathione oxidoreductase" },
  { "RPS6A", "Protein component of the small (40S) ribosomal subunit" },
  { "RLM1", "MADS-box transcription factor" },
  { "YPL088W", "Putative aryl alcohol dehydrogenase" },
  { "YDC1", "Alkaline dihydroceramidase, involved in sphingolipid metabolism" },
  { "ELP3", "Subunit of Elongator complex" },
  { "SEC16", "COPII vesicle coat protein required for ER transport vesicle budding" },
  { "BRO1", "Cytoplasmic class E vacuolar protein sorting (VPS) factor" },
  { "SEN54", "Subunit of the tRNA splicing endonuclease" },
  { "MOT1", "Essential protein involved in regulation of transcription" },
  { "RPS9A", "Protein component of the small (40S) ribosomal subunit" },
  { "YPL080C", "Putative protein of unknown function" },
  { "RPL21B", "Ribosomal 60S subunit protein L21B" },
  { "ATP4", "Subunit b of the stator stalk of mitochondrial F1F0 ATP synthase" },
  { "YPL077C", "Putative protein of unknown function" },
  { "GPI2", "Protein involved in the synthesis of GlcNAc-PI" },
  { "GCR1", "Transcriptional activator of genes involved in glycolysis" },
  { "YTA6", "Putative ATPase of the CDC48/PAS1/SEC18 (AAA) family" },
  { "YPL073C", "NULL" },
  { "UBP16", "Deubiquitinating enzyme anchored to the outer mitochondrial membrane" },
  { "YPL071C", "Putative protein of unknown function" },
  { "MUK1", "Guanine nucleotide exchange factor (GEF)" },
  { "BTS1", "Geranylgeranyl diphosphate synthase (GGPPS)" },
  { "YPL068C", "Protein of unknown function" },
  { "YPL067C", "NULL" },
  { "RGL1", "Regulator of Rho1p signaling, cofactor of Tus1p" },
  { "VPS28", "Component of the ESCRT-I complex" },
  { "CWC27", "Component of a complex containing Cef1p" },
  { "TIM50", "Essential component of the TIM23 complex" },
  { "YPL062W", "Putative protein of unknown function" },
  { "ALD6", "Cytosolic aldehyde dehydrogenase" },
  { "MFM1", "Mitochondrial inner membrane magnesium transporter" },
  { "GRX5", "Glutathione-dependent oxidoreductase" },
  { "PDR12", "Plasma membrane ATP-binding cassette (ABC) transporter" },
  { "SUR1", "Mannosylinositol phosphorylceramide (MIPC) synthase catalytic subunit" },
  { "LCL1", "Putative protein of unknown function" },
  { "LGE1", "Protein of unknown function" },
  { "LEE1", "Zinc-finger protein of unknown function" },
  { "KTR6", "Probable mannosylphosphate transferase" },
  { "OAZ1", "Regulator of ornithine decarboxylase Spe1p" },
  { "ARL3", "ARF-like small GTPase of the RAS superfamily" },
  { "MNN9", "Subunit of Golgi mannosyltransferase complex" },
  { "DIG1", "MAP kinase-responsive inhibitor of the Ste12p transcription factor" },
  { "CAM1", "One of two isoforms of the gamma subunit of eEF1B" },
  { "SGF11", "Integral subunit of SAGA histone acetyltransferase complex" },
  { "ELC1", "Elongin C, conserved among eukaryotes" },
  { "VPS16", "Subunit of the HOPS and the CORVET complexes" },
  { "YPL044C", "NULL" },
  { "NOP4", "Nucleolar protein" },
  { "SSN3", "Cyclin-dependent protein kinase" },
  { "YPL041C", "NULL" },
  { "ISM1", "Mitochondrial isoleucyl-tRNA synthetase" },
  { "YPL039W", "Putative protein of unknown function" },
  { "YPL038W-A", "Putative protein of unknown function" },
  { "MET31", "Zinc-finger DNA-binding transcription factor" },
  { "EGD1", "Subunit beta1 of the nascent polypeptide-associated complex (NAC)" },
  { "PMA2", "Plasma membrane H+-ATPase" },
  { "YPL035C", "NULL" },
  { "YPL034W", "Putative protein of unknown function" },
  { "SRL4", "Protein of unknown function" },
  { "SVL3", "Protein of unknown function" },
  { "PHO85", "Cyclin-dependent kinase" },
  { "TRM44", "tRNA(Ser) Um(44) 2'-O-methyltransferase" },
  { "SUV3", "ATP-dependent RNA helicase" },
  { "ERG10", "Acetyl-CoA C-acetyltransferase (acetoacetyl-CoA thiolase)" },
  { "SMA1", "Protein of unknown function involved in prospore membrane assembly" },
  { "SKS1", "Putative serine/threonine protein kinase" },
  { "YPL025C", "NULL" },
  { "RMI1", "Subunit of the RecQ (Sgs1p) - Topo III (Top3p) complex" },
  { "MET12", "Protein with MTHFR activity in vitro" },
  { "RAD1", "Single-stranded DNA endonuclease (with Rad10p)" },
  { "ECM23", "Non-essential protein of unconfirmed function" },
  { "ULP1", "Protease that specifically cleaves Smt3p protein conjugates" },
  { "VTC3", "Regulatory subunit of the vacuolar transporter chaperone (VTC) complex" },
  { "CTF19", "Outer kinetochore protein, needed for accurate chromosome segregation" },
  { "IRC15", "Microtubule associated protein" },
  { "SWI1", "Subunit of the SWI/SNF chromatin remodeling complex" },
  { "HST2", "Cytoplasmic NAD(+)-dependent protein deacetylase" },
  { "YPL014W", "NULL" },
  { "MRPS16", "Mitochondrial ribosomal protein of the small subunit" },
  { "RRP12", "Protein required for export of the ribosomal subunits" },
  { "TAF3", "TFIID subunit (47 kDa)" },
  { "RET3", "Zeta subunit of the coatomer complex (COPI)" },
  { "TAE2", "NULL" },
  { "CHL1", "Probable DNA helicase" },
  { "TFC8", "Subunit of RNA polymerase III transcription initiation factor complex" },
  { "NCR1", "Vacuolar membrane protein" },
  { "AEP3", "Peripheral mitochondrial inner membrane protein" },
  { "LSP1", "Eisosome core component" },
  { "ULA1", "Protein that activates Rub1p (NEDD8) before neddylation" },
  { "SNF8", "Component of the ESCRT-II complex" },
  { "HAT1", "Catalytic subunit of the Hat1p-Hat2p histone acetyltransferase complex" },
  { "CIT3", "Dual specificity mitochondrial citrate and methylcitrate synthase" },
  { "PDH1", "Putative 2-methylcitrate dehydratase" },
  { "YPR002C-A", "NULL" },
  { "YPR003C", "Putative sulfate permease" },
  { "AIM45", "Putative ortholog of mammalian ETF-alpha" },
  { "HAL1", "Cytoplasmic protein involved in halotolerance" },
  { "ICL2", "2-methylisocitrate lyase of the mitochondrial matrix" },
  { "REC8", "Meiosis-specific component of the sister chromatid cohesion complex" },
  { "HAA1", "Transcriptional activator involved in adaptation to weak acid stress" },
  { "SUT2", "Zn2Cys6 family transcription factor" },
  { "RPA135", "RNA polymerase I second largest subunit A135" },
  { "YPR010C-A", "NULL" },
  { "YPR011C", "Mitochondrial transporter" },
  { "YPR012W", "NULL" },
  { "CMR3", "Putative zinc finger protein" },
  { "YPR014C", "Putative protein of unknown function" },
  { "YPR015C", "Putative zinc finger transcription factor" },
  { "TIF6", "Constituent of 66S pre-ribosomal particles" },
  { "YPR016W-A", "NULL" },
  { "DSS4", "Guanine nucleotide dissociation stimulator for Sec4p" },
  { "RLF2", "Largest subunit (p90) of the Chromatin Assembly Complex (CAF-1)" },
  { "MCM4", "Essential helicase component of heterohexameric MCM2-7 complexes" },
  { "ATP20", "Subunit g of the mitochondrial F1F0 ATP synthase" },
  { "AGC1", "Mitochondrial amino acid transporter" },
  { "YPR022C", "NULL" },
  { "EAF3", "Component of the Rpd3S histone deacetylase complex" },
  { "YME1", "Catalytic subunit of i-AAA protease complex" },
  { "CCL1", "Cyclin associated with protein kinase Kin28p" },
  { "ATH1", "Acid trehalase required for utilization of extracellular trehalose" },
  { "YPR027C", "Putative protein of unknown function" },
  { "YOP1", "Reticulon-interacting protein" },
  { "APL4", "Gamma-adaptin" },
  { "CSR2", "Nuclear ubiquitin protein ligase binding protein" },
  { "NTO1", "Subunit of the NuA3 histone acetyltransferase complex" },
  { "SRO7", "Effector of Rab GTPase Sec4p" },
  { "HTS1", "Cytoplasmic and mitochondrial histidine tRNA synthetase" },
  { "ARP7", "Component of both the SWI/SNF and RSC chromatin remodeling complexes" },
  { "GLN1", "Glutamine synthetase (GS)" },
  { "VMA13", "Subunit H of the V1 peripheral membrane domain of V-ATPase" },
  { "YPR036W-A", "NULL" },
  { "ERV2", "Flavin-linked sulfhydryl oxidase localized to the ER lumen" },
  { "IRC16", "NULL" },
  { "YPR039W", "NULL" },
  { "TIP41", "Protein that interacts with Tap42p, which regulates PP2A" },
  { "TIF5", "Translation initiation factor eIF5" },
  { "PUF2", "PUF family mRNA-binding protein" },
  { "RPL43A", "Ribosomal 60S subunit protein L43A" },
  { "OPI11", "NULL" },
  { "THP3", "Protein that may have a role in transcription elongation" },
  { "MCM16", "Component of the Ctf19 complex and the COMA subcomplex" },
  { "MSF1", "Mitochondrial phenylalanyl-tRNA synthetase" },
  { "TAH18", "Conserved NAPDH-dependent diflavin reductase" },
  { "ATG11", "Adapter protein for pexophagy and the Cvt targeting pathway" },
  { "YPR050C", "NULL" },
  { "MAK3", "Catalytic subunit of the NatC type N-terminal acetyltransferase (NAT)" },
  { "NHP6A", "High-mobility group (HMG) protein" },
  { "YPR053C", "Putative protein of unknown function" },
  { "SMK1", "Middle sporulation-specific mitogen-activated protein kinase (MAPK)" },
  { "SEC8", "Essential 121 kDa subunit of the exocyst complex" },
  { "TFB4", "Subunit of TFIIH complex" },
  { "BRR1", "snRNP protein component of spliceosomal snRNPs" },
  { "YMC1", "Secondary mitochondrial inner membrane glycine transporter" },
  { "YPR059C", "NULL" },
  { "ARO7", "Chorismate mutase" },
  { "JID1", "Probable Hsp40p co-chaperone" },
  { "FCY1", "Cytosine deaminase" },
  { "YPR063C", "ER-localized protein of unknown function" },
  { "YPR064W", "Putative protein of unknown function" },
  { "ROX1", "Heme-dependent repressor of hypoxic genes" },
  { "UBA3", "Protein that activates Rub1p (NEDD8) before neddylation" },
  { "ISA2", "Protein required for maturation of mitochondrial [4Fe-4S] proteins" },
  { "HOS1", "Class I histone deacetylase (HDAC) family member" },
  { "SPE3", "Spermidine synthase" },
  { "MED1", "Subunit of the RNA polymerase II mediator complex" },
  { "YPR071W", "Putative membrane protein" },
  { "NOT5", "Component of the CCR4-NOT core complex, involved in mRNA decapping" },
  { "LTP1", "Protein phosphotyrosine phosphatase of unknown cellular role" },
  { "TKL1", "Transketolase" },
  { "YPR074W-A", "NULL" },
  { "OPY2", "Integral membrane protein that acts as a membrane anchor for Ste50p" },
  { "YPR076W", "NULL" },
  { "YPR077C", "NULL" },
  { "YPR078C", "Putative protein of unknown function" },
  { "MRL1", "Membrane protein" },
  { "TEF1", "Translational elongation factor EF-1 alpha" },
  { "GRS2", "Glycine-tRNA synthetase, not expressed under normal growth conditions" },
  { "DIB1", "17-kDa component of the U4/U6aU5 tri-snRNP" },
  { "MDM36", "Mitochondrial protein" },
  { "YPR084W", "Putative protein of unknown function" },
  { "ASA1", "Subunit of the ASTRA complex, involved in chromatin remodeling" },
  { "SUA7", "Transcription factor TFIIB" },
  { "VPS69", "NULL" },
  { "SRP54", "Signal recognition particle (SRP) subunit (homolog of mammalian SRP54)" },
  { "YPR089W", "Protein of unknown function" },
  { "NVJ2", "Lipid-binding ER protein, enriched at nucleus-vacuolar junctions (NVJ)" },
  { "YPR092W", "NULL" },
  { "ASR1", "Ubiquitin ligase that modifies and regulates RNA Pol II" },
  { "RDS3", "Component of the SF3b subcomplex of the U2 snRNP" },
  { "SYT1", "Guanine nucleotide exchange factor (GEF) for Arf proteins" },
  { "YPR096C", "Protein of unknown function" },
  { "YPR097W", "Protein that contains a PX domain and binds phosphoinositides" },
  { "YPR098C", "NULL" },
  { "YPR099C", "NULL" },
  { "MRPL51", "Mitochondrial ribosomal protein of the large subunit" },
  { "SNT309", "Member of the NineTeen Complex (NTC)" },
  { "RPL11A", "Ribosomal 60S subunit protein L11A" },
  { "PRE2", "Beta 5 subunit of the 20S proteasome" },
  { "FHL1", "Regulator of ribosomal protein (RP) transcription" },
  { "COG4", "Essential component of the conserved oligomeric Golgi complex" },
  { "ISR1", "Predicted protein kinase" },
  { "YTH1", "Essential RNA-binding component of cleavage and polyadenylation factor" },
  { "RPN7", "Essential non-ATPase regulatory subunit of the 26S proteasome" },
  { "YPR108W-A", "Putative protein of unknown function" },
  { "YPR109W", "NULL" },
  { "RPC40", "RNA polymerase subunit AC40" },
  { "DBF20", "Ser/Thr kinase involved in late nuclear division" },
  { "MRD1", "Essential conserved small ribosomal subunit (40s) synthesis factor" },
  { "PIS1", "Phosphatidylinositol synthase" },
  { "YPR114W", "Putative protein of unknown function" },
  { "RGC1", "Putative regulator of the Fps1p glycerol channel" },
  { "RRG8", "Protein of unknown function" },
  { "YPR117W", "Putative protein of unknown function" },
  { "MRI1", "5'-methylthioribose-1-phosphate isomerase" },
  { "CLB2", "B-type cyclin involved in cell cycle progression" },
  { "CLB5", "B-type cyclin involved in DNA replication during S phase" },
  { "THI22", "Protein with similarity to hydroxymethylpyrimidine phosphate kinases" },
  { "AXL1", "Haploid specific endoprotease of a-factor mating pheromone" },
  { "YPR123C", "NULL" },
  { "CTR1", "High-affinity copper transporter of plasma membrane" },
  { "YLH47", "Mitochondrial inner membrane protein" },
  { "YPR126C", "NULL" },
  { "YPR127W", "Putative pyridoxine 4-dehydrogenase" },
  { "ANT1", "Peroxisomal adenine nucleotide transporter" },
  { "SCD6", "Repressor of translation initiation" },
  { "YPR130C", "NULL" },
  { "NAT3", "Catalytic subunit of the NatB N-terminal acetyltransferase" },
  { "RPS23B", "Ribosomal protein 28 (rp28) of the small (40S) ribosomal subunit" },
  { "SPN1", "Protein involved in RNA polymerase II transcription" },
  { "TOM5", "Component of the TOM (translocase of outer membrane) complex" },
  { "MSS18", "Nuclear encoded protein needed for splicing of mitochondrial intron" },
  { "CTF4", "Chromatin-associated protein" },
  { "YPR136C", "NULL" },
  { "RRP9", "Protein involved in pre-rRNA processing" },
  { "MEP3", "Ammonium permease of high capacity and low affinity" },
  { "LOA1", "Lysophosphatidic acid acyltransferase" },
  { "TAZ1", "Lyso-phosphatidylcholine acyltransferase" },
  { "KAR3", "Minus-end-directed microtubule motor" },
  { "YPR142C", "NULL" },
  { "RRP15", "Nucleolar protein" },
  { "NOC4", "Nucleolar protein" },
  { "ASN1", "Asparagine synthetase" },
  { "YPR145C-A", "Putative protein of unknown function" },
  { "YPR146C", "NULL" },
  { "YPR147C", "Bifunctional triacylglycerol lipase and short chain ester hydrolase" },
  { "YPR148C", "Protein of unknown function" },
  { "NCE102", "Protein of unknown function" },
  { "YPR150W", "NULL" },
  { "SUE1", "Protein required for degradation of unstable forms of cytochrome c" },
  { "URN1", "Protein of unknown function containing WW and FF domains" },
  { "YPR153W", "NULL" },
  { "PIN3", "Negative regulator of actin nucleation-promoting factor activity" },
  { "NCA2", "Protein that regulates expression of Fo-F1 ATP synthase subunits" },
  { "TPO3", "Polyamine transporter of the major facilitator superfamily" },
  { "TDA6", "Putative protein of unknown function" },
  { "CUR1", "Sorting factor, central regulator of spatial protein quality control" },
  { "KRE6", "Type II integral membrane protein" },
  { "YPR159C-A", "Protein of unknown function" },
  { "GPH1", "Glycogen phosphorylase required for the mobilization of glycogen" },
  { "YPR160W-A", "NULL" },
  { "YPR160C-A", "NULL" },
  { "SGV1", "Cyclin (Bur2p)-dependent protein kinase" },
  { "ORC4", "Subunit of the origin recognition complex (ORC)" },
  { "TIF3", "Translation initiation factor eIF-4B" },
  { "MMS1", "Subunit of E3 ubiquitin ligase complex involved in replication repair" },
  { "RHO1", "GTP-binding protein of the rho subfamily of Ras-like proteins" },
  { "MRP2", "Mitochondrial ribosomal protein of the small subunit" },
  { "MET16", "3'-phosphoadenylsulfate reductase" },
  { "NUT2", "Subunit of the RNA polymerase II mediator complex" },
  { "JIP5", "Protein required for biogenesis of the large ribosomal subunit" },
  { "YPR170C", "NULL" },
  { "YPR169W-A", "NULL" },
  { "YPR170W-B", "Putative protein of unknown function" },
  { "YPR170W-A", "NULL" },
  { "BSP1", "Adapter that links synaptojanins to the cortical actin cytoskeleton" },
  { "YPR172W", "Protein of unknown function" },
  { "VPS4", "AAA-ATPase involved in multivesicular body (MVB) protein sorting" },
  { "YPR174C", "Protein of unknown function" },
  { "DPB2", "Second largest subunit of DNA polymerase II (DNA polymerase epsilon)" },
  { "BET2", "Beta subunit of Type II geranylgeranyltransferase" },
  { "YPR177C", "NULL" },
  { "PRP4", "Splicing factor" },
  { "HDA3", "Subunit of the HDA1 histone deacetylase complex" },
  { "AOS1", "Subunit of heterodimeric nuclear SUMO activating enzyme E1 with Uba2p" },
  { "SEC23", "GTPase-activating protein, stimulates the GTPase activity of Sar1p" },
  { "SMX3", "Core Sm protein Sm F" },
  { "DPM1", "Dolichol phosphate mannose (Dol-P-Man) synthase of ER membrane" },
  { "GDB1", "Glycogen debranching enzyme" },
  { "ATG13", "Regulatory subunit of the Atg1p signaling complex" },
  { "PZF1", "Transcription factor IIIA (TFIIIA)" },
  { "RPO26", "RNA polymerase subunit ABC23" },
  { "MLC2", "Regulatory light chain for the type II myosin Myo1p" },
  { "SKI3", "Ski complex component and TPR protein" },
  { "RPC82", "RNA polymerase III subunit C82" },
  { "QCR2", "Subunit 2 of ubiquinol cytochrome-c reductase (Complex III)" },
  { "AQY1", "Spore-specific water channel" },
  { "HPA2", "Tetrameric histone acetyltransferase" },
  { "OPT2", "Oligopeptide transporter" },
  { "YPR195C", "Putative protein of unknown function" },
  { "YPR196W", "Putative maltose-responsive transcription factor" },
  { "YPR197C", "NULL" },
  { "SGE1", "Plasma membrane multidrug transporter" },
  { "ARR1", "Transcriptional activator of the basic leucine zipper (bZIP) family" },
  { "ARR2", "Arsenate reductase required for arsenate resistance" },
  { "ARR3", "Plasma membrane metalloid/H+ antiporter" },
  { "YPR202W", "Putative protein of unknown function" },
  { "YPR203W", "Putative protein of unknown function" },
  { "YPR204W", "DNA helicase encoded within the telomeric Y' element" },
  { "YPR204C-A", "NULL" },
  { "Q0010", "NULL" },
  { "Q0017", "NULL" },
  { "Q0032", "NULL" },
  { "AI1", "Reverse transcriptase required for splicing of the COX1 pre-mRNA" },
  { "AI2", "Reverse transcriptase required for splicing of the COX1 pre-mRNA" },
  { "AI3", "Endonuclease I-SceIII" },
  { "AI4", "Endonuclease I-SceII" },
  { "AI5_ALPHA", "Endonuclease I-SceIV" },
  { "COX1", "Subunit I of cytochrome c oxidase (Complex IV)" },
  { "AI5_BETA", "Protein of unknown function" },
  { "ATP8", "Subunit 8 of the F0 sector of mitochondrial F1F0 ATP synthase" },
  { "ATP6", "Subunit a of the F0 sector of mitochondrial F1F0 ATP synthase" },
  { "Q0092", "NULL" },
  { "BI2", "Mitochondrial mRNA maturase with a role in splicing" },
  { "BI3", "Mitochondrial mRNA maturase" },
  { "BI4", "Mitochondrial mRNA maturase" },
  { "COB", "Cytochrome b" },
  { "OLI1", "F0-ATP synthase subunit c (ATPase-associated proteolipid)" },
  { "VAR1", "Mitochondrial ribosomal protein of the small subunit" },
  { "Q0142", "NULL" },
  { "Q0143", "NULL" },
  { "Q0144", "NULL" },
  { "SCEI", "I-SceI DNA endonuclease" },
  { "Q0182", "NULL" },
  { "COX2", "Subunit II of cytochrome c oxidase (Complex IV)" },
  { "Q0255", "Maturase-like protein" },
  { "COX3", "Subunit III of cytochrome c oxidase (Complex IV)" },
  { "Q0297", "NULL" }
};
  // LOG(INFO)<<leri::gene_func_map["tS(AGA)D3"];

} // namespace leri

#endif // LERI_YEAST_GENES_H_
