/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_AMINOACID_H_
#define LERI_AMINOACID_H_

#include "leri/protein/atom.h"
#include "leri/protein/bond.h"

namespace leri {

template <typename Dtype>
class AminoAcid {
public:
  explicit AminoAcid() {}
  explicit AminoAcid(string name)
  :name_(name) {}
  virtual ~AminoAcid() {}
  // AminoAcid(string _3_let, string _1_let, string full, bool _h);

  void set_amino_acid_3_letter(char* str) { this->amino_acid_3_letter_.assign(str); }
  void set_amino_acid_1_letter(char* str) { this->amino_acid_1_letter_.assign(str); }
  void set_amino_acid_full_nam(char* str) { this->amino_acid_full_nam_.assign(str); }
  void set_index(int idx) { idx_ = idx; }
  void set_chain_name(string str) { chain_name_ = str; }
  // void set_phi(Dtype phi) { phi_ = phi; }
  // void set_psi(Dtype psi) { psi_ = psi; }
  // void set_new_phsi(Dtype phi, Dtype psi) { _phsi_new[0] = phi; _phsi_new[1] = psi; }
  // void set_old_phsi(Dtype phi, Dtype psi) { _phsi_old[0] = phi; _phsi_old[1] = psi; }
  // void set_hydrophobicity(bool h) { hydrophobicity_ = h; }
  void set_num_of_atoms(int n) { num_of_atoms_ = n; }
  int get_num_of_atoms() { 
    num_of_atoms_ = atoms_.size();
    return num_of_atoms_;
  }
  int get_index() { return idx_; }
  int get_atom_index(string at) {
    for (int i = 0; i < num_of_atoms_; i++) {
      if (!atoms_[i].name_.compare(at)) {
        return i;
      }
    }
  }
  Dtype get_atom_coord(string at, int idx) {
    for (int i = 0; i < num_of_atoms_; i++) {
      if (!atoms_[i].name_.compare(at)) {
        return atoms_[i].coord_[idx];
      }
    }
  }

  void add_atom(Atom<Dtype> atom);

public:
  int  idx_;
  int  amino_acid_type_code_;
  string name_;
  string amino_acid_3_letter_;
  string amino_acid_1_letter_;
  string amino_acid_full_nam_;
  string chain_name_;

  // Side-chain
  int num_chi_;
  vector<string> atom_name_sidechain_;
  vector<vector<int> > atom_idx_sidechain_;
  vector<vector<Dtype> > internal_coord_;



  // Amino acid: Clear
  void clear() { 
    atoms_.clear();
    atom_name_sidechain_.clear();
    atom_idx_sidechain_.clear();
    internal_coord_.clear();
  }
  vector<Atom<Dtype> > atoms_;

  // Dtype _phi, _psi;   // Torsion angle
  // Dtype _phsi_old[2];
  // Dtype _phsi_new[2];
  // int NumofBond;
  // Dtype _bonds[4];
  // Dtype _bond_angles[5];
  // Dtype _hbond;
  // bool  _hydrophobicity;
  // char  _SS[2];     // secondary structure
//  char homology[MAX_HOMOLOGY];  // used for HSSP homologous family

//  map<string, Atom *> mapAtom; // map an atom name to its Atom object


  // TODO: from struct.h
  /*
   * Residue type. This should represent amino-acids in the case of protein chains, nucleic acids
   * for rna, etc.
   */
      
  char type;

  /**
   * First and last atom of this residue.
   */
  int atom_first_;
  int atom_last_;

  /**
   * Chain containing this residue.
   */   
  int chain; 

  /**
   * Numer of bonds that start at atoms within this residue.
   */
  int nbonds_;
  /**
   * List of bonds that start at atoms within this residue.
   */
  // struct BondData *bonds;
  vector<Bond<Dtype> > bonds;  // TODO: delete
  Bond<Dtype>* bonds_;

  /**
   * Arrays to hold custom integer or float properties for each residue.
   * This arrays are always NULL when the residue record is initialized. 
   */
  int niprop;
  int *iproperties;
  int nfprop;
  double *fproperties;
private:
  bool active_; // status of this amino acid
  int num_of_atoms_;
protected:
  // DISABLE_COPY_AND_ASSIGN(AminoAcid);
};  // class AminoAcid




}  // namespace leri

#endif  // LERI_AMINOACID_H_
