/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_BOND_H_
#define LERI_BOND_H_

#include "leri/common.h"

namespace leri {

template <typename Dtype>
class Bond {
public:
  explicit Bond() {}
  /**
   * Bond type.
   */
  char type; 

  /**
   * Indexes of the atoms defining the bond. It should be atom0 < atom1 < atom2 < atom3,
   * but not necessarily atom1 = atom0 + 1, etc.
   */
  int atom0; 
  int atom1; 
  int atom2;
  int atom3;

  /**
   * Index of the last atom in the chain that is affected by a torsional rotation at this bond.
   * The existence of such an index requires the atom ordering described at beginning of
   * this file, this is, i, j00 = i + 1, ..., j01, j10 = j01 + 1, ..., j11... for branches defined at i.
   */
  // int lastatom;
  int atom_last_;

  // *
  // * Current value for the torsional angle of this bond.
  Dtype angle;
  /**
   * Used to store a new value for the torsional angle.
   */
  Dtype newangle;

public:
  void set_angle(int a) { angle = a;};
  void set_angle_new(int a) { newangle = a;};


protected:
  // DISABLE_COPY_AND_ASSIGN(Bond);
};  // class Bond

}  // namespace leri

#endif  // LERI_BOND_H_
