/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_COMPUTATIONAL_BIOLOGY_UTIL_H_
#define LERI_COMPUTATIONAL_BIOLOGY_UTIL_H_

#include "leri/common.h"
#include "leri/util/malloc.h"
#include "leri/util/fileutil.h"
#include "leri/gio/figure.h"
#include "leri/gio/javas.h"
#include "leri/protein/bioutil.h"

namespace leri {

template <typename Dtype>
class CButil {
public:
  CButil(string output_dirt = "", string job_name = "", string dirt = "")
  :output_dirt_(output_dirt),
   job_name_(job_name),
   dirt_(dirt) {
    base_name_ = output_dirt_ + job_name_;
  }
  ~CButil() {}
  /*  Lines with ASG in the output of stride
  Format:  6-8  Residue name
  10-10 Protein chain identifier
  12-15 PDB	residue	number
  17-20 Ordinal residue number
  25-25 One	letter secondary structure code	**)
  43-49 Phi	angle
  53-59 Psi	angle
  65-69 Residue solvent accessible area
  */
  void generate_rama_map(string fname, string sname, bool redo = true);
  void generate_all_rama_map(string findex, bool redo = false);
public:
  void set_deg_of_bin(int v) { bin_deg_ = v; bin_size_ = 360/bin_deg_; }
private:
  void read_stride(string fname);
  void str2mtx(vector<string> str);
  void clear();
public:
  string output_dirt_;    // output directory
  string job_name_;       // job name
  string base_name_;
  string dirt_;

  string res_; // Amino acid sequence
  string sec_; // Secondary structure
  string chn_; // Chain
  int num_aa_;
  vector<int> idx_aa_;
  vector<int> idx_aa_ord_;
  vector<Dtype> phi_;
  vector<Dtype> psi_;
  vector<Dtype> saa_;

  int bin_size_;
  Dtype bin_deg_;

  int** rama_map_freq_;
protected:
  DISABLE_COPY_AND_ASSIGN(CButil);
}; // class CButil

} // namespace leri
#endif // LERI_COMPUTATIONAL_BIOLOGY_UTIL_H_
