/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_CHAIN_H_
#define LERI_CHAIN_H_

#include "leri/util/matrix.h"
#include "leri/util/fileutil.h"
#include "leri/protein/aminoacid.h"
#include "leri/protein/rotamer.h"

namespace leri {

template <typename Dtype>
class Chain {
public:
  Chain(bool idx_aa_reorder = false)
  :idx_aa_reorder_(idx_aa_reorder) {}
  ~Chain() {}
  void clear();


  void set_dirt_job(string dirt, string name) { output_dirt_ = dirt; job_name_ = name; }
  void set_name(string name) { this->name_ = name; }
  void set_num_of_atoms(int n) { num_of_atoms_ = n; }
  void set_num_of_amino_acids(int n) { num_of_amino_acids_ = n; }
  void set_active(const bool active) { active_ = active; }

  string get_name() { return name_; }
  int  get_num_of_atoms() { 
    num_of_atoms_ = 0;
    num_of_amino_acids_ = amino_acids_.size();
    for (int i = 0; i < num_of_amino_acids_; i++) {
      num_of_atoms_ += amino_acids_[i].get_num_of_atoms();
    }
    return num_of_atoms_; 
  }
  int get_num_of_amino_acids() {
    num_of_amino_acids_ = amino_acids_.size();
    return num_of_amino_acids_; 
  }
  string get_atom_name(int idx_atom, int idx_aa);
  void get_atom_in_residue(string atomtype, int* has_flag, int* idx_atom, int idx_aa);
  void add_amino_acid(AminoAcid<Dtype> aminoacid);

  void reduced_representation(); // only keep H, N, CA, C, O, CB
  void add_hn();
  void update_oxygen();
  // void add_bonds();

  void build_sidechain(string fname = "", string mod = "w", bool quiet = false);

  // TODO:
  void residue_contact(vector<vector<Dtype> >& avg_contact_matrix, 
                       string chain_label, Chain<Dtype> chain, 
                       Dtype cutoff, Dtype weight, 
                       int space_resi, string CACB, string out_path);

  void hydrogen_bond_contact(vector<vector<Dtype> >& avg_hb_matrix, 
                             char chain_label, Chain<Dtype> chain, Dtype weight, string out_path);
  void write_pdb(string fname, string mod = "w");




  // TODO: remove the same functions in the protein.h/cc
  void set_idx_coords(); // TODO: remove in the future
  void set_all_aotm_coords();
  void set_distmask(bool val = true);
  void leri_update_torsional_angle(int idx_bond, int idx_aa);
  void leri_update_all_torsional_angles_in_res(int idx_aa);
  void leri_update_all_torsional_angles();
  void calc_atom_dist(bool mask = true);
  void rotate_coords(int idx_bond, int idx_aa, Dtype alpha = 0.0);
  void leri_update_all_coords_in_res(int idx_aa, bool update_mask = true);
  void update_all_coords();

  void is_contact(Dtype* val, int idx_aa_0, int idx_aa_1, Dtype cutoff = 7.5, string type = "CB");
  void calc_aa_contact(Dtype** mat, Dtype cutoff = 7.5, string type = "CB");
  void is_NHbonded(Dtype* val, int idx_aa_0, int idx_aa_1);
  void calc_hydrogen_bond(Dtype** mat);

  // TODO: remove the same functions in the protein.h/cc

public:
  string output_dirt_;
  string job_name_;
  int idx_;
  string name_;
  vector<AminoAcid<Dtype> > amino_acids_;
  string seq_;
  bool idx_aa_reorder_;
  
  int aa_first_;
  int aa_last_;
  // TODO: from struct.h
  // char type;

  int* idx_coords_;  // 2-column, indices for amino acid and chain
  Dtype*  coords_;
  Dtype*  dist_mtx_;
  Dtype*  dist_mask_mtx_;
  string* name_coords_;
private:
  bool active_; // status of this chain
  int num_of_chains_;
  int num_of_amino_acids_;
  int num_of_atoms_;
protected:
//    DISABLE_COPY_AND_ASSIGN(Chain);

}; // class Chain


}  // namespace leri

#endif  // LERI_CHAIN_H_
