/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_MOLECULAR_COMPLEX_H_
#define LERI_MOLECULAR_COMPLEX_H_

#include "leri/common.h"
#include "leri/util/malloc.h"
// #include "leri/util/matrix.h"
#include "leri/util/strutil.h"
#include "leri/util/fileutil.h"
// #include "leri/protein/atom.h"
// #include "leri/protein/aminoacid.h"
// #include "leri/protein/bioutil.h"
// #include "leri/protein/chain.h"
// #include "leri/gio/javas.h"
// #include "leri/gio/figure.h"
// #include "leri/gio/gnuplot.h"
// #include "leri/gio/html.h"
#include "leri/folding/energy.h"
// #include "leri/folding/modules.h"
#include "leri/protein/protein.h" // TODO: remove
#include "leri/folding/mdconfig.h"
#include "leri/folding/cgmd.h"
#include "leri/folding/simulation.h"



namespace leri {

template <typename Dtype>
class Complex {
public:
  explicit Complex(string output_dirt = "", string job_name = "")
  :output_dirt_(output_dirt),
   job_name_(job_name) {
    base_name_ = output_dirt_ + job_name_;
  }
  ~Complex() {}
  
  void init();
  void driver();
  void finish();

  //  General
  void set_file_name(const string cfg_file, const string pdb_file = "", const string seq_file = "");
  // Compute energy
  void calc_energy(); // calculate energy for a given structure

  // Folding simulation
  void set_param(int max_iter, Dtype intv, Dtype temperature = 0.99);
  void actuate_discrete_folding();
  void actuate_modyfing();

private:
  //  General
  void init_system();
  void set_system();

  // Folding simulation
  // void read_cgmd_cfg();

private:
  string output_dirt_;    // output directory
  string job_name_;       // job name
  string base_name_;      // basic name
  
  string cfg_file_;
  string seq_file_;
  string pdb_file_;

  LSequence<Dtype>* sequence_;
  Protein<Dtype>* protein_;
  int num_of_proteins_;
  int num_of_chains_;



  // Folding simulation
  int max_iter_;
  int intv_;
  Dtype temperature_;

protected:
  DISABLE_COPY_AND_ASSIGN(Complex);
}; // class MDConfig

} // namespace leri
#endif // LERI_MOLECULAR_COMPLEX_H_