/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_GRID_3D_H_
#define LERI_GRID_3D_H_

#include "leri/common.h"
#include "leri/util/malloc.h"
#include "leri/util/strutil.h"
#include "leri/util/fileutil.h"
// #include "leri/gio/figure.h"
// #include "leri/gio/javas.h"
// #include "leri/protein/bioutil.h"
#include "leri/protein/protein.h"

namespace leri {

template <typename Dtype>
class Grid3D {
public:
  Grid3D(string output_dirt = "", string job_name = "", string dirt = "")
  :output_dirt_(output_dirt),
   job_name_(job_name),
   dirt_(dirt) {
    base_name_ = output_dirt_ + job_name_;
  }
  ~Grid3D() {}
public:
  void init(Dtype elength = 2068.0, int size = 512);
  void actuate(string fname, string atom_type, bool backbone = true, bool rubiks = false);
  void finish(){};

public:
  string output_dirt_;    // output directory
  string job_name_;       // job name
  string base_name_;
  string dirt_;

private:
  Dtype* mtx_voxel_;     // 3D matrix in a vector, indexed by mtx_voxel_[z*YSIZE*XSIZE+y*XSIZE+x] for (x, y, z)
  Dtype elength_;
  int n_voxel_;

protected:
  DISABLE_COPY_AND_ASSIGN(Grid3D);
}; // class Grid3D

} // namespace leri
#endif // LERI_GRID_3D_H_
