/**
 * The C++ Package of LERI (Learning Engine to Recognize Life).
 * Copyright 2015-present (C) Leri Ltd. All Rights Reserved.
 * Contact: Dr Yaan J. Jang (yaan.jang 'AT' gmail.com)
 *
 */

#ifndef LERI_RIBOSOME_DATA_H_
#define LERI_RIBOSOME_DATA_H_

#include "leri/common.h"
// #include <vector>

namespace leri {

const vector<string> ribosome_ = 
{
"# residue descriptor file for use with ribosome.  Each residue is ",
"# described in a Z-matrix style.  The first line contains the",
"# four letter code for the amino-acid ( can be less than four",
"# letters long) and the number of the atoms in the residue.",
"# Following this are N lines, one for each atom in the residue",
"# and the atoms it is connected to with the appropriate ",
"# geometric variables. The file is free-format with arguments",
"# separated by spaces or tabs.",
"# THIS FILE IS SPECIFIC FOR USE WITH LINUS",
"#DUMMY",
"name dum numatm 5",
"n      1.329  116.200  180.000   -3   -2   -4 + none    N(i)    C(i-1)   CA(i-1)   N(i-1)",
"ca     1.458  121.700  180.000    1   -3   -2 + omega   CA(i)   N(i)     C(i-1)    CA(i-1)",
"c      1.525  111.200  -64.000    2    1   -3 + phi     C(i)    CA(i)    N(i)      C(i-1) ",
"o      1.231  120.800  137.000    3    2    1 + psi     O(i)    C(i)     CA(i)     N(i)",
"cb     1.521  110.400 -122.000    2    1    3 + chi0    CB(i)   CA(i)    N(i)      C(i)",
"# ALANINE",
"name ala numatm 5",
"n      1.329  116.200  180.000   -3   -2   -4 + none    N(i)    C(i-1)   CA(i-1)   N(i-1)",
"ca     1.458  121.700  180.000    1   -3   -2 + omega   CA(i)   N(i)     C(i-1)    CA(i-1)",
"c      1.525  111.200  -64.000    2    1   -3 + phi     C(i)    CA(i)    N(i)      C(i-1) ",
"o      1.231  120.800  137.000    3    2    1 + psi     O(i)    C(i)     CA(i)     N(i)",
"cb     1.521  110.400 -122.000    2    1    3 + chi0    CB(i)   CA(i)    N(i)      C(i)",
"# ARGININE",
"name arg numatm 11",
"n      1.329  116.200  180.000   -3   -2   -4 + none    N(i)    C(i-1)   CA(i-1)   N(i-1)",
"ca     1.458  121.700  180.000    1   -3   -2 + omega   CA(i)   N(i)     C(i-1)    CA(i-1)",
"c      1.525  111.200  -64.000    2    1   -3 + phi     C(i)    CA(i)    N(i)      C(i-1) ",
"o      1.231  120.800  137.000    3    2    1 + psi     O(i)    C(i)     CA(i)     N(i)",
"cb     1.530  110.500 -122.000    2    1    3 + chi0    CB(i)   CA(i)    N(i)      C(i)",
"cg     1.520  114.100  180.000    5    2    1 + chi1",
"cd     1.520  111.300  180.000    6    5    2 + chi2",
"ne     1.460  112.000  180.000    7    6    5 + chi3",
"cz     1.329  124.200  -90.000    8    7    6 + chi4",
"nh1    1.326  120.000  180.000    9    8    7 + chi5",
"nh2    1.326  120.000  180.000    9    8   10 + none",
"#ASPARAGINE",
"name asn numatm 8",
"n      1.329  116.200  180.000   -3   -2   -4 + none    N(i)    C(i-1)   CA(i-1)   N(i-1)",
"ca     1.458  121.700  180.000    1   -3   -2 + omega   CA(i)   N(i)     C(i-1)    CA(i-1)",
"c      1.525  111.200  -64.000    2    1   -3 + phi     C(i)    CA(i)    N(i)      C(i-1) ",
"o      1.231  120.800  137.000    3    2    1 + psi     O(i)    C(i)     CA(i)     N(i)",
"cb     1.530  110.500 -122.000    2    1    3 + chi0    CB(i)   CA(i)    N(i)      C(i)",
"cg     1.520  110.500  180.000    5    2    1 + chi1",
"od1    1.231  120.800  -90.000    6    5    2 + none",
"nd2    1.328  116.400  180.000    6    5    7 + none",
"#ASPARTATE",
"name asp numatm 8",
"n      1.329  116.200  180.000   -3   -2   -4 + none    N(i)    C(i-1)   CA(i-1)   N(i-1)",
"ca     1.458  121.700  180.000    1   -3   -2 + omega   CA(i)   N(i)     C(i-1)    CA(i-1)",
"c      1.525  111.200  -64.000    2    1   -3 + phi     C(i)    CA(i)    N(i)      C(i-1) ",
"o      1.231  120.800  137.000    3    2    1 + psi     O(i)    C(i)     CA(i)     N(i)",
"cb     1.530  110.500 -122.000    2    1    3 + chi0    CB(i)   CA(i)    N(i)      C(i)",
"cg     1.520  110.500  180.000    5    2    1 + chi1",
"od1    1.249  118.400  -90.000    6    5    2 + none",
"od2    1.249  118.400  180.000    6    5    7 + none",
"# CYSTEINE",
"name cys numatm 6",
"n      1.329  116.200  180.000   -3   -2   -4 + none    N(i)    C(i-1)   CA(i-1)   N(i-1)",
"ca     1.458  121.700  180.000    1   -3   -2 + omega   CA(i)   N(i)     C(i-1)    CA(i-1)",
"c      1.525  111.200  -64.000    2    1   -3 + phi     C(i)    CA(i)    N(i)      C(i-1) ",
"o      1.231  120.800  137.000    3    2    1 + psi     O(i)    C(i)     CA(i)     N(i)",
"cb     1.530  110.500 -122.000    2    1    3 + chi0    CB(i)   CA(i)    N(i)      C(i)",
"sg     1.822  114.400  180.000    5    2    1 + chi1",
"#GLUTAMINE",
"name gln numatm 9",
"n      1.329  116.200  180.000   -3   -2   -4 + none    N(i)    C(i-1)   CA(i-1)   N(i-1)",
"ca     1.458  121.700  180.000    1   -3   -2 + omega   CA(i)   N(i)     C(i-1)    CA(i-1)",
"c      1.525  111.200  -64.000    2    1   -3 + phi     C(i)    CA(i)    N(i)      C(i-1) ",
"o      1.231  120.800  137.000    3    2    1 + psi     O(i)    C(i)     CA(i)     N(i)",
"cb     1.530  110.500 -122.000    2    1    3 + chi0    CB(i)   CA(i)    N(i)      C(i)",
"cg     1.540  110.500  180.000    5    2    1 + chi1",
"cd     1.516  112.600  180.000    6    5    2 + chi2",
"oe1    1.231  120.800  -90.000    7    6    5 + chi3",
"ne2    1.328  116.400  180.000    7    6    8 + none",
"#GLUTAMATE",
"name glu numatm 9",
"n      1.329  116.200  180.000   -3   -2   -4 + none    N(i)    C(i-1)   CA(i-1)   N(i-1)",
"ca     1.458  121.700  180.000    1   -3   -2 + omega   CA(i)   N(i)     C(i-1)    CA(i-1)",
"c      1.525  111.200  -64.000    2    1   -3 + phi     C(i)    CA(i)    N(i)      C(i-1) ",
"o      1.231  120.800  137.000    3    2    1 + psi     O(i)    C(i)     CA(i)     N(i)",
"cb     1.530  110.500 -122.000    2    1    3 + chi0    CB(i)   CA(i)    N(i)      C(i)",
"cg     1.540  110.500  180.000    5    2    1 + chi1",
"cd     1.516  112.600  180.000    6    5    2 + chi2",
"oe1    1.249  118.400  -90.000    7    6    5 + chi3",
"oe2    1.249  118.400  180.000    7    6    8 + none",
"#GLYCINE",
"name gly numatm 4",
"n      1.329  116.400  180.000   -3   -2   -4 + none    N(i)    C(i-1)   CA(i-1)   N(i-1)",
"ca     1.451  120.600  180.000    1   -3   -2 + omega   CA(i)   N(i)     C(i-1)    CA(i-1)",
"c      1.516  112.500  -64.000    2    1   -3 + phi     C(i)    CA(i)    N(i)      C(i-1) ",
"o      1.231  120.800  137.000    3    2    1 + psi     O(i)    C(i)     CA(i)     N(i)",
"#HISTIDINE",
"name his numatm 10",
"n      1.329  116.200  180.000   -3   -2   -4 + none    N(i)    C(i-1)   CA(i-1)   N(i-1)",
"ca     1.458  121.700  180.000    1   -3   -2 + omega   CA(i)   N(i)     C(i-1)    CA(i-1)",
"c      1.525  111.200  -64.000    2    1   -3 + phi     C(i)    CA(i)    N(i)      C(i-1) ",
"o      1.231  120.800  137.000    3    2    1 + psi     O(i)    C(i)     CA(i)     N(i)",
"cb     1.530  110.500 -122.000    2    1    3 + chi0    CB(i)   CA(i)    N(i)      C(i)",
"cg     1.504  112.800  180.000    5    2    1 + chi1",
"nd1    1.389  122.100   90.000    6    5    2 + chi2",
"cd2    1.372  129.900  180.000    6    5    7 + none",
"ce1    1.333  106.500    0.000    7    6    8 + none",
"ne2    1.374  107.200    0.000    8    6    7 + none",
"#cc     2.685  113.767    2.473    5    2    6 + none",
"#ISO LEUCINE",
"name ile numatm 8",
"n      1.329  116.200  180.000   -3   -2   -4 + none    N(i)    C(i-1)   CA(i-1)   N(i-1)",
"ca     1.458  121.700  180.000    1   -3   -2 + omega   CA(i)   N(i)     C(i-1)    CA(i-1)",
"c      1.525  111.200  -64.000    2    1   -3 + phi     C(i)    CA(i)    N(i)      C(i-1) ",
"o      1.231  120.800  137.000    3    2    1 + psi     O(i)    C(i)     CA(i)     N(i)",
"cb     1.540  111.500 -122.000    2    1    3 + chi0    CB(i)   CA(i)    N(i)      C(i)",
"cg1    1.530  110.400  -60.000    5    2    1 + chi1",
"cg2    1.521  110.500 -122.600    5    2    6 + none",
"cd1    1.513  113.800  180.000    6    5    2 + chi2",
"#cc     2.100  111.500 -122.000    2    1    3 + none",
"",
"#LEUCINE",
"name leu numatm 8",
"n      1.329  116.200  180.000   -3   -2   -4 + none    N(i)    C(i-1)   CA(i-1)   N(i-1)",
"ca     1.458  121.700  180.000    1   -3   -2 + omega   CA(i)   N(i)     C(i-1)    CA(i-1)",
"c      1.525  111.200  -64.000    2    1   -3 + phi     C(i)    CA(i)    N(i)      C(i-1) ",
"o      1.231  120.800  137.000    3    2    1 + psi     O(i)    C(i)     CA(i)     N(i)",
"cb     1.530  110.500 -122.000    2    1    3 + chi0    CB(i)   CA(i)    N(i)      C(i)",
"cg     1.530  116.300  180.000    5    2    1 + chi1",
"cd1    1.521  110.700  180.000    6    5    2 + chi2",
"cd2    1.521  110.700 -120.000    6    5    7 + none",
"#cc     2.176  110.500    0.000    5    2    6 + none",
"#LYSINE",
"name lys numatm 9",
"n      1.329  116.200  180.000   -3   -2   -4 + none    N(i)    C(i-1)   CA(i-1)   N(i-1)",
"ca     1.458  121.700  180.000    1   -3   -2 + omega   CA(i)   N(i)     C(i-1)    CA(i-1)",
"c      1.525  111.200  -64.000    2    1   -3 + phi     C(i)    CA(i)    N(i)      C(i-1) ",
"o      1.231  120.800  137.000    3    2    1 + psi     O(i)    C(i)     CA(i)     N(i)",
"cb     1.530  110.500 -122.000    2    1    3 + chi0    CB(i)   CA(i)    N(i)      C(i)",
"cg     1.549  113.405  180.000    5    2    1 + chi1",
"cd     1.532  114.844  180.000    6    5    2 + chi2",
"ce     1.534  109.216  180.000    7    6    5 + chi3",
"nz     1.474  109.365  180.000    8    7    6 + chi4",
"#METHIONINE",
"name met numatm 8",
"n      1.329  116.200  180.000   -3   -2   -4 + none    N(i)    C(i-1)   CA(i-1)   N(i-1)",
"ca     1.458  121.700  180.000    1   -3   -2 + omega   CA(i)   N(i)     C(i-1)    CA(i-1)",
"c      1.525  111.200  -64.000    2    1   -3 + phi     C(i)    CA(i)    N(i)      C(i-1) ",
"o      1.231  120.800  137.000    3    2    1 + psi     O(i)    C(i)     CA(i)     N(i)",
"cb     1.530  110.500 -122.000    2    1    3 + chi0    CB(i)   CA(i)    N(i)      C(i)",
"cg     1.544  115.840  -60.000    5    2    1 + chi1",
"sd     1.801  109.771 -180.000    6    5    2 + chi2",
"ce     1.798   98.504 -180.000    7    6    5 + chi3",
"#cc     2.176  110.500    0.000    5    2    6 + none",
"#PHENYL ALANINE",
"name phe numatm 12",
"n      1.329  116.200  180.000   -3   -2   -4 + none    N(i)    C(i-1)   CA(i-1)   N(i-1)",
"ca     1.458  121.700  180.000    1   -3   -2 + omega   CA(i)   N(i)     C(i-1)    CA(i-1)",
"c      1.525  111.200  -64.000    2    1   -3 + phi     C(i)    CA(i)    N(i)      C(i-1) ",
"o      1.231  120.800  137.000    3    2    1 + psi     O(i)    C(i)     CA(i)     N(i)",
"cb     1.530  110.500 -122.000    2    1    3 + chi0    CB(i)   CA(i)    N(i)      C(i)",
"cg     1.537  116.432  180.000    5    2    1 + chi1",
"cd1    1.399  119.610   90.000    6    5    2 + chi2",
"cd2    1.397  120.764  180.000    6    5    7 + none",
"ce1    1.391  120.267  180.000    7    6    5 + none",
"ce2    1.392  120.096  180.000    8    6    5 + none",
"cz     1.393  120.063   -0.000   10    8    6 + none",
"cc     2.934  116.425    0.000    5    2    6 + none",
"#PROLINE",
"name pro numatm 7",
"n      1.341  116.900  180.000   -3   -2   -4 + none    N(i)    C(i-1)   CA(i-1)   N(i-1)",
"ca     1.466  122.600  180.000    1   -3   -2 + omega   CA(i)   N(i)     C(i-1)    CA(i-1)",
"c      1.525  111.800  -64.000    2    1   -3 + phi     C(i)    CA(i)    N(i)      C(i-1) ",
"o      1.231  120.800  137.000    3    2    1 + psi     O(i)    C(i)     CA(i)     N(i)",
"cb     1.530  103.000 -122.000    2    1    3 + chi0    CB(i)   CA(i)    N(i)      C(i)",
"cg     1.529  103.600   29.200    5    2    1 + none",
"cd     1.529  103.100  -33.700    6    5    2 + none",
"#SERINE",
"name ser numatm 6",
"n      1.329  116.200  180.000   -3   -2   -4 + none    N(i)    C(i-1)   CA(i-1)   N(i-1)",
"ca     1.458  121.700  180.000    1   -3   -2 + omega   CA(i)   N(i)     C(i-1)    CA(i-1)",
"c      1.525  111.200  -64.000    2    1   -3 + phi     C(i)    CA(i)    N(i)      C(i-1) ",
"o      1.231  120.800  137.000    3    2    1 + psi     O(i)    C(i)     CA(i)     N(i)",
"cb     1.530  110.500 -122.000    2    1    3 + chi0    CB(i)   CA(i)    N(i)      C(i)",
"og     1.417  111.100  180.000    5    2    1 + chi1",
"#THREONINE",
"name thr numatm 7",
"n      1.329  116.200  180.000   -3   -2   -4 + none    N(i)    C(i-1)   CA(i-1)   N(i-1)",
"ca     1.458  121.700  180.000    1   -3   -2 + omega   CA(i)   N(i)     C(i-1)    CA(i-1)",
"c      1.525  111.200  -64.000    2    1   -3 + phi     C(i)    CA(i)    N(i)      C(i-1) ",
"o      1.231  120.800  137.000    3    2    1 + psi     O(i)    C(i)     CA(i)     N(i)",
"cb     1.540  111.500 -122.000    2    1    3 + chi0    CB(i)   CA(i)    N(i)      C(i)",
"og1    1.433  109.600  180.000    5    2    1 + chi1",
"cg2    1.521  109.300 -121.628    5    2    6 + none",
"#TRYPTOPHAN",
"name trp numatm 15",
"n      1.329  116.200  180.000   -3   -2   -4 + none    N(i)    C(i-1)   CA(i-1)   N(i-1)",
"ca     1.458  121.700  180.000    1   -3   -2 + omega   CA(i)   N(i)     C(i-1)    CA(i-1)",
"c      1.525  111.200  -64.000    2    1   -3 + phi     C(i)    CA(i)    N(i)      C(i-1) ",
"o      1.231  120.800  137.000    3    2    1 + psi     O(i)    C(i)     CA(i)     N(i)",
"cb     1.530  110.500 -122.000    2    1    3 + chi0    CB(i)   CA(i)    N(i)      C(i)",
"cg     1.532  114.792  180.000    5    2    1 + chi1",
"cd1    1.411  125.650   90.000    6    5    2 + chi2",
"cd2    1.379  128.287 -180.000    6    5    7 + none",
"ne1    1.409  111.580 -180.000    7    6    5 + none",
"ce2    1.385  103.850    0.000    9    7    6 + none",
"ce3    1.418  131.532 -180.000    8    6   10 + none",
"cz2    1.413  129.239  180.000   10    9    7 + none",
"cz3    1.395  118.392  180.000   11    8    6 + none",
"ch2    1.403  121.062   -0.000   13   11    8 + none",
"cc     3.990  111.393  -32.038    5    2    6 + none",
"#TYROSINE",
"name tyr numatm 13",
"n      1.329  116.200  180.000   -3   -2   -4 + none    N(i)    C(i-1)   CA(i-1)   N(i-1)",
"ca     1.458  121.700  180.000    1   -3   -2 + omega   CA(i)   N(i)     C(i-1)    CA(i-1)",
"c      1.525  111.200  -64.000    2    1   -3 + phi     C(i)    CA(i)    N(i)      C(i-1) ",
"o      1.231  120.800  137.000    3    2    1 + psi     O(i)    C(i)     CA(i)     N(i)",
"cb     1.530  110.500 -122.000    2    1    3 + chi0    CB(i)   CA(i)    N(i)      C(i)",
"cg     1.537  116.432  180.000    5    2    1 + chi1",
"cd1    1.399  119.610   90.000    6    5    2 + chi2",
"cd2    1.397  120.764  180.000    6    5    7 + none",
"ce1    1.391  120.267  180.000    7    6    5 + none",
"ce2    1.392  120.096  180.000    8    6    5 + none",
"cz     1.393  120.063    0.000   10    8    6 + none",
"oh     1.366  119.871  180.000   11   10    8 + none",
"cc     2.934  116.425    0.000    5    2    6 + none",
"#VALINE",
"name val numatm 7",
"n      1.329  116.200  180.000   -3   -2   -4 + none    N(i)    C(i-1)   CA(i-1)   N(i-1)",
"ca     1.458  121.700  180.000    1   -3   -2 + omega   CA(i)   N(i)     C(i-1)    CA(i-1)",
"c      1.525  111.200  -64.000    2    1   -3 + phi     C(i)    CA(i)    N(i)      C(i-1) ",
"o      1.231  120.800  137.000    3    2    1 + psi     O(i)    C(i)     CA(i)     N(i)",
"cb     1.540  111.500 -122.000    2    1    3 + chi0    CB(i)   CA(i)    N(i)      C(i)",
"cg1    1.521  110.500  180.000    5    2    1 + chi1",
"cg2    1.521  110.500  125.136    5    2    6 + none   ",
"#cc     2.100  111.500 -122.000    2    1    3 + none",
"#",
"# n-terminal acetyl group",
"#",
"name ace numatm 7",
"du     0.000    0.000    0.000    0    0    0 - none",
"ch3    1.000    0.000    0.000    1    0    0 + none",
"c      1.520  110.000    0.000    2    1    0 + none",
"o      1.231  120.500    0.000    3    2    1 + none",
"1hh3   1.080  109.600    0.000    2    3    4 - none",
"2hh3   1.080  109.600  120.000    2    3    5 - none",
"3hh3   1.080  109.600  120.000    2    3    6 - none",
"#",
"# c-terminal n-methyl amide",
"#",
"name nme numatm 6",
"n      1.329  116.200  180.000   -3   -2   -4 + none",
"ch3    1.458  121.700  180.000    1   -3   -2 + none",
"h      1.008  119.150  180.000    1   -3    2 + none",
"1hh3   1.080  110.000  120.000    2    1    3 - none",
"2hh3   1.080  110.000  120.000    2    1    4 - none",
"3hh3   1.080  110.000  120.000    2    1    5 - none",
"#",
"# c-terminal amide",
"#",
"name nh2 numatm 3",
"n      1.328  116.400  180.000   -3   -2   -4 + none",
"1h     1.008  121.800  180.000    1   -3   -2 + none",
"2h     1.008  121.800  180.000    1   -3    2 + none"
};

} // namespace leri
#endif // LERI_SEQUENCE_H_
